unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls;

type
  TWizardForm = class(TForm)
    Notebook: TNotebook;
    WelcomeStepPanel: TPanel;
    WelcomeStepLabel: TLabel;
    WelcomeStepTitle: TLabel;
    WelcomeStepImage: TImage;
    WelcomeLabel: TLabel;
    SrcStepPanel: TPanel;
    SrcStepLabel: TLabel;
    SrcStepTitle: TLabel;
    SrcStepImage: TImage;
    SrcCheckPanel: TPanel;
    SrcDriverCheck: TRadioButton;
    SrcDriverLabel: TLabel;
    SrcIniCheck: TRadioButton;
    SrcIniLabel: TLabel;
    SrcRegCheck: TRadioButton;
    SrcRegRootLabel: TLabel;
    SrcRegSectionLabel: TLabel;
    SrcDriverCombo: TComboBox;
    SrcIniEdit: TEdit;
    SrcIniButton: TButton;
    SrcRegRootCombo: TComboBox;
    SrcRegSectionEdit: TEdit;
    DestStepPanel: TPanel;
    DestStepLabel: TLabel;
    DestStepTitle: TLabel;
    DestStepImage: TImage;
    DestCheckPanel: TPanel;
    DestIniCheck: TRadioButton;
    DestIniLabel: TLabel;
    DestRegCheck: TRadioButton;
    DestRegRootLabel: TLabel;
    DestRegSectionLabel: TLabel;
    DestIniEdit: TEdit;
    DestIniButton: TButton;
    DestRegRootCombo: TComboBox;
    DestRegSectionEdit: TEdit;
    ReadyStepPanel: TPanel;
    ReadyStepLabel: TLabel;
    ReadyStepTitle: TLabel;
    ReadyStepImage: TImage;
    ReadyLabel: TLabel;
    FinishStepPanel: TPanel;
    FinishStepLabel: TLabel;
    FinishStepTitle: TLabel;
    FinishStepImage: TImage;
    FinishLabel: TLabel;
    VMLBevel: TBevel;
    VMLTitle: TLabel;
    BackButton: TButton;
    NextButton: TButton;
    CancelButton: TButton;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure SrcCheckClick(Sender: TObject);
    procedure SrcDriverComboChange(Sender: TObject);
    procedure SrcIniButtonClick(Sender: TObject);
    procedure DestIniEditChange(Sender: TObject);
    procedure DestIniButtonClick(Sender: TObject);
    procedure BackButtonClick(Sender: TObject);
    procedure NextButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
  private
    FAutoFileName: Boolean;
  public
    { Public declarations }
  end;

var
  WizardForm: TWizardForm;

implementation

uses
  VML_Modem, VML_Idents, VML_IniFile, VML_NoModem, VML_IS101,
  VML_USRobotics, VML_ZyXEL1496, VML_ZyXEL2864, VML_MotorolaSM56,
  VML_CLMD1414, VML_Rockwell, VML_IDC2814, VML_IDC2814VR, VML_AVC33600;

{$R *.DFM}

const
  RootKeyMap: array[0..1] of TIdentMapEntry = (
    (Value: Longint(HKEY_CURRENT_USER); Name: 'HKEY_CURRENT_USER'),
    (Value: Longint(HKEY_LOCAL_MACHINE); Name: 'HKEY_LOCAL_MACHINE'));

{ TWizardForm }

procedure TWizardForm.FormCreate(Sender: TObject);
begin
  GetModemDriversStringList(SrcDriverCombo.Items);
  GetIdentsStringList(RootKeyMap, SrcRegRootCombo.Items);
  GetIdentsStringList(RootKeyMap, DestRegRootCombo.Items);
end;

procedure TWizardForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
const
  SCloseQuery = 'Do you want to quit the program?';
  Flags = MB_YESNO or MB_ICONQUESTION;
begin
  if Notebook.PageIndex <> 4 then
    if Application.MessageBox(SCloseQuery, PChar(Caption), Flags) <> IDYES then
      CanClose := False;
end;

procedure TWizardForm.SrcCheckClick(Sender: TObject);
var
  I: Integer;
  Page: TControl;

  procedure EnableControls(WinControl: TWinControl);
  var
    I: Integer;
    Control: TControl;
  begin
    for I := 0 to WinControl.ControlCount - 1 do
    begin
      Control := WinControl.Controls[I];
      if (not (Control is TRadioButton)) and (Control.Tag <> 0) then
        Control.Enabled := Control.Tag = TControl(Sender).Tag;
      if Control is TWinControl then
        EnableControls(TWinControl(Control));
    end;
  end;

begin
  for I := 0 to Notebook.ControlCount - 1 do
  begin
    Page := Notebook.Controls[I];
    if TPage(Page).ContainsControl(TControl(Sender)) then
      EnableControls(TPage(Page));
  end;
end;

procedure TWizardForm.SrcDriverComboChange(Sender: TObject);
begin
  if SrcDriverCombo.ItemIndex <> -1 then
    if (Length(DestIniEdit.Text) = 0) or FAutoFileName then
    begin
      DestIniEdit.Text := ExtractFilePath(ParamStr(0));
      DestIniEdit.Text := DestIniEdit.Text + SrcDriverCombo.Text + '.ini';
      FAutoFileName := True;
    end;
end;

procedure TWizardForm.SrcIniButtonClick(Sender: TObject);
begin
  with OpenDialog do
  begin
    FileName := '';
    if Execute then SrcIniEdit.Text := FileName;
  end;
end;

procedure TWizardForm.DestIniEditChange(Sender: TObject);
begin
  FAutoFileName := False;
end;

procedure TWizardForm.DestIniButtonClick(Sender: TObject);
begin
  with SaveDialog do
  begin
    FileName := DestIniEdit.Text;
    if Execute then DestIniEdit.Text := FileName;
  end;
end;

procedure TWizardForm.BackButtonClick(Sender: TObject);
begin
  { Change page }
  Notebook.PageIndex := Notebook.PageIndex  - 1;
  { Enable Back button }
  BackButton.Enabled := Notebook.PageIndex > 0;
end;

procedure TWizardForm.NextButtonClick(Sender: TObject);
const
  SDriverError = 'Please select the modem name.';
  SDriverClassError = 'No driver class registered for selected modem name.';
  SIniFileError = 'Please enter the driver ini file name.';
  SFileNotFound = 'File %s not found.';
  SRootKeyError = 'Please select the registry root key.';
  SRootKeyInvalid = 'Selected registry root key is not valid.';
  SSectionError = 'Please enter the registry section name.';
  SSourceError = 'Please select the driver source.';
  SDestinationError = 'Please select the driver destination.';
var
  Error: string;
  Driver: TModemDriver;
  DriverClass: TModemDriverClass;
  RootKey: Longint;
begin
  { Enable Back button }
  BackButton.Enabled := True;
  { Check step params }
  case Notebook.PageIndex of
    1:
      { Check source params }
      begin
        if SrcDriverCheck.Checked then
        begin
          { Check driver selected }
          if SrcDriverCombo.ItemIndex = -1 then
          begin
            Application.MessageBox(SDriverError, PChar(Caption), MB_ICONERROR);
            { Focus driver combobox }
            SrcDriverCombo.SetFocus;
            Abort;
          end;
          { Check driver class }
          if not GetModemDriver(SrcDriverCombo.Text, DriverClass) then
          begin
            Application.MessageBox(SDriverClassError, PChar(Caption), MB_ICONERROR);
            { Focus driver combobox }
            SrcDriverCombo.SetFocus;
            Abort;
          end;
        end
        else if SrcIniCheck.Checked then
        begin
          { Check ini file name entered }
          if Length(SrcIniEdit.Text) = 0 then
          begin
            Application.MessageBox(SIniFileError, PChar(Caption), MB_ICONERROR);
            { Focus file name edit }
            SrcIniEdit.SetFocus;
            Abort;
          end;
          { Check ini file exists }
          if not FileExists(SrcIniEdit.Text) then
          begin
            Error := Format(SFileNotFound, [SrcIniEdit.Text]);
            Application.MessageBox(PChar(Error), PChar(Caption), MB_ICONERROR);
            { Focus file name edit }
            SrcIniEdit.SetFocus;
            Abort;
          end;
        end
        else if SrcRegCheck.Checked then
        begin
          { Check registry root key selected }
          if SrcRegRootCombo.ItemIndex = -1 then
          begin
            Application.MessageBox(SRootKeyError, PChar(Caption), MB_ICONERROR);
            { Focus registry root key combobox }
            SrcRegRootCombo.SetFocus;
            Abort;
          end;
          { Check registry root key }
          if not IdentToInt(SrcRegRootCombo.Text, RootKey, RootKeyMap) then
          begin
            Application.MessageBox(SRootKeyInvalid, PChar(Caption), MB_ICONERROR);
            { Focus registry root key combobox }
            SrcRegRootCombo.SetFocus;
            Abort;
          end;
          { Check registry section specified }
          if Length(SrcRegSectionEdit.Text) = 0 then
          begin
            Application.MessageBox(SSectionError, PChar(Caption), MB_ICONERROR);
            { Focus registry section edit }
            SrcRegSectionEdit.SetFocus;
            Abort;
          end;
        end
        else
        begin
          Application.MessageBox(SSourceError, PChar(Caption), MB_ICONERROR);
          Abort;
        end;
      end;
    2:
      begin
        { Check destination params }
        if DestIniCheck.Checked then
        begin
          { Check ini file name selected }
          if Length(DestIniEdit.Text) = 0 then
          begin
            Application.MessageBox(SIniFileError, PChar(Caption), MB_ICONERROR);
            { Focus file name edit }
            DestIniEdit.SetFocus;
            Abort;
          end;
        end
        else if DestRegCheck.Checked then
        begin
          { Check registry root key selected }
          if DestRegRootCombo.ItemIndex = -1 then
          begin
            Application.MessageBox(SRootKeyError, PChar(Caption), MB_ICONERROR);
            { Focus registry root key combobox }
            DestRegRootCombo.SetFocus;
            Abort;
          end;
          { Check registry root key }
          if not IdentToInt(DestRegRootCombo.Text, RootKey, RootKeyMap) then
          begin
            Application.MessageBox(SRootKeyInvalid, PChar(Caption), MB_ICONERROR);
            { Focus registry root key combobox }
            DestRegRootCombo.SetFocus;
            Abort;
          end;
          { Check registry section specified }
          if Length(DestRegSectionEdit.Text) = 0 then
          begin
            Application.MessageBox(SSectionError, PChar(Caption), MB_ICONERROR);
            { Focus registry section edit }
            DestRegSectionEdit.SetFocus;
            Abort;
          end;
        end
        else
        begin
          Application.MessageBox(SDestinationError, PChar(Caption), MB_ICONERROR);
          Abort;
        end;
        { Prepare "Ready to convert" page }
        with ReadyLabel do
        begin
          Caption := 'Ready to convert the driver.'#13#13;
          { Add driver source information }
          Caption := Caption + 'Source'#13;
          if SrcDriverCheck.Checked then
          begin
            Caption := Caption + '    Registered driver'#13;
            Caption := Caption + '    ' + SrcDriverCombo.Text + #13#13;
          end
          else if SrcIniCheck.Checked then
          begin
            Caption := Caption + '    Driver ini file'#13;
            Caption := Caption + '    ' + SrcIniEdit.Text + #13#13;
          end
          else if SrcRegCheck.Checked then
          begin
            Caption := Caption + '    Driver registry entry'#13;
            Caption := Caption + '    ' + SrcRegRootCombo.Text;
            Caption := Caption + '\' + SrcRegSectionEdit.Text + #13#13;
          end;
          { Add driver destination information }
          Caption := Caption + 'Destination'#13;
          if DestIniCheck.Checked then
          begin
            Caption := Caption + '    Driver ini file'#13;
            Caption := Caption + '    ' + DestIniEdit.Text;
          end
          else if DestRegCheck.Checked then
          begin
            Caption := Caption + '    Driver registry enrty'#13;
            Caption := Caption + '    ' + DestRegRootCombo.Text;
            Caption := Caption + '\' + DestRegSectionEdit.Text;
          end;
        end;
      end;
    3:
      { Convert the driver }
      begin
        Driver := nil;
        { Load the driver from selected source }
        if SrcDriverCheck.Checked then
        begin
          { Get driver class, create the driver instance }
          GetModemDriver(SrcDriverCombo.Text, DriverClass);
          Driver := DriverClass.Create(nil);
        end
        else if SrcIniCheck.Checked then
          { Load driver from ini file }
          Driver := ReadDriverIniFile(SrcIniEdit.Text, nil)
        else if SrcRegCheck.Checked then
        begin
          { Get source root key, load driver from registry }
          IdentToInt(SrcRegRootCombo.Text, RootKey, RootKeyMap);
          Driver := ReadDriverRegistry(RootKey, SrcRegSectionEdit.Text, nil);
        end;
        { Save driver to the selected destination }
        try
          if DestIniCheck.Checked then
            { Save driver to the ini file }
            WriteDriverIniFile(DestIniEdit.Text, Driver)
          else if DestRegCheck.Checked then
          begin
            { Get destination root key, save driver to the registry }
            IdentToInt(DestRegRootCombo.Text, RootKey, RootKeyMap);
            WriteDriverRegistry(RootKey, DestRegSectionEdit.Text, Driver);
          end;
        finally
          Driver.Free;
        end;
        { Converting complete }
        BackButton.Enabled := False;
        NextButton.Caption := '&More >';
        CancelButton.Caption := 'Close';
      end;
    4:
      { Convert one more driver }
      begin
        NextButton.Caption := '&Next >';
        CancelButton.Caption := 'Cancel';
        Notebook.PageIndex := 1;
        Exit;
      end;
  end;
  { Change page }
  Notebook.PageIndex := Notebook.PageIndex  + 1;
end;

procedure TWizardForm.CancelButtonClick(Sender: TObject);
begin
  Close;
end;

end.

