unit ConnectionDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, VML_Modem;

type
  TConnectionForm = class(TForm)
    PortGroup: TGroupBox;
    PortTimeOutLabel: TLabel;
    PortTimeOutEdit: TEdit;
    PortTimeOutUpDown: TUpDown;
    PortPollingDelayLabel: TLabel;
    PortPollingDelayEdit: TEdit;
    PortPollingDelayUpDown: TUpDown;
    PortBufferSizeLabel: TLabel;
    PortBufferSizeEdit: TEdit;
    PortBufferSizeUpDown: TUpDown;
    InitGroup: TGroupBox;
    InitCmdEdit: TEdit;
    InitLogCheck: TCheckBox;
    OKButton: TButton;
    CancelButton: TButton;
    PortTimeOutLabel2: TLabel;
    PortPollingDelayLabel2: TLabel;
    PortBufferSizeLabel2: TLabel;
    ComGroup: TGroupBox;
    ComDataBitsLabel: TLabel;
    ComParityLabel: TLabel;
    ComStopBitsLabel: TLabel;
    ComDataBitsCombo: TComboBox;
    ComParityCombo: TComboBox;
    ComStopBitsCombo: TComboBox;
    procedure FormShow(Sender: TObject);
    procedure PortEditKeyPress(Sender: TObject; var Key: Char);
    procedure OKButtonClick(Sender: TObject);
  private
    FForm: TForm;
    FProfile: TModemProfile;
  public
    { Public declarations }
  end;

{ Show connection settings dialog }

function ShowConnectionDlg(Form: TForm; Profile: TModemProfile): Boolean;

implementation

uses
  VML_Idents;

{$R *.DFM}

const
  DataBitsMap: array[0..4] of TIdentMapEntry = (
    (Value: 4; Name: '4'),
    (Value: 5; Name: '5'),
    (Value: 6; Name: '6'),
    (Value: 7; Name: '7'),
    (Value: 8; Name: '8'));

  ParityMap: array[0..4] of TIdentMapEntry = (
    (Value: NOPARITY; Name: 'No parity'),
    (Value: ODDPARITY; Name: 'Odd'),
    (Value: EVENPARITY; Name: 'Even'),
    (Value: MARKPARITY; Name: 'Mark'),
    (Value: SPACEPARITY; Name: 'Space'));

  StopBitsMap: array[0..2] of TIdentMapEntry = (
    (Value: ONESTOPBIT; Name: '1'),
    (Value: ONE5STOPBITS; Name: '1.5'),
    (Value: TWOSTOPBITS; Name: '2'));

{ Show connection settings dialog }

function ShowConnectionDlg(Form: TForm; Profile: TModemProfile): Boolean;
begin
  with TConnectionForm.Create(nil) do
  try
    FForm := Form;
    FProfile := Profile;
    { Show dialog }
    Result := ShowModal = mrOK;
  finally
    Free;
  end;
end;

{ TConnectionForm }

procedure TConnectionForm.FormShow(Sender: TObject);
var
  R: TRect;
  X, Y: Integer;
begin
  SystemParametersInfo(SPI_GETWORKAREA, 0, @R, 0);
  { Select form position }
  X := FForm.ClientOrigin.X + 15;
  if X < R.Left then X := R.Left;
  if X + Width > R.Right then X := R.Right - Width;
  Y := FForm.ClientOrigin.Y + 36;
  if Y < R.Top + 5 then Y := R.Top + 5;
  if Y + Height > R.Bottom - 5 then Y := R.Bottom - 5 - Height;
  { Change form position }
  Left := X;
  Top := Y;
  { Refresh controls }
  with FProfile do
  begin
    { Data bits }
    with ComDataBitsCombo do
    begin
      GetIdentsStringList(DataBitsMap, Items);
      ItemIndex := GetIdentValueIndex(PortDataBits, DataBitsMap);
    end;
    { Parity }
    with ComParityCombo do
    begin
      GetIdentsStringList(ParityMap, Items);
      ItemIndex := GetIdentValueIndex(PortParity, ParityMap);
    end;
    { Stop bits }
    with ComStopBitsCombo do
    begin
      GetIdentsStringList(StopBitsMap, Items);
      ItemIndex := GetIdentValueIndex(PortStopBits, StopBitsMap);
    end;
    { Port settings }
    PortTimeOutEdit.Text := IntToStr(PortTimeOut);
    PortPollingDelayEdit.Text := IntToStr(PortStatusDelay);
    PortBufferSizeEdit.Text := IntToStr(PortInBuffSize);
    { Modem init command }
    InitCmdEdit.Text := ModemInitCmd;
    InitLogCheck.Checked := ModemLogging;
  end;
end;

procedure TConnectionForm.PortEditKeyPress(Sender: TObject; var Key: Char);
begin
  if not (Key in ['0'..'9', #8]) then Key := #0;
end;

procedure TConnectionForm.OKButtonClick(Sender: TObject);
begin
  { Update profile settings }
  with FProfile do
  begin
    PortDataBits := IdentToIntDef(ComDataBitsCombo.Text, PortDataBits, DataBitsMap);
    PortParity := IdentToIntDef(ComParityCombo.Text, PortParity, ParityMap);
    PortStopBits := IdentToIntDef(ComStopBitsCombo.Text, PortStopBits, StopBitsMap);
    PortDataBits := IdentToIntDef(ComDataBitsCombo.Text, PortDataBits, DataBitsMap);
    PortParity := IdentToIntDef(ComParityCombo.Text, PortParity, ParityMap);
    PortStopBits := IdentToIntDef(ComStopBitsCombo.Text, PortStopBits, StopBitsMap);
    PortTimeOut := StrToIntDef(PortTimeOutEdit.Text, PortTimeOut);
    PortStatusDelay := StrToIntDef(PortPollingDelayEdit.Text, PortStatusDelay);
    PortReadDelay := PortStatusDelay;
    PortWriteDelay := PortReadDelay;
    PortInBuffSize := StrToIntDef(PortBufferSizeEdit.Text, PortInBuffSize);
    PortOutBuffSize := PortInBuffSize;
    ModemInitCmd := InitCmdEdit.Text;
    ModemLogging := InitLogCheck.Checked;
  end;                                                   
  { Close dialog }
  ModalResult := mrOK;
end;

end.
