unit OptionsDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls, VML_Modem;

type
  TOptionsForm = class(TForm)
    PageControl: TPageControl;
    ModemSheet: TTabSheet;
    ModemGroup: TGroupBox;
    ModemImage: TImage;
    ModemTitle: TLabel;
    ModemDriverLabel: TLabel;
    ModemDriverCombo: TComboBox;
    ModemPortCombo: TComboBox;
    ModemPortLabel: TLabel;
    ModemBaudLabel: TLabel;
    ModemBaudCombo: TComboBox;
    PortGroup: TGroupBox;
    PortImage: TImage;
    PortTitle: TLabel;
    PortButton: TButton;
    RecordingSheet: TTabSheet;
    RecordGroup: TGroupBox;
    RecordImage: TImage;
    RecordTitle: TLabel;
    RecDeviceLabel: TLabel;
    RecDeviceCombo: TComboBox;
    RecGainLabel: TLabel;
    RecGainLowLabel: TLabel;
    RecGainHighLabel: TLabel;
    RecGainTrack: TTrackBar;
    CompressionGroup: TGroupBox;
    CompressionImage: TImage;
    CompressionTitle: TLabel;
    DataFormatLabel: TLabel;
    DataFormatCombo: TComboBox;
    CompressionLabel: TLabel;
    CompressionCombo: TComboBox;
    SpeedLabel: TLabel;
    SpeedCombo: TComboBox;
    PlayingSheet: TTabSheet;
    PlayDeviceGroup: TGroupBox;
    PlayImage: TImage;
    PlayTitle: TLabel;
    PlayDeviceLabel: TLabel;
    PlayDeviceCombo: TComboBox;
    PlayVolumeLabel: TLabel;
    PlayVolumeLowLabel: TLabel;
    PlayVolumeHighLabel: TLabel;
    PlayVolumeTrack: TTrackBar;
    OKButton: TButton;
    CancelButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure PortButtonClick(Sender: TObject);
    procedure DataFormatComboChange(Sender: TObject);
    procedure CompressionComboChange(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
  private
    FProfile: TModemProfile;
    FDriver: TModemDriver;
  public
    { Public declarations }
  end;

{ Show general settings dialog }

function ShowOptionsDlg: Boolean;

implementation

uses
  ConnectionDlg, VML_IniFile, VML_Consts, VML_Idents;

{$R *.DFM}

const
  PortMap: array[0..15] of TIdentMapEntry = (
    (Value: 1; Name: 'COM1'),
    (Value: 2; Name: 'COM2'),
    (Value: 3; Name: 'COM3'),
    (Value: 4; Name: 'COM4'),
    (Value: 5; Name: 'COM5'),
    (Value: 6; Name: 'COM6'),
    (Value: 7; Name: 'COM7'),
    (Value: 8; Name: 'COM8'),
    (Value: 9; Name: 'COM9'),
    (Value: 10; Name: 'COM10'),
    (Value: 11; Name: 'COM11'),
    (Value: 12; Name: 'COM12'),
    (Value: 13; Name: 'COM13'),
    (Value: 14; Name: 'COM14'),
    (Value: 15; Name: 'COM15'),
    (Value: 16; Name: 'COM16'));

  BaudRateMap: array[0..14] of TIdentMapEntry = (
    (Value: CBR_110; Name: '110'),
    (Value: CBR_300; Name: '300'),
    (Value: CBR_600; Name: '600'),
    (Value: CBR_1200; Name: '1200'),
    (Value: CBR_2400; Name: '2400'),
    (Value: CBR_4800; Name: '4800'),
    (Value: CBR_9600; Name: '9600'),
    (Value: CBR_14400; Name: '14400'),
    (Value: CBR_19200; Name: '19200'),
    (Value: CBR_38400; Name: '38400'),
    (Value: CBR_56000; Name: '56000'),
    (Value: CBR_57600; Name: '57600'),
    (Value: CBR_115200; Name: '115200'),
    (Value: CBR_128000; Name: '128000'),
    (Value: CBR_256000; Name: '256000'));

{ Show general settings dialog }

function ShowOptionsDlg: Boolean;
begin
  with TOptionsForm.Create(nil) do
  try
    Result := ShowModal = mrOK;
  finally
    Free;
  end;
end;

{ TOptionsForm }

procedure TOptionsForm.FormCreate(Sender: TObject);
begin
  FProfile := TModemProfile.Create(nil);
end;

procedure TOptionsForm.FormDestroy(Sender: TObject);
begin
  FDriver.Free;
  FProfile.Free;
end;

procedure TOptionsForm.FormShow(Sender: TObject);
var
  Gain, Volume: Integer;
begin
  { Load profile settings }
  ReadProfileIniFile('DrvTest.ini', FProfile);
  { Create driver instance }
  FDriver := ReadDriverIniFile(FProfile.ModemIniFile, nil);
  { Refresh controls }
  with FProfile do
  begin
    { Modem type }
    ModemDriverCombo.Text := FDriver.ModemName;
    { Port list }
    with ModemPortCombo do
    begin
      GetIdentsStringList(PortMap, Items);
      ItemIndex := GetIdentValueIndex(Port, PortMap);
    end;
    { Port speed list }
    with ModemBaudCombo do
    begin
      GetIdentsStringList(BaudRateMap, Items);
      ItemIndex := GetIdentValueIndex(PortBaudRate, BaudRateMap);
    end;
    { Record device }
    with RecDeviceCombo do
    begin
      FDriver.GetRecDevicesStringList(Items);
      ItemIndex := Items.IndexOf(FDriver.GetDeviceName(FProfile.RecDevice));
    end;
    { Receive gain }
    if Length(FDriver.ReceiveGainCmd) > 0 then
    begin
      Gain := FProfile.ReceiveGain;
      if Gain = -1 then Gain := 75;
      with RecGainTrack do
      begin
        Frequency := FDriver.ReceiveGainMax - FDriver.ReceiveGainMin;
        if Frequency > 100 then Frequency := 10;
        PageSize := Frequency;
        LineSize := Frequency;
        Position := Gain;
      end;
    end
    else
    begin
      RecGainLabel.Enabled := False;
      RecGainLowLabel.Enabled := False;
      RecGainHighLabel.Enabled := False;
      RecGainTrack.Enabled := False;
      RecGainTrack.TickStyle := tsNone;
    end;
    { Voice data format }
    case FProfile.VoiceDataFormat of
      DATA_FORMAT_RMD:
        { Raw Modem Data }
        begin
          DataFormatCombo.ItemIndex := 1;
          { Record compression }
          with CompressionCombo do
          begin
            FDriver.GetCompressionsStringList(Items);
            ItemIndex := Items.IndexOf(FDriver.GetCompressionName(FProfile.RecCompression));
          end;
          { Record speed }
          with SpeedCombo do
          begin
            FDriver.GetSpeedStringList(FProfile.RecCompression, Items);
            ItemIndex := Items.IndexOf(IntToStr(FProfile.RecSpeed));
          end;
        end;
      DATA_FORMAT_WAVE:
        { WAVE Format }
        begin
          DataFormatCombo.ItemIndex := 2;
          { Record compression }
          with CompressionCombo do
          begin
            FDriver.GetWaveFormatsStringList(Items);
            ItemIndex := Items.IndexOf(FDriver.GetWaveFormatName(FProfile.RecWaveFormat));
          end;
          { Record speed }
          with SpeedCombo do
          begin
            FDriver.GetWaveSpeedStringList(FProfile.RecWaveFormat, Items);
            ItemIndex := Items.IndexOf(IntToStr(FProfile.RecWaveSpeed));
          end;
        end;
    else
      { Use default record compression }
      DataFormatCombo.ItemIndex := 0;
      { Disable format controls }
      CompressionLabel.Enabled := False;
      CompressionCombo.Enabled := False;
      SpeedLabel.Enabled := False;
      SpeedCombo.Enabled := False;
    end;
    { Play device }
    with PlayDeviceCombo do
    begin
      FDriver.GetPlayDevicesStringList(Items);
      ItemIndex := Items.IndexOf(FDriver.GetDeviceName(FProfile.PlayDevice));
    end;
    { Transmit gain }
    if Length(FDriver.TransmitGainCmd) > 0 then
    begin
      Volume := FProfile.TransmitGain;
      if Volume = -1 then Volume := 75;
      with PlayVolumeTrack do
      begin
        Frequency := FDriver.TransmitGainMax - FDriver.TransmitGainMin;
        if Frequency > 100 then Frequency := 10;
        PageSize := Frequency;
        LineSize := Frequency;
        Position := Volume;
      end;
    end
    else
    begin
      PlayVolumeLabel.Enabled := False;
      PlayVolumeLowLabel.Enabled := False;
      PlayVolumeHighLabel.Enabled := False;
      PlayVolumeTrack.Enabled := False;
      PlayVolumeTrack.TickStyle := tsNone;
    end;
  end;
end;

procedure TOptionsForm.PortButtonClick(Sender: TObject);
begin
  ShowConnectionDlg(Self, FProfile);
end;

procedure TOptionsForm.DataFormatComboChange(Sender: TObject);
begin
  { Clear comression controls }
  CompressionCombo.Items.Clear;
  SpeedCombo.Items.Clear;
  { Enable format controls }
  with DataFormatCombo do
  begin
    CompressionLabel.Enabled := ItemIndex > 0;
    CompressionCombo.Enabled := ItemIndex > 0;
    SpeedCombo.Enabled := ItemIndex > 0;
    SpeedLabel.Enabled := ItemIndex > 0;
  end;
  { Refresh compression combo box items list }
  case DataFormatCombo.ItemIndex of
    1: FDriver.GetCompressionsStringList(CompressionCombo.Items);
    2: FDriver.GetWaveFormatsStringList(CompressionCombo.Items);
  end;
end;

procedure TOptionsForm.CompressionComboChange(Sender: TObject);
var
  Compression, Speed: Integer;
begin
  { Get new compression, save selected speed value }
  Compression := FDriver.GetCompression(CompressionCombo.Text);
  Speed := StrToIntDef(SpeedCombo.Text, 0);
  { Refresh speed combo box }
  with SpeedCombo do
  begin
    Items.Clear;
    FDriver.GetSpeedStringList(Compression, Items);
    ItemIndex := Items.IndexOf(IntToStr(Speed));
  end;
end;

procedure TOptionsForm.OKButtonClick(Sender: TObject);
begin
  with FProfile do
  begin
    { Port settings }
    Port := IdentToIntDef(ModemPortCombo.Text, Port, PortMap);
    PortBaudRate := IdentToIntDef(ModemBaudCombo.Text, PortBaudRate, BaudRateMap);
    { Record device }
    RecDevice := FDriver.GetDevice(RecDeviceCombo.Text);
    { Voice data format and compression }
    case DataFormatCombo.ItemIndex of
      1:
        { Raw Modem Data }
        begin
          VoiceDataFormat := DATA_FORMAT_RMD;
          RecCompression := FDriver.GetCompression(CompressionCombo.Text);
          RecSpeed := StrToIntDef(SpeedCombo.Text, RecSpeed);
        end;
      DATA_FORMAT_WAVE:
        { WAVE Format }
        begin
          VoiceDataFormat := DATA_FORMAT_WAVE;
          RecWaveFormat := FDriver.GetWaveFormat(CompressionCombo.Text);
          RecWaveSpeed := StrToIntDef(SpeedCombo.Text, RecWaveSpeed);
        end;
    else
      { Use default compression }
      VoiceDataFormat := DATA_FORMAT_AUTO;
      RecCompression := 0;
      RecSpeed := 0;
      RecWaveFormat := 0;
      RecWaveSpeed := 0;
    end;
    { Receive gain }
    ReceiveGain := RecGainTrack.Position;
    { Play device }
    PlayDevice := FDriver.GetDevice(PlayDeviceCombo.Text);
    { Transmit gain }
    TransmitGain := PlayVolumeTrack.Position;
  end;
  { Save profile settings }
  WriteProfileIniFile('DrvTest.ini', FProfile);
  { Close dialog }
  ModalResult := mrOK;
end;

end.

