unit AR_ParamsDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, VML_Modem;

type
  TARParamsForm = class(TForm)
    TitleImage: TImage;
    TitleLabel: TLabel;
    ParamsGroup: TGroupBox;
    VRNLabel: TLabel;
    VRALabel: TLabel;
    VRNEdit: TEdit;
    VRNUpDown: TUpDown;
    VRNLabel2: TLabel;
    VRAEdit: TEdit;
    VRAUpDown: TUpDown;
    FMaxUpDown: TUpDown;
    VRALabel2: TLabel;
    FMaxLabel: TLabel;
    FMaxEdit: TEdit;
    FMaxLabel2: TLabel;
    FToneMinLabel: TLabel;
    FToneMinEdit: TEdit;
    FToneMinUpDown: TUpDown;
    FToneMinLabel2: TLabel;
    FToneMaxLabel: TLabel;
    FToneMaxEdit: TEdit;
    FToneMaxUpDown: TUpDown;
    FToneMaxLabel2: TLabel;
    ToneLevelLabel: TLabel;
    ToneLevelEdit: TEdit;
    ToneLevelUpDown: TUpDown;
    EnergyLevelLabel: TLabel;
    EnergyLevelEdit: TEdit;
    EnergyLevelUpDown: TUpDown;
    VoiceRateLabel: TLabel;
    VoiceRateEdit: TEdit;
    VoiceRateUpDown: TUpDown;
    NoiseRateLabel: TLabel;
    NoiseRateEdit: TEdit;
    NoiseRateUpDown: TUpDown;
    VoiceRequiredCheck: TCheckBox;
    OKButton: TButton;
    CancelButton: TButton;
    procedure FormShow(Sender: TObject);
    procedure ParamEditKeyPress(Sender: TObject; var Key: Char);
    procedure ParamUpDownChanging(Sender: TObject; var Position: Integer);
    procedure OKButtonClick(Sender: TObject);
  private
    FForm: TForm;
    FProfile: TModemProfile;
    FFullMode: Boolean;
  public
    { Public declarations }
  end;

{ Show answer recognition settings dialog }

function EditRecognitionParams(Form: TForm; Profile: TModemProfile; FullMode: Boolean): Boolean;

implementation

{$R *.DFM}
{$I *.INC}

{ Show answer recognition settings dialog }

function EditRecognitionParams(Form: TForm; Profile: TModemProfile; FullMode: Boolean): Boolean;
begin
  { Create dialog form }
  with TARParamsForm.Create(nil) do
  try
    { Set dialog params }
    FForm := Form;
    FProfile := Profile;
    FFullMode := FullMode;
    { Show dialog }
    Result := ShowModal = mrOK;
  finally
    Free;
  end;
end;

{ TARParamsForm }

procedure TARParamsForm.FormShow(Sender: TObject);
var
  R: TRect;
  X, Y, H, I: Integer;
  C: TControl;
begin
  SystemParametersInfo(SPI_GETWORKAREA, 0, @R, 0);
  { Select form position }
  X := FForm.ClientOrigin.X + 15;
  if X < R.Left then X := R.Left;
  if X + Width > R.Right then X := R.Right - Width;
  Y := FForm.ClientOrigin.Y + 36;
  if Y < R.Top + 5 then Y := R.Top + 5;
  if Y + Height > R.Bottom - 5 then Y := R.Bottom - 5 - Height;
  { Change form position }
  Left := X;
  Top := Y;
  { Refresh controls }
  with FProfile do
  begin
    VRNEdit.Text := IntToStr(RingbackNeverCame div 10);
    VRAEdit.Text := IntToStr(RingbackGoesAway div 10);
    FMaxEdit.Text := IntToStr(AnswerFMax);
    FToneMinEdit.Text := IntToStr(AnswerFToneMin);
    FToneMaxEdit.Text := IntToStr(AnswerFToneMax);
    ToneLevelEdit.Text := IntToStr(AnswerToneLevel);
    EnergyLevelEdit.Text := IntToStr(AnswerEnergyLevel);
    VoiceRateEdit.Text := IntToStr(AnswerVoiceRate);
    NoiseRateEdit.Text := IntToStr(AnswerNoiseRate);
    VoiceRequiredCheck.Checked := AnswerVoiceRequired;
  end;
  { Update form height }
  if not FFullMode then
  begin
    { Height decrement }
    H := VoiceRequiredCheck.Top - VRAEdit.Top - 4;
    { Hide controls }
    for I := 0 to ParamsGroup.ControlCount - 1 do
    begin
      C := ParamsGroup.Controls[I];
      if C.Tag = 1 then C.Visible := False;
    end;
    { Change height }
    ParamsGroup.Height := ParamsGroup.Height - H;
    OKButton.Top := OKButton.Top - H;
    CancelButton.Top := CancelButton.Top - H;
    Height := Height - H;
  end;
end;

procedure TARParamsForm.ParamEditKeyPress(Sender: TObject; var Key: Char);
begin
  if not (Key in ['0'..'9', #8]) then Key := #0;
end;

procedure TARParamsForm.ParamUpDownChanging(Sender: TObject; var Position: Integer);
begin
  Position := TUpDown(Sender).Increment * (Position div TUpDown(Sender).Increment);
end;

procedure TARParamsForm.OKButtonClick(Sender: TObject);

  procedure CheckParam(Edit: TEdit; Min, Max: Longint; const Error: string);
  const
    Flags = MB_OK or MB_ICONERROR;
  var
    I: Longint;
  begin
    if Edit.Visible then
    try
      { Check value }
      I := StrToInt(Edit.Text);
      if (I < Min) or (I > Max) then Abort;
    except
      { Show error message, focus control }
      Application.MessageBox(PChar(Error), PChar(Caption), Flags);
      Edit.SetFocus;
      Abort;
    end;
  end;

begin
  { Check Params }
  CheckParam(VRNEdit, 0, 25, SVRAError);
  CheckParam(VRAEdit, 0, 25, SVRNError);
  CheckParam(FMaxEdit, 1, 12000, SFMaxError);
  CheckParam(FToneMinEdit, 1, 1000, SToneMinError);
  CheckParam(FToneMaxEdit, 1, 1000, SToneMaxError);
  CheckParam(ToneLevelEdit, 1, MaxInt, SToneLevelError);
  CheckParam(EnergyLevelEdit, 1, MaxInt, SEnergyLevelError);
  CheckParam(VoiceRateEdit, 0, 100, SVoiceRateError);
  CheckParam(NoiseRateEdit, 0, 100, SNoiseRateError);
  { Update profile settings }
  with FProfile do
  begin
    RingbackNeverCame := StrToIntDef(VRNEdit.Text, RingbackNeverCame div 10) * 10;
    RingbackGoesAway := StrToIntDef(VRAEdit.Text, RingbackGoesAway div 10) * 10;
    AnswerFMax := StrToIntDef(FMaxEdit.Text, AnswerFMax);
    AnswerFToneMin := StrToIntDef(FToneMinEdit.Text, AnswerFToneMin);
    AnswerFToneMax := StrToIntDef(FToneMaxEdit.Text, AnswerFToneMax);
    AnswerToneLevel := StrToIntDef(ToneLevelEdit.Text, AnswerToneLevel);
    AnswerEnergyLevel := StrToIntDef(EnergyLevelEdit.Text, AnswerEnergyLevel);
    AnswerVoiceRate := StrToIntDef(VoiceRateEdit.Text, AnswerVoiceRate);
    AnswerNoiseRate := StrToIntDef(NoiseRateEdit.Text, AnswerNoiseRate);
    AnswerVoiceRequired := VoiceRequiredCheck.Checked;
  end;
  { Close dialog }
  ModalResult := mrOK;
end;

end.
