{
  Voice Modem Library v1.0

  This file contains expanded driver for USRobotics modems with
  software answer recognition feature.

  Note. This driver automatically replaces standard USRobotics driver.

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit AR_USRobotics;

{$I VML.INC}

interface

uses
  Windows, SysUtils, Classes, Graphics, VML_Modem, VML_Consts,
  VML_USRobotics, FFTu, voice_dll;

{$I *.INC}

type

{ TARUSRobotics }

  TARUSRobotics = class(TUSRobotics)
  private
    FAnswerStream: TMemoryStream;
    FAnswerPos: Longint;
    FLinBuff: prmas;
    FLincount: Integer;
    FAnalyzing: Boolean;
    FConvertError: Boolean;
    FToneTime: Longint;
    FVoiceTime: Longint;
    FWasVoice: Boolean;
    FAnswer: Integer;
    FTimeOut: Boolean;
    FInterrupted: Boolean;
  protected
    Fa, Fmf, Fte, Fc: Integer;
    procedure DrawSpectrum(Rect: TRect; Canvas: TCanvas);
    procedure Progress(Done: Longint); override;
  public
    constructor Create(AOwner: TComponent); override;
    function Dial(const Number: string): Integer; override;
    function HandleEvent(Event: Integer; Data: TEventData): Integer; override;
  end;

implementation

uses
  Unit3;

{ TARUSRobotics }

constructor TARUSRobotics.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  { Answer recognition method currently work only with USG/GSM
    compression method }
  Compressions.Clear;
  Compressions.Add(USROBOTICS_GSM, SUSRGSM, 'AT#VSM=128,8000', 'OK', 8000, 8, CF_8000 or CF_DEFAULT, 0, 0, 0);
end;

procedure TARUSRobotics.DrawSpectrum(Rect: TRect; Canvas: TCanvas);
const
  N = 1024;
var
  Width, Height, i: Integer;
  max: Double;
  m: array[1..N] of Word; 

  { Get X coordinate for given value index }

  function xx(x: Integer): Integer;
  begin
    xx := 2 * Round(Width / N * x);
  end;

begin
  max := Emax;
  if max = 0 then max := 1;
  for i := 1 to N div 2 do m[i] := Round(ms[i] / max * 255);

  Width := Rect.Right - Rect.Left;
  Height := Rect.Bottom - Rect.Top;

  Canvas.FillRect(Rect);

  Canvas.MoveTo(0, Height);
  for i := 1 to N do
    Canvas.LineTo(xx(i), Height - m[i]);

  Canvas.TextOut(Width - 140, 10, 'Amplitude ' + IntToStr(Fa));
  Canvas.TextOut(Width - 140, 25, 'Max. freq. ' + IntToStr(Fmf));
  Canvas.TextOut(Width - 140, 40, 'Energy ' + IntToStr(Fte));
  Canvas.TextOut(Width - 140, 55, 'Rate ' + IntToStr(Fc));
end;

procedure TARUSRobotics.Progress(Done: Longint);
var
  RMD, PVF, LIN: TInFile;
  C, I: Integer;
  Result: Integer;
  TimeOut: Longint;
begin
  if FAnalyzing then
  try
    {
      Step 1. Get next part of modem raw data
    }
    { Fill modem data structure }
    RMD.data := Pointer(Longint(FAnswerStream.Memory) + FAnswerPos);
    RMD.size := FAnswerStream.Size - FAnswerPos;
    // RMD.size := FAnswerStream.Position - FAnswerPos;
    RMD.rmd_header := FAnswerStream.Memory;
    // DebugMsgFmt(0, SGotVoiceData, [FAnswerPos, RMD.size]);
    { Shift data pointer }
    FAnswerPos := FAnswerStream.Size;
    // FAnswerPos := FAnswerStream.Position;
    {
      Step 2. Convert raw modem data to the linear data
    }
    { Clear temporary structures }
    FillChar(PVF, SizeOf(PVF), 0);
    FillChar(LIN, SizeOf(LIN), 0);
    { Set data locations }
    SetInputLocation(GETPUT_MEM);
    SetOutputLocation(GETPUT_MEM);
    { Convert modem data to binary 16 bit PVF }
    SetPVFParameters(False, 16);
    Result := RMDToPVF(PChar(@RMD), PChar(@PVF));
    { Check converting result }
    if Result <> SUCCESS then
    begin
      { Convert error occured }
      DebugMsgFmt(L_ERROR, SRMDToPVFConvertError, [Result]);
      FConvertError := True;
      { Stop dialing }
      Modem.StopCurrentAction;
      Exit;
    end;
    // DebugMsgFmt(0, SRMDToPVFSuccess, [PVF.size]);
    try
      { Convert PVF to unsigned 16 bit linear }
      SetLinParameters(True, False, TRMDHeader(FAnswerStream.Memory^).Speed);
      Result := PVFToLin(PChar(@PVF), PChar(@LIN));
      { Check converting result }
      if Result <> SUCCESS then
      begin
        { Convert error occured }
        DebugMsgFmt(L_ERROR, SPVFToLinConvertError, [Result]);
        FConvertError := True;
        { Stop dialing }
        Modem.StopCurrentAction;
        Exit;
      end;
      // DebugMsgFmt(0, SPVFToLinSuccess, [Lin.size]);
      try
        {
          Step 3. Analyze linear data
        }
        C := Lin.Size div SizeOf(Word);
        I := 0;
        while I < C do
        begin
          { Prepare 1024 points of data }
          while (FLinCount < 1024) and (I < C) do
          begin
            FLinBuff^[FLinCount + 1] := TWordArray(LIN.data^)[I];
            Inc(FLinCount);
            Inc(I);
          end;
          { Check points count }
          if FLinCount >= 1023 then
          begin
            FLinCount := 0;
            { Analyze }
            Result := FFT_AnalyzeEx(FLinBuff, Fa, Fmf, Fte, Fc);

            { **************************************************************
              WARNING! DEBUG ONLY. DRAWING WITHOUT MAIN THREAD SYNCHRONIZING
              MAY CAUSE ERRORS.
              ************************************************************** }
            with SpectrumForm do
            begin
              with PaintBox do DrawSpectrum(BoundsRect, Canvas);
              Update;
            end;

            { Check ringback }
            if Result = ANALYZE_TONE then
            begin
              DebugMsgFmt(0, SAnalyzing, [TimeToStr(Now), Fa, Fmf, Fte, Fc, Result, SToneDetected]);
              { Tone detected, reset voice times }
              FToneTime := Longint(GetTickCount);
              FVoiceTime := 0;
              FWasVoice := False;
            end
            else
            { Voice or silence detected }
            begin
              { Check for required voice }
              if Result = ANALYZE_VOICE then FWasVoice := True;
              { Check voice time }
              if FVoiceTime = 0 then
              begin
                DebugMsgFmt(0, SAnalyzing, [TimeToStr(Now), Fa, Fmf, Fte, Fc, Result, SFirstVoiceDetected]);
                { First voice detection time }
                FVoiceTime:= Longint(GetTickCount);
              end
              else
              begin
                DebugMsgFmt(0, SAnalyzing, [TimeToStr(Now), Fa, Fmf, Fte, Fc, Result, Format(SVoiceDetected, [Abs(Longint(GetTickCount) - FVoiceTime)])]);
                { Get voice detection timeout }
                TimeOut := Profile.RingbackGoesAway * 100;
                if FToneTime = 0 then TimeOut := Profile.RingbackNeverCame * 100;
                { Check voice detection interval }
                if Abs(Longint(GetTickCount) - FVoiceTime) > TimeOut then
                  { Check for voice required }
                  if (not (Profile.AnswerVoiceRequired)) or FWasVoice then
                  begin
                    { Voice answer detected }
                    FAnswer := VMA_VCON;
                    { Stop dialing }
                    Modem.StopCurrentAction;
                    Exit;
                  end;
              end;
            end;
          end;
        end;
      finally
        GlobalFreePtr(LIN.Data);
      end;
    finally
      GlobalFreePtr(PVF.Data);
    end;
  except
    on E: Exception do DebugMsg(L_ERROR, 'Progress: ' + E.Message);
  end;
end;

function TARUSRobotics.Dial(const Number: string): Integer;
var
  SaveRecMaxLen: Integer;
begin
  { Check modem specified }
  if not CheckModem('Dial', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Process dial with harware answer detection }
  if not Profile.AnswerRecognition then
  begin
    Result := inherited Dial(Number);
    Exit;
  end;
  { Save modifiable parameters }
  SaveRecMaxLen := Profile.RecMaxLen;
  { Dial number }
  if Modem.CommandFmt('ATD%s%s;', '', [Profile.DialMode, Number]) <> S_OK then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Analyze modem answer }
  case Modem.WaitAnswer('', Profile.PortTimeOut * 1000) of
    E_INTERRUPTED:
      begin
        Result := E_INTERRUPTED;
      end;
    VMA_BUSY:
      begin
        Result := S_OK;
        Modem.QueueEvent(TModemEvent.Create(BUSY_TONE));
      end;
    VMA_NO_ANSWER:
      begin
        Result := S_OK;
        Modem.QueueEvent(TModemEvent.Create(NO_ANSWER));
      end;
    VMA_NO_DIAL_TONE:
      begin
        Result := S_OK;
        Modem.QueueEvent(TModemEvent.Create(NO_DIAL_TONE));
      end;
    VMA_OK:
      begin
        { Lock convert libriary }
        if not LockVoiceLib(Self) then
        begin
          Modem.LastError := VME_CONVERT_LIB_LOCKED;
          Result := S_FAIL;
          Exit;
        end;
        try
          { Init FFT }
          if not FFT_Init(1024) then
          begin
            Modem.LastError := VME_FFT_INIT_FAILED;
            Result := S_FAIL;
            Exit;
          end;
          try
            { Init analyze params }
            ANALYZE_FMAX := Profile.AnswerFMax;
            ANALYZE_FTONEMIN := Profile.AnswerFToneMin;
            ANALYZE_FTONEMAX := Profile.AnswerFToneMax;
            ANALYZE_TONELEVEL := Profile.AnswerToneLevel;
            ANALYZE_ENERGYLEVEL := Profile.AnswerEnergyLevel;
            ANALYZE_VOICERATE := Profile.AnswerVoiceRate;
            ANALYZE_NOISERATE := Profile.AnswerNoiseRate;
            { Allocate analyze buffer }
            GetMem(FLinBuff, 1024 * SizeOf(Double));
            try
              FLinCount := 0;
              { Create answer data memory stream }
              FAnswerStream := TMemoryStream.Create;
              try
                FAnswerPos := SizeOf(TRMDHeader);
                { Set record timeout }
                Profile.RecMaxLen := Profile.DialTimeOut;
                try
                  FConvertError := False;
                  FToneTime := 0;
                  FVoiceTime := 0;
                  FWasVoice := True;
                  FAnswer := VMA_NO_ANSWER;
                  FTimeOut := False;
                  FInterrupted := False;
                  { Start recording }
                  FAnalyzing := True;
                  try
                    if Modem.RecordStream(FAnswerStream) = S_FAIL then
                    begin
                      Result := S_FAIL;
                      Exit;
                    end;
                  finally
                    FAnalyzing := False;
                  end;
                  { Check recognition result }
                  if FConvertError then
                  begin
                    { Data converting rerror occurred }
                    Modem.LastError := VME_DATA_CONVERT_FAILED;
                    Result := S_FAIL;
                  end
                  else if FTimeOut then
                  begin
                    { Analyzing time expired }
                    Result := S_OK;
                    Modem.QueueEvent(TModemEvent.Create(NO_ANSWER));
                  end
                  else if FInterrupted then
                  begin
                    { Analyzing interrupted }
                    Result := E_INTERRUPTED;
                  end
                  else if FAnswer = VMA_VCON then
                  begin
                    { Voice answer detected }
                    Result := S_OK;
                  end
                  else if (FToneTime <> 0) or (FVoiceTime <> 0) then
                  begin
                    { There are was tones but no voice answer }
                    Result := S_OK;
                    Modem.QueueEvent(TModemEvent.Create(NO_ANSWER));
                  end
                  else
                  begin
                    { Strange result }
                    Modem.LastError := VME_RECOGNITION_FAILED;
                    Result := S_FAIL;
                  end;
                finally
                  Profile.RecMaxLen := SaveRecMaxLen;
                end;
              finally
                FAnswerStream.Free;
              end;
            finally
              FreeMem(FLinBuff);
            end;
          finally
            FFT_Done;
          end;
        finally
          UnlockVoiceLib;
        end;
      end;
  else
    Modem.LastError := VME_INVALID_MODEM_ANSWER;
    Result := S_FAIL;
  end;
  { Process events queue }
  Modem.CheckEvents;
end;

function TARUSRobotics.HandleEvent(Event: Integer; Data: TEventData): Integer;
begin
  { Check for modem events }
  case Event of
    NO_VOICE_ENERGY,  { Modem could not detect voice energy on the line }
    SILENCE_DETECTED, { Modem detected silence }
    BUSY_TONE,        { Busy tone detected }
    DIAL_TONE:        { Dial tone detected }
      { Ignore modem events detection while software answer recognition
        in progress }
      if FAnalyzing then
      begin
        Result := S_OK;
        Exit;
      end;
    OPERATION_TIMEOUT:
      { Handle operation timeout event }
      if FAnalyzing then
      begin
        FTimeOut := True;
        Result := S_OK;
        Exit;
      end;
    SIGNAL_STOP:
      { Check for recognition interrrupted }
      if FAnalyzing then FInterrupted := True;
  end;
  { Default handling }
  Result := inherited HandleEvent(Event, Data);
end;

initialization
  ReplaceModemDriver(SUSRoboticsName, TARUSRobotics);

end.
