unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, CommPort, VML_Modem, VML_NoModem, VML_IS101,
  VML_Rockwell, VML_ZyXEL1496;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    Edit1: TEdit;
    Label2: TLabel;
    Edit2: TEdit;
    OpenButton: TButton;
    CloseButton: TButton;
    InitButton: TButton;
    CommandButton: TButton;
    ReadButton: TButton;
    WriteButton: TButton;
    FlushButton: TButton;
    VoiceButton: TButton;
    DataButton: TButton;
    RecordButton: TButton;
    PlayButton: TButton;
    StopButton: TButton;
    DTMFButton: TButton;
    ClearButton: TButton;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    Memo1: TMemo;
    TabSheet2: TTabSheet;
    Memo2: TMemo;
    TabSheet3: TTabSheet;
    Memo3: TMemo;
    TabSheet5: TTabSheet;
    Memo5: TMemo;
    VoiceModem1: TVoiceModem;
    ModemProfile1: TModemProfile;
    ModemPort1: TModemPort;
    ModemLogFile1: TModemLogFile;
    ModemTraceFile1: TModemLogFile;
    Rockwell1: TRockwell;
    ZyXEL14961: TZyXEL1496;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormResize(Sender: TObject);
    procedure OpenButtonClick(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
    procedure InitButtonClick(Sender: TObject);
    procedure CommandButtonClick(Sender: TObject);
    procedure ReadButtonClick(Sender: TObject);
    procedure WriteButtonClick(Sender: TObject);
    procedure FlushButtonClick(Sender: TObject);
    procedure VoiceButtonClick(Sender: TObject);
    procedure DataButtonClick(Sender: TObject);
    procedure RecordButtonClick(Sender: TObject);
    procedure PlayButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure DTMFButtonClick(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
    procedure VoiceModem1DebugMsg(Sender: TObject; Msg: Integer; const S: string);
    procedure VoiceModem1Trace(Sender: TObject; const Action, Line: string);
    procedure VoiceModem1Yield(Sender: TObject);
  private
    procedure LockModem(const Proc: string);
    procedure UnlockModem;
    procedure WMGetMinMaxInfo(var Message: TWMGetMinMaxInfo); message WM_GETMINMAXINFO;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses
  VML_Consts, VML_Idents, VML_Utils;

{$R *.DFM}

function CharToCode(C: Char): string;
begin
  if C >= ' ' then
    Result := C
  else
    Result := '#' + IntToStr(Ord(C));
end;

function StrToCode(const S: string): string;
var
  I: Integer;
begin
  Result := '';
  for I := 1 to Length(S) do
    if S[I] >= ' ' then
      Result := Result + S[I]
    else
      Result := Result + '#' + IntToStr(Ord(S[I]));
end;

procedure TForm1.LockModem(const Proc: string);
begin
  if not VoiceModem1.Lock(Self) then
  begin
    Memo2.Lines.Add(Format('%s: FAIL. %s.', [Proc, GetVMLErrorStr(VME_MODEM_LOCKED)]));
    Abort;
  end;
  Screen.Cursor := crAppStart;
end;

procedure TForm1.UnLockModem;
begin
  Screen.Cursor := crDefault;
  VoiceModem1.Unlock;
end;

procedure TForm1.WMGetMinMaxInfo(var Message: TWMGetMinMaxInfo);
begin
  inherited;
  with Message.MinMaxInfo^ do
  begin
    ptMinTrackSize.X := 350;
    ptMinTrackSize.Y := 462;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  with ModemLogFile1 do FileName := ExtractFilePath(Application.ExeName) + FileName;
  with ModemTraceFile1 do FileName := ExtractFilePath(Application.ExeName) + FileName;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  Screen.Cursor := crHourGlass;
  try
    LockModem('Quit');
    try
      if VoiceModem1.Opened then
      try
        if VoiceModem1.StopCurrentAction = S_FAIL then Exit;
        if VoiceModem1.SetDevice(NO_DEVICE) = S_FAIL then Exit;
        if VoiceModem1.HangUp = S_FAIL then Exit;
      finally
        VoiceModem1.CloseDevice;
      end;
    except
      { If no driver specified some function raise exception. It need to be
        ignored for properly closing form. }
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  Edit1.Width := ClientWidth - 179;
  Edit2.Width := ClientWidth - 179;
  with OpenButton do Left := Self.ClientWidth - (Width + 13);
  with CloseButton do Left := Self.ClientWidth - (Width + 13);
  with InitButton do Left := Self.ClientWidth - (Width + 13);
  with CommandButton do Left := Self.ClientWidth - (Width + 13);
  with ReadButton do Left := Self.ClientWidth - (Width + 13);
  with WriteButton do Left := Self.ClientWidth - (Width + 13);
  with FlushButton do Left := Self.ClientWidth - (Width + 13);
  with VoiceButton do Left := Self.ClientWidth - (Width + 13);
  with DataButton do Left := Self.ClientWidth - (Width + 13);
  with RecordButton do Left := Self.ClientWidth - (Width + 13);
  with PlayButton do Left := Self.ClientWidth - (Width + 13);
  with StopButton do Left := Self.ClientWidth - (Width + 13);
  with DTMFButton do Left := Self.ClientWidth - (Width + 13);
  with ClearButton do Left := Self.ClientWidth - (Width + 13);
  with PageControl1 do
  begin
    Width := OpenButton.Left - Left - 17;
    Height := Self.ClientHeight - Top - 11;
  end;
end;

procedure TForm1.OpenButtonClick(Sender: TObject);
begin
  case VoiceModem1.OpenDevice of
    S_OK: Memo2.Lines.Add('Open: OK');
    S_FAIL: Memo2.Lines.Add(Format('Open: FAIL. %s.', [VoiceModem1.LastErrorStr]));
  end;
end;

procedure TForm1.CloseButtonClick(Sender: TObject);
begin
  case VoiceModem1.CloseDevice of
    S_OK: Memo2.Lines.Add('Close: OK');
    S_FAIL: Memo2.Lines.Add(Format('Close: FAIL. %s.', [VoiceModem1.LastErrorStr]));
  end;
end;

procedure TForm1.InitButtonClick(Sender: TObject);
begin
  LockModem('Init');
  try
    case VoiceModem1.Init of
      S_OK: Memo2.Lines.Add('Init: OK');
      S_FAIL: Memo2.Lines.Add(Format('Init: FAIL. %s.', [VoiceModem1.LastErrorStr]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.CommandButtonClick(Sender: TObject);
var
  Result: Integer;
begin
  LockModem('Command');
  try
    Result := VoiceModem1.Command(Edit1.Text, Edit2.Text);
    if Result = VMA_FAIL then
      Memo2.Lines.Add(Format('Command: VMA_FAIL. %s.', [VoiceModem1.LastErrorStr]))
    else
      Memo2.Lines.Add(Format('Command: %s', [ModemAnswerToStr(Result)]));
  finally
    UnLockModem;
  end;
end;

procedure TForm1.ReadButtonClick(Sender: TObject);
var
  StrRead: string;
begin
  LockModem('Read');
  try
    case VoiceModem1.ReadStr(StrRead) of
      S_OK: Memo2.Lines.Add('Read: OK');
      S_FAIL: Memo2.Lines.Add(Format('Read: FAIL. %s.', [VoiceModem1.LastErrorStr]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.WriteButtonClick(Sender: TObject);
begin
  LockModem('Write');
  try
    case VoiceModem1.WriteStr(Edit1.Text) of
      S_OK: Memo2.Lines.Add('Write: OK');
      S_FAIL: Memo2.Lines.Add(Format('Write: FAIL. %s.', [VoiceModem1.LastErrorStr]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.FlushButtonClick(Sender: TObject);
begin
  LockModem('Flush');
  try
    case VoiceModem1.Flush of
      S_OK: Memo2.Lines.Add('Flush: OK');
      S_FAIL: Memo2.Lines.Add(Format('Flush: FAIL. %s', [VoiceModem1.LastErrorStr]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.VoiceButtonClick(Sender: TObject);
begin
  LockModem('Voice');
  try
    case VoiceModem1.VoiceModeOn of
      S_OK: Memo2.Lines.Add('VoiceModeOn: OK');
      S_FAIL: Memo2.Lines.Add(Format('VoiceModeOn: FAIL. %s.', [VoiceModem1.LastErrorStr]));
    end;
    case VoiceModem1.InitVoice of
      S_OK: Memo2.Lines.Add('InitVoice: OK');
      S_FAIL: Memo2.Lines.Add(Format('InitVoice: FAIL. %s.', [VoiceModem1.LastErrorStr]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.DataButtonClick(Sender: TObject);
begin
  LockModem('Data');
  try
    case VoiceModem1.VoiceModeOff of
      S_OK: Memo2.Lines.Add('VoiceModeOff: OK');
      S_FAIL: Memo2.Lines.Add(Format('VoiceModeOff: FAIL. %s.', [VoiceModem1.LastErrorStr]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.RecordButtonClick(Sender: TObject);
begin
  LockModem('Record');
  try
    case VoiceModem1.SetDevice(ModemProfile1.RecDevice) of
      S_OK: Memo2.Lines.Add('SetDevice: OK');
      S_FAIL: Memo2.Lines.Add(Format('SetDevice: FAIL. %s.', [VoiceModem1.LastErrorStr]));
    end;
    case VoiceModem1.RecordFile(Edit1.Text) of
      S_OK: Memo2.Lines.Add(Format('Record: ''%s'', OK', [Edit1.Text]));
      S_FAIL: Memo2.Lines.Add(Format('Record: ''%s'', FAIL. %s.', [Edit1.Text, VoiceModem1.LastErrorStr]));
      E_INTERRUPTED: Memo2.Lines.Add(Format('Record: ''%s'', INTERRUPTED', [Edit1.Text]));
      E_TIMEOUT: Memo2.Lines.Add(Format('Record: ''%s'', TIMEOUT', [Edit1.Text]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.PlayButtonClick(Sender: TObject);
begin
  LockModem('Play');
  try
    case VoiceModem1.SetDevice(ModemProfile1.PlayDevice) of
      S_OK: Memo2.Lines.Add('SetDevice: OK');
      S_FAIL: Memo2.Lines.Add(Format('SetDevice: FAIL. %s.', [VoiceModem1.LastErrorStr]));
    end;
    case VoiceModem1.PlayFile(Edit1.Text) of
      S_OK: Memo2.Lines.Add(Format('Play: ''%s'', OK', [Edit1.Text]));
      S_FAIL: Memo2.Lines.Add(Format('Play: ''%s'', FAIL. %s.', [Edit1.Text, VoiceModem1.LastErrorStr]));
      E_INTERRUPTED: Memo2.Lines.Add(Format('Play: ''%s'', INTERRUPTED', [Edit1.Text]));
      E_TIMEOUT: Memo2.Lines.Add(Format('Play: ''%s'', TIMEOUT', [Edit1.Text]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.StopButtonClick(Sender: TObject);
begin
  VoiceModem1.Stop;
end;

procedure TForm1.DTMFButtonClick(Sender: TObject);
var
  FileList: TStringList;
begin
  FileList := TStringList.Create;
  try
    FileList.Add('1.wav');
    FileList.Add('2.wav');
    FileList.Add('3.wav');
    VoiceModem1.PlayFiles(FileList);
  finally
    FileList.Free;
  end;
(*
  LockModem('Dial');
  try
    case VoiceModem1.Dial('123') of
      S_OK: Memo2.Lines.Add('Dial: OK');
      S_FAIL: Memo2.Lines.Add(Format('Dial: FAIL. %s.', [VoiceModem1.LastErrorStr]));
    end;
  finally
    UnLockModem;
  end;
*)
(*
  LockModem('PlayDTMF');
  try
    case VoiceModem1.SetDevice(ModemProfile1.PlayDevice) of
      S_OK: Memo2.Lines.Add('SetDevice: OK');
      S_FAIL: Memo2.Lines.Add(Format('SetDevice: FAIL. %s.', [VoiceModem1.LastErrorStr]));
    end;
    case VoiceModem1.PlayDTMF(Edit1.Text) of
      S_OK: Memo2.Lines.Add(Format('PlayDTMF: ''%s'', OK', [Edit1.Text]));
      S_FAIL: Memo2.Lines.Add(Format('PlayDTMF: ''%s'', FAIL. %s.', [Edit1.Text, VoiceModem1.LastErrorStr]));
    end;
  finally
    UnLockModem;
  end;
*)
end;

procedure TForm1.ClearButtonClick(Sender: TObject);
begin
  Memo2.Lines.Clear;
  Memo1.Lines.Clear;
  Memo3.Lines.Clear;
end;

procedure TForm1.VoiceModem1DebugMsg(Sender: TObject; Msg: Integer; const S: string);
begin
  Memo1.Lines.Add(StrToCode(S));
end;

procedure TForm1.VoiceModem1Trace(Sender: TObject; const Action, Line: string);
begin
  Memo3.Lines.Add(Action + ': ' + Line);
end;

procedure TForm1.VoiceModem1Yield(Sender: TObject);
begin
  {if VoiceModem1.State <> PLAYING then }Application.ProcessMessages;
end;

end.


