unit Unit1;

interface

uses
  Windows, Messages, ShellAPI, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, StdCtrls;

type
  TInfoForm = class(TForm)
    OpenDialog: TOpenDialog;
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    FileOpenItem: TMenuItem;
    FileN1Item: TMenuItem;
    FileExitItem: TMenuItem;
    HelpMenu: TMenuItem;
    HelpAboutItem: TMenuItem;
    InfoView: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FileOpenItemClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FileExitItemClick(Sender: TObject);
    procedure HelpAboutItemClick(Sender: TObject);
  private
    procedure OpenWAVE(const FileName: string);
    procedure WMDropFiles(var Message: TWMDropFiles); message WM_DROPFILES;
  public
    { Public declarations }
  end;

var
  InfoForm: TInfoForm;

implementation

uses
  MMSys, MMReg;

{$R *.DFM}

procedure TInfoForm.OpenWAVE(const FileName: string);
var
  Handle: Integer;
  Stream: THandleStream;
  CkPos: Longint;
  CkID, CkSize: DWORD;
  pwfx: PWaveFormatEx;
  WasFmtChunk: Boolean;

  function StreamRead(var Buffer; Size: Longint): Boolean;
  begin
    Result := True;
    if Stream.Read(Buffer, Size) <> Size then
    begin
      InfoView.Lines.Add('');
      InfoView.Lines.Add('Error reading file');
      Result := False;
    end;
  end;

begin
  Screen.Cursor := crHourGlass;
  try
    InfoView.Lines.Clear;
    { File name }
    InfoView.Lines.Add(Format('FILE: [%s]', [FileName]));
    { Check file }
    if not FileExists(FileName) then
    begin
      InfoView.Lines.Add('');
      InfoView.Lines.Add('File not found');
      Exit;
    end;
    { Open file }
    Handle := FileOpen(FileName, fmOpenRead or fmShareDenyWrite);
    { Check for file opening }
    if Handle < 0 then
    begin
      InfoView.Lines.Add('');
      InfoView.Lines.Add(Format('Error opening file (%d)', [GetLastError]));
      Exit;
    end;
    { Read file info }
    try
      Stream := THandleStream.Create(Handle);
      try
        { RIFF chunk }
        if not StreamRead(CkID, 4) then Exit;
        if CkID <> FOURCC_RIFF then
        begin
          InfoView.Lines.Add('');
          InfoView.Lines.Add('Not a RIFF format file');
          Exit;
        end;
        if not StreamRead(CkSize, 4) then Exit;
        { WAVE form type }
        if not StreamRead(CkID, 4) then Exit;
        if CkID <> FOURCC_WAVE then
        begin
          InfoView.Lines.Add('');
          InfoView.Lines.Add('Not a WAVE file');
          Exit;
        end;
        { Search chunks }
        WasFmtChunk := False;
        CkPos := Stream.Position;
        while CkPos < Stream.Size do
        begin
          Stream.Position := CkPos;
          { Chunk ID, chunk size }
          if not StreamRead(CkID, 4) then Exit;
          if not StreamRead(CkSize, 4) then Exit;
          InfoView.Lines.Add('');
          InfoView.Lines.Add(Format('----- Chunk ID "%s", Origin %d, Size %d -----', [mmioFOURCCToString(CkID), CkPos, CkSize]));
          { Next chunk offset }
          CkPos := Longint(CkSize) + Stream.Position;
          { Read chunk info }
          if CkID = FOURCC_FMT then
          begin
            InfoView.Lines.Add('Format Chunk');
            GetMem(pwfx, CkSize);
            try
              if not StreamRead(pwfx^, CkSize) then Exit;
              WasFmtChunk := True;
              { Common fields }
              InfoView.Lines.Add('Common Fields:');
              with pwfx^ do
              begin
                case wFormatTag of
                  $0000: InfoView.Lines.Add('  Format Tag $0000: WAVE_FORMAT_UNKNOWN');
                  $0001: InfoView.Lines.Add('  Format Tag $0001: WAVE_FORMAT_PCM');
                  $0006: InfoView.Lines.Add('  Format Tag $0006: WAVE_FORMAT_ALAW');
                  $0007: InfoView.Lines.Add('  Format Tag $0007: WAVE_FORMAT_MULAW');
                  $0011: InfoView.Lines.Add('  Format Tag $0011: WAVE_FORMAT_IMA_ADPCM');
                else
                  InfoView.Lines.Add(Format('  Format Tag $%0.4x: ?', [wFormatTag]));
                end;
                InfoView.Lines.Add(Format('  %d Channels', [nChannels]));
                InfoView.Lines.Add(Format('  %d Samples Per Second', [nSamplesPerSec]));
                InfoView.Lines.Add(Format('  %d Average Bytes Per Second', [nAvgBytesPerSec]));
                InfoView.Lines.Add(Format('  Block Align %d', [nBlockAlign]));
              end;
              { Format specific fields }
              InfoView.Lines.Add('Format Specific Fields:');
              case pwfx^.wFormatTag of
                $0001:
                  { PCM }
                  with PPCMWaveFormat(pwfx)^ do
                    InfoView.Lines.Add(Format('  %d Bits Per Sample', [wBitsPerSample]));
                $0011:
                  { Intel's DVI ADPCM }
                  with PDVIADPCMWaveFormat(pwfx)^ do
                  begin
                    InfoView.Lines.Add(Format('  %d Bits Per Sample', [wfx.wBitsPerSample]));
                    InfoView.Lines.Add(Format('  %d Bytes of extra information', [wfx.cbSize]));
                    InfoView.Lines.Add(Format('  %d Samples Per Block', [wSamplesPerBlock]));
                  end;
              else
                with pwfx^ do
                begin
                  InfoView.Lines.Add(Format('  %d Bits Per Sample', [wBitsPerSample]));
                  InfoView.Lines.Add(Format('  %d Bytes of extra information', [cbSize]));
                end;
              end;
            finally
              FreeMem(pwfx, CkSize);
            end;
          end
          else if CkID = FOURCC_DATA then
          begin
            { Data Chunk }
            InfoView.Lines.Add('Data Chunk');
            { Format Chunk must be before Data Chunk }
            if not WasFmtChunk then
            begin
              InfoView.Lines.Add('Warning: Format Chunk not defined before Data Chunk');
              WasFmtChunk := False;
            end;
            { Real data chunk size of all wave files I saw always was
              rounded up to a size which is a multiple of 2 bytes }
            CkPos := CkPos + CkPos mod 2;
          end
          else
            InfoView.Lines.Add('Unknown Chunk');
        end
      finally
        Stream.Free;
      end;
    finally
      FileClose(Handle);
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TInfoForm.WMDropFiles(var Message: TWMDropFiles);
var
  DropCount, DropSize: Word;
  DropName: string;
begin
  with Message do
  begin
    DropCount := DragQueryFile(Drop, $FFFFFFFF, nil, 0);
    if DropCount > 0 then
    begin
      DropSize := DragQueryFile(Drop, 0, nil, 0);
      SetLength(DropName, DropSize);
      DragQueryFile(Drop, 0, PChar(DropName), DropSize + 1);
      OpenWAVE(DropName);
    end;
    DragFinish(Drop);
  end;
  inherited;
end;

procedure TInfoForm.FormCreate(Sender: TObject);
begin
  { Allow file drop access }
  DragAcceptFiles(Handle, True);
end;

procedure TInfoForm.FormActivate(Sender: TObject);
begin
  Update;
  { Check file name in params }
  if ParamCount > 0 then
  begin
    OpenWAVE(ParamStr(1));
    Exit;
  end;
  { Otherwise open file }
  FileOpenItem.Click;
end;

procedure TInfoForm.FileOpenItemClick(Sender: TObject);
begin
  if OpenDialog.Execute then OpenWAVE(OpenDialog.FileName);
end;

procedure TInfoForm.FileExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TInfoForm.HelpAboutItemClick(Sender: TObject);
const
  SAbout =
    'WAVE file information viewer'#13 +
    'Copyright  2000 Roman M. Mochalov'#13 +
    ''#13 +
    'E-mail: roman@sar.nnov.ru';
  Flags = MB_OK or MB_ICONINFORMATION;
begin
  MessageBox(Handle, PChar(SAbout), PChar(Application.Title), Flags);
end;

end.
