{
  Fast Fourier Transform
  Voice Recognition routines

  (c) 1994 M. Mochalov
}

{$N+,E+}

unit FFTu;

interface

const

  { Definitions of analyze procedure (for 16 bit linear data) }

  ANALYZE_FMAX: Longint        = 4000;    { Max frequency div 2 }
  ANALYZE_FTONEMIN: Integer    = 390;     { Tone frequency must be between }
  ANALYZE_FTONEMAX: Integer    = 433;
  ANALYZE_TONELEVEL: Longint   = 2000000; { Tone amplitude must be higher (200000 - voice) }
  ANALYZE_ENERGYLEVEL: Longint = 2000;    { If greater -> there is signal }
  ANALYZE_VOICELEVEL: Integer  = 4;       { If greater -> it is voice }
  ANALYZE_NOISELEVEL: Integer  = 20;      { If greater -> it is noise }

  { Voice data analyze result values }

  ANALYZE_NONE  = 0; { Silence }
  ANALYZE_TONE  = 1; { Tone detected }
  ANALYZE_VOICE = 2; { Voice detected }
  ANALYZE_NOISE = 4; { Noise detected }

type
  prmas = ^rmas;
  rmas = array[1..4096] of Double;

{ Initialization and finalization routines }

function FFT_Init(N_fft: Integer): Boolean;
procedure FFT_Done;

{ Fast Fourier Transform, Back Fourier Transform }

procedure FFT(pr, pc: prmas);
procedure FFB(pr, pc: prmas);

{ Analyze linear voice data }

function FFT_Analyze(pr: prmas): Integer;
function FFT_AnalyzeEx(pr: prmas; var a, mf, te, c: Integer): Integer;

var
  ms: prmas;     { spectral energy density }
  Emax: Double;  { max Energy Density }
  imax: Integer; { Index of Emax }

implementation

type
  pimas = ^imas;
  imas = array[1..2016] of Integer;

var
  xr, xi: prmas;
  wr, wi: prmas;
  ip, jd: pimas;
  N, lgn, np, ns: Integer;

  mr, mi: prmas;

const
  tnp: array[1..12] of Integer = (0,1,2,6,12,28,56,120,240,496,992,2016);

procedure Revers(i, N: Integer; var J: Integer);
var
  k, l: Integer;
begin
  l := N shr 1;
  k :=1;
  j :=0;
  repeat
    if i - l >= 0 then
    begin
      Inc(j, k);
      Dec(i, l)
    end;
    l := l shr 1;
    Inc(k, k)
  until l <= 0;
end;

function FFT_Init(N_fft: Integer): Boolean;
label
  14, 2, 3, 5;
var
  i, id, l: Integer;
begin
  N := N_fft;
  if (N > 4096) or (N < 4) then
  begin
    FFT_Init := False;
    Exit
  end;
  lgn := 0;
  i := 1;
14:
  i := i + i;
  Inc(lgn);
  if i < N then goto 14;
  if i > N then
  begin
    Result := False;
    Exit
  end;
  np := tnp[lgn];
  ns := N div 2;
  GetMem(ip, np * SizeOf(Integer));
  GetMem(jd, np * SizeOf(Integer));
  GetMem(wr, N * SizeOf(Double));
  GetMem(wi, N * SizeOf(Double));
  GetMem(mr, N * SizeOf(Double));
  GetMem(mi, N * SizeOf(Double));
  GetMem(ms, ns * SizeOf(Double));
  np := 0;
  for i := 0 to N - 1 do
  begin
    Revers(i, N, id);
    wr^[id + 1] := Cos(2 * Pi * i / N);
    wi^[id + 1] :=-Sin(2 * Pi * i / N);
    if i = id then goto 2;
    l:=0;
3:
    Inc(l);
    if l > np then goto 5;
    if id = ip^[l] - 1 then goto 2;
    goto 3;
5:
    inc(np);
    ip^[np] := i + 1;
    jd^[np] := id + 1;
2:
  end;
  FFT_Init := True;
end;

procedure FFT_Done;
begin
  FreeMem(ms, ns * SizeOf(Double));
  FreeMem(mi, N * SizeOf(Double));
  FreeMem(mr, N * SizeOf(Double));
  FreeMem(wi, N * SizeOf(Double));
  FreeMem(wr, N * SizeOf(Double));
  FreeMem(jd, np * SizeOf(Integer));
  FreeMem(ip, np * SizeOf(Integer));
end;

procedure FFT(pr, pc: prmas);
var
  i, l, lb, js, j, k1, k2: Integer;
  wxr, wxi: Double;
  qn: Double;
begin
  qn := Sqrt(N);
  lb := N;
  xr := pr;
  xi := pc;
  for i := 1 to lgn do
  begin
    js := lb;
    lb := lb div 2;
    j := 1;
    repeat
      for l := j to j + lb - 1 do
      begin
        k1 := l + lb;
        k2 := (l - 1) div lb + 1;
        wxr := xr^[k1] * wr^[k2] - xi^[k1] * wi^[k2];
        wxi := xr^[k1] * wi^[k2] + xi^[k1] * wr^[k2];
        xr^[k1] := xr^[l] - wxr;
        xi^[k1] := xi^[l] - wxi;
        xr^[ l] := xr^[l] + wxr;
        xi^[ l] := xi^[l] + wxi;
      end;
      Inc(j, js)
    until j > N;
  end;
  for i := 1 to np do
  begin
    wxr := xr^[ip^[i]];
    wxi := xi^[ip^[i]];
    xr^[ip^[i]] := xr^[jd^[i]];
    xi^[ip^[i]] := xi^[jd^[i]];
    xr^[jd^[i]] := wxr;
    xi^[jd^[i]] := wxi
  end;
  for i := 1 to N do
  begin
    xr^[i] := xr^[i] / qn;
    xi^[i] := xi^[i] / qn
  end;
end;

procedure FFB(pr, pc: prmas);
var
  i, l, lb, js, j, k1, k2: Integer;
  wxr, wxi: Double;
  qn: Double;
begin
  qn := Sqrt(N);
  lb := N;
  xr := pr;
  xi := pc;
  for i := 1 to lgn do
  begin
    js := lb;
    lb := lb div 2;
    j := 1;
    repeat
      for l := j to j + lb - 1 do
      begin
        k1 := l + lb;
        k2 := (l - 1) div lb + 1;
        wxr := xr^[k1] * wr^[k2] + xi^[k1] * wi^[k2];
        wxi :=-xr^[k1] * wi^[k2] + xi^[k1] * wr^[k2];
        xr^[k1] := xr^[l] - wxr;
        xi^[k1] := xi^[l] - wxi;
        xr^[ l] := xr^[l] + wxr;
        xi^[ l] := xi^[l] + wxi;
      end;
      inc(j,js)
    until j > N;
  end;
  for i := 1 to np do
  begin
    wxr := xr^[ip^[i]];
    wxi := xi^[ip^[i]];
    xr^[ip^[i]] := xr^[jd^[i]];
    xi^[ip^[i]] := xi^[jd^[i]];
    xr^[jd^[i]] := wxr;
    xi^[jd^[i]] := wxi
  end;
  for i := 1 to N do
  begin
    xr^[i] := xr^[i] / qn;
    xi^[i] := xi^[i] / qn
  end;
end;

function FFT_Analyze(pr: prmas): Integer;
var
  a, mf, te, c: Integer;
begin
  FFT_Analyze := FFT_AnalyzeEx(pr, a, mf, te, c);
end;

function FFT_AnalyzeEx(pr: prmas; var a, mf, te, c: Integer): Integer;
var
  i: Integer;
  E: Double; // Total energy
  m4: Double;
  gt, gc: Boolean;

  function IndToFrec(i : Integer) : Integer;
  begin
    IndToFrec := Round(i * ANALYZE_FMAX / N * 2);
  end;

begin
  { prepare data for FFT }
  for i := 1 to N do
  begin
    mr^[i] := pr^[i]; { pr^[i] - array of initial 'lin' data }
    mi^[i] := 0;      { imagin part for calculations }
  end;
  { FFT }
  FFT(mr, mi);
  { prepare Energy Spectrum, find Emax, imax }
  Emax := 0;
  ms^[1] := 0; { Ignore first point }
  ms^[2] := 0; { Ignore second point }
  ms^[3] := 0; { Ignore point }
  for i := 4 to N div 2 do
  begin
    { Spectral density of energy }
    ms^[i] := Sqr(mr^[i]) + Sqr(mi^[i]);
    { find Energy maximum }
    if Emax < ms^[i] then
    begin
      Emax := ms^[i];
      imax := i;
    end;
  end;
  { Debug values }
  a := Round(Emax / N);
  mf := IndToFrec(iMax);
  { Calc total Energy }
  E := 0;
  for i := 1 to N div 2 do E := E + ms^[i];
  E := E / N / N;
  te := Round(E);
  //  :
  m4 := Emax / 4; //    1/4  
  c := 0;
  gc := False;
  for i := 1 to N div 2 do
  begin
    gt := ms^[i] > m4;
    { find intersection }
    if gc <> gt then
    begin
      gc := gt;
      inc(c);
    end;
  end;
  { Tone }
  if (a > ANALYZE_TONELEVEL) and (mf > ANALYZE_FTONEMIN) and (mf < ANALYZE_FTONEMAX) and (c < 5) then
  begin
    FFT_AnalyzeEx := ANALYZE_TONE;
    Exit;
  end;
  { voice or noise }
  if (E > ANALYZE_ENERGYLEVEL) and (c > ANALYZE_VOICELEVEL) then
  begin
    FFT_AnalyzeEx := ANALYZE_VOICE;
    if c > ANALYZE_NOISELEVEL then FFT_AnalyzeEx := ANALYZE_NOISE;
    Exit;
  end;
  { Silence }
  FFT_AnalyzeEx := ANALYZE_NONE;
end;

end.
