program vt;
{$APPTYPE CONSOLE}
uses
  SysUtils,
  Classes,
  VML_Consts,
  VML_Modem,
  VML_ZyXEL1496,
  VML_Omni56k,
  VML_Tools in 'VML_Tools.pas';

var
   si,so: TStream;
   s: string;
   sr: TSearchRec;

function ReplExt(Name,Ext: string): string;
var p,i: integer;
begin
   i:=1; p:=-1;
   while i < Length(Name) do begin
      if Name[i] = '.' then
         p:=i;
      inc(i);
   end;
   if p < 0 then
      Result:=Name+Ext
   else
      Result:=Copy(Name,1,p-1)+Ext;
end;

function GetParamVal(const TaggedParm : string;
                     IgnoreCase : boolean = true) : string;
var Retvar : string;
    i,Len : integer;
    Comp1,Comp2 : string;
begin
  RetVar := '';
  Comp1 := TaggedParm + '=';
  if IgnoreCase then Comp1 := UpperCase(Comp1);
  Len := length(Comp1);

  for i := 1 to ParamCount do begin
      Comp2 := copy(ParamStr(i),1,Len);
      if IgnoreCase then Comp2 := UpperCase(Comp2);
      if (Comp1 = Comp2) then begin
         RetVar := copy(ParamStr(i),Len + 1,length(ParamStr(i)));
         break;
      end;
  end;

  Result := RetVar;
end;

procedure Usage;
begin
   WriteLn('usage: ');
   WriteLn(' vt /makeraw modem=<modemname> c=<compression> if=infile [of=outfile]');
   WriteLn(' vt /makewav if=infile [of=outfile] [bits=8,16]');
   WriteLn;
   WriteLn('available modem names / compressions:');
   WriteLn('  ZyXEL1496 1,2,3,30,4 (ADPCM)');
   WriteLn('  Omni56k   4 (IMA ADPCM)');
end;

type
   vtMode = ( vtNone, vtWav, vtRmd );

var
   mode: vtMode;
   ifn, ofn: string;
   cmpr: integer;

   modem: TVoiceModem;
   driver: TModemDriver;

function FindModem(name: string): TVoiceModem;
begin
   if AnsiLowerCase(name)='zyxel1496' then
      Driver:=TZyXEL1496.Create(nil)
   else
   if AnsiLowerCase(name)='omni56k' then
      Driver:=TOmni56k.Create(nil)
   else
      raise Exception.Create('unknown modem name');

   Result:=TVoiceModem.Create(nil);
   Result.driver:=driver;
end;

begin
{  Speed:=9600;
  Bits:=4;
}
  try

     if FindCmdLineSwitch('help',['-','/'],true) or
        FindCmdLineSwitch('?',['-','/'],true) or
        FindCmdLineSwitch('h',['-','/'],true)
     then
        Usage;

     mode:=vtNone;
     if FindCmdLineSwitch('makewav',['-','/'],true) then
        mode := vtWav;
     if FindCmdLineSwitch('makeraw',['-','/'],true) then
        mode := vtRmd;

     ifn:=GetParamVal('if',true);
     ofn:=GetParamVal('of',true);
     if mode = vtRmd then begin
        modem:=FindModem(GetParamVal('modem',true));
        s:=GetParamVal('c',true);
        if s='' then Usage;
        cmpr:=StrToInt(s);
     end
     else
        modem:=nil;

     if mode = vtWav then begin
        s:=GetParamVal('bits',true);
        if s<>'' then
           wav_bits:=StrToInt(s);
     end;

     case mode of
        vtWav: Writeln(format('making wav ["%s"->"%s"]',[ifn,ofn]));
        vtRmd: Writeln(format('making rmd ["%s"->"%s"]',[ifn,ofn]));
        vtNone: Usage; //raise Exception.Create('arguments expected');
     end;

     if ofn='' then begin
        if FindFirst(ifn, faAnyFile, sr) = 0 then begin
           //f:=TStringList.Create;
           s:='';
           repeat
              if sr.Name[1] <> '.' then begin
                 si:=TFileStream.Create(sr.Name,fmOpenRead);
                 case mode of
                    vtWav: begin
                             s:=ReplExt(sr.Name,'.wav');
                             writeln('write wav ['+sr.Name+']->['+s+']');
                             so:=TFileStream.Create(s,fmCreate or fmOpenReadWrite);
                             RmdToWav(si,so);
                             so.Free;
                           end;
                    vtRmd: begin
                             s:=ReplExt(sr.Name,'.rmd');
                             writeln('write rmd ['+sr.Name+']->['+s+']');
                             so:=TFileStream.Create(s,fmCreate or fmOpenReadWrite);
                             WavToRmd(Modem,cmpr,si,so);
                             so.Free;
                           end;
                 end;
                 si.Free;
              end;
           until (FindNext(sr) <> 0);
           if s='' then writeln('no files found');
        end;
        FindClose(sr);
     end
     else begin
        si:=TFileStream.Create(ifn,fmOpenRead);
        so:=TFileStream.Create(ofn,fmCreate or fmOpenReadWrite);
        case mode of
           vtWav: RmdToWav(si,so);
           vtRmd: WavToRmd(Modem,cmpr,si,so);
        end;
        si.Free;
        so.Free;
     end;

     if modem<>nil then begin
        driver.Free;
        modem.Free;
     end;
  except
     on E: Exception do
        writeln('Error: '+E.Message);
  end;
end.

