(*==========================================================================
 *
 *  mmsystem.h -- Include file for Multimedia API's
 *
 *  Version 4.00
 *
 *  Copyright (C) 1992-1995 Microsoft Corporation 
 *
 *==========================================================================
 *
 *  Translator:
 *
 *  Copyright (C) 2000 Roman M. Mochalov
 *  E-mail: roman@sar.nnov.ru
 *
 *==========================================================================
 *)

unit MMSys;

interface

uses Windows;

{ general waveform format structure (information common to all formats) }

type
  PWaveFormat = ^TWaveFormat;
  TWaveFormat = packed record
    wFormatTag: Word;       { format type }
    nChannels: Word;        { number of channels (i.e. mono, stereo, etc.) }
    nSamplesPerSec: DWORD;  { sample rate }
    nAvgBytesPerSec: DWORD; { for buffer estimation }
    nBlockAlign: Word;      { block size of data }
  end;

{ flags for wFormatTag field of WAVEFORMAT }

const
  WAVE_FORMAT_PCM = 1;

{ specific waveform format structure for PCM data }

type
  PPCMWaveFormat = ^TPCMWaveFormat;
  TPCMWaveFormat = packed record
    wf: TWaveFormat;
    wBitsPerSample: Word;
  end;

{ general extended waveform format structure. Use this for all NON PCM
  formats (information common to all formats) }

type
  PWaveFormatEx = ^TWaveFormatEx;
  TWaveFormatEx = packed record
    wFormatTag: Word;       { format type }
    nChannels: Word;        { number of channels (i.e. mono, stereo, etc.) }
    nSamplesPerSec: DWORD;  { sample rate }
    nAvgBytesPerSec: DWORD; { for buffer estimation }
    nBlockAlign: Word;      { block size of data }
    wBitsPerSample: Word;   { number of bits per sample of mono data }
    cbSize: Word;           { the count in bytes of the size of extra information (after cbSize) }
  end;

{ MMIO data types }

type
  FOURCC = DWORD;  { a four character code }

function mmioFOURCC(ch0, ch1, ch2, ch3: Char): FOURCC;
function mmioFOURCCToString(dw: FOURCC): string;

{ standard four character codes }

const
  FOURCC_RIFF: DWORD = $46464952;   { 'RIFF' }
  FOURCC_LIST: DWORD = $5453494C;   { 'LIST' }

{ additional four character codes }

const
  FOURCC_WAVE: DWORD = $45564157;   { 'WAVE' }
  FOURCC_FMT: DWORD  = $20746D66;   { 'fmt ' }
  FOURCC_DATA: DWORD = $61746164;   { 'data' }

implementation

function mmioFOURCC(ch0, ch1, ch2, ch3: Char): FOURCC;
begin
  Result := DWORD(Byte(ch0)) or (DWORD(Byte(ch1) shl 8)) or
    DWORD(Byte(ch2) shl 16) or DWORD(Byte(ch3) shl 24);
end;

function mmioFOURCCToString(dw: FOURCC): string;
begin
  SetString(Result, nil, SizeOf(FOURCC));
  Move(dw, Result[1], SizeOf(FOURCC));
end;

end.
