{
  Voice Modem Library v1.0

  This file contains hardware driver functions for 
  Advanced Voice Courier (RC-33600) modems.

  Copyright (C) 1998-2001 Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_AVC33600;

{$I VML.INC}

interface

uses
  Windows, SysUtils, Classes, VML_Modem, VML_Consts, VML_IS101;

const

{ AVC33600 available compression modes }

  AVC33600_GSM = 2;

{ AVC33600 modem strings }

  SAVC33600Name    = 'Advanced Voice Courier (RC-33600)';
  SAVC33600RMDName = 'AVC33600';

{$I *.INC}

type

{ TAVC33600 }

  TAVC33600 = class(TIS101)
  public
    constructor Create(AOwner: TComponent); override;
    function Beep(Frequency, Length: Integer): Integer; override;
  end;

implementation

{ TAVC33600 }

constructor TAVC33600.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  { Modem name }
  ModemName := SAVC33600Name;
  ModemRMDName := SAVC33600RMDName;
  { &H1 - Transmit data flow control: Clear to Send (CTS).
    &R2 - Receive data flow control: only Request to Send (RTS).
    &I0 - Disable software flow control. }
  HardFlowCmd := 'AT&H1&R2&I0';
  { A - Answering in Voice. }
  PickPhoneCmd := 'ATA';
  PickPhoneAnsw := 'OK|VCON';
  { Device-specific initialization command }
  DevInitCmd := 'AT&F1&M5&W';
  DevInitAnsw := 'OK';
  { +BEEP= f1,l1,f2,l2,d,p,c,fd - Generate tone signals.
    f1 - First frequency value (Hz).
    l1 - First frequency level (0-63).
    f2 - Second frequency value (Hz).
    l2 - Second frequency level (0-63).
    d  - Signal duration in 0.01 sec.
    p  - Delay before processing next command in 0.01 sec..
    c  - Signal repeat counter.
    fd - Delay after final tone before processing next command in 0.01 sec. }
  BeepCmd := 'AT+BEEP%d,30,,,%d,,,';
  BeepTimeUnit := 10;
  { #VTX - Voice Transmit. }
  StartPlayCmd := 'AT#VTX';
  IntrPlayCmd := '<h16><h03>';
  IntrPlayAnsw := 'OK|VCON';
  StopPlayAnsw := 'OK|VCON';
  { #VRX - Voice Receive. }
  StartRecCmd := 'AT#VRX';
  StopRecCmd := '<DLE>';
  StopRecAnsw := 'OK|VCON';
  { #CLS - Select Data or Voice mode. }
  SwitchModeCmd := 'AT#CLS=%s';
  VoiceModeOnCmd := 'AT#CLS=8';
  VoiceModeOffCmd := 'AT#CLS=0';
  { S38=n - Inactivity timer. }
  InactivityTimerCmd := 'ATS38=%d';
  InactivityTimerAnsw := 'OK';
  { #BDR - Select a specific DTE/modem baud rate. }
  BaudRateCmd := 'AT#BDR=%d';
  BaudRateAnsw := 'OK';
  BaudRateUnit := 2400;
  { #VSS - Silence Detection. }
  SilenceDeleteCmd := 'AT#VSS=%d';
  SilenceDeleteAnsw := 'OK';
  { +X00=1  - Enable russian CID.
    +X02=45 - CID output format settings. }
  CallerIDOnCmd := 'AT+X00=1+X02=45';
  CallerIDOnAnsw := 'OK';
  CallerIDOffCmd := 'AT+X00=0';
  CallerIDOffAnsw := 'OK';
  CallerIDFormat := 'CALLER ID:';
  { #VLS=0 - Telephone line.
    #VLS=1 - Local handset.
    #VLS=4 - External speaker. }
  Devices.Clear;
  Devices.Add(NO_DEVICE, SNoDevice, 'AT#VLS=0H0', 'OK|VCON', DF_HIDDEN);
  Devices.Add(DIALUP_LINE, SDialupLine, 'AT#VLS=0', 'OK|VCON', DF_TRANSMIT_RECEIVE);
  Devices.Add(EXTERNAL_SPEAKER, SExternalSpeaker, 'AT#VLS=4','OK|VCON', DF_TRANSMIT);
  Devices.Add(LOCAL_HANDSET, SLocalHandset, 'AT#VLS=1','OK|VCON', DF_TRANSMIT_RECEIVE);
  { #VBS=2 - GSM 8000 Hz (default).
    #VSR=n - Sample rate selection (8000 Hz only). }
  Compressions.Clear;
  Compressions.Add(AVC33600_GSM, SAVC33600GSM, 'AT#VBS=2', 'OK', 8000, 8, CF_8000 or CF_DEFAULT, 0, 0, 0);
end;

function TAVC33600.Beep(Frequency, Length: Integer): Integer;
begin
  { Beep command doesn't works under voice mode. }
  if VoiceModeOff = S_FAIL then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Generate tone }
  if inherited Beep(Frequency, Length) = S_FAIL then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Restore voice mode }
  Result := RestoreVoiceMode;
end;

initialization
  RegisterModemDriver(SAVC33600Name, TAVC33600);

end.
