{
  Voice Modem Library v1.0

  This file contain definition of common constants,
  types and routines.

  Copyright (C) 1998-2001 Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_Consts;

{$I VML.INC}

interface

uses
  SysUtils, Classes, VML_Idents;

{$I *.INC}

const

{ Success codes }

  S_OK          = 0;
  S_FAIL        = -1;

  E_INTERRUPTED = $4D00; { Operation interrupted }
  E_TIMEOUT     = $4001; { Operation timeout }

  // Backward capability constants (for v1.0.4 and lower)

  OK          = S_OK;
  FAIL        = S_FAIL;
  INTERRUPTED = E_INTERRUPTED;

{ Possible errors }

  { Operation completed successfully }
  VME_SUCCESS = 0;

  { Method not implemented }
  VME_NOT_IMPLEMENTED = 1;

  { Operation timeout }
  VME_TIMEOUT = 2;

  { Operation interrupted }
  VME_INTERRUPTED = 3;

  { Unqueued event not handled }
  VME_EVENT_NOT_HANDLED = 11;

  { Event queue is full }
  VME_EVENT_QUEUE_FULL = 12;

  { No more events in event queue }
  VME_EVENT_QUEUE_EMPTY = 13;

  { Unqueued event is locked }
  VME_EVENT_LOCKED = 14;

  { Profile property not specified }
  VME_PROFILE_NOT_SPECIFIED = 21;

  { Port property not specified }
  VME_PORT_NOT_SPECIFIED = 22;

  { Driver not specified }
  VME_DRIVER_NOT_SPECIFIED = 23;

  { Can't load driver ini file }
  VME_DRIVER_INI_FILE_ERROR = 24;

  { No driver class registered for specified modem name }
  VME_DRIVER_NOT_REGISTERED = 25;

  { Port is already in use }
  VME_PORT_BUSY = 31;

  { Unexpected port error }
  VME_PORT_ERROR = 32;

  { Port not opened }
  VME_PORT_NOT_OPENED = 33;

  { Modem already locked }
  VME_MODEM_LOCKED = 41;

  { Invalid modem answer }
  VME_INVALID_MODEM_ANSWER = 42;

  { Invalid command format }
  VME_INVALID_COMMAND_FORMAT = 43;

  { Invalid output string format }
  VME_INVALID_STR_FORMAT = 44;

  { Command not supported or contains invalid parameters }
  VME_COMMAND_ERROR = 45;

  { Illegal compression method }
  VME_ILLEGAL_COMPRESSION = 51;

  { Illegal sample rate }
  VME_ILLEGAL_SAMPLE_RATE = 52;

  { Unknown device }
  VME_UNKNOWN_DEVICE = 53;

  { Stream read error }
  VME_STREAM_READ_ERR = 61;

  { Stream write error }
  VME_STREAM_WRITE_ERR = 62;

  { No raw modem data header found }
  VME_NOT_RMD_FILE = 63;

  { Voice data unsupported by current modem device }
  VME_WRONG_MODEM_TYPE = 64;

  { Given file not found }
  VME_FILE_NOT_FOUND = 71;

  { Could not open input file }
  VME_FILE_OPEN_ERROR = 72;

  { Could not create output file }
  VME_FILE_CREATE_ERROR = 73;

  { Another voice operation in progress }
  VME_OPERATION_IN_PROGRESS = 81;

  { No active voice operation }
  VME_NO_ACTIVE_OPERATION = 82;

  { Data converting library already locked }
  VME_CONVERT_LIB_LOCKED = 101;

  { Data converting error }
  VME_DATA_CONVERT_FAILED = 102;

  { FFT init failed }
  VME_FFT_INIT_FAILED = 111;

  { Answer recognition failed }
  VME_RECOGNITION_FAILED = 112;

  { Illegal WAVE Format }
  VME_ILLEGAL_WAVE_FORMAT = 121;

  { This WAVE file format is not supported }
  VME_UNSUPPORTED_WAVE_FORMAT = 122;

  { Given file is not a RIFF file }
  VME_NOT_RIFF_FILE = 123;

  { Given file is not a WAVE file }
  VME_NOT_WAVE_FILE = 124;

  { Missing fmt chunk }
  VME_MISSING_FMT_CHUNK = 125;

  { Missing data chunk }
  VME_MISSING_DATA_CHUNK = 126;

  { Unknown data format }
  VME_UNKNOWN_DATA_FORMAT = 131;

{ VML error map }

  VMLErrorMap: array[0..43] of TIdentMapEntry = (
    (Value: VME_SUCCESS; Name: SVMESuccess),
    (Value: VME_NOT_IMPLEMENTED; Name: SVMENotImplemented),
    (Value: VME_TIMEOUT; Name: SVMETimeout),
    (Value: VME_INTERRUPTED; Name: SVMEInterrupted),
    (Value: VME_EVENT_NOT_HANDLED; Name: SVMEEventNotHandled),
    (Value: VME_EVENT_QUEUE_FULL; Name: SVMEEventQueueFull),
    (Value: VME_EVENT_QUEUE_EMPTY; Name: SVMEEventQueueEmpty),
    (Value: VME_EVENT_LOCKED; Name: SVMEEventLocked),
    (Value: VME_PROFILE_NOT_SPECIFIED; Name: SVMEProfileNotSpecified),
    (Value: VME_PORT_NOT_SPECIFIED; Name: SVMEPortNotSpecified),
    (Value: VME_DRIVER_NOT_SPECIFIED; Name: SVMEDriverNotSpecified),
    (Value: VME_DRIVER_INI_FILE_ERROR; Name: SVMEDriverIniFileError),
    (Value: VME_DRIVER_NOT_REGISTERED; Name: SVMEDriverNotRegistered),
    (Value: VME_PORT_BUSY; Name: SVMEPortBusy),
    (Value: VME_PORT_ERROR; Name: SVMEPortError),
    (Value: VME_PORT_NOT_OPENED; Name:SVMEPortNotOpened),
    (Value: VME_MODEM_LOCKED; Name: SVMEModemLocked),
    (Value: VME_INVALID_MODEM_ANSWER; Name: SVMEInvalidModemAnswer),
    (Value: VME_INVALID_COMMAND_FORMAT; Name: SVMEInvalidCommandFormat),
    (Value: VME_INVALID_STR_FORMAT; Name: SVMEInvalidStrFormat),
    (Value: VME_COMMAND_ERROR; Name: SVMECommandError),
    (Value: VME_ILLEGAL_COMPRESSION; Name: SVMEIllegalCompression),
    (Value: VME_ILLEGAL_SAMPLE_RATE; Name: SVMEIllegalSampleRate),
    (Value: VME_UNKNOWN_DEVICE; Name: SVMEUnknownDevice),
    (Value: VME_STREAM_READ_ERR; Name: SVMEStreamReadErr),
    (Value: VME_STREAM_WRITE_ERR; Name: SVMEStreamWriteErr),
    (Value: VME_NOT_RMD_FILE; Name: SVMENotRMDFile),
    (Value: VME_WRONG_MODEM_TYPE; Name: SVMEWrongModemType),
    (Value: VME_FILE_NOT_FOUND; Name: SVMEFileNotFound),
    (Value: VME_FILE_OPEN_ERROR; Name: SVMEFileOpenerror),
    (Value: VME_FILE_CREATE_ERROR; Name: SVMEFileCreateError),
    (Value: VME_OPERATION_IN_PROGRESS; Name: SVMEOperationInProgress),
    (Value: VME_NO_ACTIVE_OPERATION; Name: SVMENoActiveOperation),
    (Value: VME_CONVERT_LIB_LOCKED; Name: SVMEConvertLiBlocked),
    (Value: VME_DATA_CONVERT_FAILED; Name: SVMEDataConvertFailed),
    (Value: VME_FFT_INIT_FAILED; Name: SVMEFFTInitFailed),
    (Value: VME_RECOGNITION_FAILED; Name: SVMERecognitionFailed),
    (Value: VME_ILLEGAL_WAVE_FORMAT; Name: SVMEIllegalWaveFormat),
    (Value: VME_UNSUPPORTED_WAVE_FORMAT; Name: SVMEUnsupportedWaveFormat),
    (Value: VME_NOT_RIFF_FILE; Name: SVMENotRIFFFile),
    (Value: VME_NOT_WAVE_FILE; Name: SVMENotWaveFile),
    (Value: VME_MISSING_FMT_CHUNK; Name: SVMEMissingFmtChunk),
    (Value: VME_MISSING_DATA_CHUNK; Name: SVMEMissingDataChunk),
    (Value: VME_UNKNOWN_DATA_FORMAT; Name: SVMEUnknownDataFormat));

{ VML error messages }

function GetVMLErrorStr(Error: Integer): string;

const

{ Possible input or output devices }

  NO_DEVICE           = $0001;
  DIALUP_LINE         = $0002;
  EXTERNAL_MICROPHONE = $0003;
  INTERNAL_MICROPHONE = $0004;
  EXTERNAL_SPEAKER    = $0005;
  INTERNAL_SPEAKER    = $0006;
  LOCAL_HANDSET       = $0007;

  DIALUP_WITH_EXT_SPEAKER              = $0008;
  DIALUP_WITH_INT_SPEAKER              = $0009;
  DIALUP_WITH_LOCAL_HANDSET            = $000A;
  DIALUP_WITH_EXTERNAL_MIC_AND_SPEAKER = $000B;
  DIALUP_WITH_INTERNAL_MIC_AND_SPEAKER = $000C;

  SPEAKER_PHONE        = $0010;
  SPEAKER_PHONE_MUTE   = $0011;
  SPEAKER_PHONE_UNMUTE = $0012;

  SOUND_CARD = $0020;

{ Devices map }

  DeviceMap: array[0..7] of TIdentMapEntry = (
    (Value: NO_DEVICE; Name: 'NO_DEVICE'),
    (Value: DIALUP_LINE; Name: 'DIALUP_LINE'),
    (Value: EXTERNAL_MICROPHONE; Name: 'EXTERNAL_MICROPHONE'),
    (Value: INTERNAL_MICROPHONE; Name: 'INTERNAL_MICROPHONE'),
    (Value: EXTERNAL_SPEAKER; Name: 'EXTERNAL_SPEAKER'),
    (Value: INTERNAL_SPEAKER; Name: 'INTERNAL_SPEAKER'),
    (Value: LOCAL_HANDSET; Name: 'LOCAL_HANDSET'),
    (Value: SPEAKER_PHONE; Name: 'SPEAKER_PHONE'));

{ Device flags }

  DF_TRANSMIT = $0001; { This is transmit device }
  DF_RECEIVE  = $0002; { This is receive device }
  DF_HIDDEN   = $0010; { Do not include this device into the devices string list }

  DF_TRANSMIT_RECEIVE = DF_TRANSMIT or DF_RECEIVE; 

{ Compression flags }

  CF_7200    = $0001; { Compression supports 7200 Hz sampling rate }
  CF_8000    = $0002; { Compression supports 8000 Hz sampling rate }
  CF_9600    = $0004; { Compression supports 9600 Hz sampling rate }
  CF_11025   = $0008; { Compression supports 11025 Hz sampling rate }
  CF_DEFAULT = $0100; { This is default compression method }
  CF_HIDDEN  = $0200; { Don't include this compresion into the comressions string list }

{ Voice data formats }

  DATA_FORMAT_AUTO    = 0; { Autodetect (default) data format }
  DATA_FORMAT_RMD     = 1; { Raw Modem Data }
  DATA_FORMAT_WAVE    = 2; { WAVE Fromat }

  DATA_FORMAT_UNKNOWN = -1; { Unknown format }

{ Voice modem answers }

{ The user defined ones }

  VMA_USER         = $1000;
  VMA_USER_1       = $1000;
  VMA_USER_2       = $1001;
  VMA_USER_3       = $1002;
  VMA_USER_4       = $1003;
  VMA_USER_5       = $1004;
  VMA_USER_6       = $1005;
  VMA_USER_7       = $1006;
  VMA_USER_8       = $1007;
  VMA_USER_9       = $1008;
  VMA_USER_10      = $1009;
  VMA_USER_11      = $100A;
  VMA_USER_12      = $100B;
  VMA_USER_13      = $100C;
  VMA_USER_14      = $100D;
  VMA_USER_15      = $100E;
  VMA_USER_16      = $100F;

{ The default ones }

  VMA_DEFAULT      = $2000;
  VMA_BUSY         = $2000;
  VMA_CONNECT      = $2001;
  VMA_EMPTY        = $2002;
  VMA_ERROR        = $2003;
  VMA_FAX          = $2004;
  VMA_FCON         = $2005;
  VMA_FCO          = $2006;
  VMA_NO_ANSWER    = $2007;
  VMA_NO_CARRIER   = $2008;
  VMA_NO_DIAL_TONE = $2009;
  VMA_OK           = $200A;
  VMA_RINGING      = $200B;
  VMA_RING_1       = $200C;
  VMA_RING_2       = $200D;
  VMA_RING_3       = $200E;
  VMA_RING_4       = $200F;
  VMA_RING_5       = $2010;
  VMA_RING         = $2011;
  VMA_VCON         = $2012;
  VMA_CALLERID     = $2013;

  VMA_UNKNOWN      = $2020;
  VMA_IGNORE       = $2021;

{ If something goes wrong }

  VMA_FAIL         = $4000;
  VMA_FAIL_1       = $4000;
  VMA_FAIL_2       = $4001;
  VMA_FAIL_3       = $4002;
  VMA_FAIL_4       = $4003;
  VMA_FAIL_5       = $4004;
  VMA_FAIL_6       = $4005;
  VMA_FAIL_7       = $4006;
  VMA_FAIL_8       = $4007;
  VMA_FAIL_9       = $4008;
  VMA_FAIL_10      = $4009;
  VMA_FAIL_11      = $400A;
  VMA_FAIL_12      = $400B;
  VMA_FAIL_13      = $400C;
  VMA_FAIL_14      = $400D;
  VMA_FAIL_15      = $400E;
  VMA_FAIL_16      = $400F;

{ Voice modem answers map }

  AnswerMap: array[0..22] of TIdentMapEntry = (
    (Value: VMA_BUSY; Name: 'VMA_BUSY'),
    (Value: VMA_CONNECT; Name: 'VMA_CONNECT'),
    (Value: VMA_EMPTY; Name: 'VMA_EMPTY'),
    (Value: VMA_ERROR; Name: 'VMA_ERROR'),
    (Value: VMA_FAX; Name: 'VMA_FAX'),
    (Value: VMA_FCON; Name: 'VMA_FCON'),
    (Value: VMA_FCO; Name: 'VMA_FCO'),
    (Value: VMA_NO_ANSWER; Name: 'VMA_NO_ANSWER'),
    (Value: VMA_NO_CARRIER; Name: 'VMA_NO_CARRIER'),
    (Value: VMA_NO_DIAL_TONE; Name: 'VMA_NO_DIAL_TONE'),
    (Value: VMA_OK; Name: 'VMA_OK'),
    (Value: VMA_RINGING; Name: 'VMA_RINGING'),
    (Value: VMA_RING_1; Name: 'VMA_RING_1'),
    (Value: VMA_RING_2; Name: 'VMA_RING_2'),
    (Value: VMA_RING_3; Name: 'VMA_RING_3'),
    (Value: VMA_RING_4; Name: 'VMA_RING_4'),
    (Value: VMA_RING_5; Name: 'VMA_RING_5'),
    (Value: VMA_RING; Name: 'VMA_RING'),
    (Value: VMA_VCON; Name: 'VMA_VCON'),
    (Value: VMA_CALLERID; Name: 'VMA_CALLERID'),
    (Value: VMA_UNKNOWN; Name: 'VMA_UNKNOWN'),
    (Value: VMA_IGNORE; Name: 'VMA_IGNORE'),
    (Value: VMA_FAIL; Name: 'VMA_FAIL'));

{ Standard modem responses map }

  ModemAnswers: array[0..30] of TIdentMapEntry = (
   (Value: VMA_BUSY; Name: 'BUSY'),
   (Value: VMA_CONNECT; Name: 'CONNECT'),
   (Value: VMA_ERROR; Name: 'ERROR'),
   (Value: VMA_FAX; Name: 'FAX'),
   (Value: VMA_FCON; Name: '+FCON'),
   (Value: VMA_FCO; Name: '+FCO'),
   (Value: VMA_NO_ANSWER; Name: 'NO ANSWER'),
   (Value: VMA_NO_CARRIER; Name: 'NO CARRIER'),
   (Value: VMA_NO_DIAL_TONE; Name: 'NO DIALTONE'),
   (Value: VMA_NO_DIAL_TONE; Name: 'NO DIAL TONE'),
   (Value: VMA_OK; Name: 'OK'),
   (Value: VMA_RINGING; Name: 'RINGING'),
   (Value: VMA_RING_1; Name: 'RING1'),
   (Value: VMA_RING_2; Name: 'RING2'),
   (Value: VMA_RING_3; Name: 'RING3'),
   (Value: VMA_RING_4; Name: 'RING4'),
   (Value: VMA_RING_5; Name: 'RING5'),
   (Value: VMA_RING_1; Name: 'RING 1'),
   (Value: VMA_RING_2; Name: 'RING 2'),
   (Value: VMA_RING_3; Name: 'RING 3'),
   (Value: VMA_RING_4; Name: 'RING 4'),
   (Value: VMA_RING_5; Name: 'RING 5'),
   (Value: VMA_RING_1; Name: 'RING A'),
   (Value: VMA_RING_2; Name: 'RING B'),
   (Value: VMA_RING_3; Name: 'RING C'),
   (Value: VMA_RING_4; Name: 'RING D'),
   (Value: VMA_RING_5; Name: 'RING E'),
   (Value: VMA_RING; Name: 'RING'),
   (Value: VMA_VCON; Name: 'VCON'),
   (Value: VMA_VCON; Name: '#VCON'),
   (Value: VMA_VCON; Name: 'VOICE'));

  UserAnswers: array[0..15] of TIdentMapEntry = (
   (Value: VMA_USER; Name: 'VM_USER'),
   (Value: VMA_USER_2; Name: 'VM_USER_2'),
   (Value: VMA_USER_3; Name: 'VM_USER_3'),
   (Value: VMA_USER_4; Name: 'VM_USER_4'),
   (Value: VMA_USER_5; Name: 'VM_USER_5'),
   (Value: VMA_USER_6; Name: 'VM_USER_6'),
   (Value: VMA_USER_7; Name: 'VM_USER_7'),
   (Value: VMA_USER_8; Name: 'VM_USER_8'),
   (Value: VMA_USER_9; Name: 'VM_USER_9'),
   (Value: VMA_USER_10; Name: 'VM_USER_10'),
   (Value: VMA_USER_11; Name: 'VM_USER_11'),
   (Value: VMA_USER_12; Name: 'VM_USER_12'),
   (Value: VMA_USER_13; Name: 'VM_USER_13'),
   (Value: VMA_USER_14; Name: 'VM_USER_14'),
   (Value: VMA_USER_15; Name: 'VM_USER_15'),
   (Value: VMA_USER_16; Name: 'VM_USER_16'));

{ Answer idents routines }

function StrToModemAnswer(const S: string): Longint;
function StrToModemAnswerDef(const S: string; Default: Longint): Longint;

function ModemAnswerToStr(Answer: Longint): string;
function ModemAnswerToStrDef(Answer: Longint; const Default: string): string;

{ Answer analyze routines }

function AnalyzeModemAnswer(const S: string): Longint;
function AnalyzeModemAnswerDef(const S: string; const Default: Longint): Longint;

const

{ Possible port, voice modem and shell execution states }

  DIALING          = $0000;
  IDLE             = $0001;
  INITIALIZING     = $0002;
  OFF_LINE         = $0003;
  ON_LINE          = $0004;
  PLAYING          = $0005;
  RECORDING        = $0006;
  WAITING          = $0007;

{ The different answer modes }

  ANSWER_DATA  = 1;
  ANSWER_FAX   = 2;
  ANSWER_VOICE = 4;

{ Events hadling constants }

  NO_EVENT      = $0000;
  UNKNOWN_EVENT = -2;

{ Possible event messages for the voice modem }

  VOICE_CODE_EVENT         = $1000;
  VOICE_ANSWER_PHONE       = $1000;
  VOICE_BEEP               = $1001;
  VOICE_DIAL               = $1002;
  VOICE_HANDLE_DLE         = $1003;

  VOICE_INIT               = $1004;
  VOICE_MESSAGE_LIGHT_ON   = $1005;
  VOICE_MESSAGE_LIGHT_OFF  = $1006;
  VOICE_MODE_OFF           = $1007;
  VOICE_MODE_ON            = $1008;
  VOICE_PLAY_FILE          = $1009;
  VOICE_RECORD_FILE        = $100A;
  VOICE_SET_COMPRESSION    = $100B;
  VOICE_SET_DEVICE         = $100C;
  VOICE_STOP_DIALING       = $100D;
  VOICE_STOP_PLAYING       = $100E;
  VOICE_STOP_RECORDING     = $100F;
  VOICE_STOP_WAITING       = $1010;
  VOICE_SWITCH_TO_DATA_FAX = $1011;
  VOICE_WAIT               = $1012;

{ Possible event messages from the voice modem }

  VOICE_MODEM_EVENT    = $2000;
  BONG_TONE            = $2000;
  BUSY_TONE            = $2001;
  CALL_WAITING         = $2002;
  DIAL_TONE            = $2003;
  DATA_CALLING_TONE    = $2004;
  DATA_OR_FAX_DETECTED = $2005;
  FAX_CALLING_TONE     = $2006;
  HANDSET_ON_HOOK      = $2007;
  HANDSET_OFF_HOOK     = $2008;
  LOOP_BREAK           = $2009;
  LOOP_POLARITY_CHANGE = $200A;
  NO_ANSWER            = $200B;
  NO_CARRIER           = $200C;
  NO_DIAL_TONE         = $200D;
  NO_VOICE_ENERGY      = $200E;
  RING_DETECTED        = $200F;
  RINGBACK_DETECTED    = $2010;
  RECEIVED_DTMF        = $2011;
  SILENCE_DETECTED     = $2012;
  SIT_TONE             = $2013;
  TDD_DETECTED         = $2014;
  VOICE_DETECTED       = $2015;
  REMOTE_HANGUP        = $2016;
  RINGING_TONE         = $2017;
  CONNECT_DETECTED     = $2018;

  OPERATION_TIMEOUT    = $2021;

{ Possible signal events }

  SIGNAL_EVENT   = $4000;
  RESET_WATCHDOG = $4000;
  SIGNAL_SIGCHLD = $4001;
  SIGNAL_SIGHUP  = $4002;
  SIGNAL_SIGINT  = $4003;
  SIGNAL_SIGPIPE = $4004;
  SIGNAL_SIGQUIT = $4005;
  SIGNAL_SIGTERM = $4006;
  SIGNAL_SIGUSR1 = $4007;
  SIGNAL_SIGUSR2 = $4008;

  SIGNAL_STOP    = $4010;

{ Possible events map }

  EventMap: array[0..26] of TIdentMapEntry = (
    (Value: BONG_TONE; Name: 'BONG_TONE'),
    (Value: BUSY_TONE; Name: 'BUSY_TONE'),
    (Value: CALL_WAITING; Name: 'CALL_WAITING'),
    (Value: DIAL_TONE; Name: 'DIAL_TONE'),
    (Value: DATA_CALLING_TONE; Name: 'DATA_CALLING_TONE'),
    (Value: DATA_OR_FAX_DETECTED; Name: 'DATA_OR_FAX_DETECTED'),
    (Value: FAX_CALLING_TONE; Name: 'FAX_CALLING_TONE'),
    (Value: HANDSET_ON_HOOK; Name: 'HANDSET_ON_HOOK'),
    (Value: HANDSET_OFF_HOOK; Name: 'HANDSET_OFF_HOOK'),
    (Value: LOOP_BREAK; Name: 'LOOP_BREAK'),
    (Value: LOOP_POLARITY_CHANGE; Name: 'LOOP_POLARITY_CHANGE'),
    (Value: NO_ANSWER; Name: 'NO_ANSWER'),
    (Value: NO_CARRIER; Name: 'NO_CARRIER'),
    (Value: NO_DIAL_TONE; Name: 'NO_DIAL_TONE'),
    (Value: NO_VOICE_ENERGY; Name: 'NO_VOICE_ENERGY'),
    (Value: RING_DETECTED; Name: 'RING_DETECTED'),
    (Value: RINGBACK_DETECTED; Name: 'RINGBACK_DETECTED'),
    (Value: RECEIVED_DTMF; Name: 'RECEIVED_DTMF'),
    (Value: SILENCE_DETECTED; Name: 'SILENCE_DETECTED'),
    (Value: SIT_TONE; Name: 'SIT_TONE'),
    (Value: TDD_DETECTED; Name: 'TDD_DETECTED'),
    (Value: VOICE_DETECTED; Name: 'VOICE_DETECTED'),
    (Value: REMOTE_HANGUP; Name: 'REMOTE_HANGUP'),
    (Value: RINGING_TONE; Name: 'RINGING_TONE'),
    (Value: CONNECT_DETECTED; Name: 'CONNECT_DETECTED'),
    (Value: OPERATION_TIMEOUT; Name: 'OPERATION_TIMEOUT'),
    (Value: SIGNAL_STOP; Name: 'SIGNAL_STOP'));

{ ASCII control codes }

  NUL = ^@; { $00 null (end string) }
  SOH = ^A; { $01 start of heading }
  STX = ^B; { $02 start of text }
  ETX = ^C; { $03 end of text }
  EOT = ^D; { $04 end of trasmition }
  ENQ = ^E; { $05 enquiry }
  ACK = ^F; { $06 acknowledge }
  BEL = ^G; { $07 bell }
  BS  = ^H; { $08 backspace }
  HT  = ^I; { $09 horizontal tab (TAB ) }
  LF  = ^J; { $0A line feed }
  VT  = ^K; { $0B vertical tab }
  FF  = ^L; { $0C form feed }
  CR  = ^M; { $0D carriage return }
  SO  = ^N; { $0E shift out }
  SI  = ^O; { $0F shift in }
  DLE = ^P; { $10 data line escape }
  DC1 = ^Q; { $11 device control 1 (X-ON) }
  DC2 = ^R; { $12 device control 2 }
  DC3 = ^S; { $13 device control 3 (X-OFF) }
  DC4 = ^T; { $14 device control 4 }
  NAK = ^U; { $15 negative acknowledge }
  SYN = ^V; { $16 synchronous idle }
  ETB = ^W; { $17 end transmit block }
  CAN = ^X; { $18 cancel }
  EM  = ^Y; { $19 end of medium }
  SUB = ^Z; { $1A substitute }
  ESC = ^[; { $1B escape }
  FS  = ^\; { $1C file separator }
  GS  = ^]; { $1D group separator }
  RS  = ^^; { $1E record separator }
  US  = ^_; { $1F unit separator }

  XON  = DC1; { XON char }
  XOFF = DC3; { XOFF char }

{ ASCII control codes map }

type
  PControlCodeMap = ^TControlCodeMap;
  TControlCodeMap = array[0..31] of string;

const

  { 'Dec' control codes map }

  CODEMAP_DEC: TControlCodeMap = (
    '#0', '#1', '#2', '#3', '#4', '#5', '#6', '#7',
    '#8', '#9', '#10', '#11', '#12', '#13', '#14', '#15',
    '#16', '#17', '#18', '#19', '#20', '#21', '#22', '#23',
    '#24', '#25', '#26', '#27', '#28', '#29', '#30', '#31');

  { 'Hex' control codes map }

  CODEMAP_HEX: TControlCodeMap = (
    'h00', 'h01', 'h02', 'h03', 'h04', 'h05', 'h06', 'h07',
    'h08', 'h09', 'h0a', 'h0b', 'h0c', 'h0d', 'h0e', 'h0f',
    'h10', 'h11', 'h12', 'h13', 'h14', 'h15', 'h16', 'h17',
    'h18', 'h19', 'h1a', 'h1b', 'h1c', 'h1d', 'h1e', 'h1f');

  { 'Ctl' control codes map }

  CODEMAP_CTL: TControlCodeMap = (
    '^@', '^A', '^B', '^C', '^D', '^E', '^F', '^G',
    '^H', '^I', '^J', '^K', '^L', '^M', '^N', '^O',
    '^P', '^Q', '^R', '^S', '^T', '^U', '^V', '^W',
    '^X', '^Y', '^Z', '^[', '^\', '^]', '^^', '^_');

  { 'Name' control codes map }

  CODEMAP_NAME: TControlCodeMap = (
    'NUL', 'SOH', 'STX', 'ETX', 'EOT', 'ENQ', 'ACK', 'BEL',
    'BS', 'HT', 'LF', 'VT', 'FF', 'CR', 'SO', 'SI',
    'DLE', 'XON', 'DC2', 'XOFF', 'DC4', 'NAK', 'SYN', 'ETB',
    'CAN', 'EM', 'SUB', 'ESC', 'FS', 'GS', 'RS', 'US');

{ MapControlCodes replaces all ASCII control chars in the given string
  to corresponded name with given border (framing) chars using given codes map.
  If map is not assigned (nil) control codes will be replace using CODEMAP_HEX
  control codes map, $0A will be replace to <lf>, $0D to <cr>.}

function MapControlCodes(const S, F: string; Map: PControlCodeMap): string;

{ UnMapControlCodes replaces all ASCII control char names in the given string
  to corresponded chars using control codes map. If map is not assigned (nil)
  control codes will be replace using Hexadecimal and ASCII control codes
  maps. }

function UnMapControlCodes(const S, F: string; Map: PControlCodeMap): string;

const

{ Debug message codes }

  L_MESG  = 1;
  L_NOISE = 2;
  L_WARN  = 3;
  L_ERROR = 4;
  L_JUNK  = 5;

{ Buffer length for commands, voice modem answers and so on }

  VOICE_BUF_LEN = 1024;

const

{ Header Magic }

  RMD_HEADER_MAGIC = 'RMD1';

type

{ Defines the header for raw modem data (size = 32 byte) }

  TRMDHeader = packed record          
    Magic: array[0..3] of Char;       { header magic }
    ModemType: array[0..15] of Char;  { modem identifier }
    Compression: SmallInt;            { record compression mode }
    Speed: SmallInt;                  { record speed }
    Bits: Byte;                       { bits per second }
    Reserved: array[0..6] of Char;    { reserved }
  end;

implementation

uses
  VML_Utils;

{ Error messages routines }

function GetVMLErrorStr(Error: Integer): string;
begin
  if not IntToIdent(Error, Result, VMLErrorMap) then
    Result := Format(SUnknownError, [Error]);
end;

{ Replace all ASCII control chars to corresponded name }

function MapControlCodes(const S, F: string; Map: PControlCodeMap): string;
var
  I, L: Integer;
  MS: string;
begin
  { Map control codes }
  Result := S;
  L := Length(F);
  I := Length(Result);
  while I > 0 do
  begin
    { Check control code }
    if Result[I] in [#0..#31] then
    begin
      { Prepare map string }
      if Map <> nil then MS := Map[Ord(S[I])]
      else if Result[I] = #13 then MS := 'cr'
      else if Result[I] = #10 then MS := 'lf'
      else MS := CODEMAP_HEX[Ord(S[I])];
      if L > 0 then MS := F[1] + MS;
      if L > 1 then MS := MS + F[2];
      { Replace control code to map string }
      Delete(Result, I, 1);
      Insert(MS, Result, I);
    end;
    { Next char }
    Dec(I);
  end;
end;

{ Replace all ASCII control char names to corresponded chars }

function UnMapControlCodes(const S, F: string; Map: PControlCodeMap): string;

  { Replace all occurrence of S1 in Result to C1 }

  procedure Replace(const S1: string; C2: Char);
  var
    L1, RP, P: Integer;
    SP: PChar;
  begin
    L1 := Length(S1);
    RP := 0;
    { Find first occurrence of S1 }
    SP := PChar(Result);
    P := Pos(S1, SP);
    { Replace loop }
    while P > 0 do
    begin
      { Replace occurrence }
      Delete(Result, RP + P, L1);
      Insert(C2, Result, RP + P);
      RP := RP + P;
      { Find next occurrence of S1 }
      SP := PChar(Result) + RP;
      P := Pos(S1, SP);
    end;
  end;

  procedure UnMap(const AB: string; AMap: PControlCodeMap);
  var
    I, L: Integer;
    MS: string;
  begin
    L := Length(AB);
    for I := 0 to 31 do
    begin
      { Prepare map string }
      MS := AMap^[I];
      if L > 0 then MS := AB[1] + MS;
      if L > 1 then MS := MS + AB[2];
      { Replace map string to map char }
      Replace(MS, Char(I));
    end;
  end;

begin
  Result := S;
  { Unmap codes using given map }
  if Map <> nil then
  begin
    UnMap(F, Map);
    Exit;
  end;
  { Replace <cr><lf> codes }
  Replace('cr', #13);
  Replace('lf', #10);
  { Unmap codes using all known chars }
  UnMap(F, @CODEMAP_HEX);
  UnMap(F, @CODEMAP_NAME);
end;

{ Answer idents routines }

function StrToModemAnswer(const S: string): Longint;
begin
  Result := StrToModemAnswerDef(S, VMA_FAIL);
end;

function StrToModemAnswerDef(const S: string; Default: Longint): Longint;
begin
  { Check form empty answer }
  if Length(S) = 0 then
  begin
    Result := VMA_EMPTY;
    Exit;
  end;
  { Check form modem answers }
  if IdentToInt(S, Result, ModemAnswers) then Exit;
  { Check form user answers }
  if IdentToInt(S, Result, UserAnswers) then Exit;
  { Unknown modem answer }
  Result := Default;
end;

function ModemAnswerToStr(Answer: Longint): string;
begin
  Result := ModemAnswerToStrDef(Answer, SUnknownModemAnswer);
end;

function ModemAnswerToStrDef(Answer: Longint; const Default: string): string;
begin
  { Check form modem answers }
  if IntToIdent(Answer, Result, ModemAnswers) then Exit;
  { Check form user answers }
  if IntToIdent(Answer, Result, UserAnswers) then Exit;
  { Check for VMA_FAIL value }
  if Answer = VMA_FAIL then
  begin
    Result := 'VMA_FAIL';
    Exit;
  end;
  { Unknown answer value }
  Result := Default;
end;

{ Answer analyze routines }

function AnalyzeModemAnswer(const S: string): Longint;
begin
  Result := AnalyzeModemAnswerDef(S, VMA_FAIL);
end;

function AnalyzeModemAnswerDef(const S: string; const Default: Longint): Longint;
var
  I: Integer;
begin
  { Check empty answer }
  if Length(S) = 0 then
  begin
    Result := VMA_EMPTY;
    Exit;
  end;
  { Check possible answers }
  for I := Low(ModemAnswers) to High(ModemAnswers) do
    if Pos(ModemAnswers[I].Name, S) = 1 then
    begin
      Result := ModemAnswers[I].Value;
      Exit;
    end;
  { Unknown modem answer }
  Result := Default;
end;

end.
