{
  Voice Modem Library v1.0

  This file contains the IDC-2814 Voice (with IS-101 AT+ commands set)
  specific hardware stuff.
                                       
  Copyright (C) 1998-2001 Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_IDC2814;

{$I VML.INC}

interface

uses
  Windows, SysUtils, Classes, VML_Modem, VML_Consts, VML_IS101;

const

{ IDC-2814 Voice available compression modes }

  IDC2814_PCM_8    = 128;
  IDC2814_PCM_16   = 129;
  IDC2814_ALAW     = 130;
  IDC2814_MULAW    = 131;
  IDC2814_IMAADPCM = 132;

{ IDC-2814 Voice modem strings }

  SIDC2814Name    = 'IDC-2814 Voice';
  SIDC2814RMDName = 'IDC-2814';

{$I *.INC}

type

{ TIDC2814 }

  TIDC2814 = class(TIS101)
  public
    constructor Create(AOwner: TComponent); override;
  end;

implementation

uses
  MMSys, MMReg;

{ TIDC2814 }

constructor TIDC2814.Create(AOwner: TComponent);
const
  CF_IDC2814 = CF_7200 or CF_8000 or CF_11025;
begin
  inherited Create(AOwner);
  { Modem name }
  ModemName := SIDC2814Name;
  ModemRMDName := SIDC2814RMDName;
  { &K3 - Hardware CTS/RTS flow control.
    &K4 - Software XON/XOFF flow control. }
  HardFlowCmd := 'AT&K3';
  SoftFlowCmd := 'AT&K4';
  { A - Answer. }
  PickPhoneCmd := 'ATA';
  { Interrupt playing and stop recording commands. }
  IntrPlayCmd := '<DLE><EOT>';
  StopRecCmd := 'AT';
  { +VIT=n 0 DTE/DCE (computer/modem) Inactivity Timer. }
  InactivityTimerCmd := 'AT+VIT=%d';
  InactivityTimerAnsw := 'OK';
  { +VSD=<sds>,<sdi> - Sets both the silence detection sensitivity <sds>
                       and the silence detection interval <sdi>. }
  SilenceDetectCmd := 'AT+VSD=%d,%d';
  SilenceDetectAnsw := 'OK';
  SilenceSensMin := 127;
  SilenceSensMax := 129;
  { +VGT=n - Speaker Volume Control (0-255). }
  TransmitGainCmd := 'AT+VGT=%d';
  TransmitGainAnsw := 'OK';
  TransmitGainMin := 0;
  TransmitGainMax := 255;
  { +VGR=n - Receive Gain Selection (0-255). }
  ReceiveGainCmd := 'AT+VGR=%d';
  ReceiveGainAnsw := 'OK';
  ReceiveGainMin := 0;
  ReceiveGainMax := 255;
  { +VRA - Ringback Goes Away Timer.
    +VRN - Ringback Never Appeared Timer. }
  RingbackTimersCmd := 'AT+VRA=%d+VRN=%d';
  RingbackTimersAnsw := 'OK';
  { +VCID=0 - Disable Caller ID
    +VCID=1 - Enable Caller ID, formatted caller report. }
  CallerIDOnCmd := 'AT+VCID=1';
  CallerIDOnAnsw := 'OK';
  CallerIDOffCmd := 'AT+VCID=0';
  CallerIDOffAnsw := 'OK';
  CallerIDFormat := 'NMBR=';
  { +VDR=<enable>,<report> - Distinctive Ringing and Cadence Report. }
  DRingOnCmd := 'AT+VDR=1,5';
  DRingOnAnsw := 'OK';
  DRingOffCmd := 'AT+VDR=0,0';
  DRingOffAnsw := 'OK';
  { +VNH=1 - Disable automatic hang-ups in the data and facsimile modes. }
  DevInitVoiceCmd := 'AT+VNH=1';
  DevInitVoiceAnsw := 'OK';
  { +VLS   - Analog Source/Destination Selection.
    +VLS=0 - Modem on-hook, local phone connected to Telco.
    +VLS=1 - Modem off-hook, modem connected to Telco.
    +VLS=4 - Speaker connected to modem, modem on-hook.
    +VLS=6 - Microphone connected to modem, modem on-hook (record greeting).
    +VLS=7 - Microphone and speaker connected, modem off-hook (speakerphone). }
  Devices.Clear;
  Devices.Add(NO_DEVICE, SNoDevice, 'AT+VLS=0', 'OK', DF_HIDDEN);
  Devices.Add(DIALUP_LINE, SDialupLine, 'AT+VLS=1', 'OK|VCON', DF_TRANSMIT_RECEIVE);
  Devices.Add(INTERNAL_MICROPHONE, SInternalMicrophone, 'AT+VLS=6', 'OK|VCON', DF_RECEIVE);
  Devices.Add(INTERNAL_SPEAKER, SInternalSpeaker, 'AT+VLS=4', 'OK|VCON', DF_TRANSMIT);
  Devices.Add(SPEAKER_PHONE, SSpeakerPhone, 'AT+VLS=7', 'OK|VCON', DF_TRANSMIT_RECEIVE);
  { +VSM=cml,vsr - Compression Method and Sampling Specifications.
    +VSM=128,n   - PCM 8-bit 7200, 8000, 11025 Hz.
    +VSM=129,n   - PCM 16-bit 7200, 8000, 11025 Hz (default).
    +VSM=130,n   - A-law 8-bit 8000 Hz.
    +VSM=131,n   - Mu-law 8-bit 8000 Hz.
    +VSM=132,n   - IMA ADPCM 8000 Hz. }
  Compressions.Clear;
  Compressions.Add(IDC2814_PCM_8, SIDC2814PCM8, 'AT+VSM=128,%d', 'OK', 8000, 8, CF_IDC2814, WAVE_FORMAT_PCM, 8000, CF_IDC2814);
  Compressions.Add(IDC2814_PCM_16, SIDC2814PCM16, 'AT+VSM=129,%d', 'OK', 8000, 16, CF_IDC2814 or CF_DEFAULT, WAVE_FORMAT_PCM, 8000, CF_IDC2814 or CF_DEFAULT);
  Compressions.Add(IDC2814_ALAW, SIDC2814ALAW, 'AT+VSM=130,%d', 'OK', 8000, 8, CF_8000, WAVE_FORMAT_ALAW, 8000, CF_8000);
  Compressions.Add(IDC2814_MULAW, SIDC2814MULAW, 'AT+VSM=131,%d', 'OK', 8000, 8, CF_8000, WAVE_FORMAT_MULAW, 8000, CF_8000);
  Compressions.Add(IDC2814_IMAADPCM, SIDC2814IMAADPCM, 'AT+VSM=132,%d', 'OK', 8000, 4, CF_8000, WAVE_FORMAT_IMA_ADPCM, 8000, CF_8000);
end;

initialization
  RegisterModemDriver(SIDC2814Name, TIDC2814);

end.
