{
  Voice Modem Library v1.0

  This file contains the Motorola SM56 specific hardware stuff.

  Copyright (C) 1998-2001 Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_MotorolaSM56;

{$I VML.INC}

interface

uses
  Windows, SysUtils, Classes, VML_Modem, VML_Consts, VML_IS101;

const

{ Motorola SM56 available compression modes }

  MOTOROLASM56_PCM    = 1;
  MOTOROLASM56_ADPCM  = 2;

{ Motorola SM56 modem strings }

  SMotorolaSM56Name    = 'Motorola SM56';
  SMotorolaSM56RMDName = 'Motorola SM56';

{$I *.INC}

type

{ TMotorolaSM56 }

  TMotorolaSM56 = class(TIS101)
  public
    constructor Create(AOwner: TComponent); override;
  end;

implementation

uses
  MMSys;

{ TMotorolaSM56 }

constructor TMotorolaSM56.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  { Modem name }
  ModemName := SMotorolaSM56Name;
  ModemRMDName := SMotorolaSM56RMDName;
  { &R1 -
    \Q3 - Hardware flow control }
  HardFlowCmd := 'AT&R1\Q3';
  { A - Answer }
  PickPhoneCmd := 'ATA';
  { +VSD=m,n - Specifies the silence-detection level (127-129) and
               the silence-detection duration (0-255 in 0.1 ms). }
  SilenceDetectCmd := 'AT+VSD=%d,%d';
  SilenceDetectAnsw := 'OK';
  SilenceSensMin := 127;
  SilenceSensMax := 129;
  { +VGT - Transmit volume (0-255). }
  TransmitGainCmd := 'AT+VGT=%d';
  TransmitGainAnsw := 'OK';
  TransmitGainMin := 0;
  TransmitGainMax := 255;
  { +VGR - Receive gain (0-255). }
  ReceiveGainCmd := 'AT+VGR=%d';
  ReceiveGainAnsw := 'OK';
  ReceiveGainMin := 1;
  ReceiveGainMax := 255;
  { +VRA - Ringback Goes Away Timer.
    +VRN - Ringback Never Came Timer. }
  RingbackTimersCmd := 'AT+VRA=%d+VRN=%d';
  RingbackTimersAnsw := 'OK';
  { +VCID - Caller ID Control }
  CallerIDOnCmd := 'AT+VCID=1';
  CallerIDOnAnsw := 'OK';
  CallerIDOffCmd := 'AT+VCID=0';
  CallerIDOffAnsw := 'OK';
  CallerIDFormat := 'NMBR=';
  { +VNH=1 - Disable DCE-initiated automatic hang-ups. }
  DevInitVoiceCmd := 'AT+VNH=1';
  DevInitVoiceAnsw := 'OK';
  { "TIME = " - Caller ID response.
    "NAME = " - Caller ID response. }
  Responses.Add('TIME =', VMA_IGNORE);
  Responses.Add('NAME =', VMA_IGNORE);
  { +VLS    - Select analog source and destination.
    +VLS=0  - DCE (modem) on-hook.
    +VLS=1  - DCE off-hook, DCE connected to telco.
    +VLS=8  - DCE on-hook, DCE connected to speaker.
    +VLS=9  - Speakerphone with mute enabled.
    +VLS=11 - DCE on-hook, DCE connected to microphone.
    +VLS=13 - DCE off-hook, DCE connected to telco, speaker, and
              microphone (speakerphone). }
  Devices.Clear;
  Devices.Add(NO_DEVICE, SNoDevice, 'AT+VLS=0', 'OK', DF_HIDDEN);
  Devices.Add(DIALUP_LINE, SDialupLine, 'AT+VLS=1', 'OK|VCON', DF_TRANSMIT_RECEIVE);
  Devices.Add(EXTERNAL_MICROPHONE, SExternalMicrophone, 'AT+VLS=11','OK|VCON', DF_RECEIVE);
  Devices.Add(EXTERNAL_SPEAKER, SExternalSpeaker, 'AT+VLS=8','OK|VCON', DF_TRANSMIT);
  Devices.Add(INTERNAL_SPEAKER, SInternalSpeaker, 'AT+VLS=9','OK|VCON', DF_TRANSMIT);
  Devices.Add(SPEAKER_PHONE, SSPeakerPhone, 'AT+VLS=13','OK|VCON', DF_TRANSMIT_RECEIVE);
  { +VSM              - Speech Compression Properties.
    +VSM=128,8000,0,0 - PCM 8000 Hz.
    +VSM=129,8000,0,0 - ADPCM 8000 Hz (default). }
  Compressions.Clear;
  Compressions.Add(MOTOROLASM56_PCM, SMotorolaSM56PCM, 'AT+VSM=128,8000,0,0', 'OK', 8000, 8, CF_8000, WAVE_FORMAT_PCM, 8000, CF_8000 or CF_DEFAULT);
  Compressions.Add(MOTOROLASM56_ADPCM, SMotorolaSM56ADPCM, 'AT+VSM=129,8000,0,0', 'OK', 8000, 8, CF_8000 or CF_DEFAULT, 0, 0, 0);
end;

initialization
  RegisterModemDriver(SMotorolaSM56Name, TMotorolaSM56);

end.
