{
  Voice Modem Library v1.0

  This file contains a dummy hardware driver.

  Copyright (C) 1998-2001 Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_NoModem;

{$I VML.INC}

interface

uses
  Windows, Classes, VML_Modem, VML_Consts;

{$I *.INC}

type

{ TNoModem }

  TNoModem = class(TModemDriver)
  public
    constructor Create(AOwner: TComponent); override;
    function AnswerPhone: Integer; override;
    function Beep(Frequency, Length: Integer): Integer; override;
    function Dial(const Number: string): Integer; override;
    function EnableCallerID(Enabled: Boolean): Integer; override;
    function EnableDistinctiveRing(Enabled: Boolean): Integer; override;
    function HandleDLE(Code: Char): Integer; override;
    function HandleEvent(Event: Integer; Data: TEventData): Integer; override;
    function HangUp: Integer; override;
    function Init: Integer; override;
    function InitVoice: Integer; override;
    function PlayDTMF(const Number: string): Integer; override;
    function PlayStreamEx(Stream: TStream; BPS: Integer; DataLength: Longint): Integer; override;
    function RecordStreamEx(Stream: TStream; BPS: Integer; var DataLength: Longint): Integer; override;
    function RestoreVoiceMode: Integer; override;
    function SetCompression(var Compression, Speed, Bits: Integer): Integer; override;
    function SetDevice(Device: Integer): Integer; override;
    function SetReceiveGain(Gain: Integer): Integer; override;
    function SetTransmitGain(Gain: Integer): Integer; override;
    function SetWaveFormat(var WaveFormat, Speed, Bits: Integer): Integer; override;
    function StartPlayStream: Integer; override;
    function StartRecordStream: Integer; override;
    function StopDialing: Integer; override;
    function StopPlayStream: Integer; override;
    function StopPlaying: Integer; override;
    function StopRecordStream: Integer; override;
    function StopRecording: Integer; override;
    function StopWaiting: Integer; override;
    function SwitchToDataFax(const Mode: string): Integer; override;
    function VoiceModeOff: Integer; override;
    function VoiceModeOn: Integer; override;
    function Wait(Timeout: Longint): Integer; override;
    function WaitAnswer(const ExpectedAnswers: string; Timeout: Longint): Integer; override;
  end;

implementation

{ TNoModem }

constructor TNoModem.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ModemName := 'Serial Port';
  ModemRMDName := 'Unknown';
end;

function TNoModem.AnswerPhone: Integer;
begin
  DebugMsg(L_ERROR, SAnswerPhoneCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.Beep(Frequency, Length: Integer): Integer;
begin
  DebugMsg(L_ERROR, SBeepCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.Dial(const Number: string): Integer;
begin
  DebugMsg(L_ERROR, SDialCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.EnableCallerID(Enabled: Boolean): Integer;
begin
  DebugMsg(L_ERROR, SEnableCallerIDCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.EnableDistinctiveRing(Enabled: Boolean): Integer;
begin
  DebugMsg(L_ERROR, SEnableDRingCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.HandleDLE(Code: Char): Integer;
begin
  DebugMsg(L_ERROR, SHandleDLECall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.HandleEvent(Event: Integer; Data: TEventData): Integer;
begin
  DebugMsg(L_ERROR, SHandleEventCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.HangUp: Integer;
begin
  DebugMsg(L_ERROR, SHangUpCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.Init: Integer;
begin
  DebugMsg(L_ERROR, SInitCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.InitVoice: Integer;
begin
  DebugMsg(L_ERROR, SInitVoiceCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.PlayDTMF(const Number: string): Integer;
begin
  DebugMsg(L_ERROR, SPlayDTMFCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.PlayStreamEx(Stream: TStream; BPS: Integer; DataLength: Longint): Integer;
begin
  DebugMsg(L_ERROR, SPlayStreamCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.RecordStreamEx(Stream: TStream; BPS: Integer; var DataLength: Longint): Integer;
begin
  DebugMsg(L_ERROR, SRecordStreamCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.RestoreVoiceMode: Integer;
begin
  DebugMsg(L_ERROR, SRestoreVoiceModeCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.SetCompression(var Compression, Speed, Bits: Integer): Integer;
begin
  DebugMsg(L_ERROR, SSetCompressionCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.SetDevice(Device: Integer): Integer;
begin
  DebugMsg(L_ERROR, SSetDeviceCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.SetReceiveGain(Gain: Integer): Integer;
begin
  DebugMsg(L_ERROR, SetReceiveGainCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.SetTransmitGain(Gain: Integer): Integer;
begin
  DebugMsg(L_ERROR, SetTransmitGainCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;


function TNoModem.SetWaveFormat(var WaveFormat, Speed, Bits: Integer): Integer;
begin
  DebugMsg(L_ERROR, SSetWaveFormatCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.StartPlayStream: Integer;
begin
  DebugMsg(L_ERROR, SStartPlayStreamCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.StartRecordStream: Integer;
begin
  DebugMsg(L_ERROR, SStartRecordStreamCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.StopDialing: Integer;
begin
  DebugMsg(L_ERROR, SStopDialingCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.StopPlayStream: Integer;
begin
  DebugMsg(L_ERROR, SStopPlayStreamCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.StopPlaying: Integer;
begin
  DebugMsg(L_ERROR, SStopPlayingCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.StopRecordStream: Integer;
begin
  DebugMsg(L_ERROR, SStopRecordStreamCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.StopRecording: Integer;
begin
  DebugMsg(L_ERROR, SStopRecordingCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.StopWaiting: Integer;
begin
  DebugMsg(L_ERROR, SStopWaitingCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.SwitchToDataFax(const Mode: string): Integer;
begin
  DebugMsg(L_ERROR, SSwitchToDataFaxCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.VoiceModeOff: Integer;
begin
  DebugMsg(L_ERROR, SVoiceModeOffCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.VoiceModeOn: Integer;
begin
  DebugMsg(L_ERROR, SVoiceModeOnCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.Wait(Timeout: Longint): Integer;
begin
  DebugMsg(L_ERROR, SWaitCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TNoModem.WaitAnswer(const ExpectedAnswers: string; Timeout: Longint): Integer;
begin
  DebugMsg(L_ERROR, SWaitAnswerCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

end.
