{
  Voice Modem Library v1.0
  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru

  This file contains the Omni 56k specific hardware stuff.
  Pavel Plesov <mclap@simtts.mv.ru>
}


unit VML_Omni56k;

{$I VML.INC}

interface

uses
  Windows, SysUtils, Classes, VML_Modem, VML_Consts, VML_ZyXEL1496;

const

{ Omni 56k available compression modes }

  Omni56k_ADPCM_2    = 2;
  Omni56k_ADPCM_3    = 3;
  Omni56k_ADPCM_30   = 30;
  Omni56k_ADPCM_4    = 4;
  Omni56k_DVIADPCM_4 = 40;
  Omni56k_ALAW_PCM   = 80;
  Omni56k_MULAW_PCM  = 81;

  SOmni56kName    = 'Omni56k';
  SOmni56kRMDName = 'Omni56k';


{$I *.INC}

type

{ TOmni56k }

  TOmni56k = class(TZyXEL1496)
  private
    FROMRelease: Integer;
  public
    constructor Create(AOwner: TComponent); override;
    function Init: Integer; override;
    function AnswerPhone: Integer; override;
    property ROMRelease: Integer read FROMRelease;
  end;

implementation

uses
  MMReg;

{ TOmni56k }

constructor TOmni56k.Create(AOwner: TComponent);
const
  CF_Omni56k = CF_7200 or CF_8000 or CF_9600 or CF_11025;
begin
  inherited Create(AOwner);
  { Modem name }
  ModemName := SOmni56kName;
  ModemRMDName := SOmni56kRMDName;

  ModemName := SOmni56kName;
  ModemRMDName := SOmni56kRMDName;
  { +VGT=<value> - Set the transmit gain for voice samples (56-200). }
  TransmitGainMin := 56;
  TransmitGainMax := 200;
  { +VGR=<value> - Set receive gain for voice samples (56-200). }
  ReceiveGainMin := 56;
  ReceiveGainMax := 200;
  DTMFDetectCmd := 'ATS39.6=1';
  DTMFDetectAnsw := 'OK';
  DTMFSensMin := 1;
  DTMFSensMax := 2;
  DTMFLenUnit := 1;
  { S48.5=1 - Switch to IS-101 mode. }
  DevInitCmd := 'ATS48.5=1';
  DevInitAnsw := 'OK';

  DevInitVoiceCmd := 'ATS39.7=0';
  DevInitVoiceAnsw := 'OK';

  StartRecCmd := 'AT+FLO=2||AT+VRX';
  StartRecAnsw := 'OK||CONNECT';

  { +VLS    - Select a voice I/O device.
    +VLS=0  - Disconnect all I/O devices.
    +VLS=1  - Connects to local phone.
    +VLS=2  - Modem connects to telco line (Dial-up line jack).
    +VLS=8  - External Mic. (Connect a Mic to the dial-up line RJ11 jack).
    +VLS=16 - Internal speaker. }
  Devices.Clear;
  Devices.Add(NO_DEVICE, SNoDevice, 'AT+VLS=0', 'OK', DF_HIDDEN);
  Devices.Add(DIALUP_LINE, SDialupLine, 'AT+VLS=2', 'OK|CONNECT*', DF_TRANSMIT_RECEIVE);
  Devices.Add(EXTERNAL_MICROPHONE, SExternalMicrophone, 'AT+VLS=8', 'OK|VCON', DF_RECEIVE);
  Devices.Add(INTERNAL_SPEAKER, SInternalSpeaker, 'AT+VLS=16', 'OK', DF_TRANSMIT);
  Devices.Add(LOCAL_HANDSET, SLocalHandset, 'AT+VLS=1', 'VCON', DF_TRANSMIT_RECEIVE);
  { +VSM      - Select a compression method.
    +VSM=2,s  - 2 bit ADPCM 7200 8000, 9600 or 11025 Hz (default).
    +VSM=3,s  - 3 bit ADPCM 7200 8000, 9600 or 11025 Hz.
    +VSM=30,s - New 3 bit ADPCM with resync capability 7200, 8000, 9600, 11025 Hz.
    +VSM=4,s  - 4 bit ADPCM 7200 8000, 9600 or 11025 Hz.
    +VSM=40,s - 4 bit DVI ADPCM 7200, 8000, 9600 or 11025 Hz.
    +VSM=80,s - 8 bit A-law PCM 7200, 8000, 9600 or 11025 Hz.
    +VSM=81,s - 8 bit Mu-law PCM 7200, 8000, 9600 or 11025 Hz. }
  Compressions.Clear;
  {Compressions.Add(Omni56k_ADPCM_2, SOmni56kADPCM2, 'AT+VSM=2,%d', 'OK', 9600, 2, CF_Omni56k or CF_DEFAULT, 0, 0, 0);
  Compressions.Add(Omni56k_ADPCM_3, SOmni56kADPCM3, 'AT+VSM=3,%d', 'OK', 9600, 3, CF_Omni56k, 0, 0, 0);
  Compressions.Add(Omni56k_ADPCM_30, SOmni56kADPCM30, 'AT+VSM=30,%d', 'OK', 9600, 3, CF_Omni56k, 0, 0, 0);}
  Compressions.Add(Omni56k_ADPCM_4, SOmni56kADPCM4, 'AT+VSM=4,%d', 'OK', 9600, 4, CF_Omni56k, 0, 0, 0);
  Compressions.Add(Omni56k_DVIADPCM_4, SOmni56kADPCM4, 'AT+VSM=4,%d', 'OK', 9600, 4, CF_Omni56k, WAVE_FORMAT_DVI_ADPCM, 9600, CF_Omni56k or CF_DEFAULT)
  {Compressions.Add(Omni56k_ALAW_PCM, SOmni56kALAWPCM, 'AT+VSM=80,%d', 'OK', 9600, 8, CF_Omni56k, WAVE_FORMAT_ALAW, 11025, CF_Omni56k);
  Compressions.Add(Omni56k_MULAW_PCM, SOmni56kMULAWPCM, 'AT+VSM=81,%d', 'OK', 9600, 8, CF_Omni56k, WAVE_FORMAT_MULAW, 11025, CF_Omni56k);}
end;

function TOmni56k.AnswerPhone: Integer;
begin
  { Check modem specified }
  if not CheckModem('AnswerPhone', True) then
  begin
    Result := VMA_ERROR;
    Exit;
  end;
  { Answer Phone }
  if Modem.Script('AT+VLS=2A', 'VCON', False) <> S_OK then
  begin
    Result := VMA_ERROR;
    Exit;
  end;
  Result := VMA_OK;
end;

function TOmni56k.Init: Integer;
var
  Buffer: string;
begin
  { Check modem specified }
  if not CheckModem('Init', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Detect ROM release }
  if Modem.Command('ATI1', '') <> S_OK then
  begin
    Result := S_FAIL;
    Exit;
  end;
  if Modem.ReadStr(Buffer) = S_FAIL then
  begin
    Result := S_FAIL;
    Exit;
  end;
  if Modem.ReadStr(Buffer) = S_FAIL then
  begin
    Result := S_FAIL;
    Exit;
  end;
  if Length(Buffer) >= 13 then
  begin
    FROMRelease := 0;
    FROMRelease := FROMRelease + (Ord(Buffer[10]) - Ord('0')) * 100;
    FROMRelease := FROMRelease + (Ord(Buffer[12]) - Ord('0')) * 10;
    FROMRelease := FROMRelease + (Ord(Buffer[13]) - Ord('0'));
  end
  else
    FROMRelease := 109;
  if Modem.ReadStr(Buffer) = S_FAIL then
  begin
    Result := S_FAIL;
    Exit;
  end;
  if Modem.Command('', 'OK') <> VMA_USER_1 then
  begin
    Result := S_FAIL;
    Exit;
  end;
  DebugMsgFmt(L_MESG, SROMRelease, [ModemRMDName, FROMRelease / 100]);
  { Init complete }
  Result := S_OK;
end;

initialization
  RegisterModemDriver(SOmni56kName, TOmni56k);

end.
