{
  Voice Modem Library v1.0

  This file contain Delphi VCL registration routines.

  Note: Some modem commands contain special control chars that incorrectly
  displayed by Object Inspector. These chars must be entered in one of
  following formats:
    a) using control code name framed by '<>' (for example <DLE>, <CR>,
       <LF> and etc.);
    b) using hexadecimal codes (for example <h10>, <h0D>, <h0A>,  and etc.).
    c) using control char mnemonic (for example ^P, ^C, ^J and etc.);
    d) using decimal codes (for example <#16>, <#13>, <#10> and etc.).

  Copyright (C) 1998-2001 Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_Reg;

{$I VML.INC}

interface

uses
  Windows, SysUtils, Classes, Dialogs, VML_Idents,
  {$IFDEF VML_D6_UP} DesignIntf, DesignEditors {$ELSE} DsgnIntf {$ENDIF};

type

{ TIntListProperty }

  TIntListProperty = class(TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    function GetValueFmt: string; virtual;
    procedure GetValues(Proc: TGetStrProc); override;
    function GetValuesMap(var Map: PIdentMap): Integer; virtual;
    procedure SetValue(const Value: string); override;
  end;

{ Port property }

  TPortProperty = class(TIntListProperty)
  public
    function GetValuesMap(var Map: PIdentMap): Integer; override;
  end;

{ TBaudRateProperty }

  TBaudRateProperty = class(TIntListProperty)
  public
    function GetValuesMap(var Map: PIdentMap): Integer; override;
  end;

{ TParityProperty }

  TParityProperty = class(TIntListProperty)
  public
    function GetValuesMap(var Map: PIdentMap): Integer; override;
  end;

{ TStopBitsProperty }

  TStopBitsProperty = class(TIntListProperty)
  public
    function GetValuesMap(var Map: PIdentMap): Integer; override;
  end;

{ TDataBitsProperty }

  TDataBitsProperty = class(TIntListProperty)
  public
    function GetValuesMap(var Map: PIdentMap): Integer; override;
  end;

{ TVoiceBaudRateProperty }

  TVoiceBaudRateProperty = class(TIntListProperty)
  public
    function GetValuesMap(var Map: PIdentMap): Integer; override;
  end;

{ TModemNameProperty }

  TModemNameProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

{ TModemCommandProperty }

  TModemCommandProperty = class(TStringProperty)
  public
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
  end;

{ TDataFormatProperty }

  TDataFormatProperty = class(TIntListProperty)
  public
    function GetValuesMap(var Map: PIdentMap): Integer; override;
  end;

{ TModemCompressionProperty }

  TModemCompressionProperty = class(TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

{ TModemDeviceProperty }

  TModemDeviceProperty = class(TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
  end;

{ TModemPlayDeviceProperty }

  TModemPlayDeviceProperty = class(TModemDeviceProperty)
  public
    procedure GetValues(Proc: TGetStrProc); override;
  end;

{ TModemRecDeviceProperty }

  TModemRecDeviceProperty = class(TModemDeviceProperty)
  public
    procedure GetValues(Proc: TGetStrProc); override;
  end;

{ TModemRecSpeedProperty }

  TModemRecSpeedProperty = class(TIntListProperty)
  public
    procedure GetValues(Proc: TGetStrProc); override;
  end;

{ TModemWaveFormatProperty }

  TModemWaveFormatProperty = class(TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

{ TModemWaveSpeedProperty }

  TModemWaveSpeedProperty = class(TIntListProperty)
  public
    procedure GetValues(Proc: TGetStrProc); override;
  end;

{ TFileNameProperty }

  TFileNameProperty = class(TStringProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetDefaultExt: string; virtual; abstract;
    function GetFilter: string; virtual; abstract;
  end;

{ TLogFileNameProperty }

  TLogFileNameProperty = class(TFileNameProperty)
  public
    function GetDefaultExt: string; override;
    function GetFilter: string; override;
  end;

{ TIniFileNameProperty }

  TIniFileNameProperty = class(TFileNameProperty)
  public
    function GetDefaultExt: string; override;
    function GetFilter: string; override;
  end;

{ TAnswerProperty }

  TAnswerProperty = class(TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

{ TEventProperty }

  TEventProperty = class(TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

{ TEventCode }

  TEventCodeProperty = class(TCharProperty)
  public
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
  end;

{ TDeviceProperty }

  TDeviceProperty = class(TIntListProperty)
  public
    function GetValuesMap(var Map: PIdentMap): Integer; override;
  end;

{ THexWordProperty }

  THexWordProperty = class(TIntegerProperty)
  public
    function GetValue: string; override;
  end;

{ TWaveFormatProperty }

  TWaveFormatProperty = class(TIntListProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValueFmt: string; override;
    function GetValuesMap(var Map: PIdentMap): Integer; override;
  end;

procedure Register;

implementation

uses
  VML_Consts, VML_Utils, VML_Modem, VML_NoModem, VML_IS101,
  VML_USRobotics, VML_ZyXEL1496, VML_ZyXEL2864, VML_Omni56k,
  VML_MotorolaSM56, VML_CLMD1414, VML_Rockwell, VML_IDC2814,
  VML_IDC2814VR, VML_AVC33600, MMSys, MMReg;

{ TIntListProperty }

function TIntListProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paRevertable, paReadOnly];
end;

function TIntListProperty.GetValue: string;
var
  Map: PIdentMap;
  Count: Integer;
begin
  Count := GetValuesMap(Map);
  Result := IntToIdentDef(GetOrdValue, Format(GetValueFmt, [GetOrdValue]), Slice(Map^, Count));
end;

function TIntListProperty.GetValueFmt: string;
begin
  Result := '%d';
end;

procedure TIntListProperty.GetValues(Proc: TGetStrProc);
var
  Map: PIdentMap;
  Count: Integer;
begin
  Count := GetValuesMap(Map);
  GetIdentsList(Slice(Map^, Count), Proc);
end;

const
  DummyMap: array[0..0] of TIdentMapEntry = ((Value: 0; Name: ''));

function TIntListProperty.GetValuesMap(var Map: PIdentMap): Integer;
begin
  Map := @DummyMap;
  Result := 0;
end;

procedure TIntListProperty.SetValue(const Value: string);
var
  Map: PIdentMap;
  Count: Integer;
begin
  Count := GetValuesMap(Map);
  SetOrdValue(IdentToIntDef(Value, StrToIntDef(Value, GetOrdValue), Slice(Map^, Count)));
end;

{ TPortProperty }

const
  PortMap: array[0..15] of TIdentMapEntry = (
    (Value: 1; Name: 'COM1'),
    (Value: 2; Name: 'COM2'),
    (Value: 3; Name: 'COM3'),
    (Value: 4; Name: 'COM4'),
    (Value: 5; Name: 'COM5'),
    (Value: 6; Name: 'COM6'),
    (Value: 7; Name: 'COM7'),
    (Value: 8; Name: 'COM8'),
    (Value: 9; Name: 'COM9'),
    (Value: 10; Name: 'COM10'),
    (Value: 11; Name: 'COM11'),
    (Value: 12; Name: 'COM12'),
    (Value: 13; Name: 'COM13'),
    (Value: 14; Name: 'COM14'),
    (Value: 15; Name: 'COM15'),
    (Value: 16; Name: 'COM16'));

function TPortProperty.GetValuesMap(var Map: PIdentMap): Integer;
begin
  Map := @PortMap;
  Result := High(PortMap) - Low(PortMap) + 1;
end;

{ TBaudRateProperty }

const
  BaudRateMap: array[0..14] of TIdentMapEntry = (
    (Value: CBR_110; Name: '110'),
    (Value: CBR_300; Name: '300'),
    (Value: CBR_600; Name: '600'),
    (Value: CBR_1200; Name: '1200'),
    (Value: CBR_2400; Name: '2400'),
    (Value: CBR_4800; Name: '4800'),
    (Value: CBR_9600; Name: '9600'),
    (Value: CBR_14400; Name: '14400'),
    (Value: CBR_19200; Name: '19200'),
    (Value: CBR_38400; Name: '38400'),
    (Value: CBR_56000; Name: '56000'),
    (Value: CBR_57600; Name: '57600'),
    (Value: CBR_115200; Name: '115200'),
    (Value: CBR_128000; Name: '128000'),
    (Value: CBR_256000; Name: '256000'));

function TBaudRateProperty.GetValuesMap(var Map: PIdentMap): Integer;
begin
  Map := @BaudRateMap;
  Result := High(BaudRateMap) - Low(BaudRateMap) + 1;
end;

{ TParityProperty }

const
  ParityMap: array[0..4] of TIdentMapEntry = (
    (Value: NOPARITY; Name: 'None'),
    (Value: ODDPARITY; Name: 'Odd'),
    (Value: EVENPARITY; Name: 'Even'),
    (Value: MARKPARITY; Name: 'Mark'),
    (Value: SPACEPARITY; Name: 'Space'));

function TParityProperty.GetValuesMap(var Map: PIdentMap): Integer;
begin
  Map := @ParityMap;
  Result := High(ParityMap) - Low(ParityMap) + 1;
end;

{ TStopBitsProperty }

const
  StopBitsMap: array[0..2] of TIdentMapEntry = (
    (Value: ONESTOPBIT; Name: '1'),
    (Value: ONE5STOPBITS; Name: '1.5'),
    (Value: TWOSTOPBITS; Name: '2'));

function TStopBitsProperty.GetValuesMap(var Map: PIdentMap): Integer;
begin
  Map := @StopBitsMap;
  Result := High(StopBitsMap) - Low(StopBitsMap) + 1;
end;

{ TDataBitsProperty }

const
  DataBitsMap: array[0..4] of TIdentMapEntry = (
    (Value: 4; Name: '4'),
    (Value: 5; Name: '5'),
    (Value: 6; Name: '6'),
    (Value: 7; Name: '7'),
    (Value: 8; Name: '8'));

function TDataBitsProperty.GetValuesMap(var Map: PIdentMap): Integer;
begin
  Map := @DataBitsMap;
  Result := High(DataBitsMap) - Low(DataBitsMap) + 1;
end;

{ TVoiceBaudRateProperty }

const
  VoiceBaudRateMap: array[0..7] of TIdentMapEntry = (
    (Value: 0; Name: 'Auto'),
    (Value: CBR_2400; Name: '2400'),
    (Value: CBR_4800; Name: '4800'),
    (Value: CBR_9600; Name: '9600'),
    (Value: CBR_19200; Name: '19200'),
    (Value: CBR_38400; Name: '38400'),
    (Value: CBR_57600; Name: '57600'),
    (Value: CBR_115200; Name: '115200'));

function TVoiceBaudRateProperty.GetValuesMap(var Map: PIdentMap): Integer;
begin
  Map := @VoiceBaudRateMap;
  Result := High(VoiceBaudRateMap) - Low(VoiceBaudRateMap) + 1;
end;

{ TModemNameProperty }

function TModemNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paRevertable];
end;

procedure TModemNameProperty.GetValues(Proc: TGetStrProc);
begin
  GetModemDriversList(Proc);
end;

{ TModemCommandProperty }

function TModemCommandProperty.GetValue: string;
begin
  Result := MapControlCodes(inherited GetValue, '<>', @CODEMAP_NAME);
end;

procedure TModemCommandProperty.SetValue(const Value: string);
var
  S: string;
begin
  S := Value;
  S := UnMapControlCodes(S, '<>', @CODEMAP_DEC);
  S := UnMapControlCodes(S, '', @CODEMAP_CTL);
  inherited SetValue(MapControlCodes(S, '<>', nil));
end;

{ TDataFormatProperty }

const
  DataFormatMap: array[0..2] of TIdentMapEntry = (
    (Value: DATA_FORMAT_AUTO; Name: 'Auto'),
    (Value: DATA_FORMAT_RMD; Name: 'Raw Modem Data'),
    (Value: DATA_FORMAT_WAVE; Name: 'WAVE'));

function TDataFormatProperty.GetValuesMap(var Map: PIdentMap): Integer;
begin
  Map := @DataFormatMap;
  Result := High(DataFormatMap) - Low(DataFormatMap) + 1;
end;

{ TModemCompressionProperty }

function TModemCompressionProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paRevertable];
end;

function TModemCompressionProperty.GetValue: string;
begin
  Result := GetDriverCompressionName(TModemProfile(GetComponent(0)).ModemName, GetOrdValue);
end;

procedure TModemCompressionProperty.GetValues(Proc: TGetStrProc);
begin
  GetDriverCompressionsList(TModemProfile(GetComponent(0)).ModemName, Proc);
end;

procedure TModemCompressionProperty.SetValue(const Value: string);
begin
  SetOrdValue(GetDriverCompression(TModemProfile(GetComponent(0)).ModemName, Value));
end;

{ TModemDeviceProperty }

function TModemDeviceProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paRevertable];
end;

function TModemDeviceProperty.GetValue: string;
begin
  Result := GetDriverDeviceName(TModemProfile(GetComponent(0)).ModemName, GetOrdValue);
end;

procedure TModemDeviceProperty.SetValue(const Value: string);
begin
  SetOrdValue(GetDriverDevice(TModemProfile(GetComponent(0)).ModemName, Value));
end;

{ TModemPlayDeviceProperty }

procedure TModemPlayDeviceProperty.GetValues(Proc: TGetStrProc);
begin
  GetDriverPlayDevicesList(TModemProfile(GetComponent(0)).ModemName, Proc);
end;

{ TModemRecDeviceProperty }

procedure TModemRecDeviceProperty.GetValues(Proc: TGetStrProc);
begin
  GetDriverRecDevicesList(TModemProfile(GetComponent(0)).ModemName, Proc);
end;

{ TModemRecSpeedProperty }

procedure TModemRecSpeedProperty.GetValues(Proc: TGetStrProc);
begin
  GetDriverSpeedList(TModemProfile(GetComponent(0)).ModemName,
    TModemProfile(GetComponent(0)).RecCompression, Proc);
end;

{ TModemWaveFormatProperty }

function TModemWaveFormatProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paRevertable];
end;

function TModemWaveFormatProperty.GetValue: string;
begin
  Result := GetDriverWaveFormatName(TModemProfile(GetComponent(0)).ModemName, GetOrdValue);
end;

procedure TModemWaveFormatProperty.GetValues(Proc: TGetStrProc);
begin
  GetDriverWaveFormatsList(TModemProfile(GetComponent(0)).ModemName, Proc);
end;

procedure TModemWaveFormatProperty.SetValue(const Value: string);
begin
  SetOrdValue(GetDriverWaveFormat(TModemProfile(GetComponent(0)).ModemName, Value));
end;

{ TModemWaveSpeedProperty }

procedure TModemWaveSpeedProperty.GetValues(Proc: TGetStrProc);
begin
  GetDriverWaveSpeedList(TModemProfile(GetComponent(0)).ModemName,
    TModemProfile(GetComponent(0)).RecWaveFormat, Proc);
end;

{ TFileNameProperty }

procedure TFileNameProperty.Edit;
begin
  with TOpenDialog.Create(nil) do
  try
    DefaultExt := GetDefaultExt;
    FileName := GetValue;
    Filter := GetFilter;
    Options := [ofHideReadOnly, ofPathMustExist];
    if Execute then SetValue(FileName);
  finally
    Free;
  end;
end;

function TFileNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog];
end;

{ TLogFileNameProperty }

function TLogFileNameProperty.GetDefaultExt: string;
begin
  Result := 'log';
end;

function TLogFileNameProperty.GetFilter: string;
begin
  Result := 'Log Files (*.log)|*.log|All Files (*.*)|*.*';
end;

{ TIniFileNameProperty }

function TIniFileNameProperty.GetDefaultExt: string;
begin
  Result := 'Ini';
end;

function TIniFileNameProperty.GetFilter: string;
begin
  Result := 'Ini Files (*.ini)|*.ini|All Files (*.*)|*.*';
end;

{ TAnswerProperty }

function TAnswerProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paRevertable];
end;

function TAnswerProperty.GetValue: string;
begin
  Result := IntToIdentDef(GetOrdValue, Format('$%0.4x', [GetOrdValue]), AnswerMap);
end;

procedure TAnswerProperty.GetValues(Proc: TGetStrProc);
begin
  GetIdentsList(AnswerMap, Proc);
end;

procedure TAnswerProperty.SetValue(const Value: string);
begin
  if Length(Value) = 0 then
  begin
    SetOrdValue(0);
    Exit;
  end;
  SetOrdValue(IdentToIntDef(Value, StrToIntDef(Value, GetOrdValue), AnswerMap));
end;

{ TEventProperty }

function TEventProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paRevertable];
end;

function TEventProperty.GetValue: string;
begin
  if GetOrdValue = 0 then
  begin
    Result := '';
    Exit;
  end;
  Result := IntToIdentDef(GetOrdValue, Format('$%0.4x', [GetOrdValue]), EventMap);
end;

procedure TEventProperty.GetValues(Proc: TGetStrProc);
begin
  GetIdentsList(EventMap, Proc);
end;

procedure TEventProperty.SetValue(const Value: string);
begin
  if Length(Value) = 0 then
  begin
    SetOrdValue(0);
    Exit;
  end;
  SetOrdValue(IdentToIntDef(Value, StrToIntDef(Value, GetOrdValue), EventMap));
end;

{ TEventCodeProperty }

function TEventCodeProperty.GetValue: string;
begin
  if GetOrdValue = 0 then
  begin
    Result := '';
    Exit;
  end;
  Result := MapControlCodes(Chr(GetOrdValue), '<>', @CODEMAP_NAME); ;
end;

procedure TEventCodeProperty.SetValue(const Value: string);
var
  S: string;
begin
  S := Value;
  S := UnMapControlCodes(S, '<>', @CODEMAP_DEC);
  S := UnMapControlCodes(S, '', @CODEMAP_CTL);
  inherited SetValue(UnMapControlCodes(S, '<>', nil));
end;

{ TDeviceProperty }

function TDeviceProperty.GetValuesMap(var Map: PIdentMap): Integer;
begin
  Map := @DeviceMap;
  Result := High(DeviceMap) - Low(DeviceMap) + 1;
end;

{ THexWordProperty }

function THexWordProperty.GetValue: string;
begin
  Result := Format('$%0.4x', [GetOrdValue]);
end;

{ TWaveFormatProperty }

const
  WaveFormatMap: array[0..3] of TIdentMapEntry = (
    (Value: WAVE_FORMAT_PCM; Name: 'PCM'),
    (Value: WAVE_FORMAT_ALAW; Name: 'A-Law'),
    (Value: WAVE_FORMAT_MULAW; Name: 'u-Law'),
    (Value: WAVE_FORMAT_IMA_ADPCM; Name: 'IMA ADPCM'));

function TWaveFormatProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes - [paReadOnly];
end;

function TWaveFormatProperty.GetValueFmt: string;
begin
  Result := '$%0.4x';
end;

function TWaveFormatProperty.GetValuesMap(var Map: PIdentMap): Integer;
begin
  Map := @WaveFormatMap;
  Result := High(WaveFormatMap) - Low(WaveFormatMap) + 1;
end;

procedure Register;
begin
  RegisterComponents('VML', [TVoiceModem]);
  RegisterComponents('VML', [TModemProfile]);
  RegisterComponents('VML', [TModemLogFile]);
  RegisterComponents('VML', [TModemPort]);

  RegisterComponents('VML', [TNoModem]);
  RegisterComponents('VML', [TIS101]);
  RegisterComponents('VML', [TUSRobotics]);
  RegisterComponents('VML', [TZyXEL1496]);
  RegisterComponents('VML', [TZyXEL2864]);
  RegisterComponents('VML', [TOmni56k]);
  RegisterComponents('VML', [TMotorolaSM56]);
  RegisterComponents('VML', [TCirrusLogicMD1414]);
  RegisterComponents('VML', [TRockwell]);
  RegisterComponents('VML', [TIDC2814]);
  RegisterComponents('VML', [TIDC2814VR]);
  RegisterComponents('VML', [TAVC33600]);

  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'Port', TPortProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'PortBaudRate', TBaudRateProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'PortParity', TParityProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'PortStopBits', TStopBitsProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'PortDataBits', TDataBitsProperty);

  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'VoiceBaudRate', TVoiceBaudRateProperty);

  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'PlayDevice', TModemPlayDeviceProperty);

  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'RecCompression', TModemCompressionProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'RecDevice', TModemRecDeviceProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'RecSpeed', TModemRecSpeedProperty);

  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'RecWaveFormat', TModemWaveFormatProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'RecWaveSpeed', TModemWaveSpeedProperty);

  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'VoiceDataFormat', TDataFormatProperty);

  RegisterPropertyEditor(TypeInfo(string), TModemProfile, 'ModemName', TModemNameProperty);

  RegisterPropertyEditor(TypeInfo(string), TModemProfile, 'ModemIniFile', TIniFileNameProperty);

  RegisterPropertyEditor(TypeInfo(string), TModemProfile, 'ModemInitCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemProfile, 'ModemInitAnsw', TModemCommandProperty);

  RegisterPropertyEditor(TypeInfo(string), TModemLogFile, 'FileName', TLogFileNameProperty);

  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'PickPhoneCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'PickPhoneAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'HardFlowCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'HardFlowAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'SoftFlowCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'SoftFlowAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'ResultCodesCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'ResultCodesAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'DevInitCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'DevInitAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'BeepCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'BeepAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'BeepTimeUnit', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StartPlayCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StartPlayAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'ResetPlayCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'IntrPlayCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'IntrPlayAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StopPlayCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StopPlayAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StartRecCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StartRecAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StopRecCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StopRecAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'SwitchModeCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'SwitchModeAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'VoiceModeOnCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'VoiceModeOnAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'VoiceModeOffCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'VoiceModeOffAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'PlayDTMFCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'PlayDTMFExtra', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'PlayDTMFAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'DialSetupCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'DialSetupAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'InactivityTimerCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'InactivityTimerAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'BaudRateCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'BaudRateAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'SilenceDetectCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'SilenceDetectAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'SilenceDeleteCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'SilenceDeleteAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'TransmitGainCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'TransmitGainAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'ReceiveGainCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'ReceiveGainAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'RingbackTimersCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'RingbackTimersAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'DTMFDetectCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'DTMFDetectAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'CallerIDOnCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'CallerIDOnAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'CallerIDOffCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'CallerIDOffAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'DRingOnCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'DRingOnAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'DRingOffCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'DRingOffAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'DevInitVoiceCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'DevInitVoiceAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'RestoreVoiceCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'RestoreVoiceAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'Responses', TModemCommandProperty);

  { Note: For Delphi 2.0 registering property editors below has no effect. }

  RegisterPropertyEditor(TypeInfo(string), TModemResponse, 'Response', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemResponse, 'AnswerCode', TAnswerProperty);

  RegisterPropertyEditor(TypeInfo(Char), TModemDLECode, 'Code', TEventCodeProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemDLECode, 'Event', TEventProperty);
  RegisterPropertyEditor(TypeInfo(Char), TModemDLECode, 'EventCode', TEventCodeProperty);

  RegisterPropertyEditor(TypeInfo(string), TModemDevice, 'Command', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDevice, 'Answer', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemDevice, 'Device', TDeviceProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemDevice, 'Flags', THexWordProperty);

  RegisterPropertyEditor(TypeInfo(string), TModemCompression, 'Command', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemCompression, 'Answer', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemCompression, 'Flags', THexWordProperty);

  RegisterPropertyEditor(TypeInfo(Integer), TModemCompression, 'WaveFormat', TWaveFormatProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemCompression, 'WaveFlags', THexWordProperty);
end;

end.
