{
  Voice Modem Library v1.0

  This file contains the ROCKWELL's Voice/Fax/Data chipset based hardware
  stuff.

  Copyright (C) 1998-2001 Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_Rockwell;

{$I VML.INC}

interface

uses
  Windows, SysUtils, Classes, VML_Modem, VML_Consts, VML_IS101;

const

{ Rockwell available compression modes }

  ROCKWELL_ADPCM_2 = 2;
  ROCKWELL_ADPCM_3 = 3;
  ROCKWELL_ADPCM_4 = 4;
  ROCKWELL_PCM_8   = 8;

{ Rockwell modem strings }

  SRockwellName    = 'Rockwell';
  SRockwellRMDName = 'Rockwell';

{$I *.INC}

type

{ TRockwell }

  TRockwell = class(TIS101)
  public
    constructor Create(AOwner: TComponent); override;
  end;

implementation

uses
  MMSys;

{ TRockwell }

constructor TRockwell.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  { Modem name }
  ModemName := SRockwellName;
  ModemRMDName := SRockwellRMDName;
  { &K3 - Enable RTS/CTS DTE/DCE flow control.
    &K4 - Enable XON/XOFF DTE/DCE flow control. }
  HardFlowCmd := 'AT&K3';
  SoftFlowCmd := 'AT&K4';
  { A - Answering in Voice. }
  PickPhoneCmd := 'ATA';
  PickPhoneAnsw := 'VCON';
  { #VTS - Generate Tone Signals. Duration units is 100 ms. }
  BeepCmd := 'AT#VTS=[%d,0,%d]';
  BeepTimeUnit := 100;
  { #VTX - Voice transmit mode. }
  StartPlayCmd := 'AT#VTX';
  IntrPlayAnsw := 'OK|VCON';
  StopPlayAnsw := 'OK|VCON';
  { #VRX - Voice Receive Mode. }
  StartRecCmd := 'AT#VRX';
  StopRecCmd := '!';
  StopRecAnsw := 'OK|VCON';
  { #CLS - Select Data, Fax, or Voice mode. }
  SwitchModeCmd := 'AT#CLS=%s';
  VoiceModeOnCmd := 'AT#CLS=8';
  VoiceModeOffCmd := 'AT#CLS=0';
  { #VTS - Generate DTMF tone. }
  PlayDTMFCmd := 'AT#VTS=%s';
  { S30 - Set deadman (inactivity) timer value. }
  InactivityTimerCmd := 'ATS30=%d';
  InactivityTimerAnsw := 'OK';
  { #BDR - Select baud rate (turn off autobaud) in 2400 bps units. }
  BaudRateCmd := 'AT#BDR=%d';
  BaudRateansw := 'OK';
  BaudRateUnit := 2400;
  { #VSS - Silence detection tuner (0-3).
    #VSP - Silence detection period. }
  SilenceDetectCmd := 'AT#VSS=%d||AT#VSP=%d';
  SilenceDetectAnsw := 'OK||OK';
  SilenceSensMin := 0;
  SilenceSensMax := 3;
  { #VSD - Enable silence deletion. }
  SilenceDeleteCmd := 'AT#VSD=%d';
  SilenceDeleteAnsw := 'OK';
  { #VRA - Ringback Goes Away timer.
    #VRN - Ringback Never Came timer. }
  RingbackTimersCmd := 'AT#VRA=%d#VRN=%d';
  RingbackTimersAnsw := 'OK';
  { #CID - Enable Caller ID detection and select reporting format. }
  CallerIDOnCmd := 'AT#CID=1';
  CallerIDOnAnsw := 'OK';
  CallerIDOffCmd := 'AT#CID=0';
  CallerIDOffAnsw := 'OK';
  CallerIDFormat := 'NMBR =';
  { "DATE = " - Caller ID response.
    "TIME = " - Caller ID response.
    "NAME = " - Caller ID response.
    "MESG = " - Caller ID response. }
  Responses.Add('DATE =', VMA_IGNORE);
  Responses.Add('TIME =', VMA_IGNORE);
  Responses.Add('NAME =', VMA_IGNORE);
  Responses.Add('MESG =', VMA_IGNORE);
  { <DLE>t - Local handset goes off hook. }
  DLECodes.Add('t', HANDSET_OFF_HOOK, #0, '');
  { #VLS=0 - The modem ends up on-hook as a result of issuing this command to
             connect to the telephone line
    #VLS=1 - Transmit/Receive Device (other than telephone line). This is a
             handset, headset, or speaker-phone powered directly by the modem.
    #VLS=2 - Transmit Only Device. Normally, this is the onboard speaker.
    #VLS=3 - Receive Only Device. Normally, this is a microphone.
    #VLS=4 - Telephone line with Speaker ON and handset.
    #VLS=6 - Select speakerphone mode. }
  Devices.Clear;
  Devices.Add(NO_DEVICE, SNoDevice, 'AT#VLS=0', 'OK', DF_HIDDEN);
  Devices.Add(DIALUP_LINE, SDialupLine, 'AT#VLS=4', 'OK|VCON', DF_TRANSMIT_RECEIVE);
  Devices.Add(EXTERNAL_MICROPHONE, SExternalMicrophone, 'AT#VLS=3', 'VCON', DF_RECEIVE);
  Devices.Add(INTERNAL_SPEAKER, SInternalSpeaker, 'AT#VLS=2', 'VCON', DF_TRANSMIT);
  Devices.Add(LOCAL_HANDSET, SLocalHandset, 'AT#VLS=1', 'VCON', DF_TRANSMIT_RECEIVE);
  Devices.Add(SPEAKER_PHONE, SSpeakerPhone, 'AT#VLS=6', 'OK|VCON', DF_TRANSMIT_RECEIVE);
  { #VSR   - Sampling rate selection.
    #VBS=2 - 2 bit ADPCM 7200 Hz (default).
    #VBS=3 - 3 bit ADPCM 7200 Hz.
    #VBS=4 - 4 bit ADPCM 7200 Hz.
    #VBS=8 - 8-bit unsigned linear PCM 7200, 11025 Hz. }
  Compressions.Clear;
  Compressions.Add(ROCKWELL_ADPCM_2, SRockwellADPCM2, 'AT#VSR=7200||AT#VBS=2', 'OK||OK', 7200, 2, CF_7200 or CF_DEFAULT, 0, 0, 0);
  Compressions.Add(ROCKWELL_ADPCM_3, SRockwellADPCM3, 'AT#VSR=7200||AT#VBS=3', 'OK||OK', 7200, 3, CF_7200, 0, 0, 0);
  Compressions.Add(ROCKWELL_ADPCM_4, SRockwellADPCM4, 'AT#VSR=7200||AT#VBS=4', 'OK||OK', 7200, 4, CF_7200, 0, 0, 0);
  Compressions.Add(ROCKWELL_PCM_8, SRockwellPCM8, 'AT#VSR=%d||AT#VBS=8', 'OK||OK', 7200, 8, CF_7200 or CF_11025, WAVE_FORMAT_PCM, 11025, CF_11025 or CF_DEFAULT);
end;

initialization
  RegisterModemDriver(SRockwellName, TRockwell);

end.
