unit VML_Tools;
{ Tools for Voice Modem Library                        }
{                                                      }
{ Original parts:                                      }
{ The voice extensions for the mgetty+sendfax package  }
{ Copyright (C) 1995-1998 Marc Eberhard.               }
{                                                      }
{ Delphi port                                          }
{ Copyright (C) 2001 Pavel Plesov <mclap@simtts.mv.ru> }
{ Version 0.2.2 alpha                                  }
{                                                      }
{ Supports:                                            }
{  Zyxel modems ( ZyXEL 1496, Omni56k )                }
{ Formats:                                             }
{   ZyXEL 1496 2, 3 and 4 bit ADPCM                    }
{   IMA ADPCM 4 ( Omni56k, Multitech 2834 )            }
{ Methods:                                             }
{   RmdToWav  - coverts raw modem data to wav          }
{   WavToRmd  - coverts raw modem data to wav          }
{   MergeRMD  - merge list of rmd into one big ;-)     }
{                                                      }
{ Wav produced by RmdToWav is PCM 8/16 bit, mono       }

{.$Define Debug_PVF}

interface

Uses
   Classes, SysUtils, WinSock,
   VML_Utils, VML_Consts, VML_Modem;

const
   ERROR = -1;
   NOERROR = 0;
   OK = 0;
   FAIL = -1;
   UNKNOWN_EVENT = -2;

type
   pint = ^integer;
   pvf_header = record
       ascii: boolean;
       channels: integer;
       speed: integer;
       nbits: integer;
       read_pvf_data: function(bits: integer; var fd_in: TStream): integer;
       write_pvf_data: procedure(bits: integer; var fd_out: TStream; data: integer);
   end;

   state_t = record
      data, nleft: integer;
   end;

   { modified GetRMDFileInfo from VML }
   function GetRMDStreamInfo(Stream: TStream; var Info: TRMDInfo): Integer;

   function read_bits (var fd_in: TStream; var state: state_t; nbits: integer): integer;
   { most ZyXELs }
   function zyxeltopvf (var fd_in, fd_out: TStream; nbits: integer; var header_out: pvf_header): integer;

   procedure RmdToWav(var Stream_In, Stream_Out: TStream);
   // modem depended calls
   procedure WavToRmd(modem: TVoiceModem; var Stream_In, Stream_Out: TStream); overload;
   procedure WavToRmd(modem: TVoiceModem; Compression: integer; var Stream_In, Stream_Out: TStream); overload;

   // modem independed call
   procedure WavToRmd( var Stream_In, Stream_Out: TStream; RMDName: string;
                       Speed, Compression, Bits: integer); overload;
   procedure MergeRMD( Files: TStrings; Output: TStream);

   function feof(f: TStream): boolean;

   //procedure RmdToWav2(var Stream_In, Stream_Out: TStream);


const
  Mx: array[0..2,0..7] of integer = (
     ($3800, $5600, $0000, $0000, $0000, $0000, $0000, $0000),
     ($399a, $3a9f, $4d14, $6607, $0000, $0000, $0000, $0000),
     ($3556, $3556, $399A, $3A9F, $4200, $4D14, $6607, $6607)
     );

  NLOOPS = 8;
  PRIVSIZE: integer = 100;

  function read_pvf_data(bits: integer; var fd_in: TStream): integer;
  procedure write_pvf_data(bits: integer; var fd_out: TStream; data: integer);

type
   TitleWave = record
       id_riff: array [0..3] of char;
       len_riff: longint;

       id_chuck: array [0..3] of char;
       fmt: array [0..3] of char;
       len_chuck: longint;

       wtype: shortint;
       channels: shortint;
       freq: longint;
       bytes: longint;
       align: shortint;
       bits: shortint;

       id_data: array [0..3] of char;
       len_data: longint;
   end;

   IntFunc = function: integer;
   Format_t = record
     names: array of string;  // file type names
     flags: integer;          // details about file type
     startread: IntFunc;
     read: IntFunc;
     stopread: IntFunc;
     startwrite: IntFunc;
     write: IntFunc;
     stopwrite: IntFunc;
   end;

//extern format_t formats[];

   // Signal parameters

   SignalInfo = record
     rate: LongInt;          // sampling rate
     size: integer;          // word length of data
     style: integer;         // format of sample numbers
     channels: integer;      // number of sound channels
   end;

   // Loop parameters

   LoopInfo = record
     start: integer;         // first sample
     length: integer;        // length
     count: integer;         // number of repeats, 0=forever
     ltype: integer;          // 0=no, 1=forward, 2=forward/back
   end;


     // Instrument parameters

     // vague attempt at generic information for sampler-specific info

   InstrInfo = record
      MIDInote: byte;                // for unity pitch playback
      MIDIlow, MIDIhi: byte;         // MIDI pitch-bend range
      loopmode: byte;                // semantics of loop data
      nloops: byte;                  // number of active loops
      smpte: array [0..3] of byte;   // SMPTE offset (hour:min:sec:frame)
                                     // this is a film audio thing
   end;


const ps8 = 12; // PRIVSIZE / 8;

type

   SoundStream = record
      info: SignalInfo;                   // signal specifications
      instr: InstrInfo;                   // instrument specification
      loops: array[0..7] of LoopInfo;     // Looping specificatio
      swap: boolean;                      // do byte- or word-swap
      seekable: boolean;                  // can seek on this file
      filename: string;                   // file name
      filetype: string;                   // type of file
      comment: string;                    // comment string
      fp: TStream;                        // File stream pointer
      h: Format_t;                        // format struct for this file
      priv: array [0..ps8-1] of double;   // format's private data area
   end;

   wav_t = record
     samples: LongInt;
     second_header: integer; // non-zero on second header write
   end;

   wav_p = ^wav_t;

const
   WAVE_FORMAT_UNKNOWN      =$0000;
   WAVE_FORMAT_PCM          =$0001;
   WAVE_FORMAT_ADPCM        =$0002;
   WAVE_FORMAT_ALAW         =$0006;
   WAVE_FORMAT_MULAW        =$0007;
   WAVE_FORMAT_OKI_ADPCM    =$0010;
   WAVE_FORMAT_DIGISTD      =$0015;
   WAVE_FORMAT_DIGIFIX      =$0016;
   IBM_FORMAT_MULAW         =$0101;
   IBM_FORMAT_ALAW          =$0102;
   IBM_FORMAT_ADPCM         =$0103;

   // Size field
   SZ_BYTE   = 1;
   SZ_WORD   = 2;
   SZ_LONG   = 4;
   SZ_FLOAT  = 5;
   SZ_DOUBLE = 6;
   SZ_IEEE   = 7;         // IEEE 80-bit floats.  Is it necessary?

   // Style field
   S_UNSIGNED = 1;    // unsigned linear: Sound Blaster
   S_SIGN2    = 2;    // signed linear 2's comp: Mac
   S_ULAW     = 3;    // U-law signed logs: US telephony, SPARC
   S_ALAW     = 4;    // A-law signed logs: non-US telephony

const bitmask: array [0..16] of integer = (
      $0000, $0001, $0003, $0007, $000f, $001f, $003f,
      $007f, $00ff, $01ff, $03ff, $07ff, $0fff, $1fff,
      $3fff, $7fff, $ffff);


   BLOCK_SIZE = $10000; // Blocksize for reading voice files into memory

   init_state : state_t = (data:$0000; nleft:0);

   BITS_8 = 8;
   BITS_16 = 16;
   BITS_32 = 32;
var
   wav_bits: integer;

const
   init_pvf_header: pvf_header =
      ( ascii:false;
        channels:1;
        speed:8000;
        nbits:32;
        read_pvf_data: read_pvf_data;
        write_pvf_data: write_pvf_data
      );


var
   wFormatTag: word;              // data format
   wChannels: word;               // number of channels
   wSamplesPerSecond: LongWord;   // samples per second per channel
   wAvgBytesPerSec: LongWord;     // estimate of bytes per second needed
   wBlockAlign: word;             // byte alignment of a basic sample block
   wBitsPerSample: word;          // bits per sample
   data_length: LongWord;         // length of sound data in bytes
   bytespersample: LongWord;      // bytes per sample (per channel)

implementation

Uses imaadpcm;

// dirty delphi tricks ;)
procedure putc(c: integer; var fd_out: TStream; cnt: integer);
begin
   fd_out.Write(c,cnt);
end;

function getc(var fd: TStream): integer;
begin
   Result:=0;
   fd.Read(Result,1);
end;

function feof(f: TStream): boolean;
begin
   Result:=f.position = f.size;
end;

function GetRMDStreamInfo(Stream: TStream; var Info: TRMDInfo): Integer;
var
  Header: TRMDHeader;
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
    try
      { Read file header }
      if Stream.Read(Header, SizeOf(TRMDHeader)) <> SizeOf(TRMDHeader) then
      begin
        Result := VME_STREAM_READ_ERR;
        Exit;
      end;
      { Check for header magic }
      if StrLComp(Header.Magic, RMD_HEADER_MAGIC, 4) <> 0 then
      begin
        Result := VME_NOT_RMD_FILE;
        Exit;
      end;
      { Clear result }
      FillChar(Info, SizeOf(Info), 0);
      { Get header information }
      with Info do
      begin
        { Common information }
        ModemType := StrPas(Header.ModemType);
        Compression := Header.Compression;
        Speed := Header.Speed;
        Bits := Header.Bits;
        DataSize := Stream.Size - SizeOf(TRMDHeader);
        { Driver specific information }
        if GetModemDriver(ModemType, DriverClass) then
        begin
          DriverInstance := DriverClass.Create(nil);
          try
            ModemName := DriverInstance.ModemName;
            CompressionName := DriverInstance.GetCompressionName(Compression);
            VoiceLen := DriverInstance.GetVoiceLen(Compression, Speed, Bits, DataSize);
          finally
            DriverInstance.Free;
          end;
        end
        else
        begin
          ModemName := '?';
          CompressionName := '?';
          VoiceLen := 0;
        end;
      end;
    finally
    end;
  { Success }
  Result := S_OK;
end;

//var __rb_data_new: integer;

function read_bits(var fd_in: TStream; var state: state_t; nbits: integer): integer;
var data: array[0..1] of char;
    i: integer;
begin
   while state.nleft < nbits do begin
      i:=fd_in.Read(data,1);
      if i < 1 then begin
         Result:=-1;
         Exit
      end;

      state.data := (state.data shl 8) or byte(data[0]);
      inc(state.nleft,8);
   end;

   dec(state.nleft,nbits);
   Result:= (state.data shr state.nleft) and bitmask[nbits];
end;

procedure write_bits (var fd_out: TStream; var state: state_t; nbits, data: integer);
begin
     state.data := (state.data shl nbits) or (data and bitmask[nbits]);
     inc(state.nleft,nbits);

     while state.nleft >= 8 do begin
        putc((state.data shr (state.nleft - 8)) and $ff, fd_out,1);
        dec(state.nleft,8);
     end
end;

function read_pvf_data_8(var fd_in: TStream): integer;
var data_new: ShortInt;
begin
     if fd_in.Read(data_new,1) < 1 then
        Result:=-1
     else
        Result := data_new shl 16;
end;

function read_pvf_data_16(var fd_in: TStream): integer;
var data_new: SmallInt;
begin
     if fd_in.Read(data_new,2) < 2 then
        Result:=-1
     else
        Result := data_new shl 8;
end;

function read_pvf_data_32(var fd_in: TStream): integer;
var data_new: LongInt;
begin
     if fd_in.Read(data_new,4) < 4 then
        Result:=-1
     else
        Result := ntohl(data_new);
end;

function read_pvf_data(bits: integer; var fd_in: TStream): integer;
begin
   case bits of
      BITS_8:  Result:=read_pvf_data_8(fd_in);
      BITS_16: Result:=read_pvf_data_16(fd_in);
      BITS_32: Result:=read_pvf_data_32(fd_in);
   else
      Result:=-1;
   end;
end;

procedure write_pvf_data(bits: integer; var fd_out: TStream; data: integer);
var dout: LongInt;
//    dp: pointer;
    sz: integer;
begin
//     dp:=@dout;
     case bits of
        BITS_8: begin
                   data := data shr 16;

                   if data > $7f then
                        data := $7f;

                   if data < -$80 then
                        data := -$80;

                   //putc(data, fd_out);
                   dout:=LongInt(data);
                   sz:=1;
                end;
        BITS_16: begin
                   data := data shr 8;

                   if data > $7fff then
                        data := $7fff;

                   if data < -$8000 then
                        data := -$8000;

                   dout := LongInt(htons(smallint(data)));
                   //fwrite(&out, 2, 1, fd_out);
                   sz:=2;
                end;
        BITS_32: begin
                   dout := htonl(data);
                   sz:=4;
                   //fwrite(&out, 4, 1, fd_out);
                end;
        else
            exit;
     end;

     fd_out.Write(dout,sz);
end;



{==============================================================}

function zyxeltopvf (var fd_in, fd_out: TStream; nbits: integer; var header_out: pvf_header): integer;
var
     s: state_t;
     d,sign,e: integer;
     a: LongInt;
     a2: LongWord;
//     i: integer;
begin
     a:=0; d:=5;

     if nbits = 30 then nbits := 3;

     if (nbits <> 2) and (nbits <> 3) and (nbits <> 4) then begin
        Result:=FAIL; exit;
     end;

     s := init_state;

     e := read_bits(fd_in, s, nbits);
     while e <> -1 do begin
          if (nbits = 4) and (e = 0) then
               d := 4;

          if (e shr (nbits - 1)) <> 0 then sign:=-1
          else sign:=1;

          e := e and bitmask[nbits - 1];
          a2 := (a and $80000000);
          a := (a * 4093 + 2048) shr 12;
          if a2 <> 0 then
             a:= ($FFF00000 or a);

          a2 := sign * ((e shl 1) + 1) * d;
          a2 := (a2 and $80000000) or (a2 shr 1);
          a := a + a2;

          if (d and 1) = 1 then
             a:=a+1;

          header_out.write_pvf_data(header_out.nbits, fd_out, a shl 10);

          d := (d * Mx[nbits - 2,e] + $2000) shr 14;

          if d < 5 then d := 5;

          e := read_bits(fd_in, s, nbits);
     end;

     Result:= OK;
end;

// Write long, little-endian: little end first. VAX/386 style.

function wllong(ft: SoundStream; ul: LongWord): integer;
begin
     if ft.fp.Write(ul,4) <> 4 then
        Result:=1
     else
        Result:=0;
end;

// Write short, little-endian: little end first. VAX/386 style.

function wlshort(ft: SoundStream; ul: Word): integer;
var datum: integer;
begin
     datum := (ul) and $ff; ft.fp.Write(datum,1);
     datum := (ul shr 8) and $ff;
     if ft.fp.Write(datum,1) <> 1 then
        Result:=1
     else
        Result:=0;
end;

procedure st_write(var st: TStream; data: string);
var //t: pointer;
    i: integer;
begin
  for i:=1 to Length(data) do
      putc(integer(data[i]),st,1);
end;

function rllong(ft: SoundStream): longword;
begin
   ft.fp.read(Result,4);
end;

function rlshort(ft: SoundStream): word;
begin
   ft.fp.read(Result,2);
end;

function wavstartread(ft: SoundStream): integer;
var wav: wav_t;
    magic: array [0..3] of char;
    len: longword;
    //littlendian: boolean;
    q: wav_p;
begin
     q := @ft.priv;
     wav := q^;
     //littlendian := true;

     ft.info.rate      := 0;
     ft.info.size      := -1;
     ft.info.style     := -1;
     ft.info.channels  := -1;
     ft.comment   :=  '';
     ft.swap      := false;

     ft.swap:=false;

     if (ft.fp.Read(magic,4) <> 4) or (String(magic) <> 'RIFF') then begin
        Result:=ERROR;
        Exit;
     end;

     len := rllong(ft);

     if (ft.fp.Read(magic,4) <> 4) or (String(magic) <> 'WAVE') then begin
        Result:=ERROR;
        Exit;
     end;

     // Now look for the format chunk
     while true do begin
          if ft.fp.Read(magic,4) <> 4 then begin
             //fprintf(stderr, "Sorry, missing fmt spec");
             Result:=ERROR;
             Exit;
          end;

          len := rllong(ft);

          if String(magic) = 'fmt ' then
             break; // Found the format chunk

          //skip to next chunk
          while (len > 0) and (ft.fp.position < ft.fp.size) do begin
               getc(ft.fp);
               dec(len);
          end;
     end;

     {if len < 16 then
          fprintf(stderr, "Sorry, fmt chunk is too short");}

     wFormatTag := rlshort(ft);
     case wFormatTag of
          WAVE_FORMAT_UNKNOWN: begin
               raise Exception.Create('Sorry, this WAV file is in Microsoft Official Unknown format.');
               Result:=ERROR;
          end;
          WAVE_FORMAT_PCM:    // this one, at least, I can handle
               ;
          WAVE_FORMAT_ADPCM: begin
               raise Exception.Create('Sorry, this WAV file is in Microsoft ADPCM format.');
               Result:=ERROR;
          end;
          WAVE_FORMAT_ALAW:   // Think I can handle this
               ft.info.style := S_ALAW;
          WAVE_FORMAT_MULAW:  // Think I can handle this
               ft.info.style := S_ULAW;
          WAVE_FORMAT_OKI_ADPCM: begin
               raise Exception.Create('Sorry, this WAV file is in OKI ADPCM format.');
               Result:=ERROR;
               end;
          WAVE_FORMAT_DIGISTD: begin
               raise Exception.Create('Sorry, this WAV file is in Digistd format.');
               Result:=ERROR;
               end;
          WAVE_FORMAT_DIGIFIX: begin
               raise Exception.Create('Sorry, this WAV file is in Digifix format.');
               Result:=ERROR;
               end;
          IBM_FORMAT_MULAW: begin
               raise Exception.Create('Sorry, this WAV file is in IBM U-law format.');
               Result:=ERROR;
               end;
          IBM_FORMAT_ALAW: begin
               raise Exception.Create('Sorry, this WAV file is in IBM A-law format.');
               Result:=ERROR;
               end;
          IBM_FORMAT_ADPCM: begin
               raise Exception.Create('Sorry, this WAV file is in IBM ADPCM format.');
               Result:=ERROR;
               end;
          else begin
               raise Exception.Create('Sorry, don''t understand format');
               Result:=ERROR;
               end;
     end;

     wChannels := rlshort(ft);
     ft.info.channels := wChannels;
     wSamplesPerSecond := rllong(ft);
     ft.info.rate := wSamplesPerSecond;
     wAvgBytesPerSec := rllong(ft);       // Average bytes/second
     wBlockAlign := rlshort(ft);          // Block align
     wBitsPerSample := rlshort(ft);       // bits per sample per channel
     bytespersample := Trunc((wBitsPerSample + 7)/8);
     case bytespersample of
          1: ft.info.size := SZ_BYTE;
          2: ft.info.size := SZ_WORD;
          4: ft.info.size := SZ_LONG;
          else
             raise Exception.Create('Sorry, don''t understand .wav size');
     end;

     dec(len,16);

     while (len > 0) and (ft.fp.position < ft.fp.size) do begin
          getc(ft.fp);
          dec(len);
     end;

     // Now look for the wave data chunk
     while true do begin
          if ft.fp.Read(magic,4) <> 4 then begin
             Result:=ERROR;
             Exit;
          end;

          len := rllong(ft);
          if String(magic) = 'data' then
             break;    // Found the data chunk

          // skip to next chunk
          while (len > 0) and (ft.fp.position < ft.fp.size) do begin
             getc(ft.fp);
             dec(len);
          end;
     end;

     data_length := len;
     wav.samples := trunc(data_length/ft.info.size);    // total samples

     {fprintf(stderr, "Reading Wave file: %s format, %d channel%s, %ld samp/sec\n",
             wav_format_str(wFormatTag), wChannels,
             wChannels == 1 ? "" : "s", wSamplesPerSecond);

     fprintf(stderr, "%ld byte/sec, %d block align, %d bits/samp, %lu data bytes\n",
                wAvgBytesPerSec, wBlockAlign, wBitsPerSample, data_length);
     }

     Result:=OK;
end;

function wavwritehdr(var ft: SoundStream; data_size: LongInt): integer;
begin
   case ft.info.size of
      SZ_BYTE: begin
                 wBitsPerSample := 8;
                 if (ft.info.style = -1) or (ft.info.style = S_UNSIGNED) then
                     ft.info.style := S_UNSIGNED;
                 {else
                 if (ft->info.style <> S_ALAW ) and (ft.info.style <> ULAW) then
                    fprintf(stderr, "User options overiding style written to .wav header");
                 }
               end;
      SZ_WORD: begin
                 wBitsPerSample := 16;
                 if (ft.info.style = -1) or (ft.info.style = S_SIGN2) then
                     ft.info.style := S_SIGN2;
               end;
      SZ_LONG: begin
                 wBitsPerSample := 32;
                 if (ft.info.style = -1) or (ft.info.style = S_SIGN2) then
                     ft.info.style := S_SIGN2;
               end;
   else begin
               wBitsPerSample := 32;
               if ft.info.style = -1 then
                    ft.info.style := S_SIGN2;
        end;
   end;


   case ft.info.style of
      S_UNSIGNED: begin
               wFormatTag := WAVE_FORMAT_PCM;
               {if wBitsPerSample <> 8 then
                    fprintf(stderr, "Warning - writing bad .wav file using unsigned data and %d bits/sample",wBitsPerSample);}
               end;
      S_SIGN2: begin
               wFormatTag := WAVE_FORMAT_PCM;
               {if wBitsPerSample = 8 then
                    fprintf(stderr, "Warning - writing bad .wav file using signed data and %d bits/sample",wBitsPerSample);}
               end;
      S_ALAW: begin
               wFormatTag := WAVE_FORMAT_ALAW;
               {if wBitsPerSample <> 8 then
                    fprintf(stderr, "Warning - writing bad .wav file using A-law data and %d bits/sample",wBitsPerSample);}
               end;
      S_ULAW: begin
               wFormatTag := WAVE_FORMAT_MULAW;
               {if wBitsPerSample <> 8 then
                    fprintf(stderr, "Warning - writing bad .wav file using U-law data and %d bits/sample",wBitsPerSample);}
               end;
   end;

   wSamplesPerSecond := ft.info.rate;
   bytespersample := Trunc((wBitsPerSample + 7)/8);
   wAvgBytesPerSec := ft.info.rate * ft.info.channels * bytespersample;
   wChannels := ft.info.channels;
   wBlockAlign := ft.info.channels * bytespersample;
   data_length := LongWord(data_size * bytespersample);

   // figured out header info, so write it
   st_write(ft.fp, 'RIFF');
   wllong(ft, data_length + 8+16+12+1);    // Waveform chunk size: FIXUP(4)
                                    // die 1 ist von mir karlo
   st_write(ft.fp, 'WAVE');
   st_write(ft.fp, 'fmt ');
   wllong(ft, LongInt(16));         // fmt chunk size
   wlshort(ft, wFormatTag);
   wlshort(ft, wChannels);
   wllong(ft, wSamplesPerSecond);
   wllong(ft, wAvgBytesPerSec);
   wlshort(ft, wBlockAlign);
   wlshort(ft, wBitsPerSample);

   st_write(ft.fp, 'data');

   wllong(ft, data_length);  // data chunk size: FIXUP(40)


{     fprintf(stderr, "Writing Wave file: %s format, %d channel%s, %ld samp/sec",
               wav_format_str(wFormatTag), wChannels,
               wChannels == 1 ? "" : "s", wSamplesPerSecond);
     fprintf(stderr, " %ld byte/sec, %d block align, %d bits/samp\n",
                     wAvgBytesPerSec, wBlockAlign, wBitsPerSample);

     return OK;
}
   Result:=OK;
end;

function wavstartwrite(var ft: SoundStream; data_size: LongInt): integer;
var wav: wav_t;
    q: wav_p;
begin
     q := @ft.priv;
     wav := q^;
     ft.swap:=false;

     wav.samples := 0;
     wav.second_header := 0;

     if wavwritehdr(ft,data_size) <> OK then
        Result:=ERROR
     else
        Result:=OK;
end;

function pvftowav (var fd_in, fd_out: TStream; var header_in: pvf_header; wav_bits: integer):integer;
var
   bytespersample, buffer_size,
   voice_samples, data: integer;
   data_size: longint;
   tbuf, buffer,ptr{, buff_new}: pint;
   s: SoundStream;
   q: integer;

begin
   data_size:=0;
   buffer_size := 0;
   voice_samples := 0;
   buffer := nil;
   bytespersample := Trunc((wav_bits + 7) / 8);

   case bytespersample of
      1: s.info.size := SZ_BYTE;
      2: s.info.size := SZ_WORD;
      4: s.info.size := SZ_LONG;
   else
      raise Exception.Create('sorry, don''t understand .wav size');
   end;

     s.info.rate     := header_in.speed;
     s.info.style    := -1;
     s.info.channels := header_in.channels;
     s.comment       := '';
     s.swap          := false;
     s.filetype      := '';
     s.fp            := fd_out;
     s.seekable      := false;

     while true do begin
          if feof(fd_in) then break;
          data := header_in.read_pvf_data(header_in.nbits,fd_in);

          if voice_samples >= buffer_size then begin
               inc(buffer_size, BLOCK_SIZE);

               ReallocMem(buffer,buffer_size * SizeOf(integer));

               if buffer = nil then
                  raise Exception.Create('out of memory in pvttowav');
          end;
          tbuf :=buffer; inc(tbuf,voice_samples);

          tbuf^ := data;
          inc(voice_samples);
          inc(data_size);
     end;

     if wavstartwrite(s,data_size) <> OK then begin
          FreeMem(buffer);
          Result:=ERROR;
          Exit
     end;

     ptr := buffer;

     case (s.info.size) of
          SZ_BYTE:
               while data_size > 0 do begin
                  dec(data_size);
                  q := (ptr^ and $80000000);
                  ptr^ := ptr^ shr 16;
                  if q <> 0 then
                     ptr^:= $FFFF0000 or ptr^;

                  if ptr^ > $7f then
                     ptr^ := $7f;

                  if ptr^ < -$80 then
                      ptr^ := -$80;

                  putc(ptr^+$80,fd_out,1);
                  inc(ptr);
               end;

          SZ_WORD:
               while data_size > 0 do begin
                  dec(data_size);
                  q := (ptr^ and $80000000);
                  ptr^ := ptr^ shr 8;
                  if q <> 0 then
                     ptr^:= $FF000000 or ptr^;

                  if ptr^ > $7fff then
                     ptr^ := $7fff;

                  if ptr^ < -$8000 then
                     ptr^ := -$8000;

                  //ptr^ := ptr^ + $8000;
                  // orig
                  {putc((ptr^ shr 8) and $ff, fd_out,1);
                  putc(ptr^ and $ff, fd_out,1);}

                  // low, high
                  putc(ptr^ and $ff, fd_out,1);
                  putc((ptr^ shr 8) and $ff, fd_out,1);
                  inc(ptr);
               end;

          SZ_LONG:
               while data_size > 0 do begin
                  dec(data_size);
                  putc(ptr^, fd_out,4);
                  inc(ptr);
               end;
     end;

     SysFreeMem(buffer); //,buffer_size * sizeof(int))
     Result:=OK;
end;

function pvftozyxel(var fd_in, fd_out: TStream; nbits: integer; var header_in: pvf_header): integer;
var a,d,e,nmax,sign,delta,data_new: integer;
    s: state_t;
    tt: integer;
begin
     a := 0;
     d := 5;

     s := init_state;

     while fd_in.position <> fd_in.size do begin
          data_new := header_in.read_pvf_data(header_in.nbits,fd_in);
          e := 0;
          nmax := 1 shl (nbits - 1);
          tt:=(data_new and $80000000);
          delta := (data_new shr 10);
          if tt <> 0 then delta := $FFC00000 or delta;
          delta := delta - a;

          if delta < 0 then begin
               e := nmax;
               delta := -delta;
          end;

          while true do begin
             dec(nmax);
             if (nmax = 0) or (delta <= d) then
                break;

             delta := delta - d;
             inc(e);
          end;

          if ((nbits = 4) and ((e and $0f) = 0)) then
               e := $08;

          write_bits(fd_out, s, nbits, e);
          a := (a * 4093 + 2048); tt:= (a and $80000000); a := (a shr 12);
          if tt <> 0 then a := $FFF00000 or a;

          if (e shr (nbits - 1)) <> 0 then
             sign:=-1
          else
             sign:=1;

          e := e and bitmask[nbits - 1];
          //a := a + sign * ((e << 1) + 1) * d >> 1;
          tt := sign * ((e shl 1) + 1) * d;
          if (tt and $80000000) <> 0 then
             a:= a + ($80000000 or tt shr 1)
          else
             a:= a + tt shr 1;

          if (d and 1) <> 0 then
             inc(a);

          d := (d * Mx[nbits - 2,e] + $2000) shr 14;
          if d < 5 then d := 5;
     end;

     if s.nleft > 0 then
        write_bits(fd_out, s, 8 - s.nleft, $00);

     Result:=OK;
end;

function wavtopvf(var fd_in, fd_out: TStream; var header_out: pvf_header): integer;
var  s: soundstream;
     d: integer;
begin
     d := 0;

     s.fp := fd_in;

     if wavstartread(s) <> OK then begin
        Result:=ERROR;
        Exit;
     end;

     header_out.channels := Integer(wChannels);
     header_out.speed := Integer(wSamplesPerSecond);
     //write_pvf_header(fd_out, header_out);

     while data_length > 0 do begin
        dec(data_length);
        if fd_in.position = fd_in.size then begin
           Result:=OK;
           Exit
        end;

        case wBitsPerSample of
             8: d := (getc(fd_in) - $80) shl 16;
             16: begin
                    d := getc(fd_in) + $100 * getc(fd_in);
                    if d > $7fff then
                       dec(d,$10000);
                    d:= d shl 8;
                 end;

             {else begin
                    fprintf(stderr,
                     "%s: unsupported number of bits per sample (%d)\n",
                     program_name, wBitsPerSample);
             }
        end;

        header_out.write_pvf_data(header_out.nbits,fd_out, d);
     end;

     Result:=OK;
end;

function WriteRMDHeader(Stream: TStream; ModemRMDName: string; Compression, Speed, Bits: Integer; DataLength: Longint): Integer;
var
  Header: TRMDHeader;
begin
  FillChar(Header,  SizeOf(TRMDHeader), 0);
  { Format header data }
  StrPLCopy(Header.Magic, RMD_HEADER_MAGIC, 4);
  StrPLCopy(Header.ModemType, ModemRMDName, 16);
  Header.Compression := Compression;
  Header.Speed := Speed;
  Header.Bits := Bits;
  { Write header }
  if Stream.Write(Header, SizeOf(TRMDHeader)) <> SizeOf(TRMDHeader) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Success }
  Result := S_OK;
end;

procedure raw2pvf(Info: TRMDInfo; header: pvf_header; Stream_In, Stream_Out: TStream);
begin
   if Info.ModemType = 'ZyXEL 1496' then begin
      if zyxeltopvf(Stream_In,Stream_Out,Info.Bits,header) <> OK then
         raise Exception.Create('Zyxel to pvf failed!');
   end
   else
   if Info.ModemType = 'Omni56k' then begin
      if imaadpcmtopvf(Stream_In,Stream_Out,header) <> OK then
         raise Exception.Create('IMA ADPCM to pvf failed!');
   end
   else
      raise Exception.Create('Unknown modem data format');
end;

procedure pvf2raw(Info: TRMDInfo; Stream_In, Stream_Out: TStream);
var header: pvf_header;
begin
  WriteRMDHeader( Stream_Out, info.ModemName, info.Compression,
                  info.Speed, info.Bits, 0);
  header:=init_pvf_header;
  header.speed:=Info.Speed;

  if Info.ModemType = 'ZyXEL 1496' then begin
     if pvftozyxel(Stream_In,Stream_Out,Info.Bits,header) <> OK then
        raise Exception.Create('Zyxel to pvf failed!');
  end
  else
  if Info.ModemType = 'Omni56k' then begin
     if pvftoimaadpcm(Stream_In,Stream_Out,header) <> OK then
        raise Exception.Create('IMA ADPCM to pvf failed!');
  end
  else
     raise Exception.Create('Unknown modem data format');
end;

procedure WavToRmd( var Stream_In, Stream_Out: TStream;
                    RMDName: string;
                    Speed, Compression, Bits: integer);
var
   MemTemp: TStream;
   Info: TRMDInfo;
   header: pvf_header;
   {$IfDef Debug_PVF}
   fl: TStream;
   {$EndIf}
begin
  Stream_In.Position:=0;
  MemTemp:=TMemoryStream.Create;

  header:=init_pvf_header;

  Stream_In.position:=0;
  wavtopvf(Stream_In, MemTemp, header);

  MemTemp.position:=0;
  {$IfDef Debug_PVF}
  fl:=TFileStream.Create('d:\tst-wav-rmd.pvf',fmCreate or fmOpenReadWrite);
  fl.copyfrom(MemTemp,MemTemp.size);
  fl.Free;
  MemTemp.position:=0;
  {$EndIf}

  info.ModemName:=RMDName;
  info.ModemType:=RMDName;
  info.Compression:=Compression;
  info.Speed:=Speed;
  info.Bits:=Bits;

  pvf2raw(Info, MemTemp, Stream_Out);

  MemTemp.Free;
end;

procedure WavToRmd(modem: TVoiceModem; Compression: integer; var Stream_In, Stream_Out: TStream); overload;
var
   mc: TModemCompression;
begin
  //    ...
  modem.driver.Compressions.find(Compression,mc);

  WavToRmd( Stream_In, Stream_Out, modem.ModemRMDName,
            mc.Speed,
            mc.Compression,
            mc.Bits);
end;

procedure WavToRmd(modem: TVoiceModem; var Stream_In, Stream_Out: TStream);
begin
  WavToRmd(modem, modem.profile.RecCompression, Stream_In, Stream_Out);
end;

procedure MergeRMD( Files: TStrings; Output: TStream);
var i: integer;
    v: TStream;
   MemTemp: TStream;
   Info, Info_Out: TRMDInfo;
   header: pvf_header;
   {$IfDef Debug_PVF}
   fl: TStream;
   {$EndIf}
   //tp: integer;
begin
   if Files.count < 1 then Exit;

   MemTemp:=TMemoryStream.Create;

   for i:=0 to Files.count-1 do begin
       v:=TFileStream.Create(Files[i],fmOpenRead);

       if GetRMDStreamInfo(v,Info) <> S_OK then Exit;

       if i=0 then Info_Out:=Info; // use first as shape

       header:=init_pvf_header;
       header.speed:=Info.Speed;

       raw2pvf(Info, header, v, Output);

       v.Free;
  end;

  MemTemp.position:=0;

  {$IfDef Debug_PVF}
  fl:=TFileStream.Create('d:\tst-merge.pvf',fmCreate or fmOpenReadWrite);
  fl.copyfrom(MemTemp,MemTemp.size);
  fl.Free;
  MemTemp.position:=0;
  {$EndIf}

  pvf2raw(Info_Out, MemTemp, Output);

  MemTemp.Free;
  Output.Position:=0;
end;

procedure RmdToWav(var Stream_In, Stream_Out: TStream);
var
   MemTemp: TStream;
   Info: TRMDInfo;
   header_out: pvf_header;
   {$IfDef Debug_PVF}
   fl: TStream;
   {$EndIf}
begin
  Stream_In.Position:=0;
  if GetRMDStreamInfo(Stream_In,Info) <> S_OK then Exit;

  header_out:=init_pvf_header;
  with header_out do begin
     speed:=Info.Speed;
  end;

  MemTemp:=TMemoryStream.Create;

  {!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!}
  if info.Bits = 8 then
     info.bits:=3;
  {!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!}

  raw2pvf(Info, header_out, Stream_In, MemTemp);

  MemTemp.position:=0;

  {$IfDef Debug_PVF}
  fl:=TFileStream.Create('d:\tst-rmd-wav.pvf',fmCreate or fmOpenReadWrite);
  fl.copyfrom(MemTemp,MemTemp.size);
  fl.Free;
  MemTemp.position:=0;
  {$EndIf}

  pvftowav(MemTemp,Stream_Out,header_out, wav_bits);
  //pvftowav(MemTemp,Stream_Out,header_out, 16);

  MemTemp.Free;
end;

begin
  wav_bits:=8;
end.
