{
  Voice Modem Library v1.0

  This file contain some utils procedures and funtions.

  Copyright (C) 1998-2001 Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_Utils;

{$I VML.INC}

interface

uses
  Windows, SysUtils, Classes;

type

{ RMD file information }

  TRMDInfo = packed record
    ModemType: string;       { modem type }
    ModemName: string;       { modem name }
    Compression: SmallInt;   { modem specific compression method }
    CompressionName: String; { compression name }
    Speed: SmallInt;         { sample speed }
    Bits: Byte;              { bits per sample }
    DataSize: Longint;       { voice data size in bytes }
    VoiceLen: Longint;       { voice message length in msecs }
  end;

{ Get RMD file or stream information. Returned OK if success, otherwise
  returned error number (see VME_XXX constants for details). }

function GetRMDFileInfo(const FileName: string; var Info: TRMDInfo): Integer;
function GetRMDStreamInfo(Stream: TStream; var Info: TRMDInfo): Integer;

{ Modem compression and devices routines. This functions return default
  driver's values. }

function GetDriverCompression(const ModemName: string; const Name: string): Integer;
function GetDriverCompressionName(const ModemName: string; Compression: Integer): string;
function GetDriverCompressionsList(const ModemName: string; Proc: TGetStrProc): Integer;
function GetDriverCompressionsStringList(const ModemName: string; Strings: TStrings): Integer;
function GetDriverDevice(const ModemName: string; const Name: string): Integer;
function GetDriverDeviceName(const ModemName: string; Device: Integer): string;
function GetDriverDevicesList(const ModemName: string; Proc: TGetStrProc): Integer;
function GetDriverDevicesStringList(const ModemName: string; Strings: TStrings): Integer;
function GetDriverPlayDevicesList(const ModemName: string; Proc: TGetStrProc): Integer;
function GetDriverPlayDevicesStringList(const ModemName: string; Strings: TStrings): Integer;
function GetDriverRecDevicesList(const ModemName: string; Proc: TGetStrProc): Integer;
function GetDriverRecDevicesStringList(const ModemName: string; Strings: TStrings): Integer;
function GetDriverSpeedList(const ModemName: string; Compression: Integer; Proc: TGetStrProc): Integer;
function GetDriverSpeedStringList(const ModemName: string; Compression: Integer; Strings: TStrings): Integer;
function GetDriverWaveFormat(const ModemName: string; const Name: string): Integer;
function GetDriverWaveFormatName(const ModemName: string; WaveFormat: Integer): string;
function GetDriverWaveFormatsList(const ModemName: string; Proc: TGetStrProc): Integer;
function GetDriverWaveFormatsStringList(const ModemName: string; Strings: TStrings): Integer;
function GetDriverWaveSpeedList(const ModemName: string; WaveFormat: Integer; Proc: TGetStrProc): Integer;
function GetDriverWaveSpeedStringList(const ModemName: string; WaveFormat: Integer; Strings: TStrings): Integer;

{ Modem compression and devices routines. This functions return driver ini
  file values. }

function GetIniCompression(const FileName: string; const Name: string): Integer;
function GetIniCompressionName(const FileName: string; Compression: Integer): string;
function GetIniCompressionsList(const FileName: string; Proc: TGetStrProc): Integer;
function GetIniCompressionsStringList(const FileName: string; Strings: TStrings): Integer;
function GetIniDevice(const FileName: string; const Name: string): Integer;
function GetIniDeviceName(const FileName: string; Device: Integer): string;
function GetIniDevicesList(const FileName: string; Proc: TGetStrProc): Integer;
function GetIniDevicesStringList(const FileName: string; Strings: TStrings): Integer;
function GetIniPlayDevicesList(const FileName: string; Proc: TGetStrProc): Integer;
function GetIniPlayDevicesStringList(const FileName: string; Strings: TStrings): Integer;
function GetIniRecDevicesList(const FileName: string; Proc: TGetStrProc): Integer;
function GetIniRecDevicesStringList(const FileName: string; Strings: TStrings): Integer;
function GetIniSpeedList(const FileName: string; Compression: Integer; Proc: TGetStrProc): Integer;
function GetIniSpeedStringList(const FileName: string; Compression: Integer; Strings: TStrings): Integer;
function GetIniWaveFormat(const FileName: string; const Name: string): Integer;
function GetIniWaveFormatName(const FileName: string; WaveFormat: Integer): string;
function GetIniWaveFormatsList(const FileName: string; Proc: TGetStrProc): Integer;
function GetIniWaveFormatsStringList(const FileName: string; Strings: TStrings): Integer;
function GetIniWaveSpeedList(const FileName: string; WaveFormat: Integer; Proc: TGetStrProc): Integer;
function GetIniWaveSpeedStringList(const FileName: string; WaveFormat: Integer; Strings: TStrings): Integer;

{ String parsing utilites }

function ParseIniString(const Source, Divider: string; var StartPos: Integer; const Default: string): string;
function ParseIniInteger(const Source, Divider: string; var StartPos: Integer; Default: Longint): Longint;

function ParseIniFrameStr(const Source, Divider: string; var StartPos: Integer; FrameChar: Char; const Default: string): string;

{ Some additional string routines }

function FrameString(const Str: string; FrameChar: Char): string;
function UnFrameString(const Str: string; FrameChar: Char): string;

{ Detect stream voice data format by file contents analysing. Returns one of
  DATA_FORMAT_XXXX constant (see VML_Consts.pas for constants definition). }

function DetectStreamFormat(Stream: TStream): Integer;
function DetectStreamFormatDef(Stream: TStream; Default: Integer): Integer;

{ Detect file voice data format by analysing file contents and file extension.
  Returns one of DATA_FORMAT_XXXX value (see VML_Consts.pas for constants
  definition). }

function DetectFileFormat(const FileName: string): Integer;
function DetectFileFormatDef(const FileName: string; Default: Integer): Integer;

implementation

uses
  VML_Modem, VML_IniFile, VML_Consts, MMSys;

function GetRMDFileInfo(const FileName: string; var Info: TRMDInfo): Integer;
var
  Handle: Integer;
  Stream: THandleStream;
begin
  { Check file }
  if not FileExists(FileName) then
  begin
    Result := VME_FILE_NOT_FOUND;
    Exit;
  end;
  { Open file }
  Handle := FileOpen(FileName, fmOpenRead);
  { Check for file opening }
  if Handle < 0 then
  begin
    Result := VME_FILE_OPEN_ERROR;
    Exit;
  end;
  { Get info }
  try
    Stream := THandleStream.Create(Handle);
    try
      Result := GetRMDStreamInfo(Stream, Info);
    finally
      Stream.Free;
    end;
  finally
    FileClose(Handle);
  end;
end;

function GetRMDStreamInfo(Stream: TStream; var Info: TRMDInfo): Integer;
var
  Header: TRMDHeader;
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Read file header }
  if Stream.Read(Header, SizeOf(TRMDHeader)) <> SizeOf(TRMDHeader) then
  begin
    Result := VME_STREAM_READ_ERR;
    Exit;
  end;
  { Check for header magic }
  if StrLComp(Header.Magic, RMD_HEADER_MAGIC, 4) <> 0 then
  begin
    Result := VME_NOT_RMD_FILE;
    Exit;
  end;
  { Clear result }
  FillChar(Info, SizeOf(Info), 0);
  { Get header information }
  with Info do
  begin
    { Common information }
    ModemType := StrPas(Header.ModemType);
    Compression := Header.Compression;
    Speed := Header.Speed;
    Bits := Header.Bits;
    DataSize := Stream.Size - SizeOf(TRMDHeader);
    { Driver specific information }
    if GetModemDriver(ModemType, DriverClass) then
    begin
      DriverInstance := DriverClass.Create(nil);
      try
        ModemName := DriverInstance.ModemName;
        CompressionName := DriverInstance.GetCompressionName(Compression);
        VoiceLen := DriverInstance.GetVoiceLen(Compression, Speed, Bits, DataSize);
      finally
        DriverInstance.Free;
      end;
    end
    else
    begin
      ModemName := '?';
      CompressionName := '?';
      VoiceLen := 0;
    end;
  end;
  { Success }
  Result := S_OK;
end;

{ Modem compression and devices routines (default vlaues) }

function GetDriverCompression(const ModemName: string; const Name: string): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetCompression(Name);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Try to return number from name }
  Result := StrToIntDef(Name, -1);
end;

function GetDriverCompressionName(const ModemName: string; Compression: Integer): string;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetCompressionName(Compression);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Return given number string }
  Result := IntToStr(Compression);
end;

function GetDriverCompressionsList(const ModemName: string; Proc: TGetStrProc): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetCompressionsList(Proc);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Compression list is empty }
  Result := 0;
end;

function GetDriverCompressionsStringList(const ModemName: string; Strings: TStrings): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetCompressionsStringList(Strings);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Compression list is empty }
  Result := 0;
end;

function GetDriverDevice(const ModemName: string; const Name: string): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetDevice(Name);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Try to return number from name }
  Result := StrToIntDef(Name, -1);
end;

function GetDriverDeviceName(const ModemName: string; Device: Integer): string;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetDeviceName(Device);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Return given number string }
  Result := IntToStr(Device);
end;

function GetDriverDevicesList(const ModemName: string; Proc: TGetStrProc): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetDevicesList(Proc);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Device list is empty }
  Result := 0;
end;

function GetDriverDevicesStringList(const ModemName: string; Strings: TStrings): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetDevicesStringList(Strings);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Device list is empty }
  Result := 0;
end;

function GetDriverPlayDevicesList(const ModemName: string; Proc: TGetStrProc): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetPlayDevicesList(Proc);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Device list is empty }
  Result := 0;
end;

function GetDriverPlayDevicesStringList(const ModemName: string; Strings: TStrings): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetPlayDevicesStringList(Strings);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Device list is empty }
  Result := 0;
end;

function GetDriverRecDevicesList(const ModemName: string; Proc: TGetStrProc): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetRecDevicesList(Proc);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Device list is empty }
  Result := 0;
end;

function GetDriverRecDevicesStringList(const ModemName: string; Strings: TStrings): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetRecDevicesStringList(Strings);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Device list is empty }
  Result := 0;
end;

function GetDriverSpeedList(const ModemName: string; Compression: Integer; Proc: TGetStrProc): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetSpeedList(Compression, Proc);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Speed list is empty }
  Result := 0;
end;

function GetDriverSpeedStringList(const ModemName: string; Compression: Integer; Strings: TStrings): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetSpeedStringList(Compression, Strings);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Speed list is empty }
  Result := 0;
end;

function GetDriverWaveFormat(const ModemName: string; const Name: string): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetWaveFormat(Name);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Try to return number from name }
  Result := StrToIntDef(Name, -1);
end;

function GetDriverWaveFormatName(const ModemName: string; WaveFormat: Integer): string;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetWaveFormatName(WaveFormat);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Return given number string }
  Result := IntToStr(WaveFormat);
end;

function GetDriverWaveFormatsList(const ModemName: string; Proc: TGetStrProc): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetWaveFormatsList(Proc);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Format list is empty }
  Result := 0;
end;

function GetDriverWaveFormatsStringList(const ModemName: string; Strings: TStrings): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetWaveFormatsStringList(Strings);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Format list is empty }
  Result := 0;
end;

function GetDriverWaveSpeedList(const ModemName: string; WaveFormat: Integer; Proc: TGetStrProc): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetWaveSpeedList(WaveFormat, Proc);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Speed list is empty }
  Result := 0;
end;

function GetDriverWaveSpeedStringList(const ModemName: string; WaveFormat: Integer; Strings: TStrings): Integer;
var
  DriverClass: TModemDriverClass;
  DriverInstance: TModemDriver;
begin
  { Get modem driver class }
  if GetModemDriver(ModemName, DriverClass) then
  begin
    DriverInstance := DriverClass.Create(nil);
    try
      Result := DriverInstance.GetWaveSpeedStringList(WaveFormat, Strings);
    finally
      DriverInstance.Free;
    end;
    Exit;
  end;
  { Speed list is empty }
  Result := 0;
end;

{ Modem compression and devices routines (ini files) }

function GetIniCompression(const FileName: string; const Name: string): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetCompression(Name)
    else
      { Try to return number from name }
      Result := StrToIntDef(Name, -1);
  finally
    DriverInstance.Free;
  end;
end;

function GetIniCompressionName(const FileName: string; Compression: Integer): string;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetCompressionName(Compression)
    else
      { Return given number string }
      Result := IntToStr(Compression);
  finally
    DriverInstance.Free;
  end;
end;

function GetIniCompressionsList(const FileName: string; Proc: TGetStrProc): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetCompressionsList(Proc)
    else
      { Compression list is empty }
      Result := 0;
  finally
    DriverInstance.Free;
  end;
end;

function GetIniCompressionsStringList(const FileName: string; Strings: TStrings): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetCompressionsStringList(Strings)
    else
      { Compression list is empty }
      Result := 0;
  finally
    DriverInstance.Free;
  end;
end;

function GetIniDevice(const FileName: string; const Name: string): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetDevice(Name)
    else
      { Try to return number from name }
      Result := StrToIntDef(Name, -1);
  finally
    DriverInstance.Free;
  end;
end;

function GetIniDeviceName(const FileName: string; Device: Integer): string;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetDeviceName(Device)
    else
      { Return given number string }
      Result := IntToStr(Device);
  finally
    DriverInstance.Free;
  end;
end;

function GetIniDevicesList(const FileName: string; Proc: TGetStrProc): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetDevicesList(Proc)
    else
      { Device list is empty }
      Result := 0;
  finally
    DriverInstance.Free;
  end;
end;

function GetIniDevicesStringList(const FileName: string; Strings: TStrings): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetDevicesStringList(Strings)
    else
      { Device list is empty }
      Result := 0;
  finally
    DriverInstance.Free;
  end;
end;

function GetIniPlayDevicesList(const FileName: string; Proc: TGetStrProc): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetPlayDevicesList(Proc)
    else
      { Device list is empty }
      Result := 0;
  finally
    DriverInstance.Free;
  end;
end;

function GetIniPlayDevicesStringList(const FileName: string; Strings: TStrings): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetPlayDevicesStringList(Strings)
    else
      { Device list is empty }
      Result := 0;
  finally
    DriverInstance.Free;
  end;
end;

function GetIniRecDevicesList(const FileName: string; Proc: TGetStrProc): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetRecDevicesList(Proc)
    else
      { Device list is empty }
      Result := 0;
  finally
    DriverInstance.Free;
  end;
end;

function GetIniRecDevicesStringList(const FileName: string; Strings: TStrings): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetRecDevicesStringList(Strings)
    else
      { Device list is empty }
      Result := 0;
  finally
    DriverInstance.Free;
  end;
end;

function GetIniSpeedList(const FileName: string; Compression: Integer; Proc: TGetStrProc): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetSpeedList(Compression, Proc)
    else
      { Speed list is empty }
      Result := 0;
  finally
    DriverInstance.Free;
  end;
end;

function GetIniSpeedStringList(const FileName: string; Compression: Integer; Strings: TStrings): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetSpeedStringList(Compression, Strings)
    else
      { Speed list is empty }
      Result := 0;
  finally
    DriverInstance.Free;
  end;
end;

function GetIniWaveFormat(const FileName: string; const Name: string): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetWaveFormat(Name)
    else
      { Try to return number from name }
      Result := StrToIntDef(Name, -1);
  finally
    DriverInstance.Free;
  end;
end;

function GetIniWaveFormatName(const FileName: string; WaveFormat: Integer): string;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetWaveFormatName(WaveFormat)
    else
      { Return given number string }
      Result := IntToStr(WaveFormat);
  finally
    DriverInstance.Free;
  end;
end;

function GetIniWaveFormatsList(const FileName: string; Proc: TGetStrProc): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetWaveFormatsList(Proc)
    else
      { Format list is empty }
      Result := 0;
  finally
    DriverInstance.Free;
  end;
end;

function GetIniWaveFormatsStringList(const FileName: string; Strings: TStrings): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetWaveFormatsStringList(Strings)
    else
      { Format list is empty }
      Result := 0;
  finally
    DriverInstance.Free;
  end;
end;

function GetIniWaveSpeedList(const FileName: string; WaveFormat: Integer; Proc: TGetStrProc): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetWaveSpeedList(WaveFormat, Proc)
    else
      { Speed list is empty }
      Result := 0;
  finally
    DriverInstance.Free;
  end;
end;

function GetIniWaveSpeedStringList(const FileName: string; WaveFormat: Integer; Strings: TStrings): Integer;
var
  DriverInstance: TModemDriver;
begin
  { Load modem driver }
  DriverInstance := ReadDriverIniFile(FileName, nil);
  try
    { Check driver name }
    if Length(DriverInstance.ModemName) <> 0 then
      { Get driver specific result }
      Result := DriverInstance.GetWaveSpeedStringList(WaveFormat, Strings)
    else
      { Speed list is empty }
      Result := 0;
  finally
    DriverInstance.Free;
  end;
end;

{ String parsing utilites }

function ParseIniString(const Source, Divider: string; var StartPos: Integer; const Default: string): string;
var
  SP, DP: PChar;
begin
  { Get start position }
  SP := PChar(Source) + StartPos - 1;
  { Find divider }
  DP := StrPos(SP, PChar(Divider));
  if DP = nil then DP := StrEnd(SP);
  { Check divider position }
  if DP = nil then
  begin
    Result := Default;
    Exit;
  end;
  { Get result string }
  Result := Trim(Copy(SP, 1, DP - SP));
  { Shift start position }
  StartPos := StartPos + (DP - SP) + Length(Divider);
end;

function ParseIniInteger(const Source, Divider: string; var StartPos: Integer; Default: Longint): Longint;
var
  S: string;
  E: Integer;
begin
  { Get value string }
  S := ParseIniString(Source, Divider, StartPos, IntToStr(Default));
  { Check for 0x0000 format value }
  if (Length(S) > 2) and (S[1] = '0') and (UpCase(S[2]) = 'X') then
    S := '$' + Copy(S, 3, MaxInt);
  { Convert string to integer }
  Val(S, Result, E);
  if E <> 0 then Result := Default;
end;

function ParseIniFrameStr(const Source, Divider: string; var StartPos: Integer; FrameChar: Char; const Default: string): string;
var
  SP, DP, FP1, FP2: PChar;
begin
  { Get start position }
  SP := PChar(Source) + StartPos - 1;
  { Find divider }
  DP := StrPos(SP, PChar(Divider));
  if DP = nil then DP := StrEnd(SP);
  { Check divider position }
  if DP = nil then
  begin
    Result := Default;
    Exit;
  end;
  { Find open frame char }
  FP1 := StrPos(SP, @FrameChar);
  { Check frame char exists and stands before divider }
  if (FP1 = nil) or (FP1 > DP) then
  begin
    { Open frame char not found - skip value }
    StartPos := StartPos + (DP - SP) + Length(Divider);
    Result := Default;
    Exit;
  end;
  { Find close frame char }
  FP2 := StrPos(FP1 + 1, @FrameChar);
  if FP2 = nil then
  begin
    { Close frame char not found - skip value }
    StartPos := StartPos + (DP - SP) + Length(Divider);
    Result := Default;
    Exit;
  end;
  { Find first divider after close frame char }
  DP := StrPos(FP2 + 1, PChar(Divider));
  if DP = nil then DP := StrEnd(FP2 + 1);
  { Check new divider position }
  if DP = nil then
  begin
    Result := Default;
    Exit;
  end;
  { Get unframe result string }
  Result := UnFrameString(Trim(Copy(FP1, 1, FP2 - FP1)), FrameChar);
  { Shift start position }
  StartPos := StartPos + (DP - SP) + Length(Divider);
end;

{ Some additional string routines }

function FrameString(const Str: string; FrameChar: Char): string;
begin
  Result := FrameChar + Str + FrameChar;
end;

function UnFrameString(const Str: string; FrameChar: Char): string;
var
  L: Integer;
begin
  Result := Str;
  { Check result length }
  if Length(Result) > 0 then
  begin
    { Remove first frame char }
    if Result[1] = FrameChar then Result := Copy(Result, 2, MaxInt);
    { Remove last frame char }
    L := Length(Result);
    if (L > 0) and (Result[L] = FrameChar) then SetLength(Result, L - 1);
  end;
end;

{ Detect stream voice data format }

function DetectStreamFormat(Stream: TStream): Integer;
var
  Position: Longint;
  Magic: array[0..3] of Char;
  Dummy: Longint;
begin
  { Save stream position }
  Position := Stream.Position;
  try
    { Read file magic }
    if Stream.Read(Magic, 4) = 4 then
    begin
      { Check RMD file magic }
      if StrLComp(Magic, RMD_HEADER_MAGIC, 4) = 0 then
      begin
        { Raw Modem Data }
        Result := DATA_FORMAT_RMD;
        Exit;
      end;
      { Check RIFF file magic }
      if DWORD(Magic) = FOURCC_RIFF then
        if Stream.Read(Dummy, 4) = 4 then
          { Check WAVE form type }
          if Stream.Read(Magic, 4) = 4 then
            if DWORD(Magic) = FOURCC_WAVE then
            begin
              { WAVE Format }
              Result := DATA_FORMAT_WAVE;
              Exit;
            end;
    end;
  finally
    Stream.Position := Position;
  end;
  { Data format unknown }
  Result := DATA_FORMAT_UNKNOWN;
end;

function DetectStreamFormatDef(Stream: TStream; Default: Integer): Integer;
begin
  Result := DetectStreamFormat(Stream);
  if Result = DATA_FORMAT_UNKNOWN then Result := Default;
end;

{ Detect file voice data format }

function DetectFileFormat(const FileName: string): Integer;
var
  Handle: Integer;
  Stream: THandleStream;
  Ext: string;
begin
  { Analyse file contents }
  if FileExists(FileName) then
  begin
    Handle := FileOpen(FileName, fmOpenRead or fmShareDenyNone);
    if not (Handle < 0) then
    try
      { Detect stream format }
      Stream := THandleStream.Create(Handle);
      try
        Result := DetectStreamFormat(Stream);
        if Result <> DATA_FORMAT_UNKNOWN then Exit;
      finally
        Stream.Free;
      end;
    finally
      FileClose(Handle);
    end;
  end;
  { Analyse file extension }
  Ext := ExtractFileExt(FileName);
  if CompareText(Ext, '.rmd') = 0 then
  begin
    { Raw Modem Data }
    Result := DATA_FORMAT_RMD;
    Exit;
  end;
  if CompareText(Ext, '.wav') = 0 then
  begin
    { WAVE Format }
    Result := DATA_FORMAT_WAVE;
    Exit;
  end;
  { Unknown data format }
  Result := DATA_FORMAT_UNKNOWN;
end;

function DetectFileFormatDef(const FileName: string; Default: Integer): Integer;
begin
  Result := DetectFileFormat(FileName);
  if Result = DATA_FORMAT_UNKNOWN then Result := Default;
end;

end.

