{
  Voice Modem Library v1.0

  This file contains the ZyXEL 1496 specific hardware stuff.

  Copyright (C) 1998-2001 Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_ZyXEL1496;

{$I VML.INC}

interface

uses
  Windows, SysUtils, Classes, VML_Modem, VML_Consts, VML_IS101;

const

{ ZyXEL 1496 available compression modes }

  ZYXEL1496_CELP     = 1;
  ZYXEL1496_ADPCM_2  = 2;
  ZYXEL1496_ADPCM_3  = 3;
  ZYXEL1496_ADPCM_30 = 30;
  ZYXEL1496_ADPCM_4  = 4;

{ ZyXEL 1496 modem strings }

  SZyXEL1496Name    = 'ZyXEL 1496';
  SZyXEL1496RMDName = 'ZyXEL 1496';

{$I *.INC}

type

{ TZyXEL1496 }

  TZyXEL1496 = class(TIS101)
  private
    FROMRelease: Integer;
  public
    constructor Create(AOwner: TComponent); override;
    function Init: Integer; override;
    property ROMRelease: Integer read FROMRelease;
  end;

implementation

{ TZyXEL1496 }

constructor TZyXEL1496.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  { Modem name }
  ModemName := SZyXEL1496Name;
  ModemRMDName := SZyXEL1496RMDName;
  { &H3 - Hardware CTS/RTS flow control.
    &R1 - Modem assumes RTS always ON. }
  HardFlowCmd := 'AT&H3&R1';
  { X5 - Set extended verbal result messages. }
  ResultCodesCmd := 'ATX5';
  PickPhoneAnsw := 'VCON';
  BeepTimeUnit := 100;
  { +FLO=2 - Hardware flow control.
    +VTX   - Start the voice transmission process. }
  StartPlayCmd := 'AT+FLO=2||AT+VTX';
  StartPlayAnsw := 'OK||CONNECT';
  IntrPlayCmd := '<DLE><DC4>';
  IntrPlayAnsw := 'VCON';
  StopPlayAnsw := 'VCON';
  { +FLO=2 - Hardware flow control.
    +VRX   - Start the voice reception process. }
  StartRecCmd := 'AT+FLO=2||AT+VRX';
  StopRecAnsw := 'OK||VCON';
  { +VIT=<value> - Set the DCE's initial value for the DTE/DCE Inactivity
                   timer. }
  InactivityTimerCmd := 'AT+VIT=%d';
  InactivityTimerAnsw := 'OK|NO CARRIER';
  { +VSD=<sds>,<sdi> - Set the silence detection sensitivity and the
                       required period. }
  SilenceDetectCmd := 'AT+VSD=%d,%d';
  SilenceDetectAnsw := 'OK';
  SilenceSensMin := 0;
  SilenceSensMax := 31;
  { +VGT=<value> - Set the transmit gain for voice samples (0-255). }
  TransmitGainCmd := 'AT+VGT=%d';
  TransmitGainAnsw := 'OK';
  TransmitGainMin := 0;
  TransmitGainMax := 255;
  { +VGR=<value> - Set receive gain for voice samples (0-255). }
  ReceiveGainCmd := 'AT+VGR=%d';
  ReceiveGainAnsw := 'OK';
  ReceiveGainMin := 0;
  ReceiveGainMax := 255;
  { +VRA - Ringback Goes Away Timer.
    +VRN - Ringback Never Came Timer. }
  RingbackTimersCmd := 'AT+VRA=%d+VRN=%d';
  RingbackTimersAnsw := 'OK';
  { S39.6=1 - Enable DTMF detection after AT+VLS=2.
    +VDH=x  - Set the threshold for DTMF detection (0-32).
    +VDD=x  - Set DTMF tone duration detection (0-255). }
  DTMFDetectCmd := 'ATS39.6=1||AT+VDH=%d+VDD=%d';
  DTMFDetectAnsw := 'OK||OK|NO CARRIER';
  DTMFSensMin := 0;
  DTMFSensMax := 31;
  DTMFLenUnit := 5;
  { &I2     - Enable hardware Caller ID
    S40.2=1 - Enable Caller ID }
  CallerIDOnCmd := 'AT&I2S40.2=1';
  CallerIDOnAnsw := 'OK';
  CallerIDOffCmd := 'AT&I0S40.2=0';
  CallerIDOffAnsw := 'OK';
  CallerIDFormat := 'NMBR =|CALLER NUMBER:|CALLER''S NUMBER:';
  { S40.3=1 - Enable distincitve ring type 1 (RING).
    S40.4=1 - Enable distincitve ring type 2 (RING 1).
    S40.5=1 - Enable distincitve ring type 3 (RING 2).
    S40.6=1 - Enable distincitve ring type 4 (RING 3). }
  DRingOnCmd := 'ATS40.3=1S40.4=1S40.5=1S40.6=1';
  DRingOnAnsw := 'OK';
  DRingOffCmd := 'ATS40.3=0S40.4=0S40.5=0S40.6=0';
  DRingOffAnsw := 'OK';
  { S39.7=0 - Don't include resynchronization information in recorded voice
              data.
    +VNH=1  - Disable automatic hang-ups to a varying degree in the DATA and
              FAX modes. }
  DevInitVoiceCmd := 'ATS39.7=0||AT+VNH=1';
  DevInitVoiceAnsw := 'OK||OK';
  { +VLS    - Select a voice I/O device.
    +VLS=0  - Disconnect all I/O devices.
    +VLS=2  - Telco line.
    +VLS=8  - External Mic. (Connect a Mic to the dial-up line RJ11 jack).
    +VLS=16 - Internal speaker. }
  Devices.Clear;
  Devices.Add(NO_DEVICE, SNoDevice, 'AT+VLS=0', 'OK', DF_HIDDEN);
  Devices.Add(DIALUP_LINE, SDialupLine, 'AT+VLS=2', 'OK|VCON', DF_TRANSMIT_RECEIVE);
  Devices.Add(EXTERNAL_MICROPHONE, SExternalMicrophone, 'AT+VLS=8', 'OK|VCON', DF_RECEIVE);
  Devices.Add(INTERNAL_SPEAKER, SInternalSpeaker, 'AT+VLS=16', 'OK', DF_TRANSMIT);
  { +VSM    - Select a compression method.
    +VSM=1  - CELP (Code Excited Linear Prediction) 1 bits per sample 9600 bps.
    +VSM=2  - ADPCM 2 bits per sample 9600 Hz (default).
    +VSM=3  - ADPCM 3 bits per sample 9600 Hz.
    +VSM=30 - New ADPCM 3 bits per sample 9600 Hz.
    +VSM=4  - ADPCM 4 bits per sample 9600 Hz. }
  Compressions.Clear;
  Compressions.Add(ZYXEL1496_CELP, SZyXEL1496CELP, 'AT+VSM=1', 'OK', 9600, 1, CF_9600, 0, 0, 0);
  Compressions.Add(ZYXEL1496_ADPCM_2, SZyXEL1496ADPCM2, 'AT+VSM=2', 'OK', 9600, 2, CF_9600 or CF_DEFAULT, 0, 0, 0);
  Compressions.Add(ZYXEL1496_ADPCM_3, SZyXEL1496ADPCM3, 'AT+VSM=3', 'OK', 9600, 3, CF_9600, 0, 0, 0);
  Compressions.Add(ZYXEL1496_ADPCM_30, SZyXEL1496ADPCM30, 'AT+VSM=30', 'OK', 9600, 3, CF_9600, 0, 0, 0);
  Compressions.Add(ZYXEL1496_ADPCM_4, SZyXEL1496ADPCM4, 'AT+VSM=4', 'OK', 9600, 4, CF_9600, 0, 0, 0);
end;

function TZyXEL1496.Init: Integer;
var
  Buffer: string;
begin
  { Check modem specified }
  if not CheckModem('Init', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Detect ROM release }
  if Modem.Command('ATI1', '') <> S_OK then
  begin
    Result := S_FAIL;
    Exit;
  end;
  if Modem.ReadStr(Buffer) = S_FAIL then
  begin
    Result := S_FAIL;
    Exit;
  end;
  if Modem.ReadStr(Buffer) = S_FAIL then
  begin
    Result := S_FAIL;
    Exit;
  end;
  if Modem.Command('', 'OK') <> VMA_USER_1 then
  begin
    Result := S_FAIL;
    Exit;
  end;
  if Length(Buffer) >= 13 then
  begin
    FROMRelease := 0;
    FROMRelease := FROMRelease + (Ord(Buffer[11]) - Ord('0')) * 100;
    FROMRelease := FROMRelease + (Ord(Buffer[13]) - Ord('0')) * 10;
    FROMRelease := FROMRelease + (Ord(Buffer[14]) - Ord('0'));
  end
  else
    FROMRelease := 505;
  DebugMsgFmt(L_MESG, SROMRelease, [ModemRMDName, FROMRelease / 100]);
  { Init complete }
  Result := S_OK;
end;

initialization
  RegisterModemDriver(SZyXEL1496Name, TZyXEL1496);

end.
