{
  Voice Modem Library v1.0

  This file contains the ZyXEL 2864 specific hardware stuff.

  Based on a original source code by Martin Seine <martin@erde.gun.de>,
  that used the 2864 compatible mode of the Elite.

  Copyright (C) 1998-2001 Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_ZyXEL2864;

{$I VML.INC}

interface

uses
  Windows, SysUtils, Classes, VML_Modem, VML_Consts, VML_ZyXEL1496;

const

{ ZyXEL 2864 available compression modes }

  ZYXEL2864_ADPCM_2    = 2;
  ZYXEL2864_ADPCM_3    = 3;
  ZYXEL2864_ADPCM_30   = 30;
  ZYXEL2864_ADPCM_4    = 4;
  ZYXEL2864_DVIADPCM_4 = 40;
  ZYXEL2864_ALAW_PCM   = 80;
  ZYXEL2864_MULAW_PCM  = 81;

{ ZyXEL 2864 modem strings }

  SZyXEL2864Name    = 'ZyXEL 2864';
  SZyXEL2864RMDName = 'ZyXEL 2864';

{$I *.INC}

type

{ TZyXEL2864 }

  TZyXEL2864 = class(TZyXEL1496)
  public
    constructor Create(AOwner: TComponent); override;
    function AnswerPhone: Integer; override;
  end;

implementation

uses
  MMReg;

{ TZyXEL2864 }

constructor TZyXEL2864.Create(AOwner: TComponent);
const
  CF_ZYXEL2864 = CF_7200 or CF_8000 or CF_9600 or CF_11025;
begin
  inherited Create(AOwner);
  { Modem name }
  ModemName := SZyXEL2864Name;
  ModemRMDName := SZyXEL2864RMDName;
  { +VGT=<value> - Set the transmit gain for voice samples (56-200). }
  TransmitGainMin := 56;
  TransmitGainMax := 200;
  { +VGR=<value> - Set receive gain for voice samples (56-200). }
  ReceiveGainMin := 56;
  ReceiveGainMax := 200;
  { +VDD=x,y - Set DTMF tone detection threshold and duration detection. }
  DTMFDetectCmd := 'AT+VDD=%d,%d';
  DTMFDetectAnsw := 'OK';
  DTMFSensMin := 0;
  DTMFSensMax := 15;
  DTMFLenUnit := 5;
  { S48.5=1 - Switch to IS-101 mode. }
  DevInitCmd := 'ATS48.5=1';
  DevInitAnsw := 'OK';
  { +VLS    - Select a voice I/O device.
    +VLS=0  - Disconnect all I/O devices.
    +VLS=1  - Connects to local phone.
    +VLS=2  - Modem connects to telco line (Dial-up line jack).
    +VLS=8  - External Mic. (Connect a Mic to the dial-up line RJ11 jack).
    +VLS=16 - Internal speaker. }
  Devices.Clear;
  Devices.Add(NO_DEVICE, SNoDevice, 'AT+VLS=0', 'OK', DF_HIDDEN);
  Devices.Add(DIALUP_LINE, SDialupLine, 'AT+VLS=2', 'OK|CONNECT*', DF_TRANSMIT_RECEIVE);
  Devices.Add(EXTERNAL_MICROPHONE, SExternalMicrophone, 'AT+VLS=8', 'OK|VCON', DF_RECEIVE);
  Devices.Add(INTERNAL_SPEAKER, SInternalSpeaker, 'AT+VLS=16', 'OK', DF_TRANSMIT);
  Devices.Add(LOCAL_HANDSET, SLocalHandset, 'AT+VLS=1', 'VCON', DF_TRANSMIT_RECEIVE);
  { +VSM      - Select a compression method.
    +VSM=2,s  - 2 bit ADPCM 7200 8000, 9600 or 11025 Hz (default).
    +VSM=3,s  - 3 bit ADPCM 7200 8000, 9600 or 11025 Hz.
    +VSM=30,s - New 3 bit ADPCM with resync capability 7200, 8000, 9600, 11025 Hz.
    +VSM=4,s  - 4 bit ADPCM 7200 8000, 9600 or 11025 Hz.
    +VSM=40,s - 4 bit DVI ADPCM 7200, 8000, 9600 or 11025 Hz.
    +VSM=80,s - 8 bit A-law PCM 7200, 8000, 9600 or 11025 Hz.
    +VSM=81,s - 8 bit Mu-law PCM 7200, 8000, 9600 or 11025 Hz. }
  Compressions.Clear;
  Compressions.Add(ZYXEL2864_ADPCM_2, SZyXEL2864ADPCM2, 'AT+VSM=2,%d', 'OK', 9600, 2, CF_ZYXEL2864 or CF_DEFAULT, 0, 0, 0);
  Compressions.Add(ZYXEL2864_ADPCM_3, SZyXEL2864ADPCM3, 'AT+VSM=3,%d', 'OK', 9600, 3, CF_ZYXEL2864, 0, 0, 0);
  Compressions.Add(ZYXEL2864_ADPCM_30, SZyXEL2864ADPCM30, 'AT+VSM=30,%d', 'OK', 9600, 3, CF_ZYXEL2864, 0, 0, 0);
  Compressions.Add(ZYXEL2864_ADPCM_4, SZyXEL2864ADPCM4, 'AT+VSM=4,%d', 'OK', 9600, 4, CF_ZYXEL2864, 0, 0, 0);
  Compressions.Add(ZYXEL2864_DVIADPCM_4, SZyXEL2864DVIADPCM4, 'AT+VSM=40,%d', 'OK', 9600, 4, CF_ZYXEL2864, WAVE_FORMAT_DVI_ADPCM, 11025, CF_ZYXEL2864 or CF_DEFAULT);
  Compressions.Add(ZYXEL2864_ALAW_PCM, SZyXEL2864ALAWPCM, 'AT+VSM=80,%d', 'OK', 9600, 8, CF_ZYXEL2864, WAVE_FORMAT_ALAW, 11025, CF_ZYXEL2864);
  Compressions.Add(ZYXEL2864_MULAW_PCM, SZyXEL2864MULAWPCM, 'AT+VSM=81,%d', 'OK', 9600, 8, CF_ZYXEL2864, WAVE_FORMAT_MULAW, 11025, CF_ZYXEL2864);
end;

function TZyXEL2864.AnswerPhone: Integer;
begin
  { Check modem specified }
  if not CheckModem('AnswerPhone', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Send command }
  Result := Modem.Command('AT+VLS=2', 'OK|CONNECT*');
  { Check result }
  if Result and VMA_USER <> VMA_USER then
  begin
    Result := VMA_ERROR;
    Exit;
  end;
  if Result = VMA_USER_2 then
  begin
    Result := VMA_CONNECT;
    Exit;
  end;
  Result := VMA_OK;
end;

initialization
  RegisterModemDriver(SZyXEL2864Name, TZyXEL2864);

end.
