unit imaadpcm;
{ Intel/DVI ADPCM coder/decoder.                       }
{ Converts pvf <-> IMA ADPCM                           }
{                                                      }
{ IMA ADPCM coder/decoder                              }
{ Copyright 1992 by Digispeech Inc.                    }
{ AUTHOR Yuhang Wu                                     }
{                                                      }
{ Delphi port                                          }
{ Copyright (C) 2001 Pavel Plesov <mclap@simtts.mv.ru> }
{ Version 0.2.2 alpha                                  }
{                                                      }
{ Remark:                                              }
{ Originaly this module in vgetty dedicated            }
{ to work with Multitech Modems,                       }
{ but original code is tooooo bad :)                   }

{.$Define Shift_Reg}

interface

Uses Classes, VML_Tools;

function pvftoimaadpcm(var fd_in, fd_out: TStream; var header_in: pvf_header): integer;
function imaadpcmtopvf(var fd_in, fd_out: TStream; var header_out: pvf_header): integer;

implementation

type
   adpcm_state = record
      valprev: SmallInt;    // Previous output value
      index: byte;          // Index into stepsize table

   end;

  wptr = ^SmallInt;

  int16 = SmallInt;
  int8 = ShortInt;
  pint16 = ^int16;
  pint8 = ^int8;
  pbyte = ^byte;

// Intel ADPCM step variation table
const

{  indexTable: array[0..15] of integer = (
    -1, -1, -1, -1, 2, 4, 6, 8,
    -1, -1, -1, -1, 2, 4, 6, 8
  );

  stepsizeTable: array [0..88] of integer  = (
    7,     8,     9,     10,    11,    12,    13,    14,    16,    17,
    19,    21,    23,    25,    28,    31,    34,    37,    41,    45,
    50,    55,    60,    66,    73,    80,    88,    97,    107,   118,
    130,   143,   157,   173,   190,   209,   230,   253,   279,   307,
    337,   371,   408,   449,   494,   544,   598,   658,   724,   796,
    876,   963,   1060,  1166,  1282,  1411,  1552,  1707,  1878,  2066,
    2272,  2499,  2749,  3024,  3327,  3660,  4026,  4428,  4871,  5358,
    5894,  6484,  7132,  7845,  8630,  9493,  10442, 11487, 12635, 13899,
    15289, 16818, 18500, 20350, 22385, 24623, 27086, 29794, 32767
  );
}

  // ADPCM stepsize table
 stepsizeTable: array [0..136] of SmallInt  = (
    16,    17,    18,    19,    20,    22,    23,    24,    25,    27,
    28,    30,    32,    33,    35,    37,    39,    41,    44,    46,
    49,    51,    54,    57,    61,    64,    67,    71,    75,    79,
    84,    88,    93,    98,   104,   109,   116,   122,   129,   136,
   143,   151,   159,   168,   178,   187,   198,   209,   220,   232,
   245,   258,   273,   288,   304,   320,   338,   357,   376,   397,
   419,   442,   466,   492,   519,   548,   578,   609,   643,   678,
   716,   755,   797,   841,   887,   936,   987,  1041,  1099,  1159,
  1223,  1290,  1361,  1436,  1515,  1598,  1686,  1779,  1877,  1980,
  2089,  2204,  2325,  2453,  2588,  2731,  2881,  3039,  3207,  3383,
  3569,  3765,  3973,  4191,  4422,  4665,  4921,  5192,  5478,  5779,
  6097,  6432,  6786,  7159,  7553,  7968,  8407,  8869,  9357,  9872,
 10415, 10987, 11592, 12229, 12902, 13612, 14360, 15150, 15983, 16863,
 17790, 18769, 19801, 20890, 22039, 23251, 24530
);

// Table of stepsize index
indexTable: array[0..7] of ShortInt = ( -2, -2, -2, -2, 3, 9, 13, 16 );

const BL = 16380;

procedure S16( ar: pint16; idx, value: int16);
begin
   inc(ar,idx);
   ar^:=value;
end;

function A8( ar: pbyte; idx: int16): byte;
begin
   inc(ar,idx);
   Result:=ar^;
end;

procedure fix(var v: int16);
begin
   if v < -16384 then
      v := -16384
   else
   if v >  16383 then
      v := 16383;
end;


//  encode the differential value and output an ADPCM codeword
//  return value:  char;  ADPCM codeword
//  paramenrs:
//     short Dn     :     input;  the differential value;
//     short Delta_N:     input;  the stepsize;

function encode_c( Delta_N, Dn: int16 ): int8;
var c: int8;
begin
  c := 0;
  if Dn < 0 then begin
     c := 8; Dn := -Dn;
  end;
  if Dn >= Delta_N then begin
     inc(c,4); dec(Dn,Delta_N);
  end;
  if Dn >= Delta_N shr 1 then begin
     inc(c,2); dec(Dn,Delta_N shr 1);
  end;
  if Dn >= (Delta_N shr 2) then
     inc(c);

  Result:=c;
end;

// calculate the differential value from an ADPCM codeword
// function return value:  short;  reproduced differential vale
//   char  c      :     input;  the ADPCM codeword;
//   short Delta_N:     input;  the stepsize;

function decode_c( Delta_N: int16; c: int8 ): int16;
var b: int16;
begin
   b := Delta_N shr 3;
   if (c and 4) <> 0 then
      inc(b, Delta_N);
   if (c and 2) <> 0 then
      inc(b, Delta_N shr 1);
   if (c and 1) <> 0 then
      inc(b, Delta_N shr 2);

   if (c and 8) <> 0 then
      Result:=-b
   else
      Result:=b;
end;

function pvftoimaadpcm(var fd_in, fd_out: TStream; var header_in: pvf_header): integer;
{var
   i: integer;
begin
{   while not feof(fd_in) do begin
      i:=0;
      while not feof(fd_in) and (i < SAMPLES) do begin
         sbuffer[i] := header_in.read_pvf_data(header_in.nbits,fd_in) shr 8;
         inc(i);
      end;

      adpcm_coder (@sbuffer, @cbuffer, i, state);

      if fd_out.Write(cbuffer, i div 2) <> i div 2 then begin
         Result:=ERROR;
         Exit
      end
   end;
}
var
// variables
  xn1: int16;         // signal estimate
  Delta_N: int16;     // stepsize
  stepindex: int16;   // index for stepsize table
  codeword: int8;    // ADPCM codeword
  i, j, k: int16;     // loop index
  xni: array [0..4] of int16;      // delayed speech samples
  RL: int16;          // sample index
  xn: int16;
  Dn: int16;
  c: byte;
  xv: pint16;          // input buffer
  xo: pint8;           // output buffer

begin
// initial conditions
  xn1 := 0;
  Delta_N := 16;
  stepindex := 0;
  //xni[0] := xni[1] = xni[2] = xni[3] = xni[4] = 0;
  FillChar(xni,sizeof(xni),0);

  // memory allocation for input output buffer
  GetMem(xv,BL*sizeof(int16));
  GetMem(xo,BL);

  // loop until end of file
  while true do begin
     //RL := fd_in.read(xv^, BL * 2);
     RL:=0;
     while not feof(fd_in) and (RL < BL) do begin
        pint16(integer(xv)+RL*2)^ := header_in.read_pvf_data(header_in.nbits,fd_in) shr 8;
        inc(RL);
     end;

     if RL=0 then
        break;

     k := 0;
     for j:=0 to RL-1 do begin
        xn := pint16(integer(xv)+j*2)^ shr 1;

        // calculate a prediction of input sample
        {$IfDef Shift_Reg}
        if header_in.speed < 16000 then begin
           xn1 := xni[1] + (xni[1] shr 3 ) + (xni[1] shr 4);
           dec(xn1,( xni[2] shr 3 ) + ( xni[2] shr 6 ) );
           dec(xn1,( xni[3] shr 1 ) - ( xni[3] shr 6 ) );
           inc(xn1, xni[4] shr 2 );

           fix(xn1);
        end;
        {$EndIf}

        // differential value calculation
        Dn := xn - xn1;

        // encode Dn relative to the current stepsize
        codeword := encode_c( Delta_N, Dn );

        // write ADPCM sample to output buffer
        if j mod 2 = 0 then
           c := $00F0 and ( codeword shl 4 )
        else begin
           inc(c,$000F and codeword);
           pint8(integer(xo)+k)^:=c;
           inc(k);
        end;

        // decode ADPCM code value to reproduce Dn
        // and accumulates an estimated xn

        inc(xn1,decode_c(Delta_N, codeword));
        fix(xn1);

        // shift prediction register
        {$IfDef Shift_Reg}
        if header_in.speed < 16000 then begin
           for i:=1 to 3 do
               xni[i+1] := xni[i];
           xni[1] := xn1;
        end;
        {$EndIf}

        // stepsize adaptation */
        inc(stepindex,indexTable[ codeword and 7 ]);
        if stepindex < 0 then
           stepindex := 0
        else
        if stepindex > 136 then
           stepindex := 136;
        Delta_N := stepsizeTable[stepindex];
      end;
      // write ADPCM samples from output buffer to output file
      //fwrite(xo, 1, k, file2);
      fd_out.Write(xo^, k);
  end;

  FreeMem(xv);
  FreeMem(xo);

  Result:=OK;
end;

function imaadpcmtopvf(var fd_in, fd_out: TStream; var header_out: pvf_header): integer;
var
// variables
  xo: pint16;         // output buffer
  xn1: int16;         // signal estimate
  Delta_N: int16;     // stepsize
  stepindex: int16;   // index for stepsize table
  codeword: int8;    // ADPCM codeword
  i, j, k: int16;     // loop index
  xni: array [0..4] of int16;      // delayed speech samples
  RL: int16;          // sample index
  xn: pbyte; // input buffer

begin
// initial conditions
  xn1 := 0;
  Delta_N := 16;
  stepindex := 0;
  //xni[0] := xni[1] = xni[2] = xni[3] = xni[4] = 0;
  FillChar(xni,sizeof(xni),0);

  // memory allocation for input output buffer
  GetMem(xn,BL);
  GetMem(xo,BL*sizeof(int16));

  // loop until end of file
  while true do begin
     RL := fd_in.read(xn^, BL div 2);

     if RL=0 then break;

     for j:=0 to RL-1 do begin
         for k:=0 to 1 do begin
             // extract ADPCM codeword from input buffer
             if k=0 then
                //codeword := $0F and (xn[j] shr 4);
                codeword := $0F and (A8(xn,j) shr 4)
             else
                codeword := $0F and A8(xn,j);

             //  calculate a prediction of the speech sample

             {$IfDef Shift_Reg}
             if header_out.speed < 16000 then begin
                xn1 := xni[1] + (xni[1] shr 3 ) + (xni[1] shr 4);
                dec(xn1,( xni[2] shr 3 ) + ( xni[2] shr 6 ) );
                dec(xn1,( xni[3] shr 1 ) - ( xni[3] shr 6 ) );
                inc(xn1, xni[4] shr 2 );

                fix(xn1);
             end;
             {$EndIf}

             // decode ADPCM code value to reproduce Dn
             // and accumulates an estimated xn

             inc(xn1,decode_c(Delta_N, codeword));

             fix(xn1);

             // shift prediction register
             {$IfDef Shift_Reg}
             if header_out.speed < 16000 then begin
                for i:=1 to 3 do
                    xni[i+1] := xni[i];
                xni[1] := xn1;
             end;
             {$EndIf}

             // stepsize adaptation
             inc(stepindex, indexTable[ codeword and 7 ] );

             if stepindex < 0 then
                stepindex := 0
             else
             if stepindex > 136 then
                stepindex := 136;

             Delta_N := stepsizeTable[stepindex];

             // write reproduced sample to output buffer
             if k=0 then
                S16(xo,2*j,xn1 shl 1)
             else
                S16(xo,2*j+1,xn1 shl 1);
         end;

     end;
     // write reproduced samples from output buffer to output file
     //fd_out.write(xo^, 2*RL*sizeof(int16));
     for j := 0 to 2*RL-1 do
         header_out.write_pvf_data(header_out.nbits, fd_out, integer(pint16(integer(xo)+j*2)^) shl 8);
  end;
  FreeMem(xn);
  FreeMem(xo);

  Result:=OK;
end;

end.
