{
  Voice Modem Library v1.0

  This file contains definition of voice modem library classes:
    - modem data reader/writer classes
    - modem profile class
    - modem port class
    - modem log file class
    - abstract hadrware driver class
    - voice modem class

  Copyright (C) 1998-2001 Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_Modem;
                                 
{$I VML.INC}

interface

uses
  Windows, Messages, SysUtils, Classes, CommPort, VML_Consts;

{$I *.INC}

type

{ Forward class declarations }

  TModemDriver = class;
  TVoiceModem = class;

{ Abstract modem reader class }

  TModemReader = class(TObject)
  public
    function ReadBool(const Section, Ident: string; Default: Boolean): Boolean; virtual;
    function ReadDriver(Instance: TModemDriver): TModemDriver; virtual;
    function ReadInteger(const Section, Ident: string; Default: Longint): Longint; virtual;
    function ReadMapStr(const Section, Ident, Default: string): string; virtual; 
    procedure ReadSection(const Section: string; Strings: TStrings); virtual; abstract;
    procedure ReadSectionValues(const Section: string; Strings: TStrings); virtual;
    function ReadString(const Section, Ident, Default: string): string; virtual; abstract;
  end;

{ Abstract modem writer class }

  TModemWriter = class(TObject)
  public
    procedure DeleteKey(const Section, Ident: String); virtual; abstract;
    procedure EraseSection(const Section: string); virtual; abstract;
    procedure WriteBool(const Section, Ident: string; Value: Boolean); virtual;
    procedure WriteDriver(Driver: TModemDriver); virtual;
    procedure WriteInteger(const Section, Ident: string; Value: Longint); virtual;
    procedure WriteMapStr(const Section, Ident, Value: string); virtual; 
    procedure WriteString(const Section, Ident, Value: String); virtual; abstract;
  end;

{
  Modem profile.

  Properties:

  Port                - Modem port number. The default value is 2 (i.e. COM2).
  PortBaudRate        - Baud rate at which the communications device operates.
                        The bps rate must be high enough for the compression
                        mode used. The default value is 38400. See CBR_XXX
                        constants list (Windows.pas) for more available
                        values.
  PortDataBits        - Number of bits in the bytes transmitted and received.
                        Default value is 8.
  PortParity          - Port parity scheme to be used. Default value is
                        NOPARITY. See XXXXPARITY constants list (Windows.pas)
                        for more available values.
  PortStopBits        - The number of stop bits to be used. Default value is
                        ONESTOPBIT. See XXXSTOPBIT constants list (Windows.pas)
                        for more available values.
  PortTimeout         - Default port timeout in seconds for a read or write
                        modem operation. The default value is 10 seconds.
  PortStatusDelay     - Time in msec for the delay, when checking modem
                        status. A higher value will decrease machine load by
                        increasing reaction time. The default is 50 msec.
  PortReadDelay       - Time in msec for the delay, when no new data are
                        received from the modem. The default is 75 msec.
  PortWriteDelay      - Time in msec for the delay, when no new data are
                        sended to the modem. The default is 75 msec.
  PortInBuffSize      - Port input buffer size in bytes. The default value
                        is 4096 bytes.
  PortOutBuffSize     - Port output buffer size in bytes. The default value
                        is 4096 bytes.
  ModemName           - This is default modem name for selecting modem driver
                        from the list of registered drivers. The is no default
                        value for this property.
  ModemIniFile        - This is driver INI file name for automatically creating
                        driver instance. The is no default value for this
                        property.
  ModemInitCmd        - Additional modem initialization command. Executed after
                        performing all standard and driver specific commands.
                        WARNING: Do nothing to change local echo and result
                        codes settings.
  ModemInitAnsw       - Modem answer for additional modem initialization
                        command. 
  ModemLogging        - Enable or disable logging (or tracing) of commands of
                        modem. The default value is False.
  PlayMaxLen          - Maximum playing length in seconds. There is no limit
                        by default.
  PlayDevice          - Voice playing device. The default is EXTERNAL_SPEAKER.
  PlayFileName        - Default outgoing voice message file name.
  PlayPreview         - Play preview mode. This feature is turned off by default.
  PlayPreviewLen      - Maximum playing length for preview mode in seconds. Some
                        as PlayMaxLen but turned on by PlayPreview value. Default
                        is 10 seconds.
  RecCompression      - This is the default compression mode for recording
                        voice data. The mode 0 is a special mode, that will
                        automatically choose a same default value for every
                        modem. The default is 0.
  RecSpeed            - This is the default voice data recording speed. It
                        is the number of samples per second. The speed 0 is
                        a special speed, that will automatically choose a
                        sane default value for every modem. The default is 0.
  RecSilenceLen       - Silence detection length in 0.1 seconds. If the modem
                        detects silence for this time, it sends a silence
                        detect to the host. Default is 2 seconds (20 * 0.1
                        seconds).
  RecSilenceSens      - Silence detection threshold in percent (0% to 100%).
                        Increase this value if you have a noisy phone line
                        and the silence detection doesn't work reliably. The
                        default is 40%.
  RecRemoveSilence    - If RecRemoveSilence is enabled, the trailing silence
                        of an incoming voice message as detected by the modem
                        will be deleted. This might cause you to miss parts
                        of a message if the silence threshold is high and the
                        caller is talking very quietly. To be on the safe
                        side, don't define this. This feature is turned off
                        by default.
  RecIgnoreSilence    - If RecIgnoreSilence is enabled, the trailing silence
                        of an incoming voice message as detected by the modem
                        will be ignored. Otherwise the recording will be
                        stopped. This feature is turned off by default.
  RecDevice           - Voice recording device. The default is
                        EXTERNAL_MICROPHONE.
  RecMaxLen           - Maximum recording length in seconds. Hang up if
                        somebody talks longer than this. Default is 5 minutes
                        (300 seconds).
  RecWaveFormat       - This is the default WAVE format for recording WAV voice
                        data. The mode 0 is a special mode, that will
                        automatically choose a same default value for every
                        modem. The default is 0.
  RecWaveSpeed        - This is the default WAVE voice data recording speed. It
                        is the number of samples per second. The speed 0 is
                        a special speed, that will automatically choose a
                        sane default value for every modem. The default is 0.
  TransmitGain        - Some modems support setting the transmit gain (speaker
                        volume in speaker phone mode). This value can be set in
                        percent (0% to 100%). 0% is off, 100% is maximum. To use
                        the modem default value set this to -1. The default is -1.
  ReceiveGain         - Some modems support setting the receive gain (sens of the
                        microphone in speaker phone mode). This value can be set
                        in percent (0% to 100%). 0% is off, 100% is maximum.
                        To use the modem default value set this to -1. The
                        default is -1.
  DTMFLen             - Minimum length of detected DTMF tones, in milliseconds.
                        This is currently only supported by ZyXel modems with a
                        ROM release of 6.12 or above. The default is 30
                        milliseconds.
  DTMFSens            - DTMF tone detection threshold in percent (0% to 100%).
                        Increase this if the modem erroneously detects DTMF tones,
                        decrease it if it fails to detect real ones. This is
                        currently only supported by ZyXel modems with a ROM
                        release of 6.12 or above. The default is 40%.
  DTMFWait            - Time to wait for a DTMF tone to arrive when recording or
                        waiting for DTMF input in seconds. The default is to wait
                        for 7 seconds.
  BeepFrequency       - Frequency for the beep command in Hz. The default is 933Hz.
  BeepLength          - Length for the beep command in 0.001 seconds.
                        The default is 1.5 seconds (1500 * 0.001 seconds).
  CommandDelay        - Delay before sending a new voice command to the modem
                        in milliseconds. The default is 100 milliseconds.
  CommandEcho         - Usually command echo from the modem should be enabled.
                        Since some modems sometimes forget this echo, it is
                        disabled by default. Turning this option off makes
                        things more reliable, but bugs are much harder to
                        trace. The default is to disable command echo.
                        WARNING: CommandEcho will value automatic resets when
                        modem does not return expected command echo and will
                        sets when modem received unexpected command echo.
  DialTimeout         - Default timeout for a dialout in seconds. The default
                        value is 90 seconds.
  DialMode            - Default dial mode (pulse or tone). The default value is
                        P (pulse).
  AnswerRecognition   - Some modems can not correct to decide that a dialout
                        call has been answered. For this modems may be used
                        software recognition of voice answer of abonent. Set
                        this value TRUE to enable software answer detection.
                        The default is FALSE (hardware detection of answer).
                        NOTE: This feature not implemented in version 1.0.
  AnswerFMax          - Reserved.
  AnswerFToneMin      - Reserved.
  AnswerFToneMax      - Reserved.
  AnswerToneLevel     - Reserved.
  AnswerEnergyLevel   - Reserved.
  AnswerVoiceRate     - Reserved.
  AnswerNoiseRate     - Reserved.
  AnswerVoiceRequired - Reserved.
  RingbackGoesAway    - Timeout for deciding that a dialout call has been
                        answered. If more than this many 1/10ths of a second
                        have passed since the last ringback, the modem assumes
                        the call has been answered. Default is 45 (4.5 sec).
  RingbackNeverCame   - Timeout for deciding that a dialout call has been
                        answered. If more than this many 1/10ths of a second
                        have passed without any ringback the modem assumes the
                        call has been answered. Default is 100 (10 sec).
  RawData             - Output recorded voice samples without header and expect
                        raw voice data on input for playback.
                        This feature is turned off by default.
  VoiceBaudRate       - Specific DTE/modem baud rate. If this value is nonzero
                        (no autobaud selected), the modem supports a full
                        duplex DTE interface. This means that the DTE can
                        enter commands at any time, even if the modem is in the
                        process of sending a shielded code indicating DTMF
                        detection to the DTE. The default value is 0 (enable
                        autobaud detection).
  VoiceDataFormat     - Voice recording data format and default playing data
                        format. The value may be one of DATA_FORMAT_XXXX
                        constants (see VML_Consts.pas for constants definition).
                        The default value is DATA_FORMAT_RMD (Raw Modem Data).
  InactivityTimeout   - DTE/DCE inactivity timer. This is the period of time
                        (in secsonds), which if expired causes the modem to hang
                        up the telephone line if it is off-hook and no data has
                        passed during the period.
  CallerIDDetection   - Some modems supports caller identification. Set this
                        value TRUE to enable Caller ID feature. The default is
                        FALSE (caller identification is disabled).
  DistinctiveRing     - Some voice modems offer distinctive ring support. Set
                        this value TRUE to enable this feature. The default is
                        FALSE (disabled).
 }

  TModemProfile = class(TComponent)
  private
    FPort: Integer;
    FPortBaudRate: Integer;
    FPortDataBits: Integer;
    FPortParity: Integer;
    FPortStopBits: Integer;
    FPortTimeout: Integer;
    FPortStatusDelay: Integer;
    FPortReadDelay: Integer;
    FPortWriteDelay: Integer;
    FPortInBuffSize: Integer;
    FPortOutBuffSize: Integer;
    FModemName: string;
    FModemIniFile: string;
    FModemInitCmd: string;
    FModemInitAnsw: string;
    FModemLogging: Boolean;
    FPlayMaxLen: Integer;
    FPlayDevice: Integer;
    FPlayFileName: string;
    FPlayPreview: Boolean;
    FPlayPreviewLen: Integer;
    FRecCompression: Integer;
    FRecSpeed: Integer;
    FRecSilenceLen: Integer;
    FRecSilenceSens: Integer;
    FRecRemoveSilence: Boolean;
    FRecIgnoreSilence: Boolean;
    FRecDevice: Integer;
    FRecMaxLen: Integer;
    FRecWaveFormat: Integer;
    FRecWaveSpeed: Integer;
    FTransmitGain: Integer;
    FReceiveGain: Integer;
    FDTMFLen: Integer;
    FDTMFSens: Integer;
    FDTMFWait: Integer;
    FBeepFrequency: Integer;
    FBeepLength: Integer;
    FCommandDelay: Integer;
    FCommandEcho: Boolean;
    FDialTimeout: Integer;
    FDialMode: string;
    FAnswerRecognition: Boolean;
    FAnswerFMax: Longint;
    FAnswerFToneMin: Integer;
    FAnswerFToneMax: Integer;
    FAnswerToneLevel: Longint;
    FAnswerEnergyLevel: Longint;
    FAnswerVoiceRate: Integer;
    FAnswerNoiseRate: Integer;
    FAnswerVoiceRequired: Boolean;
    FRingbackGoesAway: Integer;
    FRingbackNeverCame: Integer;
    FRawData: Boolean;
    FVoiceBaudRate: Integer;
    FVoiceDataFormat: Integer;
    FInactivityTimeout: Integer;
    FCallerIDDetection: Boolean;
    FDistinctiveRing: Boolean;
    function IsModemInitAnswStored: Boolean;
  protected
    procedure AssignTo(Dest: TPersistent); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure ReadData(Reader: TModemReader); virtual;
    procedure WriteData(Writer: TModemWriter); virtual;
  published
    property Port: Integer read FPort write FPort default 2;
    property PortBaudRate: Integer read FPortBaudRate write FPortBaudRate default 38400;
    property PortParity: Integer read FPortParity write FPortParity default NOPARITY;
    property PortDataBits: Integer read FPortDataBits write FPortDataBits default 8;
    property PortStopBits: Integer read FPortStopBits write FPortStopBits default ONESTOPBIT;
    property PortTimeout: Integer read FPortTimeout write FPortTimeout default 10;
    property PortStatusDelay: Integer read FPortStatusDelay write FPortStatusDelay default 50;
    property PortReadDelay: Integer read FPortReadDelay write FPortReadDelay default 75;
    property PortWriteDelay: Integer read FPortWriteDelay write FPortWriteDelay default 75;
    property PortInBuffSize: Integer read FPortInBuffSize write FPortInBuffSize default 4096;
    property PortOutBuffSize: Integer read FPortOutBuffSize write FPortOutBuffSize default 4096;
    property ModemName: string read FModemName write FModemName;
    property ModemIniFile: string read FModemIniFile write FModemIniFile;
    property ModemInitCmd: string read FModemInitCmd write FModemInitCmd;
    property ModemInitAnsw: string read FModemInitAnsw write FModemInitAnsw stored IsModemInitAnswStored;
    property ModemLogging: Boolean read FModemLogging write FModemLogging default False;
    property PlayDevice: Integer read FPlayDevice write FPlayDevice default EXTERNAL_SPEAKER;
    property PlayMaxLen: Integer read FPlayMaxLen write FPlayMaxLen default 0;
    property PlayFileName: string read FPlayFileName write FPlayFileName;
    property PlayPreview: Boolean read FPlayPreview write FPlayPreview default False;
    property PlayPreviewLen: Integer read FPlayPreviewLen write FPlayPreviewLen default 10;
    property RecCompression: Integer read FRecCompression write FRecCompression default 0;
    property RecSpeed: Integer read FRecSpeed write FRecSpeed default 0;
    property RecSilenceLen: Integer read FRecSilenceLen write FRecSilenceLen default 20;
    property RecSilenceSens: Integer read FRecSilenceSens write FRecSilenceSens default 40;
    property RecRemoveSilence: Boolean read FRecRemoveSilence write FRecRemoveSilence default False;
    property RecIgnoreSilence: Boolean read FRecIgnoreSilence write FRecIgnoreSilence default False;
    property RecDevice: Integer read FRecDevice write FRecDevice default EXTERNAL_MICROPHONE;
    property RecMaxLen: Integer read FRecMaxLen write FRecMaxLen default 300;
    property RecWaveFormat: Integer read FRecWaveFormat write FRecWaveFormat default 0;
    property RecWaveSpeed: Integer read FRecWaveSpeed write FRecWaveSpeed default 0;
    property TransmitGain: Integer read FTransmitGain write FTransmitGain default -1;
    property ReceiveGain: Integer read FReceiveGain write FReceiveGain default -1;
    property DTMFLen: Integer read FDTMFLen write FDTMFLen default 30;
    property DTMFSens: Integer read FDTMFSens write FDTMFSens default 40;
    property DTMFWait: Integer read FDTMFWait write FDTMFWait default 7;
    property BeepFrequency: Integer read FBeepFrequency write FBeepFrequency default 933;
    property BeepLength: Integer read FBeepLength write FBeepLength default 1500;
    property CommandDelay: Integer read FCommandDelay write FCommandDelay default 100;
    property CommandEcho: Boolean read FCommandEcho write FCommandEcho default False;
    property DialTimeout: Integer read FDialTimeout write FDialTimeout default 90;
    property DialMode: string read FDialMode write FDialMode;
    property AnswerRecognition: Boolean read FAnswerRecognition write FAnswerRecognition default False;
    property AnswerFMax: Longint read FAnswerFMax write FAnswerFMax default 4000;
    property AnswerFToneMin: Integer read FAnswerFToneMin write FAnswerFToneMin default 390;
    property AnswerFToneMax: Integer read FAnswerFToneMax write FAnswerFToneMax default 460;
    property AnswerToneLevel: Longint read FAnswerToneLevel write FAnswerToneLevel default 2000000;
    property AnswerEnergyLevel: Longint read FAnswerEnergyLevel write FAnswerEnergyLevel default 2000;
    property AnswerVoiceRate: Integer read FAnswerVoiceRate write FAnswerVoiceRate default 4;
    property AnswerNoiseRate: Integer read FAnswerNoiseRate write FAnswerNoiseRate default 20;
    property AnswerVoiceRequired: Boolean read FAnswerVoiceRequired write FAnswerVoiceRequired default True;
    property RingbackGoesAway: Integer read FRingbackGoesAway write FRingbackGoesAway default 45;
    property RingbackNeverCame: Integer read FRingbackNeverCame write FRingbackNeverCame default 120;
    property RawData: Boolean read FRawData write FRawData default False;
    property VoiceBaudRate: Integer read FVoiceBaudRate write FVoiceBaudRate default 0;
    property VoiceDataFormat: Integer read FVoiceDataFormat write FVoiceDataFormat default DATA_FORMAT_RMD;
    property InactivityTimeout: Integer read FInactivityTimeout write FInactivityTimeout default 60;
    property CallerIDDetection: Boolean read FCallerIDDetection write FCallerIDDetection default False;
    property DistinctiveRing: Boolean read FDistinctiveRing write FDistinctiveRing default False;
  end;

{ TModemPort }

  {
    Modem port class

    Note: ModemPort component used CTS signal for transmit data from
    modem. Always set hardware flow control when modem initialized.

    Properties:

    Profile           - Modem profile. For properly working a port this
                        property must be always assigned.

    Events:

    OnDebugMsg        - Debug message event.
  }

  TModemPort = class(TCustomCommPort)
  private
    FProfile: TModemProfile;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    function Close: Integer;
    function Open: Integer;
    function Read(Buffer: Pointer; Count: Integer): Integer;
    function Write(Buffer: Pointer; Count: Integer): Integer;
  published
    property Profile: TModemProfile read FProfile write FProfile;
  end;

{ TModemLogFile }

  {
    Simple log file. Opens exists log (text) file or creates new file, move
    to the file end and write log title. Then ready to add log lines.

    Procedures:

    Add            - Add line to log file.

    Properties:

    AddDateTime    - Add date/time string to the each log line except log
                     title line.
    CreateNew      - Create new log file, erase old file.
    DateTimeFormat - Date-and-time value format string (see FormatDateTime
                     function help for more details).
    FileName       - Log file name.
    Title          - Log title.

    Events:

    OnGetTitle     - Get log title. This event calls every time right after
                     log file opening.
  }

  TGetLogTitleEvent = procedure(Sender: TObject; var Value: string) of object;

  TModemLogFile = class(TComponent)
  private
    FFileName: string;
    FTitle: string;
    FAddDateTime: Boolean;
    FDateTimeFormat: string;
    FLogFile: TFileStream;
    FCreateNew: Boolean;
    FOnGetTitle: TGetLogTitleEvent;
    function IsDateTimeformatStored: Boolean;
    procedure SetFileName(const Value: string);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Add(const S: string): Boolean;
    function Open: Boolean;
    function Close: Boolean;
  published
    property AddDateTime: Boolean read FAddDateTime write FAddDateTime default False;
    property CretateNew: Boolean read FCreateNew write FCreateNew default False;
    property DateTimeFormat: string read FDateTimeFormat write FDateTimeFormat stored IsDateTimeformatStored;
    property FileName: string read FFileName write SetFileName;
    property Title: string read FTitle write FTitle;
    property OnGetTitle: TGetLogTitleEvent read FOnGetTitle write FOnGetTitle;
  end;

{ Modem event structure for information exchange between the hardware
  specific parts and the generic code. }

  TBeepData = packed record
    Frequency: Integer;
    Length: Integer;
  end;

  TEventData = packed record
    case Integer of
      0: (C: Char);
      1: (I: Integer);
      2: (P: Pointer);
      3: (Beep: TBeepData);
  end;

  TModemEvent = class(TObject)
  private
    FEvent: Integer;
    FData: TEventData;
  public
    constructor Create(AEvent: Integer);
    constructor CreateDataB(AEvent: Integer; Frequency, Length: Integer);
    constructor CreateDataC(AEvent: Integer; C: Char);
    constructor CreateDataI(AEvent: Integer; I: Integer);
    constructor CreateDataP(AEvent: Integer; P: Pointer);
    property Event: Integer read FEvent write FEvent;
    property Data: TEventData read FData write FData;
  end;

{ Modem event queue }

  PQueueItem = ^TQueueItem;
  TQueueItem = packed record
    Position: Integer;
    Next: PQueueItem;
    Event: TModemEvent;
  end;

  TEventQueue = class(TObject)
  private
    FLockSection: TRTLCriticalSection;
    FFirstItem: PQueueItem;
    FLastItem: PQueueItem;
  public
    constructor Create;
    destructor Destroy; override;
    function Queue(Event: TModemEvent): Integer;
    function UnQueue(var Event: TModemEvent): Integer;
  end;

{ TModemResponse }

  {
    Modem response item.

    Properties:

    Response   - Response string.
    AnswerCode - Corresponded modem answer code (one of VMA_XXXX consts).
  }

  TModemResponse = class(TCollectionItem)
  private
    FResponse: string;
    FAnswerCode: Integer;
  protected
    function GetDisplayName: string; {$IFNDEF VER90} override; {$ENDIF}
  public
    constructor Create(Collection: TCollection); override;
    procedure AssignTo(Dest: TPersistent); override;
  published
    property Response: string read FResponse write FResponse;
    property AnswerCode: Integer read FAnswerCode write FAnswerCode default VMA_IGNORE;
  end;

{ TModemResponses }

  {
    List of possible modem responses.

    Methods:

    Add    - Add new or replace existing response to the list.
    Delete - Remove given response from the list.
    Find   - Find response list item.

    Properties:

    Text   - Responses string. Each response separated by '|' char. 
  }

  TModemResponses = class(TCollection)
  private
    FDriver: TModemDriver;
    function GetItem(Index: Integer): TModemResponse;
    function GetText: string;
    procedure SetItem(Index: Integer; Value: TModemResponse);
  protected
    function GetOwner: TPersistent; {$IFNDEF VER90} override; {$ENDIF}
  public
    constructor Create(ADriver: TModemDriver);
    function Add(const Response: string; AnswerCode: Integer): TModemResponse;
    procedure Delete(const Response: string);
    function Find(const Response: string; var Item: TModemResponse): Boolean;
    procedure ReadData(Reader: TModemReader);
    procedure WriteData(Writer: TModemWriter);
    property Items[Index: Integer]: TModemResponse read GetItem write SetItem; default;
    property Text: string read GetText;
  end;

{ TModemDLECode }

  {
    Shielded <DLE> code item.

    Properties:

    Code      - Shielded <DLE> code char.
    Event     - Corresponded modem event (see "Possible event messages from
                the voice modem" constants )
    EventCode - Modem event code char. 
    DebugMsg  - If this string is not empty the debug message will be
                generated when code received.
  }

  TModemDLECode = class(TCollectionItem)
  private
    FCode: Char;
    FEvent: Integer;
    FEventCode: Char;
    FDebugMsg: string;
  protected
    function GetDisplayName: string; {$IFNDEF VER90} override; {$ENDIF}
  public
    constructor Create(Collection: TCollection); override;
    procedure AssignTo(Dest: TPersistent); override;
  published
    property Code: Char read FCode write FCode default #0;
    property Event: Integer read FEvent write FEvent default NO_EVENT;
    property EventCode: Char read FEventCode write FEventCode default #0;
    property DebugMsg: string read FDebugMsg write FDebugMsg;
  end;

{ TModemDLECodes }

  {
    List of shielded <DLE> codes.

    Methods:

    Add    - Add new or replace existing code to the list.
    Delete - Remove given code from the list.
    Find   - Find code list item.
  }

  TModemDLECodes = class(TCollection)
  private
    FDriver: TModemDriver;
    function GetItem(Index: Integer): TModemDLECode;
    procedure SetItem(Index: Integer; Value: TModemDLECode);
  protected
    function GetOwner: TPersistent; {$IFNDEF VER90} override; {$ENDIF}
  public
    constructor Create(ADriver: TModemDriver);
    function Add(Code: Char; Event: Integer; EventCode: Char; const DebugMsg: string): TModemDLECode;
    procedure Delete(Code: Char);
    function Find(Code: Char; var Item: TModemDLECode): Boolean;
    procedure ReadData(Reader: TModemReader);
    procedure WriteData(Writer: TModemWriter);
    property Items[Index: Integer]: TModemDLECode read GetItem write SetItem; default;
  end;

{ TModemDevice }

  {
    Modem device item.

    Properties:

    Device  - Device code (see "Possible input or output devices" constants )
    Name    - Device name.
    Command - Device setup command.
    Answer  - Modem answer for setup command.
    Flags   - Device flags (see DF_XXXX constants).
  }

  TModemDevice = class(TCollectionItem)
  private
    FDevice: Integer;
    FName: string;
    FCommand: string;
    FAnswer: string;
    FFlags: Integer;
  protected
    function GetDisplayName: string; {$IFNDEF VER90} override; {$ENDIF}
  public
    procedure AssignTo(Dest: TPersistent); override;
  published
    property Device: Integer read FDevice write FDevice default 0;
    property Name: string read FName write FName;
    property Command: string read FCommand write FCommand;
    property Answer: string read FAnswer write FAnswer;
    property Flags: Integer read FFlags write FFlags default 0;
  end;

{ TModemDevices }

  {
    List of modem devices.

    Methods:

    Add    - Add new or replace existing device to the list.
    Delete - Remove given device from the list.
    Find   - Find device list item.
  }

  TModemDevices = class(TCollection)
  private
    FDriver: TModemDriver;
    function GetItem(Index: Integer): TModemDevice;
    procedure SetItem(Index: Integer; Value: TModemDevice);
  protected
    function GetOwner: TPersistent; {$IFNDEF VER90} override; {$ENDIF}
  public
    constructor Create(ADriver: TModemDriver);
    function Add(Device: Integer; const Name: string; const Command, Answer: string; Flags: Integer): TModemDevice;
    procedure Delete(Device: Integer);
    function Find(Device: Integer; var Item: TModemDevice): Boolean;
    procedure ReadData(Reader: TModemReader);
    procedure WriteData(Writer: TModemWriter); 
    property Items[Index: Integer]: TModemDevice read GetItem write SetItem; default;
  end;

{ TModemCompression }

  {
    Modem compression item.

    Properties:

    Compression - Driver-specific compression mode number.
    Name        - Compresion name.
    Command     - Setup command.
    Answer      - Answer for setup command.
    Speed       - Default compression speed.
    Bits        - Bits per sample.
    Flags       - Compression flags (see CF_XXXX constants).
    WaveFormat  - Wave format type.
    WaveSpeed   - Default speed for wave format type.
    WaveFlags   - Wave format flags (see CF_XXXX constants).
  }

  TModemCompression = class(TCollectionItem)
  private
    FCompression: Integer;
    FName: string;        
    FCommand: string;
    FAnswer: string;
    FSpeed: Integer;
    FBits: Integer;
    FFlags: Integer;
    FWaveFormat: Integer;
    FWaveSpeed: Integer;
    FWaveFlags: Integer;  
  protected
    function GetDisplayName: string; {$IFNDEF VER90} override; {$ENDIF}
  public
    procedure AssignTo(Dest: TPersistent); override;
  published
    property Compression: Integer read FCompression write FCompression default 0;
    property Name: string read FName write FName;
    property Command: string read FCommand write FCommand;
    property Answer: string read FAnswer write FAnswer;
    property Speed: Integer read FSpeed write FSpeed default 0;
    property Bits: Integer read FBits write FBits default 0;
    property Flags: Integer read FFlags write FFlags default 0;
    property WaveFormat: Integer read FWaveFormat write FWaveFormat default 0;
    property WaveSpeed: Integer read FWaveSpeed write FWaveSpeed default 0;
    property WaveFlags: Integer read FWaveFlags write FWaveFlags default 0;
  end;

{ TModemCompressions }

  {
    List of supported modem compressions.

    Methods:

    Add      - Add new or replace existing compression to the list.
    Delete   - Remove given compression from the list.
    Find     - Find list item for giver compression. If comression mode
               not specified (i.e. zero) returns default (with CF_DEFAULT
               flag) compression item.
    FindWave - Find list item for given wave format. If format not
               specified (i.e. zero) returns default wave format item.
  }

  TModemCompressions = class(TCollection)
  private
    FDriver: TModemDriver;
    function GetItem(Index: Integer): TModemCompression;
    procedure SetItem(Index: Integer; Value: TModemCompression);
  protected
    function GetOwner: TPersistent; {$IFNDEF VER90} override; {$ENDIF}
  public
    constructor Create(ADriver: TModemDriver);
    function Add(Compression: Integer; const Name: string;
      const Command, Answer: string; Speed, Bits, Flags: Integer;
      WaveFormat, WaveSpeed, WaveFlags: Integer): TModemCompression;
    procedure Delete(Compression: Integer);
    function Find(Compression: Integer; var Item: TModemCompression): Boolean;
    function FindWave(WaveFormat: Integer; var Item: TModemCompression): Boolean;
    procedure ReadData(Reader: TModemReader);
    procedure WriteData(Writer: TModemWriter);
    property Items[Index: Integer]: TModemCompression read GetItem write SetItem; default;
  end;

{ TModemDriver }

  {
    Hardware driver abstract class.

    Procedures:

    Analyze               - Analyze the string in buffer for the modem-specific
                            answers (for example various caller ID reporting
                            answers).
    AnswerPhone           - Answering in voice.
    Beep                  - Generate tone signals.
    CheckModem            - Check modem property. Generate debug error message
                            if property is not assigned.
    Dial                  - Dial number and wait answer.
    EnableCallerID        - Enable/disable Caller ID reporting.
    EnableDistinctiveRing - Enable/disable distinctive ringing reporting.
    GetCompression        - Get compression ID for given name. Tries to convert
                            given string into number if compression with given
                            method name is not supported.
    GetCompressionName    - Get compression name for given ID. Return string
                            with given number if given compression method not
                            supported.
    GetCompressionsList   - Get supported compression name list. Return number
                            of supported methods.
    GetDevice             - Get device ID for given name. Tries to convert
                            given string into number if device with given name
                            not supported.
    GetDeviceName         - Get device name for given ID. Return string with
                            given number if given device not supported.
    GetDevicesList        - Get supported device name list. Return number of
                            supported devices.
    GetPlayDevicesList    - Get subset of supported devices for playing voice
                            data.
    GetRecDevicesList     - Get subset of supported devices for recording
                            voice data.
    GetSpeedLiet          - Get available recording speed list for specified
                            compression.
    GetWaveFormat         - Get wave format tag for given name. Tries to convert
                            given string into number if format with given name
                            is not supported.
    GetWaveFormatName     - Get compression name for given wave format. Return
                            string with given number if given WaveFormat not
                            supported.
    GetWaveFormatsList    - Get supported wave formats list. Return number of
                            supported formats.
    GetWaveSpeedList      - Get available recording speed list for specified
                            wave format.
    GetVoiceLen           - Calculate duration of voice for given modem data.
    HandleDLE             - Handle DLE code.
    HandleEvent           - Handle given event.
    HangUp                - Hang up (goes on hook).
    Init                  - Hardware specific modem init. WARNING: do nothing
                            to change local echo and result codes settings.
    InitVoice             - Init modem voice params (called in voice mode).
    PlayDTMF              - Play one of more DTMF tones.
    PlayStream            - Play data from stream.
    PlayStreamEx          - Play given number of bytes of the voice data stream.
    RecordStream          - Record data to stream.
    RecordStreamEx        - Record data to stream, return number of bytes of the
                            voice data.
    RestoreVoiceMode      - Switch modem to voice mode, quick reset voice
                            preferences, which are forgotten after leaving voice
                            mode.
    SetCompression        - Set selected compression. Return valid bits per
                            sample value.
    SetDevice             - Set selected device.
    SetReceiveGain        - Set receive gain value (sens of the microphone, in
                            percent).
    SetTransmitGain       - Set transmit gain value (speaketr volume, in percent).
    SetWaveFormat         - Set compression for given wave format. Return valid
                            bits per sample value.
    StartPlayStream       - Write to modem start playing command.
    StartRecordStream     - Write to modem start recording command.
    StopDialing           - Stop dialing operation.
    StopPlayStream        - Write to modem stop play voice command.
    StopPlaying           - Stop playing voice data.
    StopRecordStream      - Write to modem stop voice recording sequence.
    StopRecording         - Stop recording voice data.
    StopWaiting           - Stop waintg operation.
    SwitchToDataFax       - Switch modem to selected mode (data, fax or voice).
    VoiceModeOff          - Switch modem to voice mode.
    VoiceModeOn           - Return to previous mode
    Wait                  - Wait and handle modem <DLE> sequence during given
                            timeout in msec.
    WaitAnswer            - Wait modem standard or given answer, detect <DLE>
                            sequences during given timeout in msec.

    Properties:

    ModemName             - Modem full name.
    ModemRMDName          - Modem short name for RMD file header.
    PickPhoneCmd          - Manual answer: goes off hook in answer mode and
                            answer.
    PickPhoneAnsw
    HangUpCmd             - Hangs up (goes on hook) command and answer.
    HangUpAnsw
    HardFlowCmd           - Turn on handware flow control command and answer
    HardFlowAnsw
    SoftFlowCmd           - Turn on software flow control command and answer.
    SoftFlowAnsw
    ResultCodesCmd        - Init extended result codes command and answer.
    ResultCodesAnsw
    DevInitCmd            - Additional device-specific initialization command
                            and answers.
    DevInitAnsw
    BeepCmd               - Play tone command and answer.
    BeepAnsw
    BeepTimeUnit          - Divisor for calculation of value of beep duration.
                            By default, duration of beep signal (Length
                            parameter in Beep() function) is measured at
                            milliseconds. But some modems required beep
                            duration parameter in other units (for example in
                            0.01 second). The true length of beep signal will
                            be calculated as Length / BeepTimeUnit.
    StartPlayCmd          - Start, stop or interrupt playing voice data
                            commands and answers.
    StartPlayAnsw
    IntrPlayCmd
    IntrPlayAnsw
    StopPlayCmd
    StopPlayAnsw
    StartRecCmd           - Start, stop recording voice data commands and
                            answers.
    StartRecAnsw
    StopRecCmd
    StopRecAnsw
    SwitchModeCmd         - Select data or fax mode command and answer.
    SwitchModeAnsw
    VoiceModeOnCmd        - Entering voice mode command and answer.
    VoiceModeOnAnsw
    VoiceModeOffCmd       - Leaving voice mode command and answer.
    VoiceModeOffAnsw
    PlayDTMFCmd           - Command to generate a DTMF digit during a voice
                            call and answer.
    PlayDTMFAnsw
    PlayDTMFExtra         - Play DTMF digits separator.
    DialSetupCmd          - Dialing number setup command and answer. This
                            command will be sent every time before dialing
                            number.
    DialSetupAnsw
    InactivityTimerCmd    - Set the DCE's initial value for the DTE/DCE
                            inactivity timer command and answer.
    InactivityTimerAnsw
    BaudRateCmd           - Select a specific DTE/modem baud rate command ans
                            answer.
    BaudRateAnsw
    BaudRateUnit          - Divisor for calculation of baud rate value. The
                            true baud rate value will be calculated as
                            Profile.VoiceBaudRate / BaudRateUnit.
    SilenceDetectCmd      - Set the silence detection sensitivity and silence
                            detection interval command ans answer.
    SilenceDetectAnsw
    SilenceSensMin        - Silence detection sensitive ranges.
    SilenceSensMax
    SilenceDeleteCmd      - Silence deletion command ans answer.
    SilenceDeleteAnsw
    TransmitGainCmd       - Speaker volume control command and answer.
    TransmitGainAnsw
    TransmitGainMin       - Speaker volume ranges.
    TransmitGainMax
    ReceiveGainCmd        - Receive microphone gain control command and answer.
    ReceiveGainAnsw
    ReceiveGainMin        - Receive microphone gain ranges.
    ReceiveGainMax
    RingbackTimersCmd     - Ringback timeouts command and answer.
    RingbackTimersAnsw
    DTMFDetectCmd         - Set DTMF tone detection (threshold and duration)
                            command and answer.
    DTMFDetectAnsw
    DTMFSensMin           - DTMF tone detection threshold ranges.
    DTMFSensMax
    DTMFLenUnit           - Divisor for calculation of the DTMF tones duration
                            value. By default, duration of DTMF tones is measured
                            at milliseconds. But some modems required this value
                            in other units (for example in 5 msecs). The true
                            length of DTMF tone will
                            be calculated as Profile.DTMFLen / DTMFLenUnit.
    CallerIDOnCmd         - Enable caller ID identification command and answer.
    CallerIDOnAnsw
    CallerIDOffCmd        - Disable caller ID identification command and answer.
    CallerIDOffAnsw
    CallerIDFormat        - Caller ID reporting format.
    DRingOnCmd            - Enable Distinctive Ring command and answer.
    DRingOnAnsw
    DRingOffCmd           - Disable Distinctive Ring command and answer.
    DRingOffAnsw
    DevInitVoiceCmd       - Additional device-specific voice initialization
                            command ans answers.
    DevInitVoiceAnsw
    RestoreVoiceCmd       - Additional restoring voice mode commands and answers.
    RestoreVoiceAnsw
    Responses             - Possible modem non-standard responses which are not
                            included in any command answers.
    DLECodes              - List of supported shielded <DLE> codes.
    Devices               - List of supported devices.
    Compressions          - List of supported compression methods.
    Modem                 - Modem component link. For driver properly working
                            this property must be always assigned.
    Profile               - Modem profile. If property not specified at design
                            time it will be taken from modem at runtime.
  }

  TModemDriverClass = class of TModemDriver;

  TModemDriver = class(TComponent)
  private
    FModemName: string;
    FModemRMDName: string;
    FPickPhoneCmd: string;
    FPickPhoneAnsw: string;
    FHangUpCmd: string;
    FHangUpAnsw: string;
    FHardFlowCmd: string;
    FHardFlowAnsw: string;
    FSoftFlowCmd: string;
    FSoftFlowAnsw: string;
    FResultCodesCmd: string;
    FResultCodesAnsw: string;
    FDevInitCmd: string;
    FDevInitAnsw: string;
    FBeepCmd: string;
    FBeepAnsw: string;
    FBeepTimeUnit: Integer;
    FStartPlayCmd: string;
    FStartPlayAnsw: string;
    FIntrPlayCmd: string;
    FIntrPlayAnsw: string;
    FStopPlayCmd: string;
    FStopPlayAnsw: string;
    FStartRecCmd: string;
    FStartRecAnsw: string;
    FStopRecCmd: string;
    FStopRecAnsw: string;
    FSwitchModeCmd: string;
    FSwitchModeAnsw: string;
    FVoiceModeOnCmd: string;
    FVoiceModeOnAnsw: string;
    FVoiceModeOffCmd: string;
    FVoiceModeOffAnsw: string;
    FPlayDTMFCmd: string;
    FPlayDTMFAnsw: string;
    FPlayDTMFExtra: string;
    FDialSetupCmd: string;
    FDialSetupAnsw: string;
    FInactivityTimerCmd: string;
    FInactivityTimerAnsw: string;
    FBaudRateCmd: string;
    FBaudRateAnsw: string;
    FBaudRateUnit: Integer;
    FSilenceDetectCmd: string;
    FSilenceDetectAnsw: string;
    FSilenceSensMin: Integer;
    FSilenceSensMax: Integer;
    FSilenceDeleteCmd: string;
    FSilenceDeleteAnsw: string;
    FTransmitGainCmd: string;
    FTransmitGainAnsw: string;
    FTransmitGainMin: Integer;
    FTransmitGainMax: Integer;
    FReceiveGainCmd: string;
    FReceiveGainAnsw: string;
    FReceiveGainMin: Integer;
    FReceiveGainMax: Integer;
    FRingbackTimersCmd: string;
    FRingbackTimersAnsw: string;
    FDTMFDetectCmd: string;
    FDTMFDetectAnsw: string;
    FDTMFSensMin: Integer;
    FDTMFSensMax: Integer;
    FDTMFLenUnit: Integer;
    FCallerIDOnCmd: string;
    FCallerIDOnAnsw: string;
    FCallerIDOffCmd: string;
    FCallerIDOffAnsw: string;
    FCallerIDFormat: string;
    FDRingOnCmd: string;
    FDRingOnAnsw: string;
    FDRingOffCmd: string;
    FDRingOffAnsw: string;
    FDevInitVoiceCmd: string;
    FDevInitVoiceAnsw: string;
    FRestoreVoiceCmd: string;
    FRestoreVoiceAnsw: string;
    FResponses: TModemResponses;
    FDLECodes: TModemDLECodes;
    FDevices: TModemDevices;
    FCompressions: TModemCompressions;
    FModem: TVoiceModem;
    FProfile: TModemProfile;
    function GetProfile: TModemProfile;
    function IsResponsesStored: Boolean;
    function IsDLECodesStored: Boolean;
    function IsDevicesStored: Boolean;
    function IsCompressionsStored: Boolean;
    procedure SetCompressions(Value: TModemCompressions);
    procedure SetDevices(Value: TModemDevices);
    procedure SetDLECodes(Value: TModemDLECodes);
    procedure SetModem(Value: TVoiceModem);
    procedure SetResponses(Value: TModemResponses);
  protected
    procedure DebugMsg(Msg: Integer; const S: string); virtual;
    procedure DebugMsgFmt(Msg: Integer; const S: string; const Args: array of const);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Progress(Done: Longint); virtual;
    procedure YieldProc; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Analyze(const Buffer: string): Integer; virtual;
    function AnswerPhone: Integer; virtual; abstract;
    function Beep(Frequency, Length: Integer): Integer; virtual; abstract;
    function CheckModem(const CheckFunc: string; DebugError: Boolean): Boolean;
    function Dial(const Number: string): Integer; virtual; abstract;
    function EnableCallerID(Enabled: Boolean): Integer; virtual; abstract;
    function EnableDistinctiveRing(Enabled: Boolean): Integer; virtual; abstract;
    function GetCompression(const Name: string): Integer; virtual;
    function GetCompressionName(Compression: Integer): string; virtual;
    function GetCompressionsList(Proc: TGetStrProc): Integer; virtual;
    function GetCompressionsStringList(Strings: TStrings): Integer; virtual;
    function GetDevice(const Name: string): Integer; virtual;
    function GetDeviceName(Device: Integer): string; virtual;
    function GetDevicesList(Proc: TGetStrProc): Integer; virtual;
    function GetDevicesStringList(Strings: TStrings): Integer; virtual;
    function GetPlayDevicesList(Proc: TGetStrProc): Integer; virtual;
    function GetPlayDevicesStringList(Strings: TStrings): Integer; virtual;
    function GetRecDevicesList(Proc: TGetStrProc): Integer; virtual;
    function GetRecDevicesStringList(Strings: TStrings): Integer; virtual;
    function GetSpeedList(Compression: Integer; Proc: TGetStrProc): Integer; virtual;
    function GetSpeedStringList(Compression: Integer; Strings: TStrings): Integer; virtual;
    function GetWaveFormat(const Name: string): Integer; virtual;
    function GetWaveFormatName(WaveFormat: Integer): string; virtual;
    function GetWaveFormatsList(Proc: TGetStrProc): Integer; virtual;
    function GetWaveFormatsStringList(Strings: TStrings): Integer; virtual;
    function GetWaveSpeedList(WaveFormat: Integer; Proc: TGetStrProc): Integer; virtual;
    function GetWaveSpeedStringList(WaveFormat: Integer; Strings: TStrings): Integer; virtual;
    function GetVoiceLen(Compression, Speed, Bits: Integer; DataSize: Longint): Integer; virtual;
    function HandleDLE(Code: Char): Integer; virtual; abstract;
    function HandleEvent(Event: Integer; Data: TEventData): Integer; virtual; abstract;
    function HangUp: Integer; virtual; abstract;
    function Init: Integer; virtual; abstract;
    function InitVoice: Integer; virtual; abstract;
    function PlayDTMF(const Number: string): Integer; virtual; abstract;
    function PlayStream(Stream: TStream; BPS: Integer): Integer;
    function PlayStreamEx(Stream: TStream; BPS: Integer; DataLength: Longint): Integer; virtual; abstract;
    procedure ReadData(Reader: TModemReader); virtual;
    function RecordStream(Stream: TStream; BPS: Integer): Integer;
    function RecordStreamEx(Stream: TStream; BPS: Integer; var DataLength: Longint): Integer; virtual; abstract;
    function RestoreVoiceMode: Integer; virtual; abstract;
    function SetCompression(var Compression, Speed, Bits: Integer): Integer; virtual; abstract;
    function SetDevice(Device: Integer): Integer; virtual; abstract;
    function SetReceiveGain(Gain: Integer): Integer; virtual; abstract;
    function SetTransmitGain(Gain: Integer): Integer; virtual; abstract;
    function SetWaveFormat(var WaveFormat, Speed, Bits: Integer): Integer; virtual; abstract;
    function StartPlayStream: Integer; virtual; abstract;
    function StartRecordStream: Integer; virtual; abstract;
    function StopDialing: Integer; virtual; abstract;
    function StopPlayStream: Integer; virtual; abstract;
    function StopPlaying: Integer; virtual; abstract;
    function StopRecordStream: Integer; virtual; abstract;
    function StopRecording: Integer; virtual; abstract;
    function StopWaiting: Integer; virtual; abstract;
    function SwitchToDataFax(const Mode: string): Integer; virtual; abstract;
    function VoiceModeOff: Integer; virtual; abstract;
    function VoiceModeOn: Integer; virtual; abstract;
    function Wait(Timeout: Longint): Integer; virtual; abstract;
    function WaitAnswer(const ExpectedAnswers: string; Timeout: Longint): Integer; virtual; abstract;
    procedure WriteData(Writer: TModemWriter); virtual;
  published
    property ModemName: string read FModemName write FModemName;
    property ModemRMDName: string read FModemRMDName write FModemRMDName;
    property PickPhoneCmd: string read FPickPhoneCmd write FPickPhoneCmd;
    property PickPhoneAnsw: string read FPickPhoneAnsw write FPickPhoneAnsw;
    property HangUpCmd: string read FHangUpCmd write FHangUpCmd;
    property HangUpAnsw: string read FHangUpAnsw write FHangUpAnsw;
    property HardFlowCmd: string read FHardFlowCmd write FHardFlowCmd;
    property HardFlowAnsw: string read FHardFlowAnsw write FHardFlowAnsw;
    property SoftFlowCmd: string read FSoftFlowCmd write FSoftFlowCmd;
    property SoftFlowAnsw: string read FSoftFlowAnsw write FSoftFlowAnsw;
    property ResultCodesCmd: string read FResultCodesCmd write FResultCodesCmd;
    property ResultCodesAnsw: string read FResultCodesAnsw write FResultCodesAnsw;
    property DevInitCmd: string read FDevInitCmd write FDevInitCmd;
    property DevInitAnsw: string read FDevInitAnsw write FDevInitAnsw;
    property BeepCmd: string read FBeepCmd write FBeepCmd;
    property BeepAnsw: string read FBeepAnsw write FBeepAnsw;
    property BeepTimeUnit: Integer read FBeepTimeUnit write FBeepTimeUnit default 0;
    property StartPlayCmd: string read FStartPlayCmd write FStartPlayCmd;
    property StartPlayAnsw: string read FStartPlayAnsw write FStartPlayAnsw;
    property IntrPlayCmd: string read FIntrPlayCmd write FIntrPlayCmd;
    property IntrPlayAnsw: string read FIntrPlayAnsw write FIntrPlayAnsw;
    property StopPlayCmd: string read FStopPlayCmd write FStopPlayCmd;
    property StopPlayAnsw: string read FStopPlayAnsw write FStopPlayAnsw;
    property StartRecCmd: string read FStartRecCmd write FStartRecCmd;
    property StartRecAnsw: string read FStartRecAnsw write FStartRecAnsw;
    property StopRecCmd: string read FStopRecCmd write FStopRecCmd;
    property StopRecAnsw: string read FStopRecAnsw write FStopRecAnsw;
    property SwitchModeCmd: string read FSwitchModeCmd write FSwitchModeCmd;
    property SwitchModeAnsw: string read FSwitchModeAnsw write FSwitchModeAnsw;
    property VoiceModeOnCmd: string read FVoiceModeOnCmd write FVoiceModeOnCmd;
    property VoiceModeOnAnsw: string read FVoiceModeOnAnsw write FVoiceModeOnAnsw;
    property VoiceModeOffCmd: string read FVoiceModeOffCmd write FVoiceModeOffCmd;
    property VoiceModeOffAnsw: string read FVoiceModeOffAnsw write FVoiceModeOffAnsw;
    property PlayDTMFCmd: string read FPlayDTMFCmd write FPlayDTMFCmd;
    property PlayDTMFAnsw: string read FPlayDTMFAnsw write FPlayDTMFAnsw;
    property PlayDTMFExtra: string read FPlayDTMFExtra write FPlayDTMFExtra;
    property DialSetupCmd: string read FDialSetupCmd write FDialSetupCmd;
    property DialSetupAnsw: string read FDialSetupAnsw write FDialSetupAnsw;
    property InactivityTimerCmd: string read FInactivityTimerCmd write FInactivityTimerCmd;
    property InactivityTimerAnsw: string read FInactivityTimerAnsw write FInactivityTimerAnsw;
    property BaudRateCmd: string read FBaudRateCmd write FBaudRateCmd;
    property BaudRateAnsw: string read FBaudRateAnsw write FBaudRateAnsw;
    property BaudRateUnit: Integer read FBaudRateUnit write FBaudRateUnit default 0;
    property SilenceDetectCmd: string read FSilenceDetectCmd write FSilenceDetectCmd;
    property SilenceDetectAnsw: string read FSilenceDetectAnsw write FSilenceDetectAnsw;
    property SilenceSensMin: Integer read FSilenceSensMin write FSilenceSensMin default 0;
    property SilenceSensMax: Integer read FSilenceSensMax write FSilenceSensMax default 0;
    property SilenceDeleteCmd: string read FSilenceDeleteCmd write FSilenceDeleteCmd;
    property SilenceDeleteAnsw: string read FSilenceDeleteAnsw write FSilenceDeleteAnsw;
    property TransmitGainCmd: string read FTransmitGainCmd write FTransmitGainCmd;
    property TransmitGainAnsw: string read FTransmitGainAnsw write FTransmitGainAnsw;
    property TransmitGainMin: Integer read FTransmitGainMin write FTransmitGainMin default 0;
    property TransmitGainMax: Integer read FTransmitGainMax write FTransmitGainMax default 0;
    property ReceiveGainCmd: string read FReceiveGainCmd write FReceiveGainCmd;
    property ReceiveGainAnsw: string read FReceiveGainAnsw write FReceiveGainAnsw;
    property ReceiveGainMin: Integer read FReceiveGainMin write FReceiveGainMin default 0;
    property ReceiveGainMax: Integer read FReceiveGainMax write FReceiveGainMax default 0;
    property RingbackTimersCmd: string read FRingbackTimersCmd write FRingbackTimersCmd;
    property RingbackTimersAnsw: string read FRingbackTimersAnsw write FRingbackTimersAnsw;
    property DTMFDetectCmd: string read FDTMFDetectCmd write FDTMFDetectCmd;
    property DTMFDetectAnsw: string read FDTMFDetectAnsw write FDTMFDetectAnsw;
    property DTMFSensMin: Integer read FDTMFSensMin write FDTMFSensMin default 0;
    property DTMFSensMax: Integer read FDTMFSensMax write FDTMFSensMax default 0;
    property DTMFLenUnit: Integer read FDTMFLenUnit write FDTMFLenUnit default 0;
    property CallerIDOnCmd: string read FCallerIDOnCmd write FCallerIDOnCmd;
    property CallerIDOnAnsw: string read FCallerIDOnAnsw write FCallerIDOnAnsw;
    property CallerIDOffCmd: string read FCallerIDOffCmd write FCallerIDOffCmd;
    property CallerIDOffAnsw: string read FCallerIDOffAnsw write FCallerIDOffAnsw;
    property CallerIDFormat: string read FCallerIDFormat write FCallerIDFormat;
    property DRingOnCmd: string read FDRingOnCmd write FDRingOnCmd;
    property DRingOnAnsw: string read FDRingOnAnsw write FDRingOnAnsw;
    property DRingOffCmd: string read FDRingOffCmd write FDRingOffCmd;
    property DRingOffAnsw: string read FDRingOffAnsw write FDRingOffAnsw;
    property DevInitVoiceCmd: string read FDevInitVoiceCmd write FDevInitVoiceCmd;
    property DevInitVoiceAnsw: string read FDevInitVoiceAnsw write FDevInitVoiceAnsw;
    property RestoreVoiceCmd: string read FRestoreVoiceCmd write FRestoreVoiceCmd;
    property RestoreVoiceAnsw: string read FRestoreVoiceAnsw write FRestoreVoiceAnsw;
    property Responses: TModemResponses read FResponses write SetResponses stored IsResponsesStored;
    property DLECodes: TModemDLECodes read FDLECodes write SetDLECodes stored IsDLECodesStored;
    property Devices: TModemDevices read FDevices write SetDevices stored IsDevicesStored;
    property Compressions: TModemCompressions read FCompressions write SetCompressions stored IsCompressionsStored;
    property Modem: TVoiceModem read FModem write SetModem;
    property Profile: TModemProfile read GetProfile write FProfile;
  end;

{ TVoiceModem }

  {
    Voice Modem component.

    Procedures:

    Analyze               - Analyze the string in buffer for the appearance of
                            one of the possibilities given in expected_answers
                            in the form "OK|BUSY". If this fails, try to detect
                            driver-specific answer or some standard voice modem
                            answers.
    AnswerPhone           - Set voice modem to answer mode.
    Beep                  - Play default Beep signal (with Profile settings).
    CheckDriver           - Check Driver property. Generate debug error message
                            if driver does not exists (assigned).
    CheckEvents           - Process event queue.
    CheckPort             - Check Port portperty. Generate debug error message
                            if port does not exists (assigned).
    Close                 - Hang up modem and close voice device.
    CloseDevice           - Close the voice device.
    Command               - Execute the given command and wait for an answer.
                            Returned OK if command output success and no
                            expected answers, VMA_USER_XXX if answer is one of
                            expected answer, VMA_FAIL if error. Command can
                            contains hexadecimal of ASCII control code mnemonic
                            framed by '<>'. Answers must be separated by '|'.
    CommandFmt            - Format given command and execute it.
    DetectModemType       - Autodetect the modem type we are connected to. Not
                            implemented in version 1.0.
    Dial                  - Dial given number.
    EnableCallerID        - Enable/disable Caller ID reporting.
    EnableDistinctiveRing - Enable/disable distinctive ringing reporting.
    Flush                 - Read input from the voice modem device.
    FlushEvents           - Flush event queue.
    FlushEx               - Read input from the voice modem device during
                            given timeout in msec.
    HandleDLE             - Handle given DLE code.
    HandleEvent           - Handle given event.
    HangUp                - Hang up (go on hook).
    Init                  - Detect the type of voice modem connected (not
                            implemented in version 1.0) and initialize the
                            modem.
    InitVoice             - Set modem voice params (must be called only in
                            voice mode).
    Lock                  - Lock modem for private usage. Use this function
                            for multiplie (multithreaded) using one
                            TVoiceModem instance or for prevent start new
                            modem operations
                            before any other operation still active.
    ModemName             - Return modem full name for current modem driver.
    ModemRMDName          - Return modem short name for current modem driver.
                            This name is stored in RMF file header.
    Open                  - Open voice device, automatically initialize modem
                            and switch to voice mode.
    OpenDevice            - Open voice device.
    PlayDTMF              - Play one of more DTMF tones.
    PlayFile              - Play the given file. The voice data format will be
                            detected automatically by analysing file contents
                            or file extension. If autodetection data type
                            failed the Profile.VoiceDataFormat format will be
                            used for playing file.
    PlayFiles             - Play the files sequentially from the given list.
                            The voice data format will be detected automatically
                            by analysing first file contents or extension. If
                            autodetection data type failed the VoiceDtataFormat
                            will be used for playing files. Note that all files
                            must be recorded using one compression settings.
    PlayStream            - Play the given stream. The voice data format will
                            be detected automatically by analysing stream
                            contents. If autodetection data type failed the
                            Profile.VoiceDataFormat format will be used for
                            playing file.
    QueueEvent            - Post given event into event queue.
    Read                  - Read data from the voice modem device. Returns
                            amount read byte or -1 when error.
    ReadChar              - Read char from the voice modem device.
    ReadRMDHeader         - Read raw modem data header from the given stream.
    ReadStr               - Read string from the voice modem device.
    ReadWaveHeader        - Read WAVE file header from the given stream.
    RecordFile            - Record voice data from the voice modem and save it
                            as Profile.VoiceDataFormat format file with given
                            name.
    RecordStream          - Record voice data into the given stream of
                            Profile.VoiceDataFormat format.
    RestoreVoiceMode      - Switch modem to voice mode, quick reset voice
                            preferences, which are forgotten after leaving
                            voice mode.
    Script                - Execute the multiple commands. Each command must
                            be separated by '||'. Returned OK if all
                            commands executed successfull, VMA_FAIL + N if
                            error (where N is failed command number).
    ScriptFmt             - Format given command script and execute it.
    SetCompression        - Set compression mode. Return valid bits per
                            sample value.
    SetDevice             - Set device for voice playback and recording.
    SetReceiveGain        - Set receive gain value (sens of the microphone).
    SetTransmitGain       - Set transmit gain value (speaketr volume).
    SetWaveFormat         - Set compression for given wave format. Return
                            valid bits per sample value.
    Stop                  - Queue stop operation event.
    StopCurrentAction     - Check for current modem action and stop it.
    StopDialing           - Stop dialing.
    StopPlaying           - Stop playing.
    StopRecording         - Stop recording.
    StopWaiting           - Stop waiting.
    SwitchToDataFax       - Turn on data/fax mode.
    Unlock                - Unlock modem private usage. Return previous locked
                            state.
    UnqueueEvent          - Get fist event and remove it from event queue.
    VoiceModeOn           - Entering voice mode.
    VoiceModeOff          - Leaving voice mode.
    Wait                  - Wait modem char and handle DLE events during given
                            timeout in msec.
    WaitAnswer            - Wait modem standard or given answer, detect <DLE>
                            sequences during given timeout in msec.
    Write                 - Write data to voice modem device.
    WriteChar             - Write char to voice modem device.
    WriteCmd              - Unmap control codes and write command to voice modem
                            device (with CR). Command can contains hexadecimal
                            of ASCII control code mnemonic framed by '<>';
    WriteRMDHeader        - Write raw modem data header to the given stream.
    WriteStr              - Write string to voice modem device (with CR).
    WriteStrFmt           - Format give string and write it to voice modem
                            device.
    WriteWaveHeader       - Write WAVE header to the given stream.

    Properties:

    LastCallerID          - Telephone number of the last caller (last Caller ID
                            report). 
    LastDTMF              - Last received DTMF codes sequence.
    LastError             - Last error code.
    LastErorStr           - Last error description.
    State                 - Current modem (component) state.
    Driver                - Hardware driver. If there are not value assigned at
                            design time the driver component instance will be
                            created at runtime automatically unig the specified
                            ModemIniFile or ModemName property of profile
                            settings. 
    LogFile               - Log file. All debug messages may be writed into
                            specified log file.
    Port                  - Modem port. If there are not value assigned at
                            design time the standard TModemPort component
                            instance will be created at runtime. 
    Profile               - Modem profile. If property not specified at design
                            time a new instance of standard TModemProfile will
                            be created automatically with default settings at
                            runtime.
    TraceFile             - Trace file. Modem text input/output may be writed
                            into specified file.

    Events:

    OnChangeState         - Modem state changed.
    OnEvent               - Modem event queue handler.
    OnFlushChar           - Flush modem char event. Before calling this event
                            you get OnRead event.
    OnFlushString         - Flush modem string event. Before calling this event
                            you get OnFlushChar event. Event string does not
                            inclided chars such as CR, LF.
    OnDebugMsg            - Debug message event. WARNING: the Sender parameter
                            of event procedure not always is Self.
    OnProgress            - Progress callback for voice playing and recording.
    OnRead                - Read data from modem.
    OnReadChar            - Read single char from modem. Before calling this
                            event you get OnRead event.
    OnReadString          - Read string (command) from modem. Before calling
                            this event you get OnReadChar event for each char
                            readed from modem while reading string. Event
                            string does not inclided chars such as CR, LF, XON,
                            XOFF, DLE, ETX.   
    OnTrace               - Got one more modem input/output trace line. 
    OnWrite               - Write data to modem.
    OnWriteChar           - Write single char to modem. Before calling this
                            event you get OnWrite event.
    OnWriteString         - Write string (command) to modem. Before calling
                            this event you get OnWrite event. Event string
                            does not inclided CR char.
    OnYield               - Yield event. Use this event for process Windows
                            messages for singlethreaded applications.
  }

  TReadDataHeaderProc = function(Stream: TStream; var Compression, Speed, Bits: Integer; var DataLen: Longint): Integer of object;
  TWriteDataHeaderProc = function(Stream: TStream; Compression, Speed, Bits: Integer; DataLen: Longint): Integer of object;
  TSetCompressionProc = function(var Compression, Speed, Bits: Integer): Integer of object;

  TModemEventHandler = procedure(Sender: TObject; Event: Integer; Data: TEventData; var Result: Integer) of object;

  TModemDataEvent = procedure(Sender: TObject; Buffer: Pointer; Count: Integer) of object;
  TModemCharEvent = procedure(Sender: TObject; C: Char) of object;
  TModemStringEvent = procedure(Sender: TObject; const S: string) of object;
  TModemTraceEvent = procedure(Sender: TObject; const Action, Line: string) of object;

  TModemMessageEvent = procedure(Sender: TObject; Msg: Integer; const S: string) of object;
  TModemProgressEvent = procedure(Sender: TObject; Done: Longint) of object;

  TVoiceModem = class(TComponent)
  private
    FPort: TModemPort;
    FDriver: TModemDriver;
    FProfile: TModemProfile;
    FLogFile: TModemLogFile;
    FTraceFile: TModemLogFile;
    FTraceWasCR: Boolean;
    FTraceLine: string;
    FTraceAction: string;
    FEventQueue: TEventQueue;
    FState: Integer;
    FStateSection: TRTLCriticalSection;
    FLocked: Boolean;
    FLockOwner: TObject;
    FLockSection: TRTLCriticalSection;
    FLastError: Integer;
    FLastCallerID: string;
    FLastCallerIDSection: TRTLCriticalSection;
    FLastDTMF: string;
    FLastDTMFSection: TRTLCriticalSection;
    FOnChangeState: TNotifyEvent;
    FOnEvent: TModemEventHandler;
    FOnDebugMsg: TModemMessageEvent;
    FOnRead: TModemDataEvent;
    FOnReadChar: TModemCharEvent;
    FOnReadString: TModemStringEvent;
    FOnWrite: TModemDataEvent;
    FOnWriteChar: TModemCharEvent;
    FOnWriteString: TModemStringEvent;
    FOnFlushChar: TModemCharEvent;
    FOnFlushString: TModemStringEvent;
    FOnTrace: TModemTraceEvent;
    FOnProgress: TModemProgressEvent;
    FOnYield: TNotifyEvent;
    function GetDriver: TModemDriver;
    function GetLastCallerID: string;
    function GetLastDTMF: string;
    function GetLastErrorStr: string;
    function GetLocked: Boolean;
    function GetLockOwner: TObject;
    function GetOpened: Boolean;
    function GetPort: TModemPort;
    function GetProfile: TModemProfile;
    function GetState: Integer;
    procedure SetDriver(Value: TModemDriver);
    procedure SetLastCallerID(const Value: string);
    procedure SetLastDTMF(const Value: string);
    procedure SetLastError(Value: Integer);
    procedure SetLogFile(Value: TModemLogFile);
    procedure SetPort(Value: TModemPort);
    procedure SetProfile(Value: TModemProfile);
    procedure SetState(Value: Integer);
    procedure SetTraceFile(Value: TModemLogFile);
  protected
    procedure ChangeState; virtual;
    procedure DebugMsg(Msg: Integer; const S: string); virtual;
    procedure DebugMsgFmt(Msg: Integer; const S: string; const Args: array of const); virtual;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function InternalCreateDriver: TModemDriver; virtual;
    function InternalCreatePort: TModemPort; virtual;
    function InternalCreateProfile: TModemProfile; virtual;
    function InternalPlayFile(const FileName: string; Compression, Speed, Bits: Integer; ReadHeaderProc: TReadDataHeaderProc; SetCompressionProc: TSetCompressionProc): Integer;
    function InternalPlayFiles(FileList: TStrings; Compression, Speed, Bits: Integer; ReadHeaderProc: TReadDataHeaderProc; SetCompressionProc: TSetCompressionProc): Integer;
    function InternalPlayStream(Stream: TStream; Compression, Speed, Bits: Integer; ReadHeaderProc: TReadDataHeaderProc; SetCompressionProc: TSetCompressionProc): Integer;
    function InternalRecordFile(const FileName: string; Compression, Speed, Bits: Integer; WriteHeaderProc: TWriteDataHeaderProc; SetCompressionProc: TSetCompressionProc): Integer;
    function InternalRecordStream(Stream: TStream; Compression, Speed, Bits: Integer; WriteHeaderProc: TWriteDataHeaderProc; SetCompressionProc: TSetCompressionProc): Integer;
    procedure PortError(Sender: TObject; Errors: Integer); virtual;
    procedure Progress(Done: Longint); virtual;
    procedure TraceChar(const Action: string; C: Char); virtual;
    procedure YieldProc; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Analyze(const Buffer, ExpectedAnswers: string): Integer;
    function AnswerPhone: Integer;
    function Beep(Frequency, Length: Integer): Integer;
    function CheckDataFormat(DataFormat: Integer; var Compression, Speed, Bits: Integer;
      var ReadHeaderProc: TReadDataHeaderProc; var WriteHeaderProc: TWriteDataHeaderProc;
      var SetCompressionProc: TSetCompressionProc): Boolean; virtual;
    function CheckDriver(const CheckFunc: string; DebugError: Boolean): Boolean;
    function CheckEvents: Integer;
    function CheckPort(const CheckFunc: string; DebugError: Boolean): Boolean;
    function Close: Integer;
    function CloseDevice: Integer;
    function Command(const Command, ExpectedAnswers: string): Integer;
    function CommandFmt(const Command, ExpectedAnswers: string; const Args: array of const): Integer;
    function DetectModemType: Integer;
    function Dial(const Number: string): Integer;
    function EnableCallerID(Enabled: Boolean): Integer;
    function EnableDistinctiveRing(Enabled: Boolean): Integer;
    function Flush: Integer;
    function FlushEvents: Integer;
    function FlushEx(Timeout: Longint): Integer;
    function HandleDLE(Code: Char): Integer;
    function HandleEvent(Event: Integer; Data: TEventData): Integer; virtual;
    function HangUp: Integer;
    function Init: Integer;
    function InitVoice: Integer;
    function Lock(Owner: TObject): Boolean;
    function ModemName: string;
    function ModemRMDName: string;
    function Open: Integer;
    function OpenDevice: Integer;
    function PlayDTMF(const Number: string): Integer;
    function PlayFile(const FileName: string): Integer;
    function PlayFiles(FileList: TStrings): Integer;
    function PlayStream(Stream: TStream): Integer;
    function QueueEvent(Event: TModemEvent): Integer;
    function Read(Buffer: Pointer; Count: Integer): Integer;
    function ReadChar(var CharIn: Char): Integer;
    function ReadRMDHeader(Stream: TStream; var Compression, Speed, Bits: Integer; var DataLength: Longint): Integer;
    function ReadStr(var S: string): Integer;
    function ReadWaveHeader(Stream: TStream; var WaveFormat, Speed, Bits: Integer; var DataLength: Longint): Integer;
    function RecordFile(const FileName: string): Integer;
    function RecordStream(Stream: TStream): Integer;
    function RestoreVoiceMode: Integer;
    function Script(const Commands, ExpectedAnswers: string; ExecuteAll: Boolean): Integer;
    function ScriptFmt(const Commands, ExpectedAnswers: string; ExecuteAll: Boolean; const Args: array of const): Integer;
    function SetCompression(var Compression, Speed, Bits: Integer): Integer;
    function SetDevice(Device: Integer): Integer;
    function SetReceiveGain(Gain: Integer): Integer;
    function SetTransmitGain(Gain: Integer): Integer; 
    function SetWaveFormat(var WaveFormat, Speed, Bits: Integer): Integer;
    function Stop: Integer;
    function StopCurrentAction: Integer;
    function StopDialing: Integer;
    function StopPlaying: Integer;
    function StopRecording: Integer;
    function StopWaiting: Integer;
    function SwitchToDataFax(Mode: string): Integer;
    function Unlock: Boolean;
    function UnqueueEvent(var Event: TModemEvent): Integer;
    function VoiceModeOff: Integer;
    function VoiceModeOn: Integer;
    function Wait(Timeout: Longint): Integer;
    function WaitAnswer(const ExpectedAnswers: string; Timeout: Longint): Integer;
    function Write(Buffer: Pointer; Count: Integer): Integer;
    function WriteChar(CharOut: Char): Integer;
    function WriteCmd(const Command: string): Integer;
    function WriteRMDHeader(Stream: TStream; Compression, Speed, Bits: Integer; DataLength: Longint): Integer;
    function WriteStr(const S: string): Integer;
    function WriteStrFmt(const Format: string; const Args: array of const): Integer;
    function WriteWaveHeader(Stream: TStream; WaveFormat, Speed, Bits: Integer; DataLength: Longint): Integer;
    property LastCallerID: string read GetLastCallerID write SetLastCallerID;
    property LastDTMF: string read GetLastDTMF write SetLastDTMF;
    property LastError: Integer read FLastError write SetLastError;
    property LastErrorStr: string read GetLastErrorStr;
    property Locked: Boolean read GetLocked;
    property LockOwner: TObject read GetLockOwner;
    property Opened: Boolean read GetOpened;
    property State: Integer read GetState write SetState;
  published
    property Driver: TModemDriver read GetDriver write SetDriver;
    property LogFile: TModemLogFile read FLogFile write SetLogFile;
    property Port: TModemPort read GetPort write SetPort;
    property Profile: TModemProfile read GetProfile write SetProfile;
    property TraceFile: TModemLogFile read FTraceFile write SetTraceFile;
    property OnChangeState: TNotifyEvent read FOnChangeState write FOnChangeState;
    property OnEvent: TModemEventHandler read FOnEvent write FOnEvent;
    property OnFlushChar: TModemCharEvent read FOnFlushChar write FOnFlushChar;
    property OnFlushString: TModemStringEvent read FOnFlushString write FOnFlushString;
    property OnDebugMsg: TModemMessageEvent read FOnDebugMsg write FOnDebugMsg;
    property OnProgress: TModemProgressEvent read FOnProgress write FOnProgress;
    property OnRead: TModemDataEvent read FOnRead write FOnRead;
    property OnReadChar: TModemCharEvent read FOnReadChar write FOnReadChar;
    property OnReadString: TModemStringEvent read FOnReadString write FOnReadString;
    property OnTrace: TModemTraceEvent read FOnTrace write FOnTrace;
    property OnWrite: TModemDataEvent read FOnWrite write FOnWrite;
    property OnWriteChar: TModemCharEvent read FOnWriteChar write FOnWriteChar;
    property OnWriteString: TModemStringEvent read FOnWriteString write FOnWriteString;
    property OnYield: TNotifyEvent read FOnYield write FOnYield;
  end;

{ Modem drivers registration routines }

procedure RegisterModemDriver(const ModemName: string; DriverClass: TModemDriverClass);
procedure UnRegisterModemDriver(const ModemName: string);

procedure ReplaceModemDriver(const ModemName: string; NewDriverClass: TModemDriverClass);
procedure ReplaceModemDriverEx(const ModemName: string; NewDriverClass: TModemDriverClass; var OldDriverClass: TModemDriverClass);

function GetModemDriver(const ModemName: string; var DriverClass: TModemDriverClass): Boolean;
function GetModemDriversList(Proc: TGetStrProc): Integer;
function GetModemDriversStringList(Strings: TStrings): Integer;

implementation

uses
  VML_IS101, VML_IniFile, VML_Utils, MMSys, MMReg, MMCodec;

{ Modem drivers registration routines }

var
  FDriversList: TStringList = nil;

function DriversList: TStringList;
begin
  if FDriversList = nil then FDriversList := TStringList.Create;
  Result := FDriversList;
end;

procedure RegisterModemDriver(const ModemName: string; DriverClass: TModemDriverClass);
begin
  DriversList.AddObject(ModemName, TObject(DriverClass));
end;

procedure UnRegisterModemDriver(const ModemName: string);
var
  Index: Integer;
begin
  Index := DriversList.IndexOf(ModemName);
  { Unregister driver }
  if Index <> -1 then DriversList.Delete(Index);
end;

procedure ReplaceModemDriver(const ModemName: string; NewDriverClass: TModemDriverClass);
var 
  OldDriverClass: TModemDriverClass;
begin
  ReplaceModemDriverEx(ModemName, NewDriverClass, OldDriverClass);
end;

procedure ReplaceModemDriverEx(const ModemName: string; NewDriverClass: TModemDriverClass; var OldDriverClass: TModemDriverClass);
var
  Index: Integer;
begin
  { Search registered driver }
  Index := DriversList.IndexOf(ModemName);
  if (Index <> -1) then 
  begin
    { Unregister old driver }
    OldDriverClass := TModemDriverClass(DriversList.Objects[Index]);
    DriversList.Delete(Index);
  end
  else
    { No driver registered for this modem }
    OldDriverClass := nil;
  { Register new driver }
  DriversList.AddObject(ModemName, TObject(NewDriverClass));
end;

function GetModemDriver(const ModemName: string; var DriverClass: TModemDriverClass): Boolean;
var
  Index: Integer;
begin
  Index := DriversList.IndexOf(ModemName);
  { Driver class not found }
  if Index = -1 then
  begin
    DriverClass := nil;
    Result := False;
    Exit;
  end;
  { Success }
  DriverClass := TModemDriverClass(DriversList.Objects[Index]);
  Result := True;
end;

function GetModemDriversList(Proc: TGetStrProc): Integer;
var
  I: Integer;
begin
  Result := 0;
  for I := 0 to DriversList.Count - 1 do
  begin
    Proc(DriversList[I]);
    Inc(Result);
  end;
end;

function GetModemDriversStringList(Strings: TStrings): Integer;
type
  TGetStrFunc = function(const S: string): Integer of object;
var
  Proc: TGetStrFunc;
begin
  Proc := Strings.Add;
  Result := GetModemDriversList(TGetStrProc(Proc));
end;

{ TModemReader }

function TModemReader.ReadBool(const Section, Ident: string; Default: Boolean): Boolean;
begin
  Result := ReadInteger(Section, Ident, Ord(Default)) <> 0;
end;

function TModemReader.ReadDriver(Instance: TModemDriver): TModemDriver;
var
  ModemName: string;
  DriverClass: TModemDriverClass;

  function GetAncestorDriver: TModemDriverClass;
  var
    KeyList: TStringList;
    I: Integer;
    ModemName: string;
    DriverClass: TModemDriverClass;
  begin
    Result := nil;
    { Find any registered ancestor driver class }
    KeyList := TStringList.Create;
    try
      ReadSection('Ancestors', KeyList);
      for I := 0 to KeyList.Count - 1 do
      begin
        ModemName := ReadString('Ancestors', KeyList[I], '');
        if GetModemDriver(ModemName, DriverClass) then
          if (Result = nil) or DriverClass.InheritsFrom(Result) then
            { Result driver class may be first registered class or most
               ? ancestor clas }
            Result := DriverClass;
      end;
    finally
      KeyList.Free;
    end;
    { No any driver ancestor class registered but driver instance is
      required. So use standard IS-101 driver class. }
    if Result = nil then Result := TIS101;
  end;

begin
  { Read driver class information }
  ModemName := ReadString('Driver', 'ModemName', '');
  { Check driver instance }
  if Instance = nil then
  begin
    { Find driver class }
    if not GetModemDriver(ModemName, DriverClass) then
      DriverClass := GetAncestorDriver;
    { Create driver instance }
    Result := DriverClass.Create(nil);
  end
  else
    { Use existing instance }
    Result := Instance;
  { Set modem name }
  Result.ModemName := ModemName;
  Result.ModemRMDName := ReadString('Driver', 'ModemRMDName', Result.ModemRMDName);
  { Write driver params }
  Result.ReadData(Self);
end;

function TModemReader.ReadInteger(const Section, Ident: string; Default: Longint): Longint;
var
  S: string;
begin
  S := ReadString(Section, Ident, '');
  if (Length(S) > 2) and (S[1] = '0') and (UpCase(S[2]) = 'X') then
    S := '$' + Copy(S, 3, MaxInt);
  Result := StrToIntDef(S, Default);
end;

function TModemReader.ReadMapStr(const Section, Ident, Default: string): string;
begin
  Result := UnMapControlCodes(ReadString(Section, Ident, Default), '<>', nil);
end;

procedure TModemReader.ReadSectionValues(const Section: string; Strings: TStrings);
var
  KeyList: TStringList;
  I, Index: Integer;
begin
  KeyList := TStringList.Create;
  try
    ReadSection(Section, KeyList);
    { Read key values }
    Strings.BeginUpdate;
    try
      for I := 0 to KeyList.Count - 1 do
      begin
        { Find existing key }
        Index := Strings.IndexOfName(KeyList[I]);
        if Index <> -1 then
          { Key exists - replace value }
          Strings[I] := KeyList[I] + '=' + ReadString(Section, KeyList[I], '')
        else
          { Key not found - add new }
          Strings.Add(KeyList[I] + '=' + ReadString(Section, KeyList[I], ''));
      end;
    finally
      Strings.EndUpdate;
    end;
  finally
    KeyList.Free;
  end;
end;

{ TModemWriter }

procedure TModemWriter.WriteBool(const Section, Ident: string; Value: Boolean);
const
  Values: array[Boolean] of string = ('0', '1');
begin
  WriteString(Section, Ident, Values[Value]);
end;

procedure TModemWriter.WriteDriver(Driver: TModemDriver);
var
  Count: Integer;
  ClassType: TClass;
begin
  { Write modem name }
  WriteString('Driver', 'ModemName', Driver.ModemName);
  WriteString('Driver', 'ModemRMDName', Driver.ModemRMDName);
  { Write driver params }
  Driver.WriteData(Self);
  { Clear old ancestors list }
  EraseSection('Ancestors');
  { Write driver class ancestors list }
  Count := 0;
  ClassType := Driver.ClassParent;
  while ClassType.InheritsFrom(TIS101) do
  begin
    Inc(Count);
    { Write ancestor modem name }
    with TModemDriverClass(ClassType).Create(nil) do
    try
      WriteString('Ancestors', IntToStr(Count), ModemName);
    finally
      Free;
    end;
    { Next ancestor }
    ClassType := ClassType.ClassParent;
  end;
end;

procedure TModemWriter.WriteInteger(const Section, Ident: string; Value: Longint);
begin
  WriteString(Section, Ident, IntToStr(Value));
end;

procedure TModemWriter.WriteMapStr(const Section, Ident, Value: string);
begin
  WriteString(Section, Ident, MapControlCodes(Value, '<>', nil));
end;

{ TModemProfile }

constructor TModemProfile.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPort := 2;
  FPortBaudRate := 38400;
  FPortDataBits := 8;
  FPortParity := NOPARITY;
  FPortStopBits := ONESTOPBIT;
  FPortTimeout := 10;
  FPortStatusDelay := 50;
  FPortReadDelay := 75;
  FPortWriteDelay := 75;
  FPortInBuffSize := 4096;
  FPortOutBuffSize := 4096;
  FModemName := '';
  FModemIniFile := '';
  FModemInitCmd := '';
  FModemInitAnsw := 'OK';
  FModemLogging := False;
  FPlayDevice := EXTERNAL_SPEAKER;
  FPlayMaxLen := 0;
  FPlayFileName := '';
  FPlayPreviewLen := 10;
  FRecCompression := 0;
  FRecSpeed := 0;
  FRecIgnoreSilence := False;
  FRecSilenceLen := 20;
  FRecSilenceSens := 40;
  FRecDevice := EXTERNAL_MICROPHONE;
  FRecMaxLen := 300;
  FRecWaveFormat := 0;
  FRecWaveSpeed := 0;
  FTransmitGain := -1;
  FReceiveGain := -1;
  FDTMFLen := 30;
  FDTMFSens := 40;
  FDTMFWait := 7;
  FBeepFrequency := 933;
  FBeepLength := 1500;
  FCommandDelay := 100;
  FCommandEcho := False;
  FDialTimeout := 90;
  FDialMode := 'P';
  FAnswerRecognition := False;
  FAnswerFMax := 4000;
  FAnswerFToneMin := 390;
  FAnswerFToneMax := 460;
  FAnswerToneLevel := 2000000;
  FAnswerEnergyLevel := 2000;
  FAnswerVoiceRate := 4;
  FAnswerNoiseRate := 20;
  FAnswerVoiceRequired := True;
  FRingbackGoesAway := 45;
  FRingbackNeverCame := 120;
  FRawData := False;
  FVoiceBaudRate := 0;
  FVoiceDataFormat := DATA_FORMAT_RMD;
  FInactivityTimeout := 60;
  FCallerIDDetection := False;
  FDistinctiveRing := False;
end;

function TModemProfile.IsModemInitAnswStored: Boolean;
begin
  Result := FModemInitAnsw <> 'OK';
end;

procedure TModemProfile.AssignTo(Dest: TPersistent);
begin
  if Dest is TModemProfile then
  begin
    TModemProfile(Dest).Port := Port;
    TModemProfile(Dest).PortBaudRate := PortBaudRate;
    TModemProfile(Dest).PortDataBits := PortDataBits;
    TModemProfile(Dest).PortParity := PortParity;
    TModemProfile(Dest).PortStopBits := PortStopBits;
    TModemProfile(Dest).PortTimeout := PortTimeout;
    TModemProfile(Dest).PortStatusDelay := PortStatusDelay;
    TModemProfile(Dest).PortReadDelay := PortReadDelay;
    TModemProfile(Dest).PortWriteDelay := PortWriteDelay;
    TModemProfile(Dest).PortInBuffSize := PortInBuffSize;
    TModemProfile(Dest).PortOutBuffSize := PortOutBuffSize;
    TModemProfile(Dest).ModemName := ModemName;
    TModemProfile(Dest).ModemIniFile := ModemIniFile;
    TModemProfile(Dest).ModemInitCmd := ModemInitCmd;
    TModemProfile(Dest).ModemInitAnsw := ModemInitAnsw;
    TModemProfile(Dest).ModemLogging := ModemLogging;
    TModemProfile(Dest).PlayDevice := PlayDevice;
    TModemProfile(Dest).PlayMaxLen := PlayMaxLen;
    TModemProfile(Dest).PlayFileName := PlayFileName;
    TModemProfile(Dest).PlayPreview := PlayPreview;
    TModemProfile(Dest).PlayPreviewLen := PlayPreviewLen;
    TModemProfile(Dest).RecCompression := RecCompression;
    TModemProfile(Dest).RecSpeed := RecSpeed;
    TModemProfile(Dest).RecSilenceLen := RecSilenceLen;
    TModemProfile(Dest).RecSilenceSens := RecSilenceSens;
    TModemProfile(Dest).RecIgnoreSilence := RecIgnoreSilence;
    TModemProfile(Dest).RecDevice := RecDevice;
    TModemProfile(Dest).RecMaxLen := RecMaxLen;
    TModemProfile(Dest).RecWaveFormat := RecWaveFormat;
    TModemProfile(Dest).RecWaveSpeed := RecWaveSpeed;
    TModemProfile(Dest).TransmitGain := TransmitGain;
    TModemProfile(Dest).ReceiveGain := ReceiveGain;
    TModemProfile(Dest).DTMFLen := DTMFLen;
    TModemProfile(Dest).DTMFSens := DTMFSens;
    TModemProfile(Dest).DTMFWait := DTMFWait;
    TModemProfile(Dest).BeepFrequency := BeepFrequency;
    TModemProfile(Dest).BeepLength := BeepLength;
    TModemProfile(Dest).CommandDelay := CommandDelay;
    TModemProfile(Dest).CommandEcho := CommandEcho;
    TModemProfile(Dest).DialTimeout := DialTimeout;
    TModemProfile(Dest).DialMode := DialMode;
    TModemProfile(Dest).AnswerRecognition := AnswerRecognition;
    TModemProfile(Dest).AnswerFMax := AnswerFMax;
    TModemProfile(Dest).AnswerFToneMin := AnswerFToneMin;
    TModemProfile(Dest).AnswerFToneMax := AnswerFToneMax;
    TModemProfile(Dest).AnswerToneLevel := AnswerToneLevel;
    TModemProfile(Dest).AnswerEnergyLevel := AnswerEnergyLevel;
    TModemProfile(Dest).AnswerVoiceRate := AnswerVoiceRate;
    TModemProfile(Dest).AnswerNoiseRate := AnswerNoiseRate;
    TModemProfile(Dest).AnswerVoiceRequired := AnswerVoiceRequired;
    TModemProfile(Dest).RingbackGoesAway := RingbackGoesAway;
    TModemProfile(Dest).RingbackNeverCame := RingbackNeverCame;
    TModemProfile(Dest).RawData := RawData;
    TModemProfile(Dest).VoiceBaudRate := VoiceBaudRate;
    TModemProfile(Dest).VoiceDataFormat := VoiceDataFormat;
    TModemProfile(Dest).InactivityTimeout := InactivityTimeout;
    TModemProfile(Dest).CallerIDDetection := CallerIDDetection;
    TModemProfile(Dest).DistinctiveRing := DistinctiveRing;
    Exit;
  end;
  inherited AssignTo(Dest);
end;

procedure TModemProfile.ReadData(Reader: TModemReader);
begin
  with Reader do
  begin
    Port := ReadInteger('Profile', 'Port', Port);
    PortBaudRate := ReadInteger('Profile', 'PortBaudRate', PortBaudRate);
    PortParity := ReadInteger('Profile', 'PortParity', PortParity);
    PortDataBits := ReadInteger('Profile', 'PortDataBits', PortDataBits);
    PortStopBits := ReadInteger('Profile', 'PortStopBits', PortStopBits);
    PortTimeout := ReadInteger('Profile', 'PortTimeout', PortTimeout);
    PortStatusDelay := ReadInteger('Profile', 'PortStatusDelay', PortStatusDelay);
    PortReadDelay := ReadInteger('Profile', 'PortReadDelay', PortReadDelay);
    PortWriteDelay := ReadInteger('Profile', 'PortWriteDelay', PortWriteDelay);
    PortInBuffSize := ReadInteger('Profile', 'PortInBuffSize', PortInBuffSize);
    PortOutBuffSize := ReadInteger('Profile', 'PortOutBuffSize', PortOutBuffSize);
    ModemName := ReadString('Profile', 'ModemName', ModemName);
    ModemIniFile := ReadString('Profile', 'ModemIniFile', ModemIniFile);
    ModemInitCmd := ReadString('Profile', 'ModemInitCmd', ModemInitCmd);
    ModemInitAnsw := ReadString('Profile', 'ModemInitAnsw', ModemInitAnsw);
    ModemLogging := ReadBool('Profile', 'ModemLogging', ModemLogging);
    PlayDevice := ReadInteger('Profile', 'PlayDevice', PlayDevice);
    PlayMaxLen := ReadInteger('Profile', 'PlayMaxLen', PlayMaxLen);
    PlayFileName := ReadString('Profile', 'PlayFileName', PlayFileName);
    PlayPreview := ReadBool('Profile', 'PlayPreview', PlayPreview);
    PlayPreviewLen := ReadInteger('Profile', 'PlayPreviewLen', PlayPreviewLen);
    RecCompression := ReadInteger('Profile', 'RecCompression', RecCompression);
    RecSpeed := ReadInteger('Profile', 'RecSpeed', RecSpeed);
    RecSilenceLen := ReadInteger('Profile', 'RecSilenceLen', RecSilenceLen);
    RecSilenceSens := ReadInteger('Profile', 'RecSilenceSens', RecSilenceSens);
    RecIgnoreSilence := ReadBool('Profile', 'RecIgnoreSilence', RecIgnoreSilence);
    RecDevice := ReadInteger('Profile', 'RecDevice', RecDevice);
    RecMaxLen := ReadInteger('Profile', 'RecMaxLen', RecMaxLen);
    RecWaveFormat := ReadInteger('Profile', 'RecWaveFormat', RecWaveFormat);
    RecWaveSpeed := ReadInteger('Profile', 'RecWaveSpeed', RecWaveSpeed);
    TransmitGain := ReadInteger('Profile', 'TransmitGain', TransmitGain);
    ReceiveGain := ReadInteger('Profile', 'ReceiveGain', ReceiveGain);
    DTMFLen := ReadInteger('Profile', 'DTMFLen', DTMFLen);
    DTMFSens := ReadInteger('Profile', 'DTMFSens', DTMFSens);
    DTMFWait := ReadInteger('Profile', 'DTMFWait', DTMFWait);
    BeepFrequency := ReadInteger('Profile', 'BeepFrequency', BeepFrequency);
    BeepLength := ReadInteger('Profile', 'BeepLength', BeepLength);
    CommandDelay := ReadInteger('Profile', 'CommandDelay', CommandDelay);
    CommandEcho := ReadBool('Profile', 'CommandEcho', CommandEcho);
    DialTimeout := ReadInteger('Profile', 'DialTimeOut', DialTimeout);
    DialMode := ReadString('Profile', 'DialMode', DialMode);
    AnswerRecognition := ReadBool('Profile', 'AnswerRecognition', AnswerRecognition);
    AnswerFMax := ReadInteger('Profile', 'AnswerFMax', AnswerFMax);
    AnswerFToneMin := ReadInteger('Profile', 'AnswerFToneMin', AnswerFToneMin);
    AnswerFToneMax := ReadInteger('Profile', 'AnswerFToneMax', AnswerFToneMax);
    AnswerToneLevel := ReadInteger('Profile', 'AnswerToneLevel', AnswerToneLevel);
    AnswerEnergyLevel := ReadInteger('Profile', 'AnswerEnergyLevel', AnswerEnergyLevel);
    AnswerVoiceRate := ReadInteger('Profile', 'AnswerVoiceRate', AnswerVoiceRate);
    AnswerNoiseRate := ReadInteger('Profile', 'AnswerNoiseRate', AnswerNoiseRate);
    AnswerVoiceRequired := ReadBool('Profile', 'AnswerVoiceRequired', AnswerVoiceRequired);
    RingbackGoesAway := ReadInteger('Profile', 'RingbackGoesAway', RingbackGoesAway);
    RingbackNeverCame := ReadInteger('Profile', 'RingbackNeverCame', RingbackNeverCame);
    RawData := ReadBool('Profile', 'RawData', RawData);
    VoiceBaudRate := ReadInteger('Profile', 'VoiceBaudRate', VoiceBaudRate);
    VoiceDataFormat := ReadInteger('Profile', 'VoiceDataFormat', VoiceDataFormat);
    InactivityTimeout := ReadInteger('Profile', 'InactivityTimeout', InactivityTimeout);
    CallerIDDetection := ReadBool('Profile', 'CallerIDDetection', CallerIDDetection);
    DistinctiveRing := ReadBool('Profile', 'DistinctiveRing', DistinctiveRing);
  end;
end;

procedure TModemProfile.WriteData(Writer: TModemWriter);
begin
  with Writer do
  begin
    WriteInteger('Profile', 'Port', Port);
    WriteInteger('Profile', 'PortBaudRate', PortBaudRate);
    WriteInteger('Profile', 'PortParity', PortParity);
    WriteInteger('Profile', 'PortDataBits', PortDataBits);
    WriteInteger('Profile', 'PortStopBits', PortStopBits);
    WriteInteger('Profile', 'PortTimeout', PortTimeout);
    WriteInteger('Profile', 'PortStatusDelay', PortStatusDelay);
    WriteInteger('Profile', 'PortReadDelay', PortReadDelay);
    WriteInteger('Profile', 'PortWriteDelay', PortWriteDelay);
    WriteInteger('Profile', 'PortInBuffSize', PortInBuffSize);
    WriteInteger('Profile', 'PortOutBuffSize', PortOutBuffSize);
    WriteString('Profile', 'ModemName', ModemName);
    WriteString('Profile', 'ModemIniFile', ModemIniFile);
    WriteMapStr('Profile', 'ModemInitCmd', ModemInitCmd);
    WriteMapStr('Profile', 'ModemInitAnsw', ModemInitAnsw);
    WriteBool('Profile', 'ModemLogging', ModemLogging);
    WriteInteger('Profile', 'PlayDevice', PlayDevice);
    WriteInteger('Profile', 'PlayMaxLen', PlayMaxLen);
    WriteString('Profile', 'PlayFileName', PlayFileName);
    WriteBool('Profile', 'PlayPreview', PlayPreview);
    WriteInteger('Profile', 'PlayPreviewLen', PlayPreviewLen);
    WriteInteger('Profile', 'RecCompression', RecCompression);
    WriteInteger('Profile', 'RecSpeed', RecSpeed);
    WriteInteger('Profile', 'RecSilenceLen', RecSilenceLen);
    WriteInteger('Profile', 'RecSilenceSens', RecSilenceSens);
    WriteBool('Profile', 'RecIgnoreSilence', RecIgnoreSilence);
    WriteInteger('Profile', 'RecDevice', RecDevice);
    WriteInteger('Profile', 'RecMaxLen', RecMaxLen);
    WriteInteger('Profile', 'RecWaveFormat', RecWaveFormat);
    WriteInteger('Profile', 'RecWaveSpeed', RecWaveSpeed);
    WriteInteger('Profile', 'TransmitGain', TransmitGain);
    WriteInteger('Profile', 'ReceiveGain', ReceiveGain);
    WriteInteger('Profile', 'DTMFLen', DTMFLen);
    WriteInteger('Profile', 'DTMFSens', DTMFSens);
    WriteInteger('Profile', 'DTMFWait', DTMFWait);
    WriteInteger('Profile', 'BeepFrequency', BeepFrequency);
    WriteInteger('Profile', 'BeepLength', BeepLength);
    WriteInteger('Profile', 'CommandDelay', CommandDelay);
    WriteBool('Profile', 'CommandEcho', CommandEcho);
    WriteInteger('Profile', 'DialTimeout', DialTimeout);
    WriteString('Profile', 'DialMode', DialMode);
    WriteBool('Profile', 'AnswerRecognition', AnswerRecognition);
    WriteInteger('Profile', 'AnswerFMax', AnswerFMax);
    WriteInteger('Profile', 'AnswerFToneMin', AnswerFToneMin);
    WriteInteger('Profile', 'AnswerFToneMax', AnswerFToneMax);
    WriteInteger('Profile', 'AnswerToneLevel', AnswerToneLevel);
    WriteInteger('Profile', 'AnswerEnergyLevel', AnswerEnergyLevel);
    WriteInteger('Profile', 'AnswerVoiceRate', AnswerVoiceRate);
    WriteInteger('Profile', 'AnswerNoiseRate', AnswerNoiseRate);
    WriteBool('Profile', 'AnswerVoiceRequired', AnswerVoiceRequired);
    WriteInteger('Profile', 'RingbackGoesAway', RingbackGoesAway);
    WriteInteger('Profile', 'RingbackNeverCame', RingbackNeverCame);
    WriteBool('Profile', 'RawData', RawData);
    WriteInteger('Profile', 'VoiceBaudRate', VoiceBaudRate);
    WriteInteger('Profile', 'VoiceDataFormat', VoiceDataFormat);
    WriteInteger('Profile', 'InactivityTimeout', InactivityTimeout);
    WriteBool('Profile', 'CallerIDDetection', CallerIDDetection);
    WriteBool('Profile', 'DistinctiveRing', DistinctiveRing);
  end;
end;

{ TModemPort }

procedure TModemPort.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FProfile) then FProfile := nil;
end;

function TModemPort.Close: Integer;
begin
  ClosePort;
  Result := S_OK;
end;

function TModemPort.Open: Integer;
begin
  { Check profile specified }
  if Profile = nil then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Setup port number }
  if not (Profile.Port in [1..16]) then
  begin
    Result := S_FAIL;
    Exit;
  end
  else
    ComNumber := TComNumber(Profile.Port - 1);
  { Setup baud rate value }
  case Profile.PortBaudRate of
    CBR_110: BaudRate := br110;
    CBR_300: BaudRate := br300;
    CBR_600: BaudRate := br600;
    CBR_1200: BaudRate := br1200;
    CBR_2400: BaudRate := br2400;
    CBR_4800: BaudRate := br4800;
    CBR_9600: BaudRate := br9600;
    CBR_14400: BaudRate := br14400;
    CBR_19200: BaudRate := br19200;
    CBR_38400: BaudRate := br38400;
    CBR_56000: BaudRate := br56000;
    CBR_57600: BaudRate := br57600;
    CBR_115200: BaudRate := br115200;
    CBR_128000: BaudRate := br128000;
    CBR_256000: BaudRate := br256000;
  else
    Result := S_FAIL;
    Exit;
  end;
  { Setup data bits value }
  case Profile.PortDataBits of
    4: DataBits := db4;
    5: DataBits := db5;
    6: DataBits := db6;
    7: DataBits := db7;
    8: DataBits := db8;
  else
    Result := S_FAIL;
    Exit;
  end;
  { Setup parity value }
  case Profile.PortParity of
    NOPARITY: Parity := paNone;
    ODDPARITY: Parity := paOdd;
    EVENPARITY: Parity := paEven;
    MARKPARITY: Parity := paMark;
    SPACEPARITY: Parity := paSpace;
  else
    Result := S_FAIL;
    Exit;
  end;
  { Setup stop bits value }
  case Profile.PortStopBits of
    ONESTOPBIT: StopBits := sb1_0;
    ONE5STOPBITS: StopBits := sb1_5;
    TWOSTOPBITS: StopBits := sb2_0;
  else
    Result := S_FAIL;
    Exit;
  end;
  { Setup timeouts }
  StatusDelay := Profile.PortStatusDelay;
  ReadDelay := Profile.PortReadDelay;
  WriteDelay := Profile.PortWriteDelay;
  { Setup buffers size }
  InBuffSize := Profile.PortInBuffSize;
  OutBuffSize := Profile.PortOutBuffSize;
  { Open device }
  OpenPort;
  { Check result }
  if not Enabled then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Success }
  Result := S_OK;
end;

function TModemPort.Read(Buffer: Pointer; Count: Integer): Integer;
begin
  if not Enabled then
    Result := -1
  else if CharReady then
    Result := inherited GetBlock(Buffer^, Count)
  else
    Result := 0;
end;

function TModemPort.Write(Buffer: Pointer; Count: Integer): Integer;
begin
  if not Enabled then
    Result := -1
  else
    Result := inherited PutBlock(Buffer^, Count);
end;

{ TModemLogFile }

constructor TModemLogFile.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDateTimeFormat := 'MM-DD-YYYY hh:mm:ss';
end;

destructor TModemLogFile.Destroy;
begin
  Close;
  inherited Destroy;
end;

function TModemLogFile.IsDateTimeformatStored: Boolean;
begin
  Result := FAddDateTime;
end;

procedure TModemLogFile.SetFileName(const Value: string);
begin
  if FFileName <> Value then
  begin
    FFileName := Value;
    { Close log file }
    FLogFile.Free;
    FLogFile := nil;
  end;
end;

function TModemLogFile.Add(const S: string): Boolean;
var
  L: string;
begin
  try
    { Open log file }
    if (not Open) or (FLogFile = nil) then
    begin
      Result := False;
      Exit;
    end;
    { Create log line }
    L := '';
    if FAddDateTime then L := FormatDateTime(FDateTimeFormat, Now) + ' - ';
    L := L + S;
    { Write line }
    FLogFile.Write(Pointer(L)^, Length(L));
    FLogFile.Write(#13#10, 2);
    { Success }
    Result := True;
  except
    Result := False;
  end;
end;

function TModemLogFile.Open: Boolean;
var
  S: string;
begin
  { Check log already opened }
  if FLogFile <> nil then
  begin
    Result := True;
    Exit;
  end;
  { Check log file name }
  if Length(FFileName) = 0 then
  begin
    Result := False;
    Exit;
  end;
  { Open file }
  try
    if not FCreateNew then
    begin
      { Check for new file }
      if not FileExists(FFileName) then
      begin
        FLogFile := TFileStream.Create(FFileName, fmCreate);
        FLogFile.Free;
      end;
      { Open log file, allow other users read log }
      FLogFile := TFileStream.Create(FFileName, fmOpenWrite or fmShareDenyWrite);
      { Move to end of file end write empty line }
      if FLogFile.Size > 0 then
      begin
        FLogFile.Position := FLogFile.Size;
        FLogFile.Write(#13#10, 2);
      end;
    end
    else
    begin
      { Create new file }
      FLogFile := TFileStream.Create(FFileName, fmCreate);
      FLogFile.Free;
      { Open log file, allow other users read log }
      FLogFile := TFileStream.Create(FFileName, fmOpenWrite or fmShareDenyWrite);
    end;
    { Get log title }
    S := FTitle;
    if Assigned(FOnGetTitle) then FOnGetTitle(Self, S);
    { Write title string }
    if Length(S) > 0 then
    begin
      FLogFile.Write(Pointer(S)^, Length(S));
      FLogFile.Write(#13#10, 2);
      FLogFile.Write(#13#10, 2);
    end;
    { Success }
    Result := True;
  except
    FLogFile := nil;
    Result := False;
  end;
end;

function TModemLogFile.Close: Boolean;
begin
  try
    { Close file }
    FLogFile.Free;
    FLogFile := nil;
    { Success }
    Result := True;
  except
    Result := False;
  end;
end;

{ TModemEvent }

constructor TModemEvent.Create(AEvent: Integer);
begin
  FEvent := AEvent;
end;

constructor TModemEvent.CreateDataB(AEvent: Integer; Frequency, Length: Integer);
begin
  FEvent := AEvent;
  FData.Beep.Frequency := Frequency;
  FData.Beep.Length := Length;
end;

constructor TModemEvent.CreateDataC(AEvent: Integer; C: Char);
begin
  FEvent := AEvent;
  FData.C := C;
end;

constructor TModemEvent.CreateDataI(AEvent: Integer; I: Integer);
begin
  FEvent := AEvent;
  FData.I := I;
end;

constructor TModemEvent.CreateDataP(AEvent: Integer; P: Pointer);
begin
  FEvent := AEvent;
  FData.P := P;
end;

{ TEventQueue }

constructor TEventQueue.Create;
begin
  InitializeCriticalSection(FLockSection);
end;

destructor TEventQueue.Destroy;
begin
  DeleteCriticalSection(FLockSection);
end;

function TEventQueue.Queue(Event: TModemEvent): Integer;
var
  Item: PQueueItem;
begin
  EnterCriticalSection(FLockSection);
  try
    New(Item);
    Item^.Next := nil;
    Item^.Event := Event;
    if FLastItem <> nil then
    begin
      Item^.Position := FLastItem^.Position + 1;
      FLastItem^.Next := Item;
      FLastItem := Item;
    end
    else
    begin
      Item^.Position := 1;
      FLastItem := Item;
      FFirstItem := Item;
    end;
    Result := Item^.Position;
  finally
    LeaveCriticalSection(FLockSection);
  end;
end;

function TEventQueue.UnQueue(var Event: TModemEvent): Integer;
var
  Item: PQueueItem;
begin
  EnterCriticalSection(FLockSection);
  try
    if FFirstItem <> nil then
    begin
      Item := FFirstItem;
      if Item = FLastItem then
      begin
        FLastItem := nil;
        FFirstItem := nil;
      end
      else
        FFirstItem := Item^.Next;
      Result := Item^.Position;
      Event := Item^.Event;
      Dispose(Item);
    end
    else
    begin
      Result := 0;
      Event := nil;
    end;
  finally
    LeaveCriticalSection(FLockSection);
  end;
end;

{ TModemResponse }

constructor TModemResponse.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FAnswerCode := VMA_IGNORE;
end;

function TModemResponse.GetDisplayName: string;
begin
  Result := MapControlCodes(Response, '<>', @CODEMAP_NAME);
end;

procedure TModemResponse.AssignTo(Dest: TPersistent);
begin
  if Dest is TModemResponse then
  begin
    TModemResponse(Dest).Response := Response;
    TModemResponse(Dest).AnswerCode := AnswerCode;
    Exit;
  end;
  inherited AssignTo(Dest);
end;

{ TModemResponses }

constructor TModemResponses.Create(ADriver: TModemDriver);
begin
  inherited Create(TModemResponse);
  FDriver := ADriver;
end;

function TModemResponses.GetItem(Index: Integer): TModemResponse;
begin
  Result := TModemResponse(inherited GetItem(Index));
end;

function TModemResponses.GetText: string;
var
  I: Integer;
begin
  Result := '';
  if Count > 0 then
  begin
    Result := Items[0].Response;
    for I := 0 to Count - 1 do Result := Result + '|' + Items[I].Response;
  end;
end;

procedure TModemResponses.SetItem(Index: Integer; Value: TModemResponse);
begin
  inherited SetItem(Index, Value);
end;

function TModemResponses.GetOwner: TPersistent;
begin
  Result := FDriver;
end;

function TModemResponses.Add(const Response: string; AnswerCode: Integer): TModemResponse;
begin
  if not Find(Response, Result) then Result := TModemResponse(inherited Add);
  Result.Response := Response;
  Result.AnswerCode := AnswerCode;
end;

procedure TModemResponses.Delete(const Response: string);
var
  Item: TModemResponse;
begin
  if Find(Response, Item) then Item.Free;
end;

function TModemResponses.Find(const Response: string; var Item: TModemResponse): Boolean;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
  begin
    Item := Items[I];
    if Length(Item.Response) <> 0 then
      if AnsiCompareText(Item.Response, Response) = 0 then
      begin
        Result := True;
        Exit;
      end;
  end;
  Item := nil;
  Result := False;
end;

procedure TModemResponses.ReadData(Reader: TModemReader);
var
  KeyList: TStringList;
  I, P: Integer;
  Value: string;
  Response: string;
  AnswerCode: Integer;
begin
  KeyList := TStringList.Create;
  try
    Reader.ReadSection('Responses', KeyList);
    Clear;
    { Read responses }
    for I := 0 to KeyList.Count - 1 do
    begin
      { Read value string }
      Value := Reader.ReadString('Responses', KeyList[I], '');
      if Length(Value) <> 0 then
      begin
        P := 1;
        { Parse value string }
        Response := ParseIniFrameStr(Value, ',', P, '"', '');
        AnswerCode := ParseIniInteger(Value, ',', P, VMA_IGNORE);
        { Add response to the list }
        Add(Response, AnswerCode);
      end;
    end;
  finally
    KeyList.Free;
  end;
end;

procedure TModemResponses.WriteData(Writer: TModemWriter);
var
  I: Integer;
  Item: TModemResponse;
  RS, ACS, Value: string;
begin
  { Clear responses section }
  Writer.EraseSection('Responses');
  { Write list }
  for I := 0 to Count - 1 do
  begin
    Item := Items[I];
    if Length(Item.Response) <> 0 then
    begin
      { Format response strings }
      with Item do
      begin
        RS := FrameString(MapControlCodes(Response, '<>', nil), '"');
        ACS := Format('$%0.4x', [AnswerCode]);
      end;
      { Format value string }
      Value := RS + ',' + ACS;
      { Write value }
      Writer.WriteString('Responses', IntToStr(I + 1), Value);
    end;
  end;
end;

{ TModemDLECode }

constructor TModemDLECode.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  Event := NO_EVENT;
end;

function TModemDLECode.GetDisplayName: string;
begin
  Result := MapControlCodes(Code, '<>', @CODEMAP_NAME);
end;

procedure TModemDLECode.AssignTo(Dest: TPersistent);
begin
  if Dest is TModemDLECode then
  begin
    TModemDLECode(Dest).Code := Code;
    TModemDLECode(Dest).Event := Event;
    TModemDLECode(Dest).EventCode := EventCode;
    TModemDLECode(Dest).DebugMsg := DebugMsg;
    Exit;
  end;
  inherited AssignTo(Dest);
end;

{ TModemDLECodes }

constructor TModemDLECodes.Create(ADriver: TModemDriver);
begin
  inherited Create(TModemDLECode);
  FDriver := ADriver;
end;

function TModemDLECodes.GetItem(Index: Integer): TModemDLECode;
begin
  Result := TModemDLECode(inherited GetItem(Index));
end;

procedure TModemDLECodes.SetItem(Index: Integer; Value: TModemDLECode);
begin
  inherited SetItem(Index, Value);
end;

function TModemDLECodes.GetOwner: TPersistent;
begin
  Result := FDriver;
end;

function TModemDLECodes.Add(Code: Char; Event: Integer; EventCode: Char; const DebugMsg: string): TModemDLECode;
begin
  if not Find(Code, Result) then Result := TModemDLECode(inherited Add);
  Result.Code := Code;
  Result.Event := Event;
  Result.EventCode := EventCode;
  Result.DebugMsg := DebugMsg;
end;

procedure TModemDLECodes.Delete(Code: Char);
var
  Item: TModemDLECode;
begin
  if Find(Code, Item) then Item.Free;
end;

function TModemDLECodes.Find(Code: Char; var Item: TModemDLECode): Boolean;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
  begin
    Item := Items[I];
    if Item.Code <> #0 then
      if Item.Code = Code then
      begin
        Result := True;
        Exit;
      end;
  end;
  Item := nil;
  Result := False;
end;

procedure TModemDLECodes.ReadData(Reader: TModemReader);
var
  KeyList: TStringList;
  I, P: Integer;
  Value: string;
  Code, EventCode: Char;
  Event: Integer;
  DebugMsg: string;

  function StrToCode(const S: string): Char;
  var
    CS: string;
  begin
    CS := UnMapControlCodes(S, '<>', nil);
    if Length(CS) > 0 then Result := CS[1] else Result := #0;
  end;

begin
  KeyList := TStringList.Create;
  try
    Reader.ReadSection('DLECodes', KeyList);
    Clear;
    { Read DLE codes }
    for I := 0 to KeyList.Count - 1 do
    begin
      { Read value string }
      Value := Reader.ReadString('DLECodes', KeyList[I], '');
      if Length(Value) <> 0 then
      begin
        P := 1;
        { Parse value string }
        Code := StrToCode(ParseIniString(Value, ',', P, ''));
        Event := ParseIniInteger(Value, ',', P, NO_EVENT);
        EventCode := StrToCode(ParseIniString(Value, ',', P, ''));
        DebugMsg := ParseIniFrameStr(Value, ',', P, '"', '');
        { Add code to the list }
        Add(Code, Event, EventCode, DebugMsg);
      end;
    end;
  finally
    KeyList.Free;
  end;
end;

procedure TModemDLECodes.WriteData(Writer: TModemWriter);
var
  I: Integer;
  Item: TModemDLECode;
  CS, ES, ECS, DMS, Value: string;

  function CodeToStr(C: Char): string;
  begin
    Result := MapControlCodes(C, '<>', nil);
  end;

begin
  { Clear DLE codes section }
  Writer.EraseSection('DLECodes');
  { Write list }
  for I := 0 to Count - 1 do
  begin
    Item := Items[I];
    if Item.Code <> #0 then
    begin
      { Format DLE code strings }
      with Item do
      begin
        CS := CodeToStr(Code);
        ES := Format('$%0.4x', [Event]);
        ECS := CodeToStr(EventCode);
        DMS := FrameString(DebugMsg, '"');
      end;
      { Format value string }
      Value := CS + ',' + ES + ',' + ECS + ',' + DMS;
      { Write value }
      Writer.WriteString('DLECodes', IntToStr(I + 1), Value);
    end;
  end;
end;

{ TModemDevice }

function TModemDevice.GetDisplayName: string;
begin
  Result := Name;
end;

procedure TModemDevice.AssignTo(Dest: TPersistent);
begin
  if Dest is TModemDevice then
  begin
    TModemDevice(Dest).Device := Device;
    TModemDevice(Dest).Name := Name;
    TModemDevice(Dest).Command := Command;
    TModemDevice(Dest).Answer := Answer;
    TModemDevice(Dest).Flags := Flags;
    Exit;
  end;
  inherited AssignTo(Dest);
end;

{ TModemDevices }

constructor TModemDevices.Create(ADriver: TModemDriver);
begin
  inherited Create(TModemDevice);
  FDriver := ADriver;
end;

function TModemDevices.GetItem(Index: Integer): TModemDevice;
begin
  Result := TModemDevice(inherited GetItem(Index));
end;

procedure TModemDevices.SetItem(Index: Integer; Value: TModemDevice);
begin
  inherited SetItem(Index, Value);
end;

function TModemDevices.GetOwner: TPersistent;
begin
  Result := FDriver;
end;

function TModemDevices.Add(Device: Integer; const Name: string; const Command, Answer: string; Flags: Integer): TModemDevice;
begin
  if not Find(Device, Result) then Result := TModemDevice(inherited Add);
  Result.Device := Device;
  Result.Name := Name;
  Result.Command := Command;
  Result.Answer := Answer;
  Result.Flags := Flags;
end;

procedure TModemDevices.Delete(Device: Integer);
var
  Item: TModemDevice;
begin
  if Find(Device, Item) then Item.Free;
end;

function TModemDevices.Find(Device: Integer; var Item: TModemDevice): Boolean;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
  begin
    Item := Items[I];
    if Item.Device <> 0 then
      if Item.Device = Device then
      begin
        Result := True;
        Exit;
      end;
  end;
  Item := nil;
  Result := False;
end;

procedure TModemDevices.ReadData(Reader: TModemReader);
var
  KeyList: TStringList;
  I, P: Integer;
  Value: string;
  Device: Integer;
  Name, Command, Answer: string;
  Flags: Integer;
begin
  KeyList := TStringList.Create;
  try
    Reader.ReadSection('Devices', KeyList);
    Clear;
    { Read devices }
    for I := 0 to KeyList.Count - 1 do
    begin
      { Read value string }
      Value := Reader.ReadString('Devices', KeyList[I], '');
      if Length(Value) <> 0 then
      begin
        P := 1;
        { Parse value string }
        Device := ParseIniInteger(Value, ',', P, 0);
        Name := ParseIniFrameStr(Value, ',', P, '"', '');
        Command := ParseIniFrameStr(Value, ',', P, '"', '');
        Answer := ParseIniFrameStr(Value, ',', P, '"', '');
        Flags := ParseIniInteger(Value, ',', P, 0);
        { Add device to the list }
        Add(Device, Name, Command, Answer, Flags);
      end;
    end;
  finally
    KeyList.Free;
  end;
end;

procedure TModemDevices.WriteData(Writer: TModemWriter);
var
  I: Integer;
  Item: TModemDevice;
  DS, NS, CS, CAS, FS, Value: string;
begin
  { Clear devices section }
  Writer.EraseSection('Devices');
  { Write list }
  for I := 0 to Count - 1 do
  begin
    Item := Items[I];
    if Item.Device <> 0 then
    begin
      { Format device strings }
      with Item do
      begin
        DS := Format('$%0.4x', [Device]);
        NS := FrameString(Name, '"');
        CS := FrameString(MapControlCodes(Command, '<>', nil), '"');
        CAS := FrameString(MapControlCodes(Answer, '<>', nil), '"');
        FS := Format('$%0.4x', [Flags]);
      end;
      { Format value string }
      Value := DS + ',' + NS + ',' + CS + ',' + CAS + ',' + FS;
      { Write value }
      Writer.WriteString('Devices', IntToStr(I + 1), Value);
    end;
  end;
end;

{ Modem compressions list }

function TModemCompression.GetDisplayName: string;
begin
  Result := Name;
end;

procedure TModemCompression.AssignTo(Dest: TPersistent);
begin
  if Dest is TModemCompression then
  begin
    TModemCompression(Dest).Compression := Compression;
    TModemCompression(Dest).Name := Name;
    TModemCompression(Dest).Command := Command;
    TModemCompression(Dest).Answer := Answer;
    TModemCompression(Dest).Speed := Speed;
    TModemCompression(Dest).Bits := Bits;
    TModemCompression(Dest).Flags := Flags;
    TModemCompression(Dest).WaveFormat := WaveFormat;
    TModemCompression(Dest).WaveSpeed := WaveSpeed;
    TModemCompression(Dest).WaveFlags := WaveFlags;
    Exit;
  end;
  inherited AssignTo(Dest);
end;

{ TModemCompressions }

constructor TModemCompressions.Create(ADriver: TModemDriver);
begin
  inherited Create(TModemCompression);
  FDriver := ADriver;
end;

function TModemCompressions.GetItem(Index: Integer): TModemCompression;
begin
  Result := TModemCompression(inherited GetItem(Index));
end;

procedure TModemCompressions.SetItem(Index: Integer; Value: TModemCompression);
begin
  inherited SetItem(Index, Value);
end;

function TModemCompressions.GetOwner: TPersistent;
begin
  Result := FDriver;
end;

function TModemCompressions.Add(Compression: Integer; const Name: string;
  const Command, Answer: string; Speed, Bits, Flags: Integer;
  WaveFormat, WaveSpeed, WaveFlags: Integer): TModemCompression;
begin
  if not Find(Compression, Result) then Result := TModemCompression(inherited Add);
  Result.Compression := Compression;
  Result.Name := Name;
  Result.Command := Command;
  Result.Answer := Answer;
  Result.Speed := Speed;
  Result.Bits := Bits;
  Result.Flags := Flags;
  Result.WaveFormat := WaveFormat;
  Result.WaveSpeed := WaveSpeed;
  Result.WaveFlags := WaveFlags;
end;

procedure TModemCompressions.Delete(Compression: Integer);
var
  Item: TModemCompression;
begin
  if Find(Compression, Item) then Item.Free;
end;

function TModemCompressions.Find(Compression: Integer; var Item: TModemCompression): Boolean;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
  begin
    Item := Items[I];
    if Item.Compression <> 0 then
      if (Item.Compression = Compression) or
        ((Compression = 0) and (Item.Flags and CF_DEFAULT <> 0)) then
      begin
        Result := True;
        Exit;
      end;
  end;
  Item := nil;
  Result := False;
end;

function TModemCompressions.FindWave(WaveFormat: Integer; var Item: TModemCompression): Boolean;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
  begin
    Item := Items[I];
    if Item.WaveFormat <> 0 then
      if (Item.WaveFormat = WaveFormat) or
        ((WaveFormat = 0) and (Item.WaveFlags and CF_DEFAULT <> 0)) then
      begin
        Result := True;
        Exit;
      end;
  end;
  Item := nil;
  Result := False;
end;

procedure TModemCompressions.ReadData(Reader: TModemReader);
var
  KeyList: TStringList;
  I, P: Integer;
  Value: string;
  Compression: Integer;
  Name, Command, Answer: string;
  Speed, Bits, Flags: Integer;
  WaveFormat, WaveSpeed, WaveFlags: Integer;
begin
  KeyList := TStringList.Create;
  try
    Reader.ReadSection('Compressions', KeyList);
    Clear;
    { Read compressions }
    for I := 0 to KeyList.Count - 1 do
    begin
      { Read value string }
      Value := Reader.ReadString('Compressions', KeyList[I], '');
      if Length(Value) <> 0 then
      begin
        P := 1;
        { Parse value string }
        Compression := ParseIniInteger(Value, ',', P, 0);
        Name := ParseIniFrameStr(Value, ',', P, '"', '');
        Command := ParseIniFrameStr(Value, ',', P, '"', '');
        Answer := ParseIniFrameStr(Value, ',', P, '"', '');
        Speed := ParseIniInteger(Value, ',', P, 0);
        Bits := ParseIniInteger(Value, ',', P, 0);
        Flags := ParseIniInteger(Value, ',', P, 0);
        WaveFormat := ParseIniInteger(Value, ',', P, 0);
        WaveSpeed := ParseIniInteger(Value, ',', P, 0);
        WaveFlags := ParseIniInteger(Value, ',', P, 0);
        { Add compression to the list }
        Add(Compression, Name, Command, Answer, Speed, Bits, Flags,
          WaveFormat, WaveSpeed, WaveFlags);
      end;
    end;
  finally
    KeyList.Free;
  end;
end;

procedure TModemCompressions.WriteData(Writer: TModemWriter);
var
  I: Integer;
  Item: TModemCompression;
  CMS, NS, CS, CAS, SS, BS, FS, WMS, WSS, WFS, Value: string;
begin
  { Clear compressions section }
  Writer.EraseSection('Compressions');
  { Write list }
  for I := 0 to Count - 1 do
  begin
    Item := Items[I];
    if Item.Compression <> 0 then
    begin
      { Format compression strings }
      with Item do
      begin
        CMS := IntToStr(Compression);
        NS := FrameString(Name, '"');
        CS := FrameString(MapControlCodes(Command, '<>', nil), '"');
        CAS := FrameString(MapControlCodes(Answer, '<>', nil), '"');
        SS := IntToStr(Speed);
        BS := IntToStr(Bits);
        FS := Format('$%0.4x', [Flags]);
        WMS := Format('$%0.4x', [WaveFormat]);
        WSS := IntToStr(WaveSpeed);
        WFS := Format('$%0.4x', [WaveFlags]);
      end;
      { Format value string }
      Value := CMS + ',' + NS + ',' + CS + ',' + CAS + ',' + SS + ',' +
        BS + ',' + FS + ',' + WMS + ',' + WSS + ',' + WFS;
      { Write value }
      Writer.WriteString('Compressions', IntToStr(I + 1), Value);
    end;
  end;
end;

{ TModemDriver }

constructor TModemDriver.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FResponses := TModemResponses.Create(Self);
  FDLECodes := TModemDLECodes.Create(Self);
  FDevices := TModemDevices.Create(Self);
  FCompressions := TModemCompressions.Create(Self);
end;

destructor TModemDriver.Destroy;
begin
  FCompressions.Free;
  FDevices.Free;
  FDLECodes.Free;
  FResponses.Free;
  inherited Destroy;
end;

function TModemDriver.GetProfile: TModemProfile;
begin
  if FProfile <> nil then
  begin
    Result := FProfile;
    Exit;
  end;
  if not (csDesigning in ComponentState) and (Modem <> nil) then
  begin
    Result := Modem.Profile;
    Exit;
  end;
  Result := nil;
end;

function TModemDriver.IsResponsesStored: Boolean;
begin
  Result := Responses.Count > 0;
end;

function TModemDriver.IsDLECodesStored: Boolean;
begin
  Result := DLECodes.Count > 0;
end;

function TModemDriver.IsDevicesStored: Boolean;
begin
  Result := Devices.Count > 0;
end;

function TModemDriver.IsCompressionsStored: Boolean;
begin
  Result := Compressions.Count > 0;
end;

procedure TModemDriver.SetCompressions(Value: TModemCompressions);
begin
  FCompressions.Assign(Value);
end;

procedure TModemDriver.SetDevices(Value: TModemDevices);
begin
  FDevices.Assign(Value);
end;

procedure TModemDriver.SetDLECodes(Value: TModemDLECodes);
begin
  FDLECodes.Assign(Value);
end;

procedure TModemDriver.SetModem(Value: TVoiceModem);
var
  SModem: TVoiceModem;
begin
  if FModem <> Value then
  begin
    SModem := FModem;
    FModem := Value;
    if SModem <> nil then SModem.Driver := nil;
    if FModem <> nil then FModem.Driver := Self;
  end;
end;

procedure TModemDriver.SetResponses(Value: TModemResponses);
begin
  FResponses.Assign(Value);
end;

procedure TModemDriver.DebugMsg(Msg: Integer; const S: string);
begin
  { Redirect message to modem }
  if FModem <> nil then FModem.DebugMsg(Msg, S);
end;

procedure TModemDriver.DebugMsgFmt(Msg: Integer; const S: string; const Args: array of const);
begin
  { Redirect message to modem }
  if FModem <> nil then FModem.DebugMsgFmt(Msg, S, Args);
end;

procedure TModemDriver.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
  begin
    if AComponent = FModem then FModem := nil;
    if AComponent = FProfile then FProfile := nil;
  end;
end;

procedure TModemDriver.Progress(Done: Longint);
begin
  { Redirect event to modem }
  if FModem <> nil then FModem.Progress(Done);
end;

procedure TModemDriver.YieldProc;
begin
  { Redirect event to modem }
  if FModem <> nil then FModem.YieldProc;
end;

function TModemDriver.Analyze(const Buffer: string): Integer;
begin
  Result := VMA_UNKNOWN;
end;

function TModemDriver.CheckModem(const CheckFunc: string; DebugError: Boolean): Boolean;
begin
  Result := Modem <> nil;
  { Generate error }
  if (not Result) and DebugError then DebugMsgFmt(L_ERROR, SNoModemError, [CheckFunc]);
end;

function TModemDriver.GetCompression(const Name: string): Integer;
var
  I: Integer;
  ModemCompression: TModemCompression;
begin
  { Empty name length is default compression }
  if Length(Name) = 0 then
  begin
    Result := 0;
    Exit;
  end;
  { Find compression number }
  for I := 0 to Compressions.Count - 1 do
  begin
    ModemCompression := Compressions[I];
    if ModemCompression.Compression <> 0 then
      if AnsiCompareText(ModemCompression.Name, Name) = 0 then
      begin
         Result := ModemCompression.Compression;
         Exit;
      end;
  end;
  { Convert name to number }
  Result := StrToIntDef(Name, -1);
end;

function TModemDriver.GetCompressionName(Compression: Integer): string;
var
  I: Integer;
  ModemCompression: TModemCompression;
begin
  for I := 0 to Compressions.Count - 1 do
  begin
    ModemCompression := Compressions[I];
    if ModemCompression.Compression <> 0 then
      if ModemCompression.Compression = Compression then
      begin
         Result := ModemCompression.Name;
         Exit;
      end;
  end;
  Result := IntToStr(Compression);
end;

function TModemDriver.GetCompressionsList(Proc: TGetStrProc): Integer;
var
  I: Integer;
  ModemCompression: TModemCompression;
begin
  Result := 0;
  for I := 0 to Compressions.Count - 1 do
  begin
    ModemCompression := Compressions[I];
    if ModemCompression.Compression <> 0 then
      if ModemCompression.Flags and CF_HIDDEN = 0 then
      begin
        Proc(ModemCompression.Name);
        Inc(Result);
      end;
  end;
end;

function TModemDriver.GetCompressionsStringList(Strings: TStrings): Integer;
type
  TGetStrFunc = function(const S: string): Integer of object;
var
  Proc: TGetStrFunc;
begin
  Proc := Strings.Add;
  Result := GetCompressionsList(TGetStrProc(Proc));
end;

function TModemDriver.GetDevice(const Name: string): Integer;
var
  I: Integer;
  ModemDevice: TModemDevice;
begin
  for I := 0 to Devices.Count - 1 do
  begin
    ModemDevice := Devices[I];
    if ModemDevice.Device <> 0 then
      if AnsiCompareText(ModemDevice.Name, Name) = 0 then
      begin
         Result := ModemDevice.Device;
         Exit;
      end;
  end;
  Result := StrToIntDef(Name, -1);
end;

function TModemDriver.GetDeviceName(Device: Integer): string;
var
  I: Integer;
  ModemDevice: TModemDevice;
begin
  for I := 0 to Devices.Count - 1 do
  begin
    ModemDevice := Devices[I];
    if ModemDevice.Device <> 0 then
      if ModemDevice.Device = Device then
      begin
         Result := ModemDevice.Name;
         Exit;
      end;
  end;
  Result := IntToStr(Device);
end;

function TModemDriver.GetDevicesList(Proc: TGetStrProc): Integer;
var
  I: Integer;
  ModemDevice: TModemDevice;
begin
  Result := 0;
  for I := 0 to Devices.Count - 1 do
  begin
    ModemDevice := Devices[I];
    if ModemDevice.Device <> 0 then
      if ModemDevice.Flags and DF_HIDDEN = 0 then
      begin
        Proc(ModemDevice.Name);
        Inc(Result);
      end;
  end;
end;

function TModemDriver.GetDevicesStringList(Strings: TStrings): Integer;
type
  TGetStrFunc = function(const S: string): Integer of object;
var
  Proc: TGetStrFunc;
begin
  Proc := Strings.Add;
  Result := GetDevicesList(TGetStrProc(Proc));
end;

function TModemDriver.GetPlayDevicesList(Proc: TGetStrProc): Integer;
var
  I: Integer;
  ModemDevice: TModemDevice;
begin
  Result := 0;
  for I := 0 to Devices.Count - 1 do
  begin
    ModemDevice := Devices[I];
    if ModemDevice.Device <> 0 then
      if (ModemDevice.Flags and DF_HIDDEN = 0) and (ModemDevice.Flags and DF_TRANSMIT <> 0) then
      begin
        Proc(ModemDevice.Name);
        Inc(Result);
      end;
  end;
end;

function TModemDriver.GetPlayDevicesStringList(Strings: TStrings): Integer;
type
  TGetStrFunc = function(const S: string): Integer of object;
var
  Proc: TGetStrFunc;
begin
  Proc := Strings.Add;
  Result := GetPlayDevicesList(TGetStrProc(Proc));
end;

function TModemDriver.GetRecDevicesList(Proc: TGetStrProc): Integer;
var
  I: Integer;
  ModemDevice: TModemDevice;
begin
  Result := 0;
  for I := 0 to Devices.Count - 1 do
  begin
    ModemDevice := Devices[I];
    if ModemDevice.Device <> 0 then
      if (ModemDevice.Flags and DF_HIDDEN = 0) and (ModemDevice.Flags and DF_RECEIVE <> 0) then
      begin
        Proc(ModemDevice.Name);
        Inc(Result);
      end;
  end;
end;

function TModemDriver.GetRecDevicesStringList(Strings: TStrings): Integer;
type
  TGetStrFunc = function(const S: string): Integer of object;
var
  Proc: TGetStrFunc;
begin
  Proc := Strings.Add;
  Result := GetRecDevicesList(TGetStrProc(Proc));
end;

function TModemDriver.GetSpeedList(Compression: Integer; Proc: TGetStrProc): Integer;
var
  ModemCompression: TModemCompression;

  procedure CheckFlag(Flag: Integer; const SpeedName: string);
  begin
    if (ModemCompression.Flags and Flag) <> 0 then
    begin
      Proc(SpeedName);
      Inc(Result);
    end;
  end;

begin
  Result := 0;
  if Compressions.Find(Compression, ModemCompression) then
  begin
    CheckFlag(CF_7200, '7200');
    CheckFlag(CF_8000, '8000');
    CheckFlag(CF_9600, '9600');
    CheckFlag(CF_11025, '11025');
  end;
end;

function TModemDriver.GetSpeedStringList(Compression: Integer; Strings: TStrings): Integer;
type
  TGetStrFunc = function(const S: string): Integer of object;
var
  Proc: TGetStrFunc;
begin
  Proc := Strings.Add;
  Result := GetSpeedList(Compression, TGetStrProc(Proc));
end;

function TModemDriver.GetWaveFormat(const Name: string): Integer;
var
  I: Integer;
  ModemCompression: TModemCompression;
begin
  { Empty name length is default format }
  if Length(Name) = 0 then
  begin
    Result := 0;
    Exit;
  end;
  { Find format }
  for I := 0 to Compressions.Count - 1 do
  begin
    ModemCompression := Compressions[I];
    if ModemCompression.WaveFormat <> 0 then
      if AnsiCompareText(ModemCompression.Name, Name) = 0 then
      begin
         Result := ModemCompression.WaveFormat;
         Exit;
      end;
  end;
  { Convert name to number }
  Result := StrToIntDef(Name, -1);
end;

function TModemDriver.GetWaveFormatName(WaveFormat: Integer): string;
var
  I: Integer;
  ModemCompression: TModemCompression;
begin
  for I := 0 to Compressions.Count - 1 do
  begin
    ModemCompression := Compressions[I];
    if ModemCompression.WaveFormat <> 0 then
      if ModemCompression.WaveFormat = WaveFormat then
      begin
         Result := ModemCompression.Name;
         Exit;
      end;
  end;
  Result := IntToStr(WaveFormat);
end;

function TModemDriver.GetWaveFormatsList(Proc: TGetStrProc): Integer;
var
  I: Integer;
  ModemCompression: TModemCompression;
begin
  Result := 0;
  for I := 0 to Compressions.Count - 1 do
  begin
    ModemCompression := Compressions[I];
    if ModemCompression.WaveFormat <> 0 then
      if ModemCompression.WaveFlags and CF_HIDDEN = 0 then
      begin
        Proc(ModemCompression.Name);
        Inc(Result);
      end;
  end;
end;

function TModemDriver.GetWaveFormatsStringList(Strings: TStrings): Integer;
type
  TGetStrFunc = function(const S: string): Integer of object;
var
  Proc: TGetStrFunc;
begin
  Proc := Strings.Add;
  Result := GetWaveFormatsList(TGetStrProc(Proc));
end;

function TModemDriver.GetWaveSpeedList(WaveFormat: Integer; Proc: TGetStrProc): Integer;
var
  ModemCompression: TModemCompression;

  procedure CheckFlag(Flag: Integer; const SpeedName: string);
  begin
    if (ModemCompression.WaveFlags and Flag) <> 0 then
    begin
      Proc(SpeedName);
      Inc(Result);
    end;
  end;

begin
  Result := 0;
  if Compressions.FindWave(WaveFormat, ModemCompression) then
  begin
    CheckFlag(CF_7200, '7200');
    CheckFlag(CF_8000, '8000');
    CheckFlag(CF_9600, '9600');
    CheckFlag(CF_11025, '11025');
  end;
end;

function TModemDriver.GetWaveSpeedStringList(WaveFormat: Integer; Strings: TStrings): Integer;
type
  TGetStrFunc = function(const S: string): Integer of object;
var
  Proc: TGetStrFunc;
begin
  Proc := Strings.Add;
  Result := GetWaveSpeedList(WaveFormat, TGetStrProc(Proc));
end;

function TModemDriver.GetVoiceLen(Compression, Speed, Bits: Integer; DataSize: Longint): Integer;
begin
  Result := 0;
end;

function TModemDriver.PlayStream(Stream: TStream; BPS: Integer): Integer;
begin
  Result := PlayStreamEx(Stream, BPS, 0);
end;

procedure TModemDriver.ReadData(Reader: TModemReader);
begin
  with Reader do
  begin
    { Answer }
    PickPhoneCmd := ReadString('Answer', 'PickPhoneCmd', PickPhoneCmd);
    PickPhoneAnsw := ReadString('Answer', 'PickPhoneAnsw', PickPhoneAnsw);
    { Hangup }
    HangUpCmd := ReadString('Hangup', 'HangUpCmd', HangUpCmd);
    HangUpAnsw := ReadString('Hangup', 'HangUpAnsw', HangUpAnsw);
    { Init }
    HardFlowCmd := ReadString('Init', 'HardFlowCmd', HardFlowCmd);
    HardFlowAnsw := ReadString('Init', 'HardFlowAnsw', HardFlowAnsw);
    SoftFlowCmd := ReadString('Init', 'SoftFlowCmd', SoftFlowCmd);
    SoftFlowAnsw := ReadString('Init', 'SoftFlowAnsw', SoftFlowAnsw);
    ResultCodesCmd := ReadString('Init', 'ResultCodesCmd', ResultCodesCmd);
    ResultCodesAnsw := ReadString('Init', 'ResultCodesAnsw', ResultCodesAnsw);
    DevInitCmd := ReadString('Init', 'DevInitCmd', DevInitCmd);
    DevInitAnsw := ReadString('Init', 'DevInitAnsw', DevInitAnsw);
    { InitVoice }
    SilenceDetectCmd := ReadString('InitVoice', 'SilenceDetectCmd', SilenceDetectCmd);
    SilenceDetectAnsw := ReadString('InitVoice', 'SilenceDetectAnsw', SilenceDetectAnsw);
    SilenceSensMin := ReadInteger('InitVoice', 'SilenceSensMin', SilenceSensMin);
    SilenceSensMax := ReadInteger('InitVoice', 'SilenceSensMax', SilenceSensMax);
    SilenceDeleteCmd := ReadString('InitVoice', 'SilenceDeleteCmd', SilenceDeleteCmd);
    SilenceDeleteAnsw := ReadString('InitVoice', 'SilenceDeleteAnsw', SilenceDeleteAnsw);
    TransmitGainCmd := ReadString('InitVoice', 'TransmitGainCmd', TransmitGainCmd);
    TransmitGainAnsw := ReadString('InitVoice', 'TransmitGainAnsw', TransmitGainAnsw);
    TransmitGainMin := ReadInteger('InitVoice', 'TransmitGainMin', TransmitGainMin);
    TransmitGainMax := ReadInteger('InitVoice', 'TransmitGainMax', TransmitGainMax);
    ReceiveGainCmd := ReadString('InitVoice', 'ReceiveGainCmd', ReceiveGainCmd);
    ReceiveGainAnsw := ReadString('InitVoice', 'ReceiveGainAnsw', ReceiveGainAnsw);
    ReceiveGainMin := ReadInteger('InitVoice', 'ReceiveGainMin', ReceiveGainMin);
    ReceiveGainMax := ReadInteger('InitVoice', 'ReceiveGainMax', ReceiveGainMax);
    RingbackTimersCmd := ReadString('InitVoice', 'RingbackTimersCmd', RingbackTimersCmd);
    RingbackTimersAnsw := ReadString('InitVoice', 'RingbackTimersAnsw', RingbackTimersAnsw);
    DTMFDetectCmd := ReadString('InitVoice', 'DTMFDetectCmd', DTMFDetectCmd);
    DTMFDetectAnsw := ReadString('InitVoice', 'DTMFDetectAnsw', DTMFDetectAnsw);
    DTMFSensMin := ReadInteger('InitVoice', 'DTMFSensMin', DTMFSensMin);
    DTMFSensMax := ReadInteger('InitVoice', 'DTMFSensMax', DTMFSensMax);
    DTMFLenUnit := ReadInteger('InitVoice', 'DTMFLenUnit', DTMFLenUnit);
    DevInitVoiceCmd := ReadString('InitVoice', 'DevInitVoiceCmd', DevInitVoiceCmd);
    DevInitVoiceAnsw := ReadString('InitVoice', 'DevInitVoiceAnsw', DevInitVoiceAnsw);
    RestoreVoiceCmd := ReadString('InitVoice', 'RestoreVoiceCmd', RestoreVoiceCmd);
    RestoreVoiceAnsw := ReadString('InitVoice', 'RestoreVoiceAnsw', RestoreVoiceAnsw);
    { Settings }
    InactivityTimerCmd := ReadString('Settings', 'InactivityTimerCmd', InactivityTimerCmd);
    InactivityTimerAnsw := ReadString('Settings', 'InactivityTimerAnsw', InactivityTimerAnsw);
    BaudRateCmd := ReadString('Settings', 'BaudRateCmd', BaudRateCmd);
    BaudRateAnsw := ReadString('Settings', 'BaudRateAnsw', BaudRateAnsw);
    BaudRateUnit := ReadInteger('Settings', 'BaudRateUnit', BaudRateUnit);
    { VoiceMode }
    SwitchModeCmd := ReadString('VoiceMode', 'SwitchModeCmd', SwitchModeCmd);
    SwitchModeAnsw := ReadString('VoiceMode', 'SwitchModeAnsw', SwitchModeAnsw);
    VoiceModeOnCmd := ReadString('VoiceMode', 'VoiceModeOnCmd', VoiceModeOnCmd);
    VoiceModeOnAnsw := ReadString('VoiceMode', 'VoiceModeOnAnsw', VoiceModeOnAnsw);
    VoiceModeOffCmd := ReadString('VoiceMode', 'VoiceModeOffCmd', VoiceModeOffCmd);
    VoiceModeOffAnsw := ReadString('VoiceMode', 'VoiceModeOffAnsw', VoiceModeOffAnsw);
    { Beep }
    BeepCmd := ReadString('Beep', 'BeepCmd', BeepCmd);
    BeepAnsw := ReadString('Beep', 'BeepAnsw', BeepAnsw);
    BeepTimeUnit := ReadInteger('Beep', 'BeepTimeUnit', BeepTimeUnit);
    { Play }
    StartPlayCmd := ReadString('Play', 'StartPlayCmd', StartPlayCmd);
    StartPlayAnsw := ReadString('Play', 'StartPlayAnsw', StartPlayAnsw);
    IntrPlayCmd := ReadString('Play', 'IntrPlayCmd', IntrPlayCmd);
    IntrPlayAnsw := ReadString('Play', 'IntrPlayAnsw', IntrPlayAnsw);
    StopPlayCmd := ReadString('Play', 'StopPlayCmd', StopPlayCmd);
    StopPlayAnsw := ReadString('Play', 'StopPlayAnsw', StopPlayAnsw);
    { Record }
    StartRecCmd := ReadString('Record', 'StartRecCmd', StartRecCmd);
    StartRecAnsw := ReadString('Record', 'StartRecAnsw', StartRecAnsw);
    StopRecCmd := ReadString('Record', 'StopRecCmd', StopRecCmd);
    StopRecAnsw := ReadString('Record', 'StopRecAnsw', StopRecAnsw);
    { PlayDTMF }
    PlayDTMFCmd := ReadString('PlayDTMF', 'PlayDTMFCmd', PlayDTMFCmd);
    PlayDTMFAnsw := ReadString('PlayDTMF', 'PlayDTMFAnsw', PlayDTMFAnsw);
    PlayDTMFExtra := ReadString('PlayDTMF', 'PlayDTMFExtra', PlayDTMFExtra);
    { Dial }
    DialSetupCmd := ReadString('Dial', 'DialSetupCmd', DialSetupCmd);
    DialSetupAnsw := ReadString('Dial', 'DialSetupAnsw', DialSetupAnsw);
    { CallerID }
    CallerIDOnCmd := ReadString('CallerID', 'CallerIDOnCmd', CallerIDOnCmd);
    CallerIDOnAnsw := ReadString('CallerID', 'CallerIDOnAnsw', CallerIDOnAnsw);
    CallerIDOffCmd := ReadString('CallerID', 'CallerIDOffCmd', CallerIDOffCmd);
    CallerIDOffAnsw := ReadString('CallerID', 'CallerIDOffAnsw', CallerIDOffAnsw);
    CallerIDFormat := ReadString('CallerID', 'CallerIDFormat', CallerIDFormat);
    { DistinctiveRing }
    DRingOnCmd := ReadString('DistinctiveRing', 'DRingOnCmd', DRingOnCmd);
    DRingOnAnsw := ReadString('DistinctiveRing', 'DRingOnAnsw', DRingOnAnsw);
    DRingOffCmd := ReadString('DistinctiveRing', 'DRingOffCmd', DRingOffCmd);
    DRingOffAnsw := ReadString('DistinctiveRing', 'DRingOffAnsw', DRingOffAnsw);
    { Responses }
    Responses.ReadData(Reader);
    { DLECodes }
    DLECodes.ReadData(Reader);
    { Devices }
    Devices.ReadData(Reader);
    { Compressions }
    Compressions.ReadData(Reader);
  end;
end;

function TModemDriver.RecordStream(Stream: TStream; BPS: Integer): Integer;
var
  DataLength: Longint;
begin
  Result := RecordStreamEx(Stream, BPS, DataLength);
end;

procedure TModemDriver.WriteData(Writer: TModemWriter);
begin
  with Writer do
  begin
    { Answer }
    WriteMapStr('Answer', 'PickPhoneCmd', PickPhoneCmd);
    WriteMapStr('Answer', 'PickPhoneAnsw', PickPhoneAnsw);
    { Hangup }
    WriteMapStr('Hangup', 'HangUpCmd', HangUpCmd);
    WriteMapStr('Hangup', 'HangUpAnsw', HangUpAnsw);
    { Init }
    WriteMapStr('Init', 'HardFlowCmd', HardFlowCmd);
    WriteMapStr('Init', 'HardFlowAnsw', HardFlowAnsw);
    WriteMapStr('Init', 'SoftFlowCmd', SoftFlowCmd);
    WriteMapStr('Init', 'SoftFlowAnsw', SoftFlowAnsw);
    WriteMapStr('Init', 'ResultCodesCmd', ResultCodesCmd);
    WriteMapStr('Init', 'ResultCodesAnsw', ResultCodesAnsw);
    WriteMapStr('Init', 'DevInitCmd', DevInitCmd);
    WriteMapStr('Init', 'DevInitAnsw', DevInitAnsw);
    { InitVoice }
    WriteMapStr('InitVoice', 'SilenceDetectCmd', SilenceDetectCmd);
    WriteMapStr('InitVoice', 'SilenceDetectAnsw', SilenceDetectAnsw);
    WriteInteger('InitVoice', 'SilenceSensMin', SilenceSensMin);
    WriteInteger('InitVoice', 'SilenceSensMax', SilenceSensMax);
    WriteMapStr('InitVoice', 'SilenceDeleteCmd', SilenceDeleteCmd);
    WriteMapStr('InitVoice', 'SilenceDeleteAnsw', SilenceDeleteAnsw);
    WriteMapStr('InitVoice', 'TransmitGainCmd', TransmitGainCmd);
    WriteMapStr('InitVoice', 'TransmitGainAnsw', TransmitGainAnsw);
    WriteInteger('InitVoice', 'TransmitGainMin', TransmitGainMin);
    WriteInteger('InitVoice', 'TransmitGainMax', TransmitGainMax);
    WriteMapStr('InitVoice', 'ReceiveGainCmd', ReceiveGainCmd);
    WriteMapStr('InitVoice', 'ReceiveGainAnsw', ReceiveGainAnsw);
    WriteInteger('InitVoice', 'ReceiveGainMin', ReceiveGainMin);
    WriteInteger('InitVoice', 'ReceiveGainMax', ReceiveGainMax);
    WriteMapStr('InitVoice', 'RingbackTimersCmd', RingbackTimersCmd);
    WriteMapStr('InitVoice', 'RingbackTimersAnsw', RingbackTimersAnsw);
    WriteMapStr('InitVoice', 'DTMFDetectCmd', DTMFDetectCmd);
    WriteMapStr('InitVoice', 'DTMFDetectAnsw', DTMFDetectAnsw);
    WriteInteger('InitVoice', 'DTMFSensMin', DTMFSensMin);
    WriteInteger('InitVoice', 'DTMFSensMax', DTMFSensMax);
    WriteInteger('InitVoice', 'DTMFLenUnit', DTMFLenUnit);
    WriteMapStr('InitVoice', 'DevInitVoiceCmd', DevInitVoiceCmd);
    WriteMapStr('InitVoice', 'DevInitVoiceAnsw', DevInitVoiceAnsw);
    WriteMapStr('InitVoice', 'RestoreVoiceCmd', RestoreVoiceCmd);
    WriteMapStr('InitVoice', 'RestoreVoiceAnsw', RestoreVoiceAnsw);
    { Settings }
    WriteMapStr('Settings', 'InactivityTimerCmd', InactivityTimerCmd);
    WriteMapStr('Settings', 'InactivityTimerAnsw', InactivityTimerAnsw);
    WriteMapStr('Settings', 'BaudRateCmd', BaudRateCmd);
    WriteMapStr('Settings', 'BaudRateAnsw', BaudRateAnsw);
    WriteInteger('Settings', 'BaudRateUnit', BaudRateUnit);
    { VoiceMode }
    WriteMapStr('VoiceMode', 'SwitchModeCmd', SwitchModeCmd);
    WriteMapStr('VoiceMode', 'SwitchModeAnsw', SwitchModeAnsw);
    WriteMapStr('VoiceMode', 'VoiceModeOnCmd', VoiceModeOnCmd);
    WriteMapStr('VoiceMode', 'VoiceModeOnAnsw', VoiceModeOnAnsw);
    WriteMapStr('VoiceMode', 'VoiceModeOffCmd', VoiceModeOffCmd);
    WriteMapStr('VoiceMode', 'VoiceModeOffAnsw', VoiceModeOffAnsw);
    { Beep }
    WriteMapStr('Beep', 'BeepCmd', BeepCmd);
    WriteMapStr('Beep', 'BeepAnsw', BeepAnsw);
    WriteInteger('Beep', 'BeepTimeUnit', BeepTimeUnit);
    { Play }
    WriteMapStr('Play', 'StartPlayCmd', StartPlayCmd);
    WriteMapStr('Play', 'StartPlayAnsw', StartPlayAnsw);
    WriteMapStr('Play', 'IntrPlayCmd', IntrPlayCmd);
    WriteMapStr('Play', 'IntrPlayAnsw', IntrPlayAnsw);
    WriteMapStr('Play', 'StopPlayCmd', StopPlayCmd);
    WriteMapStr('Play', 'StopPlayAnsw', StopPlayAnsw);
    { Record }
    WriteMapStr('Record', 'StartRecCmd', StartRecCmd);
    WriteMapStr('Record', 'StartRecAnsw', StartRecAnsw);
    WriteMapStr('Record', 'StopRecCmd', StopRecCmd);
    WriteMapStr('Record', 'StopRecAnsw', StopRecAnsw);
    { PlayDTMF }
    WriteMapStr('PlayDTMF', 'PlayDTMFCmd', PlayDTMFCmd);
    WriteMapStr('PlayDTMF', 'PlayDTMFAnsw', PlayDTMFAnsw);
    WriteMapStr('PlayDTMF', 'PlayDTMFExtra', PlayDTMFExtra);
    { Dial }
    WriteMapStr('Dial', 'DialSetupCmd', DialSetupCmd);
    WriteMapStr('Dial', 'DialSetupAnsw', DialSetupAnsw);
    { CallerID }
    WriteMapStr('CallerID', 'CallerIDOnCmd', CallerIDOnCmd);
    WriteMapStr('CallerID', 'CallerIDOnAnsw', CallerIDOnAnsw);
    WriteMapStr('CallerID', 'CallerIDOffCmd', CallerIDOffCmd);
    WriteMapStr('CallerID', 'CallerIDOffAnsw', CallerIDOffAnsw);
    WriteMapStr('CallerID', 'CallerIDFormat', CallerIDFormat);
    { DistinctiveRing }
    WriteMapStr('DistinctiveRing', 'DRingOnCmd', DRingOnCmd);
    WriteMapStr('DistinctiveRing', 'DRingOnAnsw', DRingOnAnsw);
    WriteMapStr('DistinctiveRing', 'DRingOffCmd', DRingOffCmd);
    WriteMapStr('DistinctiveRing', 'DRingOffAnsw', DRingOffAnsw);
    { Responces }
    Responses.WriteData(Writer);
    { DLECodes }
    DLECodes.WriteData(Writer);
    { Devices }
    Devices.WriteData(Writer);
    { Compressions }
    Compressions.WriteData(Writer);
  end;
end;

{ TVoiceModem }

constructor TVoiceModem.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  { Init event queue }
  FEventQueue := TEventQueue.Create;
  { Init modem state }
  FState := IDLE;
  { Init critical sections }
  InitializeCriticalSection(FStateSection);
  InitializeCriticalSection(FLockSection);
  InitializeCriticalSection(FLastCallerIDSection);
  InitializeCriticalSection(FLastDTMFSection);
end;

destructor TVoiceModem.Destroy;
begin
  DeleteCriticalSection(FLastDTMFSection);
  DeleteCriticalSection(FLastCallerIDSection);
  DeleteCriticalSection(FLockSection);
  DeleteCriticalSection(FStateSection);
  FEventQueue.Free;
  inherited Destroy;
end;

function TVoiceModem.GetDriver: TModemDriver;
begin
  if (not (csDesigning in ComponentState)) and (FDriver = nil) then
  begin
    FDriver := InternalCreateDriver;
    if FDriver <> nil then
    begin
      if FDriver.Owner = nil then InsertComponent(FDriver);
      FDriver.Modem := Self;
      FDriver.Profile := Profile;
    end;
  end;
  Result := FDriver;
end;

function TVoiceModem.GetLastCallerID: string;
begin
  EnterCriticalSection(FLastCallerIDSection);
  Result := FLastCallerID;
  LeaveCriticalSection(FLastCallerIDSection);
end;

function TVoiceModem.GetLastDTMF: string;
begin
  EnterCriticalSection(FLastDTMFSection);
  Result := FLastDTMF;
  LeaveCriticalSection(FLastDTMFSection);
end;

function TVoiceModem.GetLastErrorStr: string;
begin
  Result := GetVMLErrorStr(LastError);
end;

function TVoiceModem.GetLocked: Boolean;
begin
  EnterCriticalSection(FLockSection);
  Result := FLocked;
  LeaveCriticalSection(FLockSection);
end;

function TVoiceModem.GetLockOwner: TObject;
begin
  EnterCriticalSection(FLockSection);
  Result := FLockOwner;
  LeaveCriticalSection(FLockSection);
end;

function TVoiceModem.GetOpened: Boolean;
begin
  Result := Port.Enabled;
end;

function TVoiceModem.GetPort: TModemPort;
begin
  if (not (csDesigning in ComponentState)) and (FPort = nil) then
  begin
    FPort := InternalCreatePort;
    if FPort <> nil then
    begin
      if FPort.Owner = nil then InsertComponent(FPort);
      FPort.Profile := Profile;
    end;
  end;
  Result := FPort;
end;

function TVoiceModem.GetProfile: TModemProfile;
begin
  if (not (csDesigning in ComponentState)) and (FProfile = nil) then
  begin
    FProfile := InternalCreateProfile;
    if FProfile <> nil then
    begin
      if FProfile.Owner = nil then InsertComponent(FProfile);
      if (FPort <> nil) and (FPort.Profile = nil) then FPort.Profile := FProfile;
      if (FDriver <> nil) and (FDriver.Profile = nil) then FDriver.Profile := FProfile;
    end;
  end;
  Result := FProfile;
end;

function TVoiceModem.GetState: Integer;
begin
  EnterCriticalSection(FStateSection);
  Result := FState;
  LeaveCriticalSection(FStateSection);
end;

procedure TVoiceModem.SetDriver(Value: TModemDriver);
var
  SDriver: TModemDriver;
begin
  if FDriver <> Value then
  begin
    SDriver := FDriver;
    { Set new profile }
    FDriver := Value;
    if FDriver <> nil then FDriver.FModem := Self;
    { Free default profile component }
    if SDriver <> nil then
    begin
      SDriver.FModem := nil;
      if SDriver.Owner = Self then SDriver.Free;
    end;
  end;
end;

procedure TVoiceModem.SetLastCallerID(const Value: string);
begin
  EnterCriticalSection(FLastCallerIDSection);
  FLastCallerID := Value;
  LeaveCriticalSection(FLastCallerIDSection);
end;

procedure TVoiceModem.SetLastDTMF(const Value: string);
begin
  EnterCriticalSection(FLastDTMFSection);
  FLastDTMF := Value;
  LeaveCriticalSection(FLastDTMFSection);
end;

procedure TVoiceModem.SetLastError(Value: Integer);
begin
  InterlockedExchange(FLastError, Value);
end;

procedure TVoiceModem.SetLogFile(Value: TModemLogFile);
var
  SLogFile: TModemLogFile;
begin
  if FLogFile <> Value then
  begin
    SLogFile := FLogFile;
    { Set new log file }
    FLogFile := Value;
    { Close old log file, free default log component }
    if SLogFile <> nil then
    begin
      SLogFile.Close;
      if SLogFile.Owner = Self then SLogFile.Free;
    end;
  end;
end;

procedure TVoiceModem.SetPort(Value: TModemPort);
var
  SPort: TModemPort;
begin
  if FPort <> Value then
  begin
    SPort := FPort;
    { Set new profile }
    FPort := Value;
    { Free default port component }
    if (SPort <> nil) and (SPort.Owner = Self) then SPort.Free;
  end;
end;

procedure TVoiceModem.SetProfile(Value: TModemProfile);
var
  SProfile: TModemProfile;
begin
  if FProfile <> Value then
  begin
    SProfile := FProfile;
    { Set new profile }
    FProfile := Value;
    { Free default profile component }
    if (SProfile <> nil) and (SProfile.Owner = Self) then SProfile.Free;
  end;
end;

procedure TVoiceModem.SetState(Value: Integer);
begin
  EnterCriticalSection(FStateSection);
  try
    if FState <> Value then
    begin
      FState := Value;
      ChangeState;
    end;
  finally
    LeaveCriticalSection(FStateSection);
  end;
end;

procedure TVoiceModem.SetTraceFile(Value: TModemLogFile);
var
  STraceFile: TModemLogFile;
begin
  if FTraceFile <> Value then
  begin
    STraceFile := FTraceFile;
    { Set new trace file }
    FTraceFile := Value;
    { Close old trace file, free default log component }
    if STraceFile <> nil then
    begin
      STraceFile.Close;
      if STraceFile.Owner = Self then STraceFile.Free;
    end;
  end;
end;

procedure TVoiceModem.ChangeState;
begin
  if not (csDestroying in ComponentState) and Assigned(FOnChangeState) then
  try
    FOnChangeState(Self);
  except
    on E: Exception do DebugMsg(L_ERROR, 'ChangeState: ' + E.Message);
  end;
  { Yield event }
  YieldProc;
end;

procedure TVoiceModem.DebugMsg(Msg: Integer; const S: string);
var
  Str: string;
  Mapped: Boolean;
begin
  Str := S;
  Mapped := False;
  { Write log message }
  if (FLogFile <> nil) and Profile.ModemLogging then
  begin
    { Do control chars mapping only if necessary }
    Str := MapControlCodes(Str, '', @CODEMAP_DEC);
    Mapped := True;
    { Write log line }
    FLogFile.Add(Str);
  end;
  { Call debug event }
  if not (csDestroying in ComponentState) and Assigned(FOnDebugMsg) then
  try
    { Map control chars }
    if not Mapped then Str := MapControlCodes(Str, '', @CODEMAP_DEC);
    { Call event }
    FOnDebugMsg(Self, Msg, Str);
  except
    { Ignore exceptions }
  end;
  { Yield event }
  YieldProc;
end;

procedure TVoiceModem.DebugMsgFmt(Msg: Integer; const S: string; const Args: array of const);
begin
  { The Format() is very slow function. Calling this function can get
    unnecessary delay if there is no debug message handling. }
  if ((FLogFile <> nil) and Profile.ModemLogging) or Assigned(FOnDebugMsg) then
  try
    DebugMsg(Msg, Format(S, Args));
  except
    { Ignore exceptions }
  end;
end;

procedure TVoiceModem.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
  begin
    if AComponent = FPort then FPort := nil;
    if AComponent = FDriver then FDriver := nil;
    if AComponent = FProfile then FProfile := nil;
    if AComponent = FLogFile then FLogFile := nil;
  end;
end;

function TVoiceModem.InternalCreateDriver: TModemDriver;
var
  DriverClass: TModemDriverClass;
begin
  { Load driver from ini file }
  if Length(Profile.ModemIniFile) <> 0 then
  begin
    Result := ReadDriverIniFile(Profile.ModemIniFile, nil);
    { Check driver name }
    if Length(Result.ModemName) <> 0 then Exit;
    { There is no driver loaded }
    Result.Free;
  end;
  { Create registered driver }
  if GetModemDriver(Profile.ModemName, DriverClass) then
  begin
    Result := DriverClass.Create(Self);
    Exit;
  end;
  { Can't create driver automatically }
  Result := nil;
end;

function TVoiceModem.InternalCreatePort: TModemPort;
begin
  Result := TModemPort.Create(Self);
end;

function TVoiceModem.InternalCreateProfile: TModemProfile;
begin
  Result := TModemProfile.Create(Self);
end;

function TVoiceModem.InternalPlayFile(const FileName: string; Compression, Speed, Bits: Integer; ReadHeaderProc: TReadDataHeaderProc; SetCompressionProc: TSetCompressionProc): Integer;
var
  Handle: Integer;
  Stream: THandleStream;
begin
  DebugMsgFmt(L_MESG, SPlayingFile, [FileName]);
  { Check driver specified }
  if not CheckDriver('InternalPlayFile', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Check file }
  if not FileExists(FileName) then
  begin
    DebugMsgFmt(L_ERROR, SFileNotFound, [FileName]);
    LastError := VME_FILE_NOT_FOUND;
    Result := S_FAIL;
    Exit;
  end;
  { Open file }
  Handle := FileOpen(FileName, fmOpenRead or fmShareDenyWrite);
  { Check for file opening }
  if Handle < 0 then
  begin
    DebugMsgFmt(L_ERROR, SFileOpenError, [FileName, GetLastError]);
    { Playing S_FAILed }
    LastError := VME_FILE_OPEN_ERROR;
    Result := S_FAIL;
    Exit;
  end;
  try
    { Playing file }
    Stream := THandleStream.Create(Handle);
    try
      Result := InternalPlayStream(Stream, Compression, Speed, Bits, ReadHeaderProc, SetCompressionProc);
    finally
      Stream.Free;
    end;
  finally
    FileClose(Handle);
  end;
end;

type
  PHandleArray = ^THandleArray;
  THandleArray = array[0..65535] of Integer;

function TVoiceModem.InternalPlayFiles(FileList: TStrings; Compression, Speed, Bits: Integer; ReadHeaderProc: TReadDataHeaderProc; SetCompressionProc: TSetCompressionProc): Integer;
var
  I, Count: Integer;
  Handles: PHandleArray;
  Stream: THandleStream;
  DataLength: Longint;
begin
  { Check driver specified }
  if not CheckDriver('InternalPlayFiles', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Check file count }
  if FileList.Count = 0 then
  begin
    Result := S_OK;
    Exit;
  end;
  { Play files }
  GetMem(Handles, FileList.Count * SizeOf(THandle));
  try
    FillChar(Handles^, FileList.Count * SizeOf(THandle), $FF);
    Count := 0;
    try
      DebugMsgFmt(L_MESG, SPlayingFiles, [FileList.Count]);
      { Prepare files }
      for I := 0 to FileList.Count - 1 do
      begin
        { Check file }
        if not FileExists(FileList[I]) then
        begin
          DebugMsgFmt(L_ERROR, SFileNotFound, [FileList[I]]);
          LastError := VME_FILE_NOT_FOUND;
          Result := S_FAIL;
          Exit;
        end;
        { Open file }
        Handles^[I] := FileOpen(FileList[I], fmOpenRead or fmShareDenyWrite);
        { Check file opening result }
        if Handles^[I] < 0 then
        begin
          DebugMsgFmt(L_ERROR, SFileOpenError, [FileList[I], GetLastError]);
          { Playing failed }
          LastError := VME_FILE_OPEN_ERROR;
          Result := S_FAIL;
          Exit;
        end;
        { File opened successfull }
        Inc(Count);
      end;
      { Update modem state }
      State := PLAYING;
      try
        { Get compression method of first file }
        Stream := THandleStream.Create(Handles^[0]);
        try
          DataLength := 0;
          { Read voice data header }
          if Assigned(ReadHeaderProc) then
            if ReadHeaderProc(Stream, Compression, Speed, Bits, DataLength) = S_FAIL then
            begin
              Result := S_FAIL;
              Exit;
            end;
        finally
          Stream.Free;
        end;
        { Set compression }
        if Assigned(SetCompressionProc) then
          if SetCompressionProc(Compression, Speed, Bits) = S_FAIL then
          begin
            Result := S_FAIL;
            Exit;
          end;
        { Start playing }
        if Driver.StartPlayStream = S_FAIL then
        begin
          DebugMsg(L_ERROR, SStartPlayStreamFailed);
          Result := S_FAIL;
          Exit;
        end;
        { Play files }
        for I := 0 to Count - 1 do
        begin
          Stream := THandleStream.Create(Handles^[I]);
          try
            DebugMsgFmt(L_MESG, SPlayingFile, [FileList[I]]);
            { Skip voice data header (not for first file) }
            if (I <> 0) and Assigned(ReadHeaderProc) then
              if ReadHeaderProc(Stream, Compression, Speed, Bits, DataLength) = S_FAIL then
                { Skip this file if error occurred }
                Continue;
            { Play file }
            if Driver.PlayStreamEx(Stream, Speed * Bits, DataLength) = S_FAIL then
            begin
              Result := S_FAIL;
              Exit;
            end;
          finally
            Stream.Free;
          end;
        end;
        { Stop playing }
        if Driver.StopPlayStream = S_FAIL then
        begin
          DebugMsg(L_ERROR, SStopPlayStreamFailed);
          Result := S_FAIL;
          Exit;
        end;
      finally
        State := IDLE;
      end;
    finally
      for I := 0 to Count - 1 do
        if not (Handles^[I] < 0) then FileClose(Handles^[I]);
    end;
  finally
    FreeMem(Handles, FileList.Count * SizeOf(THandle));
  end;
  { Success }
  Result := S_OK;
end;

function TVoiceModem.InternalPlayStream(Stream: TStream; Compression, Speed, Bits: Integer; ReadHeaderProc: TReadDataHeaderProc; SetCompressionProc: TSetCompressionProc): Integer;
var
  DataLength: Longint;
begin
  { Check driver specified }
  if not CheckDriver('InternalPlayStream', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Update modem state }
  State := PLAYING;
  try
    DataLength := 0;
    { Read voice data header }
    if Assigned(ReadHeaderProc) then
      if ReadHeaderProc(Stream, Compression, Speed, Bits, DataLength) = S_FAIL then
      begin
        Result := S_FAIL;
        Exit;
      end;
    { Set compression }
    if Assigned(SetCompressionProc) then
      if SetCompressionProc(Compression, Speed, Bits) = S_FAIL then
      begin
        Result := S_FAIL;
        Exit;
      end;
    { Start playing }
    if Driver.StartPlayStream = S_FAIL then
    begin
      DebugMsg(L_ERROR, SStartPlayStreamFailed);
      Result := S_FAIL;
      Exit;
    end;
    { Play stream }
    Result := Driver.PlayStreamEx(Stream, Speed * Bits, DataLength);
    { Check playing result }
    if Result = S_FAIL then
    begin
      DebugMsg(L_ERROR, SPlayStreamFailed);
      Exit;
    end;
    { Stop playing }
    if Driver.StopPlayStream = S_FAIL then
    begin
      DebugMsg(L_ERROR, SStopPlayStreamFailed);
      Result := S_FAIL;
      Exit;
    end;
  finally
    State := IDLE;
  end;
end;

function TVoiceModem.InternalRecordFile(const FileName: string; Compression, Speed, Bits: Integer; WriteHeaderProc: TWriteDataHeaderProc; SetCompressionProc: TSetCompressionProc): Integer;
var
  Handle: Integer;
  Stream: THandleStream;
begin
  DebugMsgFmt(L_MESG, SRecordingFile, [FileName]);
  { Create output file }
  Handle := FileCreate(FileName);
  { Check file create error }
  if Handle < 0 then
  begin
    DebugMsgFmt(L_ERROR, SFileCreateError, [FileName, GetLastError]);
    { Recording failed }
    LastError := VME_FILE_CREATE_ERROR;
    Result := S_FAIL;
    Exit;
  end;
  try
    { Recording file }
    Stream := THandleStream.Create(Handle);
    try
      Result := InternalRecordStream(Stream, Compression, Speed, Bits, WriteHeaderProc, SetCompressionProc);
    finally
      Stream.Free;
    end;
  finally
    FileClose(Handle);
  end;
end;

function TVoiceModem.InternalRecordStream(Stream: TStream; Compression, Speed, Bits: Integer; WriteHeaderProc: TWriteDataHeaderProc; SetCompressionProc: TSetCompressionProc): Integer;
var
  DataLength, HeaderPos, StreamPos: Longint;
begin
  { Check driver specified }
  if not CheckDriver('RecordStream', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Update modem state }
  State := RECORDING;
  try
    HeaderPos := 0;
    { Setup compression }
    if Assigned(SetCompressionProc) then
      if SetCompressionProc(Compression, Speed, Bits) = S_FAIL then
      begin
        Result := S_FAIL;
        Exit;
      end;
    { Write voice data header }
    if Assigned(WriteHeaderProc) then
    begin
      HeaderPos := Stream.Position;
      { Note: DataLength param is unknown now }
      if WriteHeaderProc(Stream, Compression, Speed, Bits, 0) = S_FAIL then
      begin
        Result := S_FAIL;
        Exit;
      end;
    end;
    { Start recording }
    if Driver.StartRecordStream = S_FAIL then
    begin
      DebugMsg(L_ERROR, SStartRecordStreamFailed);
      Result := S_FAIL;
      Exit;
    end;
    { Record stream }
    Result := Driver.RecordStreamEx(Stream, Speed * Bits, DataLength);
    { Check recording result }
    if Result = S_FAIL then
    begin
      DebugMsg(L_ERROR, SRecordStreamFailed);
      Exit;
    end;
    { Stop recording }
    if Driver.StopRecordStream = S_FAIL then
    begin
      DebugMsg(L_ERROR, SStopRecordStreamFailed);
      Result := S_FAIL;
      Exit;
    end;
    { Now we know voice data length so place it to the data header }
    if Assigned(WriteHeaderProc) then
    begin
      StreamPos := Stream.Position;
      { Return to header position }
      Stream.Position := HeaderPos;
      { Write header again }
      if WriteHeaderProc(Stream, Compression, Speed, Bits, DataLength) = S_FAIL then
      begin
        Result := S_FAIL;
        Exit;
      end;
      { Restore stream position }
      Stream.Position := StreamPos;
    end;
  finally
    State := IDLE;
  end;
end;

procedure TVoiceModem.PortError(Sender: TObject; Errors: Integer);
begin
  DebugMsgFmt(L_ERROR, SPortError, [Errors]);
  QueueEvent(TModemEvent.Create(SIGNAL_SIGQUIT));
end;

procedure TVoiceModem.Progress(Done: Longint);
begin
  if Assigned(FOnProgress) then
  try
    FOnProgress(Self, Done);
  except
    on E: Exception do DebugMsg(L_ERROR, 'Progress: ' + E.Message);
  end;
  { Yield event }
  YieldProc;
end;

procedure TVoiceModem.TraceChar(const Action: string; C: Char);

  procedure LineComplete;
  begin
    { Write trace to file }
    if (FTraceFile <> nil) and Profile.ModemLogging then FTraceFile.Add(FTraceLine);
    { Trace message }
    if Assigned(FOnTrace) then
    try
      FOnTrace(Self, FTraceAction, FTraceLine);
    except
      on E: Exception do DebugMsg(L_ERROR, 'Trace: ' + E.Message);
    end;
    { Clear trace line }
    FTraceLine := '';
  end;

begin
  { Check for CR and LF }
  case C of
    CR:
      { CR char }
      begin
        FTraceWasCR := True;
        Exit;
      end;
    LF:
      { LF char }
      if FTraceWasCR then
      begin
        { Check line complete }
        if FTraceAction = Action then
        begin
          FTraceWasCR := False;
          LineComplete;
        end
        else
          { Trace direction changed - ignore char }
          Exit;
      end
      else
        { Add char to trace line }
        FTraceLine := FTraceLine + MapControlCodes(C, '<>', @CODEMAP_HEX);
  else
    { Check single CR without LF or action reverse }
    if FTraceWasCR or ((FTraceAction <> Action) and (Length(FTraceLine) <> 0)) then
    begin
      FTraceWasCR := False;
      LineComplete;
    end;
    { Add char to trace line }
    FTraceLine := FTraceLine + MapControlCodes(C, '<>', @CODEMAP_HEX);
  end;
  { Save current action }
  FTraceAction := Action;
end;

procedure TVoiceModem.YieldProc;
begin
  if Assigned(FOnYield) then
  try
    FOnYield(Self);
  except
    on E: Exception do DebugMsg(L_ERROR, 'Yield: ' + E.Message);
  end;
end;

function TVoiceModem.Analyze(const Buffer, ExpectedAnswers: string): Integer;
var
  AnswerPos: Integer;
  Responses: string;
  CallerFmt: string;

  function FindAnswer(Answers: string; var AnswerPos: Integer): Boolean;
  var
    Current: Integer;
    CurrentAnswer: string;
    DividerPos: Integer;
  begin
    Current := 0;
    { Analyze answers }
    while Length(Answers) > 0 do
    begin
      DividerPos := Pos('|', Answers);
      { Get next expected answer }
      if DividerPos = 0 then
      begin
        CurrentAnswer := Answers;
        Answers := '';
      end
      else
      begin
        CurrentAnswer := Copy(Answers, 1, DividerPos - 1);
        Answers := Copy(Answers, DividerPos + 1, MaxInt);
      end;
      { Check buffer for current answer }
      if Pos(CurrentAnswer, Buffer) = 1 then
      begin
        AnswerPos := Current;
        Result := True;
        Exit;
      end;
      Inc(Current);
    end;
    { Answer not found }
    Result := False;
  end;

  function ExtractAnswer(Answers: string; Index: Integer): string;
  var
    DividerPos: Integer;
  begin
    { Locate answer }
    DividerPos := Pos('|', Answers);
    while (Index > 0) and (DividerPos <> 0) do
    begin
      Answers := Copy(Answers, DividerPos + 1, MaxInt);
      DividerPos := Pos('|', Answers);
      Dec(Index);
    end;
    if DividerPos = 0 then DividerPos := MaxInt;
    { Result }
    Result := Copy(Answers, 1, DividerPos);
  end;

begin
  { Check for buffer length }
  if Length(Buffer) = 0 then
  begin
    Result := VMA_EMPTY;
    Exit;
  end;
  { Analyze user answers }
  if FindAnswer(ExpectedAnswers, AnswerPos) then
  begin
    Result := VMA_USER + AnswerPos;
    Exit;
  end;
  { Driver-specific answer analyzing }
  if CheckDriver('Analyze', False) then
  begin
    Result := Driver.Analyze(Buffer);
    if Result <> VMA_UNKNOWN then Exit;
    { Check for Caller ID reporting }
    if FindAnswer(Driver.CallerIDFormat, AnswerPos) then
    begin
      CallerFmt := ExtractAnswer(Driver.CallerIDFormat, AnswerPos);
      LastCallerID := Trim(Copy(Buffer, Length(CallerFmt) + 1, MaxInt));
      Result := VMA_CALLERID;
      Exit;
    end;
    { Non-standard modem responses }
    Responses := UnMapControlCodes(Driver.Responses.Text, '<>', nil);
    if FindAnswer(Responses, AnswerPos) then
    begin
      Result := Driver.Responses[AnswerPos].AnswerCode;
      Exit;
    end;
  end;
  { Get standard modem answer }
  Result := AnalyzeModemAnswer(Buffer);
end;

function TVoiceModem.AnswerPhone: Integer;
begin
  DebugMsg(L_MESG, SAnsweringCall);
  { Check driver specified }
  if not CheckDriver('AnswerPhone', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Answer phone }
  Result := Driver.AnswerPhone;
end;

function TVoiceModem.Beep(Frequency, Length: Integer): Integer;
begin
  DebugMsg(L_MESG, SCheckingBeep);
  { Check driver specified }
  if not CheckDriver('Beep', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Beep }
  Result := Driver.Beep(Frequency, Length);
end;

function TVoiceModem.CheckDataFormat(DataFormat: Integer; var Compression, Speed, Bits: Integer;
  var ReadHeaderProc: TReadDataHeaderProc; var WriteHeaderProc: TWriteDataHeaderProc;
  var SetCompressionProc: TSetCompressionProc): Boolean;
begin
  Result := True;
  { Get format params }
  case DataFormat of
    DATA_FORMAT_RMD:
      { Raw Modem Data }
      begin
        Compression := Profile.RecCompression;
        Speed := Profile.RecSpeed;
        Bits := 8;
        if Profile.RawData then
        begin
          ReadHeaderProc := nil;
          WriteHeaderProc := nil;
        end
        else
        begin
          ReadHeaderProc := ReadRMDHeader;
          WriteHeaderProc := WriteRMDHeader;
        end;
        SetCompressionProc := SetCompression;
      end;
    DATA_FORMAT_WAVE:
      { WAVE Format }
      begin
        Compression := Profile.RecWaveFormat;
        Speed := Profile.RecWaveSpeed;
        Bits := 8;
        ReadHeaderProc := ReadWaveHeader;
        WriteHeaderProc := WriteWaveHeader;
        SetCompressionProc := SetWaveFormat;
      end;
  else
    DebugMsgFmt(L_ERROR, SUnknownDataFormat, [DataFormat]);
    LastError := VME_UNKNOWN_DATA_FORMAT;
    Result := False;
  end;
end;

function TVoiceModem.CheckDriver(const CheckFunc: string; DebugError: Boolean): Boolean;
begin
  Result := Driver <> nil;
  { Generate error }
  if (not Result) and DebugError then
  begin
    { Check driver must be created automatically }
    if Profile <> nil then
    begin
      if Length(Profile.ModemIniFile) <> 0 then
      begin
        { Can't load driver ini file }
        DebugMsgFmt(L_ERROR, SDriverIniFileError, [CheckFunc]);
        LastError := VME_DRIVER_INI_FILE_ERROR;
        Exit;
      end;
      if Length(Profile.ModemName) <> 0 then
      begin
        { Driver class is not registered for specified modem name }
        DebugMsgFmt(L_ERROR, SDriverNotRegistered, [CheckFunc]);
        LastError := VME_DRIVER_NOT_REGISTERED;
        Exit;
      end;
    end;
    { Driver property is not assigned }
    DebugMsgFmt(L_ERROR, SNoDriverError, [CheckFunc]);
    LastError := VME_DRIVER_NOT_SPECIFIED;
  end;
end;

function TVoiceModem.CheckEvents: Integer;
var
  Event: TModemEvent;
begin
  while UnqueueEvent(Event) = S_OK do
  begin
    if HandleEvent(Event.Event, Event.Data) = S_FAIL then
    begin
      DebugMsg(L_ERROR, SHandleEventFailed);
      LastError := VME_EVENT_NOT_HANDLED;
      Result := S_FAIL;
      Exit; {?}
    end;
    Event.Free;
  end;
  LastError := VME_SUCCESS;
  Result := S_OK;
end;

function TVoiceModem.CheckPort(const CheckFunc: string; DebugError: Boolean): Boolean;
begin
  Result := Port <> nil;
  { Generate error }
  if (not Result) and DebugError then
  begin
    DebugMsgFmt(L_ERROR, SNoPortError, [CheckFunc]);
    LastError := VME_PORT_NOT_SPECIFIED;
  end
end;

function TVoiceModem.Close: Integer;
begin
  { Check modem already closed }
  if not Opened then
  begin
    DebugMsg(L_MESG, SClosingPort);
    DebugMsg(L_WARN, SNoOpenedPort);
    { Close log files }
    if FLogFile <> nil then FLogFile.Close;
    if FTraceFile <> nil then FTraceFile.Close;
    { Close successfull }
    LastError := VME_SUCCESS;
    Result := S_OK;
    Exit;
  end;
  { Hang up the modem and close device }
  try
    HangUp;
  finally
    Result := CloseDevice;
  end;
end;

function TVoiceModem.CloseDevice: Integer;
begin
  DebugMsg(L_MESG, SClosingPort);
  { Check port specified }
  if not CheckPort('CloseDevice', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Check device opened }
  if Opened then
  begin
    { Close device }
    Port.Close;
    { Check port closing result }
    if Port.Enabled then
    begin
      DebugMsg(L_ERROR, SClosingPortError);
      LastError := VME_PORT_ERROR;
      Result := S_FAIL;
      Exit;
    end;
  end
  else
    DebugMsg(L_WARN, SNoOpenedPort);
  { Close log files }
  if FLogFile <> nil then FLogFile.Close;
  if FTraceFile <> nil then FTraceFile.Close;
  { Result }
  LastError := VME_SUCCESS;
  Result := S_OK;
end;

function TVoiceModem.Command(const Command, ExpectedAnswers: string): Integer;
var
  ExecuteCmd, ExecuteAnsw: string; 
  CmdLen, AnswLen: Integer;
  Buffer: string;
  WasEcho: Boolean;
begin
  DebugMsgFmt(L_NOISE, SCommand, [Command, ExpectedAnswers]);
  Result := S_OK;
  { Unmap control codes in the command and answers for executing }
  ExecuteCmd := UnMapControlCodes(Command, '<>', nil);
  ExecuteAnsw := UnMapControlCodes(ExpectedAnswers, '<>', nil);
  { Command delay }
  if Profile.CommandDelay > 0 then Sleep(Profile.CommandDelay);
  { Out command }
  CmdLen := Length(ExecuteCmd);
  if CmdLen <> 0 then
    if WriteStr(ExecuteCmd) = S_FAIL then
    begin
      Flush;
      Result := VMA_FAIL;
      Exit;
    end;
  { Process modem answer }
  AnswLen := Length(ExecuteAnsw);
  if ((CmdLen <> 0) and Profile.CommandEcho) or (AnswLen <> 0) then
  begin
    WasEcho := False;
    repeat
      { Read modem answer }
      if ReadStr(Buffer) = S_FAIL then
      begin
        Flush;
        Result := VMA_FAIL;
        Exit;
      end;
      { Check command echo }
      if (not WasEcho) and (CmdLen <> 0) then
      begin
        WasEcho := True;
        if Analyze(Buffer, ExecuteCmd) = VMA_USER_1 then
        begin
          { Partially fixing unexpected command echo problem }
          if not Profile.CommandEcho then
          begin
            Profile.CommandEcho := True;
            DebugMsg(L_WARN, SUnexpectedEchoError);
          end;
          { Check command completion }
          if AnswLen = 0 then Break else Continue;
        end
        else
          { Modem does not return echo }
          if Profile.CommandEcho then
          begin
            Profile.CommandEcho := False;
            DebugMsg(L_WARN, SNoEchoError);
          end;
      end;
      { Check expected answers }
      Result := Analyze(Buffer, ExecuteAnsw);
      { Test checking result }
      if Result = VMA_FAIL then
      begin
        DebugMsg(L_ERROR, SInvalidAnswer);
        Flush;
        LastError := VME_INVALID_MODEM_ANSWER;
        Exit;
      end;
      if Result = VMA_ERROR then
      begin
        DebugMsg(L_ERROR, SErrorReturned);
        Flush;
        LastError := VME_COMMAND_ERROR;
        Result := VMA_FAIL;
        Exit;
      end;
    until Result and VMA_USER = VMA_USER;
  end;
end;

function TVoiceModem.CommandFmt(const Command, ExpectedAnswers: string; const Args: array of const): Integer;
var
  FormattedCmd: string;
begin
  { Format command }
  try
    FormattedCmd := Format(Command, Args);
  except
    DebugMsg(L_ERROR, SInvalidCommandFormat);
    LastError := VME_INVALID_COMMAND_FORMAT;
    Result := VMA_FAIL;
    Exit;
  end;
  { Execute the command }
  Result := Self.Command(FormattedCmd, ExpectedAnswers);
end;

function TVoiceModem.DetectModemType: Integer;
begin
  LastError := VME_NOT_IMPLEMENTED;
  Result := S_FAIL;
end;

function TVoiceModem.Dial(const Number: string): Integer;
begin
  { Check driver specified }
  if not CheckDriver('Dial', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Update modem state }
  State := DIALING;
  try
    { Dial number }
    Result := Driver.Dial(Number);
  finally
     State := IDLE;
  end;
end;

function TVoiceModem.EnableCallerID(Enabled: Boolean): Integer;
begin
  DebugMsg(L_MESG, SEnablingCallerID);
  { Check driver specified }
  if not CheckDriver('EnableCallerID', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Enable/disable caller id detection }
  Result := Driver.EnableCallerID(Enabled);
end;

function TVoiceModem.EnableDistinctiveRing(Enabled: Boolean): Integer;
begin
  DebugMsg(L_MESG, SEnablingDRing);
  { Check driver specified }
  if not CheckDriver('EnableDistinctiveRing', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Enable/disable distinctive ring }
  Result := Driver.EnableDistinctiveRing(Enabled);
end;

function TVoiceModem.Flush: Integer;
begin
  Result := FlushEx(0);
end;

function TVoiceModem.FlushEvents: Integer;
var
  Event: TModemEvent;
begin
  while UnqueueEvent(Event) = S_OK do
  begin
    DebugMsgFmt(L_WARN, SFlushEvent, [Event.Event, MapControlCodes(Event.Data.C, '', @CODEMAP_DEC)]);
    Event.Free;
  end;
  LastError := VME_SUCCESS;
  Result := S_OK;
end;

function TVoiceModem.FlushEx(Timeout: Longint): Integer;
var
  StartTime: Longint;
  FlushChar: Char;
  FlushStr: string;
  FirstChar: Boolean;
begin
  { Check port specified }
  if not CheckPort('FlushEx', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  StartTime := Longint(GetTickCount);
  FlushStr := '';
  FirstChar := True;
  { Flush data by one char }
  while (Timeout = 0) or (Abs(Longint(GetTickCount) - StartTime) < Timeout) do
  begin
    { Check for more flush data }
    if not Port.CharReady then
    begin
      Sleep(Profile.PortStatusDelay + Profile.PortReadDelay);
      if not Port.CharReady then
        if Timeout = 0 then Break else Continue;
    end;
    { Read modem char }
    if Read(@FlushChar, 1) <> 1 then
    begin
      Result := S_FAIL;
      Exit;
    end;
    { Trace input char }
    TraceChar(SRecvAction, FlushChar);
    { Flush char event }
    if Assigned(FOnFlushChar) then
    try
      FOnFlushChar(Self, FlushChar);
    except
      on E: Exception do DebugMsg(L_ERROR, 'FlushChar: ' + E.Message);
    end;
    { Check for line break }
    if FlushChar in [LF, CR] then
    begin
      if not FirstChar then
      begin
        DebugMsgFmt(L_JUNK, SFlushString, [FlushStr]);
        { Flush string event }
        if Assigned(FOnFlushString) then
        try
          FOnFlushString(Self, FlushStr);
        except
          on E: Exception do DebugMsg(L_ERROR, 'FlushString: ' + E.Message);
        end;
      end;
      FlushStr := '';
      FirstChar := True;
    end
    else
      if FirstChar then
      begin
        FlushStr := FlushChar;
        FirstChar := False;
      end
      else
        FlushStr := FlushStr + FlushChar;
  end;
  { Check for lost debug message }
  if Length(FlushStr) > 0 then DebugMsgFmt(L_JUNK, SFlushString, [FlushStr]);
  { Success }
  Result := S_OK;
end;

function TVoiceModem.HandleDLE(Code: Char): Integer;
begin
  { Check driver specified }
  if not CheckDriver('HandleDLE', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Handle DLE code }
  Result := Driver.HandleDLE(Code);
end;

function TVoiceModem.HandleEvent(Event: Integer; Data: TEventData): Integer;
begin
  DebugMsgFmt(L_JUNK, SGotEvent, [Event, MapControlCodes(Data.C, '', @CODEMAP_DEC)]);
  { Driver handling }
  if CheckDriver('HandleEvent', True) then
  begin
    Result := Driver.HandleEvent(Event, Data);
    if Result <> UNKNOWN_EVENT then Exit;
  end;
  { User handling }
  if Assigned(FOnEvent) then
  begin
    Result := UNKNOWN_EVENT;
    try
      FOnEvent(Self, Event, Data, Result);
    except
      on E: Exception do DebugMsg(L_ERROR, 'HandleEvent: ' + E.Message);
    end;
    if Result <> UNKNOWN_EVENT then Exit;
  end;
  { Default handling }
  case Event of
    SIGNAL_STOP:
      begin
        StopCurrentAction;
        Result := S_OK;
        Exit;
      end;
  end;
  case State of
    DIALING:
      case Event of
        BUSY_TONE,
        DATA_CALLING_TONE,
        FAX_CALLING_TONE:
          begin
            Result := StopDialing;
            Exit;
          end;
      end;
    PLAYING:
      case Event of
        BUSY_TONE,
        DIAL_TONE,
        DATA_CALLING_TONE,
        FAX_CALLING_TONE:
          begin
            Result := StopPlaying;
            Exit;
          end;
      end;
    RECORDING:
      case Event of
        BUSY_TONE,
        DIAL_TONE,
        DATA_CALLING_TONE,
        FAX_CALLING_TONE,
        NO_VOICE_ENERGY:
          begin
            Result := StopRecording;
            Exit;
          end;
        SILENCE_DETECTED:
          if not Profile.RecIgnoreSilence then
          begin
            Result := StopRecording;
            Exit;
          end
          else
          begin
            Result := S_OK;
            Exit;
          end;
      end;
    WAITING:
      case Event of
        BUSY_TONE,
        DIAL_TONE,
        DATA_CALLING_TONE,
        FAX_CALLING_TONE,
        RING_DETECTED:
          begin
            Result := StopWaiting;
            Exit;
          end;
      end;
  end;
  { Unknown event  }
  DebugMsgFmt(L_WARN, SUnknownEvent, [Event, MapControlCodes(Data.C, '', @CODEMAP_DEC)]);
  Result := UNKNOWN_EVENT;
end;

function TVoiceModem.HangUp: Integer;
begin
  { Check driver specified }
  if not CheckDriver('HangUp', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Hang up (goes on hook) }
  Result := Driver.HangUp;
end;

function TVoiceModem.Init: Integer;
var
  EchoCmd, EchoAnsw: string;
begin
  { Update modem state }
  State := INITIALIZING;
  try
    { Sel local echo }
    EchoCmd := Format('ATE%d', [Ord(Profile.CommandEcho)]);
    EchoAnsw := EchoCmd + '|OK';
    if WriteStr(EchoCmd) = S_FAIL then
    begin
      DebugMsg(L_WARN, SlocalEchoError);
      Result := S_FAIL;
      Exit;
    end;
    if (Command('', EchoAnsw) and VMA_USER) <> VMA_USER then
    begin
      Result := S_FAIL;
      Exit;
    end;
    Flush;
    { Set verbal result messages }
    if Command('ATQ0V1', 'OK') <> VMA_USER_1 then
    begin
      DebugMsg(L_WARN, SResultCodesError);
      Result := S_FAIL;
      Exit;
    end;
    { Common params init complete }
    Result := S_OK;
    { Driver specific initialization }
    if CheckDriver('Init', False) then
    begin
      Result := Driver.Init;
      if Result = S_FAIL then Exit;
    end;
    { Additional initialization command }
    if Length(Profile.ModemInitCmd) > 0 then
      if Script(Profile.ModemInitCmd, Profile.ModemInitAnsw, True) <> S_OK then
        DebugMsg(L_WARN, SAdditionalCmdFailed);
  finally
    State := IDLE;
  end;
end;

function TVoiceModem.InitVoice: Integer;
begin
  { Check driver specified }
  if not CheckDriver('InitVoice', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Update modem state }
  State := INITIALIZING;
  try
    { Driver specific initialization }
    Result := Driver.InitVoice;
  finally
    State := IDLE;
  end;
end;

function TVoiceModem.Lock(Owner: TObject): Boolean;
begin
  EnterCriticalSection(FLockSection);
  { Check for modem already locked }
  Result := not FLocked;
  { Check lock owner }
  if Result then
  begin
    FLocked := True;
    FLockOwner := Owner;
  end;
  LeaveCriticalSection(FLockSection);
end;

function TVoiceModem.ModemName: string;
begin
  { Check driver specified }
  if not CheckDriver('ModemName', True) then
  begin
    Result := '';
    Exit;
  end;
  { Get modem name }
  Result := Driver.ModemName;
end;

function TVoiceModem.ModemRMDName: string;
begin
  { Check driver specified }
  if not CheckDriver('ModemRMDName', True) then
  begin
    Result := '';
    Exit;
  end;
  { Get modem name (for RMD file header) }
  Result := Driver.ModemRMDName;
end;

function TVoiceModem.Open: Integer;
begin
  { Check device already opened }
  if not Opened then
  begin
    { Open voice device }
    if OpenDevice = S_FAIL then
    begin
      Result := S_FAIL;
      Exit;
    end;
    { Init modem }
    if Init = S_FAIL then
    begin
      Result := S_FAIL;
      Exit;
    end;
    { Switch to voice mode }
    if VoiceModeOn = S_FAIL then
    begin
      Result := S_FAIL;
      Exit;
    end;
    { Init voice }
    if InitVoice = S_FAIL then
    begin
      Result := S_FAIL;
      Exit;
    end;
  end
  else
  begin
    { Device already opened }
    DebugMsgFmt(L_MESG, SOpeningPort, [Profile.Port]);
    DebugMsgFmt(L_WARN, SPortAlreadyOpen, [Ord(Port.ComNumber) + 1]);
  end;
  { Result }
  LastError := VME_SUCCESS;
  Result := S_OK;
end;

function TVoiceModem.OpenDevice: Integer;
begin
  DebugMsgFmt(L_MESG, SOpeningPort, [Profile.Port]);
  { Check port specified }
  if not CheckPort('OpenDevice', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Check for device closed }
  if not Opened then
  begin
    { Open port }
    if Port.Open = S_FAIL then
    begin
      DebugMsg(L_ERROR, SOpenPortError);
      LastError := VME_PORT_ERROR;
      Result := S_FAIL;
      Exit;
    end;
  end
  else
    DebugMsgFmt(L_WARN, SPortAlreadyOpen, [Ord(Port.ComNumber) + 1]);
  { Result }
  LastError := VME_SUCCESS;
  Result := S_OK;
end;

function TVoiceModem.PlayDTMF(const Number: string): Integer;
begin
  DebugMsg(L_MESG, SPlayingDTMF);
  { Check driver specified }
  if not CheckDriver('PlayDTMF', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Play DTMF tone }
  Result := Driver.PlayDTMF(Number);
end;

function TVoiceModem.PlayFile(const FileName: string): Integer;
var
  DataFormat: Integer;
  Compression, Speed, Bits: Integer;
  ReadHeaderProc: TReadDataHeaderProc;
  WriteHeaderProc: TWriteDataHeaderProc;
  SetCompressionProc: TSetCompressionProc;
begin
  { Detect data format }
  DataFormat := DetectFileFormatDef(FileName, Profile.VoiceDataFormat);
  { Check data format }
  if not CheckDataFormat(DataFormat, Compression, Speed, Bits,
    ReadHeaderProc, WriteHeaderProc, SetCompressionProc) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Play voice file }
  Result := InternalPlayFile(FileName, Compression, Speed, 8, ReadHeaderProc,
    SetCompressionProc);
end;

function TVoiceModem.PlayFiles(FileList: TStrings): Integer;
var
  DataFormat: Integer;
  Compression, Speed, Bits: Integer;
  ReadHeaderProc: TReadDataHeaderProc;
  WriteHeaderProc: TWriteDataHeaderProc;
  SetCompressionProc: TSetCompressionProc;
begin
  { Check files count }
  if FileList.Count = 0 then
  begin
    Result := S_OK;
    Exit;
  end;
  { Detect data format }
  DataFormat := DetectFileFormatDef(FileList[0], Profile.VoiceDataFormat);
  { Check data format }
  if not CheckDataFormat(DataFormat, Compression, Speed, Bits,
    ReadHeaderProc, WriteHeaderProc, SetCompressionProc) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Play files }
  Result := InternalPlayFiles(FileList, Compression, Speed, Bits,
    ReadHeaderProc, SetCompressionProc);
end;

function TVoiceModem.PlayStream(Stream: TStream): Integer;
var
  DataFormat: Integer;
  Compression, Speed, Bits: Integer;
  ReadHeaderProc: TReadDataHeaderProc;
  WriteHeaderProc: TWriteDataHeaderProc;
  SetCompressionProc: TSetCompressionProc;
begin
  { Detect file data format }
  DataFormat := DetectStreamFormatDef(Stream, Profile.VoiceDataFormat);
  { Check data format }
  if not CheckDataFormat(DataFormat, Compression, Speed, Bits,
    ReadHeaderProc, WriteHeaderProc, SetCompressionProc) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Play stream }
  Result := InternalPlayStream(Stream, Compression, Speed, Bits,
    ReadHeaderProc, SetCompressionProc);
end;

function TVoiceModem.QueueEvent(Event: TModemEvent): Integer;
var
  EventNumber: Integer;
begin
  EventNumber := FEventQueue.Queue(Event);
  DebugMsgFmt(L_JUNK, SQueuedEvent, [Event.Event, EventNumber]);
  LastError := VME_SUCCESS;
  Result := S_OK;
end;

function TVoiceModem.Read(Buffer: Pointer; Count: Integer): Integer;
var
  Timeout, StartTime: Longint;
begin
  { Check port opened }
  if not Opened then
  begin
    DebugMsg(L_ERROR, SBufferReadError);
    LastError := VME_PORT_NOT_OPENED;
    Result := S_FAIL;
    Exit;
  end;
  { Read data until success or timeout }
  Timeout := Profile.PortTimeout * 1000;
  StartTime := Longint(GetTickCount);
  while (Timeout = 0) or (Abs(Longint(GetTickCount) - StartTime) < Timeout) do
  begin
    Result := Port.Read(Buffer, Count);
    { Check reading result }
    if Result < 0 then
    begin
      DebugMsg(L_ERROR, SBufferReadError);
      LastError := VME_PORT_ERROR;
      Exit;
    end
    else if Result > 0 then
    begin
      { Read event }
      if Assigned(FOnRead) then
      try
        FOnRead(Self, Buffer, Result);
      except
        on E: Exception do DebugMsg(L_ERROR, 'Read: ' + E.Message);
      end;
      { Yield callback }
      YieldProc;
      { Success }
      LastError := VME_SUCCESS;
      Exit;
    end
    else
      Sleep(Profile.PortReadDelay);
    { Yield callback }
    YieldProc;
  end;
  { Reading timeout }
  DebugMsg(L_ERROR, SBufferReadTimeout);
  LastError := VME_TIMEOUT;
  Result := -1;
end;

function TVoiceModem.ReadChar(var CharIn: Char): Integer;
begin
  if Read(@CharIn, 1) < 0 then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Trace modem input }
  TraceChar(SRecvAction, CharIn);
  { Read char event }
  if Assigned(FOnReadChar) then
  try
    FOnReadChar(Self, CharIn);
  except
    on E: Exception do DebugMsg(L_ERROR, 'ReadChar: ' + E.Message);
  end;
  { Success }
  LastError := VME_SUCCESS;
  Result := S_OK;
  Exit;
end;

function TVoiceModem.ReadRMDHeader(Stream: TStream; var Compression, Speed, Bits: Integer; var DataLength: Longint): Integer;
var
  Header: TRMDHeader;
begin
  { Read header }
  if Stream.Read(Header, SizeOf(TRMDHeader)) <> SizeOf(TRMDHeader) then
  begin
    DebugMsgFmt(L_ERROR, SHeaderReadError, [GetLastError]);
    LastError := VME_STREAM_READ_ERR;
    Result := S_FAIL;
    Exit;
  end;
  { Check for header magic }
  if StrLComp(Header.Magic, RMD_HEADER_MAGIC, 4) <> 0 then
  begin
    DebugMsg(L_ERROR, SNotRMDFile);
    LastError := VME_NOT_RMD_FILE;
    Result := S_FAIL;
    Exit;
  end;
  { Check modem name }
  if StrLComp(Header.ModemType, PChar(ModemRMDName), Length(ModemRMDName)) <> 0 then
  begin
    DebugMsgFmt(L_ERROR, SWrongModemType, [Header.ModemType]);
    LastError := VME_WRONG_MODEM_TYPE;
    Result := S_FAIL;
    Exit;
  end;
  { Get data compression }
  Compression := Header.Compression;
  Speed := Header.Speed;
  Bits := Header.Bits;
  { GetDataLength }
  DataLength := Stream.Size - SizeOf(TRMDHeader);
  { Success }
  Result := S_OK;
end;

function TVoiceModem.ReadStr(var S: string): Integer;
var
  CharRead: Char;
  CharCount: Integer;
begin
  S := '';
  CharCount := 0;
  { Reading string }
  repeat
    { Read char }
    if ReadChar(CharRead) < 0 then
    begin
      Result := S_FAIL;
      Exit;
    end;
    { Check for <DLE> codes }
    if CharRead = DLE then
    begin
      { Read <DLE> code }
      if ReadChar(CharRead) = S_FAIL then
      begin
        Result := S_FAIL;
        Exit;
      end;
      DebugMsgFmt(L_JUNK, SReadDLE, [CharRead]);
      { Handle <DLE> code }
      HandleDLE(CharRead);
      Result := S_OK;
      Exit;
    end
    { Check for valid chars }
    else if not (CharRead in [LF, CR, XON, XOFF]) then
    begin
      S := S + CharRead;
      Inc(CharCount);
    end
    { Check for unexpected XON/XOFF codes }
    else if not (CharRead in [LF, CR]) then
      DebugMsgFmt(L_WARN, SUnexpectedByte, [CharRead]);
  until (CharRead = LF) and (CharCount > 0);
  { Read string event }
  if Assigned(FOnReadString) then
  try
    FOnReadString(Self, S);
  except
    on E: Exception do DebugMsg(L_ERROR, 'ReadString: ' + E.Message);
  end;
  { Success }
  DebugMsgFmt(L_JUNK, SReadString, [S]);
  Result := S_OK;
end;

function TVoiceModem.ReadWaveHeader(Stream: TStream; var WaveFormat, Speed, Bits: Integer; var DataLength: Longint): Integer;
var
  CkPos: Longint;
  CkID, CkSize: DWORD;
  pwfx: PWaveFormatEx;
  WasFmtChunk, WasDataChunk: Boolean;

  function StreamRead(var Buffer; Size: Longint): Boolean;
  begin
    Result := True;
    if Stream.Read(Buffer, Size) <> Size then
    begin
      DebugMsgFmt(L_ERROR, SWaveHeaderReadError, [GetLastError]);
      LastError := VME_STREAM_READ_ERR;
      Result := False;
    end;
  end;
                                
begin
  Result := S_FAIL;
  { RIFF chunk }
  if not StreamRead(CkID, 4) then Exit;
  if CkID <> FOURCC_RIFF then
  begin
    DebugMsg(L_ERROR, SNotRIFFFile);
    LastError := VME_NOT_RIFF_FILE;
    Exit;
  end;
  if not StreamRead(CkSize, 4) then Exit;
  { WAVE form type }
  if not StreamRead(CkID, 4) then Exit;
  if CkID <> FOURCC_WAVE then
  begin
    DebugMsg(L_ERROR, SNotWaveFile);
    LastError := VME_NOT_WAVE_FILE;
    Exit;
  end;
  { Search chunks }
  WasFmtChunk := False;
  WasDataChunk := False;
  CkPos := Stream.Position;
  while CkPos < Stream.Size do
  begin
    Stream.Position := CkPos;
    { Chunk ID, chunk size }
    if not StreamRead(CkID, 4) then Exit;
    if not StreamRead(CkSize, 4) then Exit;
    { Next chunk offset }
    CkPos := Longint(CkSize) + Stream.Position;
    { StreamRead chunk info }
    if CkID = FOURCC_FMT then
    begin
      WasFmtChunk := True;
      { Format Chunk }
      GetMem(pwfx, CkSize);
      try
        if not StreamRead(pwfx^, CkSize) then Exit;
        { Get compression info }
        with pwfx^ do
        begin
          if (wFormatTag <> WAVE_FORMAT_PCM) and
            (wFormatTag <> WAVE_FORMAT_ALAW) and
            (wFormatTag <> WAVE_FORMAT_MULAW) and
            (wFormatTag <> WAVE_FORMAT_IMA_ADPCM) then
          begin
            { Currently supported formats are PCM, A-Law, u-Law, IMA ADPCM }
            DebugMsgFmt(L_ERROR, SWrongWaveFormat, [wFormatTag]);
            LastError := VME_UNSUPPORTED_WAVE_FORMAT;
            Exit;
          end;
          WaveFormat := wFormatTag;
          Speed := nSamplesPerSec;
          Bits := wBitsPerSample;
        end;
      finally
        FreeMem(pwfx, CkSize);
      end;
    end
    else if CkID = FOURCC_DATA then
    begin
      WasDataChunk := True;
      { Data Chunk }
      if not WasFmtChunk then
      begin
        { Format Chunk must be before Data Chunk }
        DebugMsg(L_ERROR, SMissingFmtChunk);
        LastError := VME_MISSING_FMT_CHUNK;
        Exit;
      end;
      { Data size }
      DataLength := CkSize;
      Break;
    end;
  end;
  { Sheck errors }
  if not WasFmtChunk then
  begin
    { Dormat chunk not found }
    DebugMsg(L_ERROR, SFmtChunkNotFound);
    LastError := VME_MISSING_FMT_CHUNK;
    Exit;
  end;
  if not WasDataChunk then
  begin
    { Data chunk not found }
    DebugMsg(L_ERROR, SDataChunkNotFound);
    LastError := VME_MISSING_DATA_CHUNK;
    Exit;
  end;
  { Success }
  Result := S_OK;
end;

function TVoiceModem.RecordFile(const FileName: string): Integer;
var
  DataFormat: Integer;
  Compression, Speed, Bits: Integer;
  ReadHeaderProc: TReadDataHeaderProc;
  WriteHeaderProc: TWriteDataHeaderProc;
  SetCompressionProc: TSetCompressionProc;
begin
  DataFormat := Profile.VoiceDataFormat;
  { Detect data format by analysing file extension }
  if DataFormat = DATA_FORMAT_AUTO then
    DataFormat := DetectFileFormatDef(FileName, DATA_FORMAT_UNKNOWN);
  { Check data format }
  if not CheckDataFormat(DataFormat, Compression, Speed, Bits,
    ReadHeaderProc, WriteHeaderProc, SetCompressionProc) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Record voice file }
  Result := InternalRecordFile(FileName, Compression, Speed, Bits,
    WriteHeaderProc, SetCompressionProc);
end;

function TVoiceModem.RecordStream(Stream: TStream): Integer;
var
  DataFormat: Integer;
  Compression, Speed, Bits: Integer;
  ReadHeaderProc: TReadDataHeaderProc;
  WriteHeaderProc: TWriteDataHeaderProc;
  SetCompressionProc: TSetCompressionProc;
begin
  DataFormat := Profile.VoiceDataFormat;
  { Check data format }
  if not CheckDataFormat(DataFormat, Compression, Speed, Bits,
    ReadHeaderProc, WriteHeaderProc, SetCompressionProc) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Record voice stream }
  Result := InternalRecordStream(Stream, Compression, Speed, Bits,
    WriteHeaderProc, SetCompressionProc);
end;

function TVoiceModem.RestoreVoiceMode: Integer;
begin
  DebugMsg(L_MESG, SRestoringVoiceMode);
  { Check driver specified }
  if not CheckDriver('RestoreVoiceMode', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Restoring voice mode }
  Result := Driver.RestoreVoiceMode;
end;

function TVoiceModem.Script(const Commands, ExpectedAnswers: string; ExecuteAll: Boolean): Integer;
var
  ScriptCmd, ScriptAnsw: string;
  Current: Integer;
  CurrentCmd, CurrentAnsw: string;
  CmdPos, AnswPos: Integer;
begin
  DebugMsgFmt(L_NOISE, SScript, [Commands, ExpectedAnswers]);
  ScriptCmd := Commands;
  ScriptAnsw := ExpectedAnswers;
  Result := S_OK;
  Current := 0;
  { Process all script commands }
  while Length(ScriptCmd) > 0 do
  begin
    { Get next command }
    CmdPos := Pos('||', ScriptCmd);
    if CmdPos = 0 then
    begin
      CurrentCmd := ScriptCmd;
      ScriptCmd := '';
    end
    else
    begin
      CurrentCmd := Copy(ScriptCmd, 1, CmdPos - 1);
      ScriptCmd := Copy(ScriptCmd, CmdPos + 2, MaxInt);
    end;
    { Get next expected answer }
    AnswPos := Pos('||', ScriptAnsw);
    if AnswPos = 0 then
    begin
      CurrentAnsw := ScriptAnsw;
      ScriptAnsw := '';
    end
    else
    begin
      CurrentAnsw := Copy(ScriptAnsw, 1, AnswPos - 1);
      ScriptAnsw := Copy(ScriptAnsw, AnswPos + 2, MaxInt);
    end;
    { Execute command }
    if Command(CurrentCmd, CurrentAnsw) = VMA_FAIL then
    begin
      DebugMsgFmt(L_ERROR, SScriptError, [Current]);
      Result := VMA_FAIL + Current;
      { Error occured, exit }
      if not ExecuteAll then Exit;
    end;
    { Command processed }
    Inc(Current);
  end;
end;

function TVoiceModem.ScriptFmt(const Commands, ExpectedAnswers: string; ExecuteAll: Boolean; const Args: array of const): Integer;
var
  FormattedCmds: string;
begin
  { Format command Script }
  try
    FormattedCmds := Format(Commands, Args);
  except
    DebugMsg(L_ERROR, SInvalidCommandFormat);
    LastError := VME_INVALID_COMMAND_FORMAT;
    Result := VMA_FAIL;
    Exit;
  end;
  { Execute the command Script }
  Result := Script(FormattedCmds, ExpectedAnswers, ExecuteAll);
end;

function TVoiceModem.SetCompression(var Compression, Speed, Bits: Integer): Integer;
begin
  DebugMsg(L_MESG, SSettingCompression);
  { Check driver specified }
  if not CheckDriver('SetCompression', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Set compression mode }
  Result := Driver.SetCompression(Compression, Speed, Bits);
  { Check result }
  if Result = S_FAIL then
  begin
    DebugMsgFmt(L_ERROR, SCompressionError, [Compression, Speed]);
    Exit;
  end;
  { Success }
  DebugMsgFmt(L_MESG, SCompression, [Compression, Speed]);
end;

function TVoiceModem.SetDevice(Device: Integer): Integer;
begin
  DebugMsg(L_MESG, SSettingDevice);
  { Check driver specified }
  if not CheckDriver('SetDevice', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Set device }
  Result := Driver.SetDevice(Device);
end;

function TVoiceModem.SetReceiveGain(Gain: Integer): Integer;
begin
  DebugMsg(L_MESG, SSettingReceiveGain);
  { Check driver specified }
  if not CheckDriver('SetReceiveGain', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Set receive gain value }
  Result := Driver.SetReceiveGain(Gain);
end;

function TVoiceModem.SetTransmitGain(Gain: Integer): Integer;
begin
  DebugMsg(L_MESG, SSettingTransmitGain);
  { Check driver specified }
  if not CheckDriver('SetTransmitGain', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Set transmit gain value }
  Result := Driver.SetTransmitGain(Gain);
end;

function TVoiceModem.SetWaveFormat(var WaveFormat, Speed, Bits: Integer): Integer;
begin
  DebugMsg(L_MESG, SSettingWaveFormat);
  { Check driver specified }
  if not CheckDriver('SetWaveFormat', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Set compression for given wave format }
  Result := Driver.SetWaveFormat(WaveFormat, Speed, Bits);
  { Check result }
  if Result = S_FAIL then
  begin
    DebugMsgFmt(L_ERROR, SWaveFormatError, [WaveFormat, Speed]);
    Exit;
  end;
  { Success }
  DebugMsgFmt(L_MESG, SWaveFormat, [WaveFormat, Speed]);
end;

function TVoiceModem.Stop: Integer;
begin
  Result := QueueEvent(TModemEvent.Create(SIGNAL_STOP));
end;

function TVoiceModem.StopCurrentAction: Integer;
begin
  case State of
    DIALING: Result := StopDialing;
    PLAYING: Result := StopPlaying;
    RECORDING: Result := StopRecording;
    WAITING: Result := StopWaiting;
  else
    LastError := VME_SUCCESS;
    Result := S_OK;
  end;
end;

function TVoiceModem.StopDialing: Integer;
begin
  { Check driver specified }
  if not CheckDriver('StopDialing', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Stop dialing }
  Result := Driver.StopDialing;
end;

function TVoiceModem.StopPlaying: Integer;
begin
  { Check driver specified }
  if not CheckDriver('StopPlaying', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Stop playing }
  Result := Driver.StopPlaying;
end;

function TVoiceModem.StopRecording: Integer;
begin
  { Check driver specified }
  if not CheckDriver('StopRecording', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Stop recording }
  Result := Driver.StopRecording;
end;

function TVoiceModem.StopWaiting: Integer;
begin
  { Check driver specified }
  if not CheckDriver('StopWaiting', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Stop waiting }
  Result := Driver.StopWaiting;
end;

function TVoiceModem.SwitchToDataFax(Mode: string): Integer;
begin
  DebugMsg(L_MESG, SSwitchingToDataFax);
  { Check driver specified }
  if not CheckDriver('SwitchToDataFax', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Set data mode }
  Result := Driver.SwitchToDataFax(Mode);
end;

function TVoiceModem.Unlock: Boolean;
begin
  EnterCriticalSection(FLockSection);
  { Unlocking the modem }
  Result := FLocked;
  { Check result }
  if Result then
  begin
    FLocked := False;
    FLockOwner := nil;
  end;
  LeaveCriticalSection(FLockSection);
end;

function TVoiceModem.UnqueueEvent(var Event: TModemEvent): Integer;
var
  EventNumber: Integer;
begin
  EventNumber := FEventQueue.UnQueue(Event);
  if Event = nil then
  begin
    LastError := VME_EVENT_QUEUE_EMPTY;
    Result := S_FAIL;
    Exit;
  end;
  DebugMsgFmt(L_JUNK, SUnqueuedEvent, [Event.Event, EventNumber]);
  LastError := VME_SUCCESS;
  Result := S_OK;
end;

function TVoiceModem.VoiceModeOff: Integer;
begin
  DebugMsg(L_MESG, SLeavingVoiceMode);
  { Check driver specified }
  if not CheckDriver('VoiceModeOff', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Leaving voice mode }
  Result := Driver.VoiceModeOff;
end;

function TVoiceModem.VoiceModeOn: Integer;
begin
  DebugMsg(L_MESG, SEnteringVoiceMode);
  { Check driver specified }
  if not CheckDriver('VoiceModeOn', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Entering voice mode }
  Result := Driver.VoiceModeOn;
end;

function TVoiceModem.Wait(Timeout: Longint): Integer;
begin
  DebugMsgFmt(L_MESG, SWaiting, [Timeout]);
  { Check driver specified }
  if not CheckDriver('Wait', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Update modem state }
  State := WAITING;
  try
    { Wait event }
    Result := Driver.Wait(Timeout);
  finally
     State := IDLE;
  end;
end;

function TVoiceModem.WaitAnswer(const ExpectedAnswers: string; Timeout: Longint): Integer;
begin
  DebugMsgFmt(L_MESG, SWaitingAnswer, [ExpectedAnswers, Timeout]);
  { Check driver specified }
  if not CheckDriver('WaitAnswer', True) then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Update modem state }
  State := WAITING;
  try
    { Wait event }
    Result := Driver.WaitAnswer(ExpectedAnswers, Timeout);
  finally
     State := IDLE;
  end;
end;

function TVoiceModem.Write(Buffer: Pointer; Count: Integer): Integer;
var
  Timeout, StartTime: Longint;
begin
  { Check port opened }
  if not Opened then
  begin
    DebugMsg(L_ERROR, SBufferWriteError);
    LastError := VME_PORT_NOT_OPENED;
    Result := S_FAIL;
    Exit;
  end;
  { Write data until success or timeout }
  Timeout := Profile.PortTimeout * 1000;
  StartTime := Longint(GetTickCount);
  while ((Timeout = 0) or (Abs(Longint(GetTickCount) - StartTime) < Timeout)) and (Count > 0) do
  begin
    Result := Port.Write(Buffer, Count);
    { Check result }
    if Result < 0 then
    begin
      DebugMsg(L_ERROR, SBufferWriteError);
      LastError := VME_PORT_ERROR;
      Result := S_FAIL;
      Exit;
    end
    else if Result > 0 then
    begin
      { Write data event }
      if Assigned(FOnWrite) then
      try
        FOnWrite(Self, Buffer, Result);
      except
        on E: Exception do DebugMsg(L_ERROR, 'Write: ' + E.Message);
      end;
      { Shift buffer pointer }
      Buffer := Pointer(Longint(Buffer) + Result);
      Count := Count - Result;
    end
    else
      Sleep(Profile.PortWriteDelay);
    { Yield callback }
    YieldProc;
  end;
  { Check finally result }
  if Count = 0 then
  begin
    LastError := VME_SUCCESS;
    Result := S_OK;
    Exit;
  end;
  { Writing timeout }
  DebugMsg(L_ERROR, SBufferWriteTimeout);
  LastError := VME_TIMEOUT;
  Result := S_FAIL;
end;

function TVoiceModem.WriteChar(CharOut: Char): Integer;
begin
  { Writing char }
  if Write(@CharOut, 1) = S_FAIL then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Trace modem output }
  TraceChar(SSendAction, CharOut);
  { Write char event }
  if Assigned(FOnWriteChar) then
  try
    FOnWriteChar(Self, CharOut);
  except
    on E: Exception do DebugMsg(L_ERROR, 'WriteChar: ' + E.Message);
  end;
  { Success }
  Result := S_OK;
end;

function TVoiceModem.WriteCmd(const Command: string): Integer;
begin
  Result := WriteStr(UnMapControlCodes(Command, '<>', nil));
end;

function TVoiceModem.WriteRMDHeader(Stream: TStream; Compression, Speed, Bits: Integer; DataLength: Longint): Integer;
var
  Header: TRMDHeader;
begin
  FillChar(Header,  SizeOf(TRMDHeader), 0);
  { Format header data }
  StrPLCopy(Header.Magic, RMD_HEADER_MAGIC, 4);
  StrPLCopy(Header.ModemType, ModemRMDName, 16);
  Header.Compression := Compression;
  Header.Speed := Speed;
  Header.Bits := Bits;
  { Write header }
  if Stream.Write(Header, SizeOf(TRMDHeader)) <> SizeOf(TRMDHeader) then
  begin
    DebugMsgFmt(L_ERROR, SHeaderWriteError, [GetLastError]);
    LastError := VME_STREAM_WRITE_ERR;
    Result := S_FAIL;
    Exit;
  end;
  { Success }
  Result := S_OK;
end;

function TVoiceModem.WriteStr(const S: string): Integer;
var
  I: Integer;
begin
  DebugMsgFmt(L_JUNK, SWriteString, [S]);
  { Writing string }
  if Write(PChar(S), Length(S)) = S_FAIL then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Trace output }
  for I := 1 to Length(S) do TraceChar(SSendAction, S[I]);
  { Write string event }
  if Assigned(FOnWriteString) then
  try
    FOnWriteString(Self, S);
  except
    on E: Exception do DebugMsg(L_ERROR, 'WriteStr: ' + E.Message);
  end;
  { Writing line break }
  if WriteChar(CR) = S_FAIL then
  begin
    Result := S_FAIL;
    Exit;
  end;
  { Success }
  Result := S_OK;
end;

function TVoiceModem.WriteStrFmt(const Format: string; const Args: array of const): Integer;
var
  FormattedStr: string;
begin
  { Format string }
  try
    FormattedStr := SysUtils.Format(Format, Args);
  except
    DebugMsg(L_ERROR, SInvalidStrFormat);
    LastError := VME_INVALID_STR_FORMAT;
    Result := S_FAIL;
    Exit;
  end;
  { Write string }
  Result := WriteStr(FormattedStr);
end;

function TVoiceModem.WriteWaveHeader(Stream: TStream; WaveFormat, Speed, Bits: Integer; DataLength: Longint): Integer;
var
  nSize: Longint;
  pwfx: PWaveFormatEx;
  bps: DWORD;
  ckSize: Longint;

  function StreamWrite(const Buffer; Size: Longint): Boolean;
  begin
    Result := True;
    if Stream.Write(Buffer, Size) <> Size then
    begin
      DebugMsgFmt(L_ERROR, SWaveHeaderWriteError, [GetLastError]);
      LastError := VME_STREAM_WRITE_ERR;
      Result := False;
    end;
  end;

begin
  Result := S_FAIL;
  { Fmt chunk size }
  case WaveFormat of
    WAVE_FORMAT_PCM: nSize := SizeOf(TPCMWaveFormat);
    WAVE_FORMAT_ALAW: nSize := SizeOf(TWaveFormatEx);
    WAVE_FORMAT_MULAW: nSize := SizeOf(TWaveFormatEx);
    WAVE_FORMAT_IMA_ADPCM: nSize := SizeOf(TIMAADPCMWaveFormat);
  else
    { This wave format is not supported }
    DebugMsgFmt(L_ERROR, SUnsupportedWaveFormat, [WaveFormat]);
    LastError := VME_UNSUPPORTED_WAVE_FORMAT;
    Exit;
  end;
  { Alloc fmt chunk }
  GetMem(pwfx, nSize);
  try
    FillChar(pwfx^, nSize, 0);
    { Common fields }
    pwfx^.wFormatTag := WaveFormat;
    pwfx^.nChannels := 1;
    pwfx^.nSamplesPerSec := Speed;
    pwfx^.wBitsPerSample := Bits;
    { Format specific fields }
    case WaveFormat of
      WAVE_FORMAT_PCM:
        with PPCMWaveFormat(pwfx)^ do
        begin
          wf.nBlockAlign := pcmBlockAlignment(pwfx);
          wf.nAvgBytesPerSec := pcmAvgBytesPerSec(pwfx);
        end;
      WAVE_FORMAT_ALAW,
      WAVE_FORMAT_MULAW:
        with pwfx^ do
        begin
          bps := (wBitsPerSample + 7) div 8;
          nAvgBytesPerSec := nSamplesPerSec * nChannels * bps;
          nBlockAlign := nChannels * bps;
          cbSize := 0;
        end;
      WAVE_FORMAT_IMA_ADPCM:
        with PIMAADPCMWaveFormat(pwfx)^ do
        begin
          wfx.nBlockAlign := imaadpcmBlockAlign(pwfx);
          wSamplesPerBlock := imaadpcmSamplesPerBlock(pwfx);
          wfx.nAvgBytesPerSec := pcmAvgBytesPerSec(pwfx);
          wfx.cbSize := IMAADPCM_WFX_EXTRA_BYTES;
        end;
    end;
    { RIFF chunk size }
    ckSize := 4 + 8 + nSize + 8 + DataLength;
    { RIFF chunk }
    if not StreamWrite(FOURCC_RIFF, 4) then Exit;
    if not StreamWrite(ckSize, 4) then Exit;
    { WAVE form type }
    if not StreamWrite(FOURCC_WAVE, 4) then Exit;
    { Format chunk }
    if not StreamWrite(FOURCC_FMT, 4) then Exit;
    if not StreamWrite(nSize, 4) then Exit;
    if not StreamWrite(pwfx^, nSize) then Exit;
    { Data chunk }
    if not StreamWrite(FOURCC_DATA, 4) then Exit;
    if not StreamWrite(DataLength, 4) then Exit;
    { Success }
    Result := S_OK;
  finally
    FreeMem(pwfx, nSize);
  end;
end;

initialization

finalization
  FDriversList.Free;
  FDriversList := nil;

end.

